Package: projectLSA
Title: Shiny Application for Latent Structure Analysis with a Graphical
        User Interface
Version: 0.1.1
Authors@R: 
    c(
      person(
        given = "Hasan",
        family = "Djidu",
        email = "hasandjidu@gmail.com",
        role = c("aut", "cre"),
        comment = c(ORCID = "0000-0003-1110-6815")
      ),
      person(
        given = "Heri",
        family = "Retnawati",
        role = "ctb",
        comment = c(ORCID = "0000-0002-1792-5873")
      ),
      person(
        given = "Samsul",
        family = "Hadi",
        role = "ctb",
        comment = c(ORCID = "0000-0003-3437-2542")
      ),
      person(
        given = "Haryanto",
        family = "",
        role = "ctb",
        comment = c(ORCID = "0000-0003-3322-904X")
      )
    )
Description: Provides an interactive Shiny-based toolkit for conducting latent structure analyses, including Latent Profile Analysis (LPA), Latent Class Analysis (LCA), Latent Trait Analysis (LTA/IRT), Exploratory Factor Analysis (EFA), Confirmatory Factor Analysis (CFA), and Structural Equation Modeling (SEM). The implementation is grounded in established methodological frameworks: LPA is supported through 'tidyLPA' (Rosenberg et al., 2018) <doi:10.21105/joss.00978>, LCA through 'poLCA' (Linzer & Lewis, 2011) <doi:10.32614/CRAN.package.poLCA> & 'glca' (Kim & Kim, 2024) <doi:10.32614/CRAN.package.glca>, LTA/IRT via 'mirt' (Chalmers, 2012) <doi:10.18637/jss.v048.i06>, and EFA via 'psych' (Revelle, 2025). SEM and CFA functionalities build upon the 'lavaan' framework (Rosseel, 2012) <doi:10.18637/jss.v048.i02>. The CFA/SEM module additionally supports multi-group invariance testing, latent growth modelling, modification indices, and path diagram visualisation. Every module can save and restore an analysis session, export an 'R Markdown' HTML report, and consult an optional AI assistant that interprets the current results through a user-supplied large language model API key. Users can upload datasets or use built-in examples, fit models, compare fit indices, visualize results, and export outputs without programming.
License: MIT + file LICENSE
Depends: R (>= 4.0.0)
Encoding: UTF-8
Imports: shinyBS, colourpicker, data.table, dplyr, DT, flextable,
        ggiraph, ggplot2, glca, haven, httr, jsonlite, kableExtra,
        knitr, lavaan, magick, mclust, mirt, officer, plotly, poLCA,
        psych, purrr, readr, readxl, rlang, rmarkdown, scales, semPlot,
        semptools, shiny, shinycssloaders, shinyWidgets, stats,
        stringr, tibble, tidyr, tidyLPA, viridisLite, writexl
Suggests: pdftools, pkgdown, semTools, testthat (>= 3.0.0)
VignetteBuilder: knitr
URL: https://github.com/hdmeasure/projectLSA
BugReports: https://github.com/hdmeasure/projectLSA/issues
Config/roxygen2/version: 8.0.0
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-08-22 08:47:01 UTC; hasandjidu
Author: Hasan Djidu [aut, cre] (ORCID: <https://orcid.org/0000-0003-1110-6815>),
  Heri Retnawati [ctb] (ORCID: <https://orcid.org/0000-0002-1792-5873>),
  Samsul Hadi [ctb] (ORCID: <https://orcid.org/0000-0003-3437-2542>),
  Haryanto [ctb] (ORCID: <https://orcid.org/0000-0003-3322-904X>)
Maintainer: Hasan Djidu <hasandjidu@gmail.com>
Repository: CRAN
Date/Publication: 2026-08-22 09:20:08 UTC

# Generated by roxygen2: do not edit by hand

export(run_projectLSA)
import(shiny)
importFrom(DT,datatable)
importFrom(colourpicker,colourInput)
importFrom(data.table,data.table)
importFrom(dplyr,select)
importFrom(flextable,flextable)
importFrom(ggiraph,girafe)
importFrom(ggplot2,ggplot)
importFrom(glca,glca)
importFrom(haven,read_sav)
importFrom(httr,POST)
importFrom(jsonlite,fromJSON)
importFrom(kableExtra,kbl)
importFrom(knitr,knit)
importFrom(lavaan,sem)
importFrom(magick,image_read)
importFrom(mclust,Mclust)
importFrom(mirt,mirt)
importFrom(officer,read_docx)
importFrom(plotly,plot_ly)
importFrom(poLCA,poLCA)
importFrom(psych,fa)
importFrom(purrr,map)
importFrom(readr,read_csv)
importFrom(readxl,read_excel)
importFrom(rlang,.data)
importFrom(rmarkdown,render)
importFrom(scales,percent)
importFrom(semPlot,semPaths)
importFrom(semptools,mark_sig)
importFrom(shinyBS,bsTooltip)
importFrom(shinyWidgets,pickerInput)
importFrom(shinycssloaders,withSpinner)
importFrom(stats,sd)
importFrom(stringr,str_detect)
importFrom(tibble,tibble)
importFrom(tidyLPA,estimate_profiles)
importFrom(tidyr,pivot_longer)
importFrom(viridisLite,viridis)
importFrom(writexl,write_xlsx)

# projectLSA <img src="man/figures/logoProjectLSA.png" align="right" width="120" />

<!-- badges: start -->

[![CRAN
status](https://www.r-pkg.org/badges/version/projectLSA)](https://CRAN.R-project.org/package=projectLSA)
[![CRAN
release](https://www.r-pkg.org/badges/last-release/projectLSA)](https://CRAN.R-project.org/package=projectLSA)
[![Downloads](https://cranlogs.r-pkg.org/badges/projectLSA)](https://cranlogs.r-pkg.org/badges/projectLSA)
[![Total
Downloads](https://cranlogs.r-pkg.org/badges/grand-total/projectLSA)](https://cranlogs.r-pkg.org/badges/grand-total/projectLSA)

[![R-CMD-check](https://github.com/hdmeasure/projectLSA/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/hdmeasure/projectLSA/actions/workflows/R-CMD-check.yaml)
[![License:
MIT](https://img.shields.io/badge/License-MIT-blue.svg)](LICENSE)
[![Lifecycle:
stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html)

<!-- badges: end -->

projectLSA is an R package that provides a complete graphical user
interface (GUI) for conducting **Latent Structure Analysis (LSA)**
through a Shiny application. It integrates multiple latent variable
methods, including:

- **Latent Profile Analysis (LPA)**
- **Latent Class Analysis (LCA)**
- **Latent Trait Analysis (LTA / IRT)**
- **Exploratory Factor Analysis (EFA)**
- **Confirmatory Factor Analysis (CFA)**

All analyses can be performed **without writing any code**, making the
package accessible for researchers, students, and applied analysts.

------------------------------------------------------------------------

## Installation

``` r
# Install from CRAN (when available)
install.packages("projectLSA")

# Install development version from GitHub (optional)
remotes::install_github("hdmeasure/projectLSA")
```

------------------------------------------------------------------------

## Launch the Application

``` r
library(projectLSA)
run_projectLSA()
```

This opens the full Shiny application, including all LSA modules, data
upload, built-in datasets, interactive plots, and reporting features.

------------------------------------------------------------------------

## Video Tutorial

[![projectLSA – Installation and Quick
Start](https://img.youtube.com/vi/Rqj_ZPSXVaA/maxresdefault.jpg)](https://www.youtube.com/watch?v=Rqj_ZPSXVaA)

🎬 *Click the image to watch the installation and quick-start tutorial
for **projectLSA***.

------------------------------------------------------------------------

## Features

### ✔ Latent Profile Analysis (LPA)

- Upload your own dataset or use built-in examples.
- Fit multiple LPA models automatically.
- Compare AIC, BIC, entropy, and class size.
- Visualize the best model with customizable class names.

### ✔ Latent Class Analysis (LCA)

- Supports categorical indicators.
- Fits multiple class solutions.
- Interactive plots with **ggiraph**.
- Probability tables and class membership export.

### ✔ Latent Trait Analysis (LTA / IRT)

- Supports dichotomous and polytomous items.
- Automatically fits Rasch, 2PL, 3PL (or PCM/GRM/GPCM).
- ICC plots, test information, factor scores.
- Test information, marginal reliability, and conditional standard error
  of measurement.
- Differential item functioning (DIF) analysis with anchor-item
  selection.
- Score new respondents from a fitted model using a downloadable Excel
  template.
- Multi-dimensional visualization with 3D surfaces and heatmaps.

### ✔ Exploratory Factor Analysis (EFA)

- KMO, Bartlett test, parallel analysis.
- Factor extraction with rotation.
- Factor scores and loading matrix export.
- Data summary visualisations and composite-variable calculation.
- Score new respondents from a fitted solution.
- Clean HTML summaries for clearer interpretation.

### ✔ Confirmatory Factor Analysis (CFA / SEM)

- Lavaan model editor with a built-in methodological guide.
- Fit measures, loadings, factor scores, AVE/CR, and HTMT.
- Modification indices with an editable model-comparison history.
- **Multi-group CFA** with configural, metric, scalar, and strict
  invariance testing, plus a one-click automated all-levels comparison.
- **Latent growth modelling** via `lavaan::growth()`.
- Data summary visualisations, composite-variable calculation, and
  scoring of new data.
- Fully customized SEM path diagrams (full model or structural model
  only).

### ✔ Save & Restore Analysis Sessions

- Every module can save the whole analysis session (data, fitted models,
  and settings) to an `.rds` workspace file.
- Reload the workspace later by uploading it as the data source and
  continue where you left off.

### ✔ HTML Reports

- Every module renders an R Markdown report that you can preview in-app
  before downloading.
- Reports embed the underlying R console output and, optionally, an
  AI-written summary.

### ✔ AI Assistant (optional)

- A floating assistant available in every module: ask questions about
  the current results, or generate a summary in manuscript, APA 7,
  paragraph, bullet, or table format (English or Indonesian).
- Works with Google Gemini, OpenAI, Groq, or OpenRouter.
- **You supply your own API key at runtime.** It is used only for that
  session, is never stored by the package, and nothing is sent anywhere
  until you enter a key and ask a question. The application is fully
  usable without it.

------------------------------------------------------------------------

## Live Demo (Shiny Application)

All features of **projectLSA** can be explored through an interactive
Shiny web application.

👉 **Launch the live application:**  
<https://measure.shinyapps.io/ProjectLSA/>

The web interface provides access to Latent Profile Analysis (LPA),
Latent Class Analysis (LCA), Confirmatory Factor Analysis (CFA),
Structural Equation Modeling (SEM), and Latent Trait Analysis (IRT),
allowing users to explore the full workflow without local installation.

------------------------------------------------------------------------

## Citation

If you use projectLSA in publications, please cite the following:

**Journal Article (primary):**

> Djidu, H., Retnawati, H., Hadi, S., & Haryanto. (2026). projectLSA: A
> Shiny Application for Integrated Latent Structure Analysis. *Applied
> Psychological Measurement*.
> <https://doi.org/10.1177/01466216261446305>

**R Package:**

> Djidu, H., Retnawati, H., Hadi, S., & Haryanto (2026). *projectLSA: R
> Shiny application for latent structure analysis with a graphical user
> interface*. R package.
> <https://doi.org/10.32614/CRAN.package.projectLSA>

### BibTeX

``` bibtex
@article{Djidu2026projectLSA,
  title     = {projectLSA: A Shiny Application for Integrated Latent Structure Analysis},
  author    = {Djidu, Hasan and Retnawati, Heri and Hadi, Samsul and Haryanto},
  journal   = {Applied Psychological Measurement},
  year      = {2026},
  doi       = {10.1177/01466216261446305}
}
```

------------------------------------------------------------------------

## Contributing

Bug reports and feature requests are welcome:

<https://github.com/hdmeasure/projectLSA/issues>

------------------------------------------------------------------------

## License

MIT License © 2026 Hasan Djidu

# projectLSA <img src="man/figures/logoProjectLSA.png" align="right" width="120" />

<!-- badges: start -->

[![CRAN
status](https://www.r-pkg.org/badges/version/projectLSA)](https://CRAN.R-project.org/package=projectLSA)
[![CRAN
release](https://www.r-pkg.org/badges/last-release/projectLSA)](https://CRAN.R-project.org/package=projectLSA)
[![Downloads](https://cranlogs.r-pkg.org/badges/projectLSA)](https://cranlogs.r-pkg.org/badges/projectLSA)
[![Total
Downloads](https://cranlogs.r-pkg.org/badges/grand-total/projectLSA)](https://cranlogs.r-pkg.org/badges/grand-total/projectLSA)

[![R-CMD-check](https://github.com/hdmeasure/projectLSA/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/hdmeasure/projectLSA/actions/workflows/R-CMD-check.yaml)
[![License:
MIT](https://img.shields.io/badge/License-MIT-blue.svg)](LICENSE)
[![Lifecycle:
stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html)

<!-- badges: end -->

projectLSA is an R package that provides a complete graphical user
interface (GUI) for conducting **Latent Structure Analysis (LSA)**
through a Shiny application. It integrates multiple latent variable
methods, including:

- **Latent Profile Analysis (LPA)**
- **Latent Class Analysis (LCA)**
- **Latent Trait Analysis (LTA / IRT)**
- **Exploratory Factor Analysis (EFA)**
- **Confirmatory Factor Analysis (CFA)**

All analyses can be performed **without writing any code**, making the
package accessible for researchers, students, and applied analysts.

------------------------------------------------------------------------

## Installation

``` r
# Install from CRAN (when available)
install.packages("projectLSA")

# Install development version from GitHub (optional)
remotes::install_github("hdmeasure/projectLSA")
```

------------------------------------------------------------------------

## Launch the Application

``` r
library(projectLSA)
run_projectLSA()
```

This opens the full Shiny application, including all LSA modules, data
upload, built-in datasets, interactive plots, and reporting features.

------------------------------------------------------------------------

## Video Tutorial

[![projectLSA – Installation and Quick
Start](https://img.youtube.com/vi/Rqj_ZPSXVaA/maxresdefault.jpg)](https://www.youtube.com/watch?v=Rqj_ZPSXVaA)

🎬 *Click the image to watch the installation and quick-start tutorial
for **projectLSA***.

------------------------------------------------------------------------

## Features

### ✔ Latent Profile Analysis (LPA)

- Upload your own dataset or use built-in examples.
- Fit multiple LPA models automatically.
- Compare AIC, BIC, entropy, and class size.
- Visualize the best model with customizable class names.

### ✔ Latent Class Analysis (LCA)

- Supports categorical indicators.
- Fits multiple class solutions.
- Interactive plots with **ggiraph**.
- Probability tables and class membership export.

### ✔ Latent Trait Analysis (LTA / IRT)

- Supports dichotomous and polytomous items.
- Automatically fits Rasch, 2PL, 3PL (or PCM/GRM/GPCM).
- ICC plots, test information, factor scores.
- Multi-dimensional visualization with 3D surfaces and heatmaps.

### ✔ Exploratory Factor Analysis (EFA)

- KMO, Bartlett test, parallel analysis.
- Factor extraction with rotation.
- Factor scores and loading matrix export.
- Clean HTML summaries for clearer interpretation.

### ✔ Confirmatory Factor Analysis (CFA)

- Lavaan model editor.
- Fit measures, loadings, factor scores.
- Fully customized SEM path diagrams.

------------------------------------------------------------------------

## Live Demo (Shiny Application)

All features of **projectLSA** can be explored through an interactive
Shiny web application.

👉 **Launch the live application:**  
<https://measure.shinyapps.io/ProjectLSA/>

The web interface provides access to Latent Profile Analysis (LPA),
Latent Class Analysis (LCA), Confirmatory Factor Analysis (CFA),
Structural Equation Modeling (SEM), and Latent Trait Analysis (IRT),
allowing users to explore the full workflow without local installation.

------------------------------------------------------------------------

## Citation

If you use projectLSA in publications, please cite the following:

**Journal Article (primary):**

> Djidu, H., Retnawati, H., Hadi, S., & Haryanto. (2026). projectLSA: A
> Shiny Application for Integrated Latent Structure Analysis. *Applied
> Psychological Measurement*.
> <https://doi.org/10.1177/01466216261446305>

**R Package:**

> Djidu, H., Retnawati, H., Hadi, S., & Haryanto (2026). *projectLSA: R
> Shiny application for latent structure analysis with a graphical user
> interface*. R package.
> <https://doi.org/10.32614/CRAN.package.projectLSA>

### BibTeX

``` bibtex
@article{Djidu2026projectLSA,
  title     = {projectLSA: A Shiny Application for Integrated Latent Structure Analysis},
  author    = {Djidu, Hasan and Retnawati, Heri and Hadi, Samsul and Haryanto},
  journal   = {Applied Psychological Measurement},
  year      = {2026},
  doi       = {10.1177/01466216261446305}
}
```

------------------------------------------------------------------------

## Contributing

Bug reports and feature requests are welcome:

<https://github.com/hdmeasure/projectLSA/issues>

------------------------------------------------------------------------

## License

MIT License © 2026 Hasan Djidu

# ==== Reactive Data LCA ====
server_lca <- function(input, output, session) {
  library(poLCA)
  library(tidyverse)
  library(ggiraph)
  library(data.table)
  library(glca)
  library(stats)
  library(haven)
  
  best_c_r <- reactiveVal(NULL)
  set.seed(100)
observeEvent(input$run_lca, {
  req(data_lca(), input$vars_lca)
  updateTabsetPanel(session, "main_tab_lca", selected = "fit_tab_lca")
})
data_source <- reactive({
  if (input$data_source_lca == "cheat") {
    data("cheating", package = "poLCA")
    df <- cheating %>% rownames_to_column("id_auto") 
  } else if (input$data_source_lca == "lazar") {
    freq <- data.frame(
      V1   = c(1, 1, 1, 1, 2, 2, 2, 2),
      V2   = c(1, 1, 2, 2, 1, 1, 2, 2),
      V3   = c(1, 2, 1, 2, 1, 2, 1, 2),
      Freq = c(220, 160, 60, 160, 60, 60, 60, 220)
    )
    df <- freq[rep(seq_len(nrow(freq)), freq$Freq), c("V1", "V2", "V3")]
    rownames(df) <- NULL
    df$id_auto <- seq_len(nrow(df))
  
  } else if(input$data_source_lca == "gss82") {
    data("gss82", package = "poLCA")
    df <- gss82 %>% rownames_to_column("id_auto") %>%
      dplyr::mutate(
        PURPOSE  = dplyr::recode(PURPOSE,  "Waste of time"=1,"Depends"=2,"Good"=3),
        ACCURACY = dplyr::recode(ACCURACY, "Not true"=1,"Mostly true"=2),
        UNDERSTA = dplyr::recode(UNDERSTA, "Fair/Poor"=1,"Good"=2),
        COOPERAT = dplyr::recode(COOPERAT, "Impatient"=1,"Cooperative"=2,"Interested"=3)
      )
    
  } else {
    req(input$datafile_lca)
    showModal(modalDialog(title = NULL, "Reading Your File, Please wait...", footer = NULL, easyClose = FALSE))
    ext <- tolower(tools::file_ext(input$datafile_lca$name))
    df <- switch(
      ext,
      "csv"  = data.table::fread(input$datafile_lca$datapath,data.table = FALSE),
      "xls"  = readxl::read_excel(input$datafile_lca$datapath),
      "xlsx" = readxl::read_excel(input$datafile_lca$datapath),
      "sav"  = haven::read_sav(input$datafile_lca$datapath),
      "rds"  = readRDS(input$datafile_lca$datapath),
      stop("Unsupported file type. Please upload CSV, Excel, SPSS (.sav), or RDS file.")
    )
    removeModal()
    df <- df %>% mutate(across(everything(), ~ifelse(.x=="", NA, .x)),
                        id_auto = paste0("id_", sprintf("%04d", 1:n())))
  }
  return(df)
})
data_lca <- reactive({
  df <- data_source()
  # Jika pakai covariat → buang baris NA di covariat
  if (isTRUE(input$use_cov_lca) && length(input$cov_lca) > 0) {
    df <- df %>% tidyr::drop_na(all_of(input$cov_lca))
  }
  
  df
})

# ==== Pilih ID ====
output$id_select_ui_lca <- renderUI({
  req(data_source())
  selectInput(
    "id_lca",
    label = "Select ID Columns (Optional):",
    choices = names(data_lca()),
    selected = names(data_source())[str_detect(names(data_source()), "id")],
    multiple = TRUE
    )
})

# ==== Pilih variabel ====
output$var_select_ui_lca <- renderUI({
  req(data_source())
  selectInput(
    "vars_lca",
    label = "Select Variables for LCA:",
    choices = names(data_source()),
    selected = names(data_source()%>% dplyr::select(-c(id_auto)))[1:min(2,ncol(data_source()))],
    multiple = TRUE  )
})

output$cov_lca_ui <- renderUI({
  req(data_source())
  
  tagList(
    checkboxInput(
      "use_cov_lca",
      label = "Include Covariate?",
      value = FALSE
    ),
    
    conditionalPanel(
      condition = "input.use_cov_lca == true",
      selectInput(
        "cov_lca",
        label = "Select Covariate (Numeric):",
        choices = names(data_source()),
        multiple = TRUE
      )
    )
  )
})


observeEvent(c(input$data_source_lca, input$datafile_lca), {
  updateSelectInput(session,"vars_lca",selected = "") 
}, ignoreInit = TRUE)

# ==== Preview Data ====
output$data_preview_lca <- DT::renderDT({
  req(data_lca(), input$vars_lca)
  df <- data_lca() %>% dplyr::select(input$vars_lca)
  numeric_cols <- which(sapply(df, function(x) is.numeric(x)))
  
  DT::datatable(df,#extensions = 'Buttons',
                options = list(scrollX = TRUE, dom = 'rtp',
                               # buttons = list(
                               #   list(
                               #     extend = 'csv',
                               #     text = 'Export CSV',
                               #     filename = paste0('Data LCA')  
                               #   ),
                               #   list(
                               #     extend = 'excel',
                               #     text = 'Export Excel',
                               #     filename = paste0('Data LCA')
                               #   ))
                               ),
                rownames = TRUE) %>% 
    formatRound(columns = numeric_cols, digits = 0)
}, server = TRUE)


output$data_summary_lca <- DT::renderDT({
  req(data_lca(), input$vars_lca)
  df <- data_lca() %>% dplyr::select(input$vars_lca)
  # Pastikan semuanya character (aman untuk join)
  df_chr <- df %>% dplyr::mutate(across(everything(), as.character))
  # Buat kolom join (kombinasi respons)
  df_chr$Respon <- apply(df_chr, 1, paste0, collapse = "-")
  
  # Tabel frekuensi
  freq_table <- df_chr %>%
    dplyr::count(Respon, name = "Freq") %>%
    dplyr::mutate(
      Percent = round(100 * Freq / sum(Freq), 2)
    ) %>%
    dplyr::arrange(dplyr::desc(Freq))
  
  DT::datatable(freq_table,extensions = 'Buttons',
                options = list(scrollX = TRUE, dom = 'Brtp',
                               buttons = list(
                                 list(
                                   extend = 'csv',
                                   text = 'Export CSV',
                                   filename = paste0('Data LCA')  
                                 ),
                                 list(
                                   extend = 'excel',
                                   text = 'Export Excel',
                                   filename = paste0('Data LCA')
                                 ))),
                rownames = TRUE)
}, server = FALSE)

output$data_description <- renderUI({
  desc_html <- if (input$data_source_lca == "cheat") {
    desc_cheating
  } else if (input$data_source_lca == "lazar") {
    desc_lazar
  } else if (input$data_source_lca == "gss82") {
    desc_gss82
  } else {
    ""
  }
  
  div(
    class = "badge-info",
    HTML(desc_html)
  )
})

# ==== Fit LCA ====
lca_models <- eventReactive(input$run_lca, { 
  req(data_lca(), input$vars_lca)
  df <- data_lca()
  showModal(modalDialog(title = NULL, "Please wait, (Running LCA)...", footer = NULL, easyClose = FALSE))
  
  # =========================
  # 1. Tentukan kolom yang dipakai
  # =========================
  vars_ind  <- input$vars_lca
  vars_cov  <- if (isTRUE(input$use_cov_lca)) input$cov_lca else NULL

  used_vars <- unique(c(vars_ind, vars_cov))

  dat <- df[, used_vars, drop = FALSE]

  # =========================
  # 2. Konversi indikator → factor
  # =========================
  dat <- dat %>%
    mutate(across(all_of(vars_ind), as.factor))

  # =========================
  # 3. Kovariat: JANGAN dipaksa factor
  #    (biarkan numeric tetap numeric)
  # =========================
  # OPTIONAL: jika mau auto-detect kovariat nominal
  if (!is.null(vars_cov)) {
    dat <- dat %>%
      mutate(across(
        all_of(vars_cov),
        ~ if (is.numeric(.x) && dplyr::n_distinct(.x) <= 3) {
          as.factor(.x)
        } else {
          .x
        }
      ))
  }

  # formula <- as.formula(paste("cbind(", paste(input$vars_lca, collapse=","), ") ~ 1"))
  # =========================
  # FORMULA (dengan / tanpa covariat)
  # =========================
  if (isTRUE(input$use_cov_lca) && length(input$cov_lca) > 0) {

    formula_polca <- as.formula(
      paste("cbind(", paste(input$vars_lca, collapse = ","), ") ~ ",
        paste(input$cov_lca)
      )
    )
    formula_glca <- as.formula(
      paste("item(", paste(input$vars_lca, collapse = ","), ") ~ ",
            paste(input$cov_lca)
      )
    )
  } else {
    formula_polca <- as.formula(
      paste("cbind(", paste(input$vars_lca, collapse = ","), ") ~ 1")
    )
    formula_glca <- as.formula(
      paste("item(", paste(input$vars_lca, collapse = ","), ") ~ 1")
    )
  }

  min_k <- input$min_class_lca
  max_k <- input$max_class_lca
  # model_list <- list()
  polca_models <- list()
  glca_models  <- list()
  withProgress(message = "Estimating LCA models...", {
    for (k in min_k:max_k) {
      
      incProgress(1 / (max_k - min_k + 1),
                  detail = paste("Classes:", k))
      
      polca_models[[as.character(k)]] <- tryCatch(
        poLCA::poLCA(formula = formula_polca, data= dat, nclass = k, verbose = FALSE),
        error = function(e) NULL
      )
      
      glca_models[[as.character(k)]] <- tryCatch(
        glca::glca(formula=formula_glca, data=dat, nclass = k, seed = 1, verbose = FALSE),
        error = function(e) NULL
      )
    }
  })
  # =========================
  # GOFGLCA (SEMUA MODEL SEKALIGUS)
  # =========================
  withProgress(message = "Comparing Multiple Model...", {
  glca_valid <- glca_models[!sapply(glca_models, is.null)]
  gof <- if (length(glca_valid) >= 2) {
    do.call(
      glca::gofglca,
      c(
        unname(glca_valid), 
        list(test = "boot", seed = 1)
      )
    )
  } else {
    NULL
  }
  gtable <- as.data.frame(gof$gtable) %>% dplyr::rename(p_Gsq = `Boot p-value`)
  dtable <- as.data.frame(gof$dtable) %>% dplyr::rename(p_Deviance = `Boot p-value`) %>% dplyr::select(Deviance,p_Deviance)
  gof_glca <- cbind(gtable,dtable)
  if (input$min_class_lca == 1) {
    fit_1_class <- data.frame(
      loglik     = polca_models[["1"]]$llik,
      AIC        = polca_models[["1"]]$aic,
      BIC        = polca_models[["1"]]$bic,
      entropy    = NA,
      df         = polca_models[["1"]]$resid.df,
      Gsq        = polca_models[["1"]]$Gsq,
      p_Gsq      = NA,
      Deviance   = NA,
      p_Deviance = NA)
    gof_glca <- rbind(fit_1_class, gof_glca)
  }
  })
  removeModal()
  list(
    polca = polca_models,
    glca   = gof_glca
  )
})


# FUNCTION TO CALCULATION OF LCA -----
APCP_poLCA <- function(fit) {
  post <- fit$posterior
  cls  <- fit$predclass
  K <- ncol(post)
  APCP_class <- sapply(1:K, function(k) {
    mean(post[cls == k, k], na.rm=TRUE) })
  APCP_overall <- mean(post[cbind(1:nrow(post), cls)])
  APCP_byclass <- mean(APCP_class,  na.rm = TRUE)
  min_Prob <- min(post[cbind(1:nrow(post), cls)])
  list(
    APCP_per_class = APCP_class,
    APCP_overall  = APCP_overall,
    APCP_byClass = APCP_byclass,
    min_Prob = min_Prob
  )
}
# Smallest Class Size ===
smallest_class <- reactive({ 
  req(lca_models())
  purrr::map_df(names(lca_models()$polca), function(k) {
    model_k <- lca_models()$polca[[k]]
    membership <- model_k$predclass
    class_table <- as.data.frame(table(membership))
    names(class_table) <- c("Class", "N")
    
    class_table$Percent <- round(100 * class_table$N / sum(class_table$N), 1)
    min_class <- class_table %>% slice_min(N, n = 1)
    min_class$N_class <- as.numeric(k)
    min_class
  })
})


# ==== Fit Table ====

# fit_lca <- reactive({
#   req(lca_models())
#   
#   fit <- data.frame(
#     N_class = integer(),
#     Smallest_class_size = numeric(),
#     Av_Prob = numeric()
#   )
#   
#   gof <- lca_models()$glca
#   model_polca <- lca_models()$polca
#   smallest_df <- smallest_class()
#   
#   for (k in names(model_polca)) {
#     n <- model_polca[[k]]
#     if (!is.null(n)) {
#       
#       scs <- smallest_df %>%
#         dplyr::filter(N_class == as.numeric(k)) %>%
#         dplyr::pull(Percent)
#       
#       fit <- rbind(
#         fit,
#         data.frame(
#           N_class = as.numeric(k),
#           Av_Prob = round(APCP_poLCA(n)$APCP_byClass, 2),
#           Smallest_class_size = scs
#         )
#       )
#     }
#   }
#   
#   cbind(fit, gof)
# })

fit_lca <- reactive({
  req(lca_models())
  
  fit <- data.frame(
    N_class = integer(),
    Smallest_class_size = numeric(),
    Av_Prob = numeric()
  )
  
  gof <- lca_models()$glca
  model_polca <- lca_models()$polca
  smallest_df <- smallest_class()
  
  for (k in names(model_polca)) {
    n <- model_polca[[k]]
    if (!is.null(n)) {
      
      scs <- smallest_df %>%
        dplyr::filter(N_class == as.numeric(k)) %>%
        dplyr::pull(Percent)
      
      fit <- rbind(
        fit,
        data.frame(
          N_class = as.numeric(k),
          Av_Prob = round(APCP_poLCA(n)$APCP_byClass, 2),
          Smallest_class_size = scs
        )
      )
    }
  }
  
  cbind(fit, gof)
})


# ==== Fit Table ====
output$fit_table_lca <- renderDT({
    df <- fit_lca() %>% 
      dplyr::select(N_class, loglik:p_Deviance, Smallest_class_size, Av_Prob) %>% 
      dplyr::mutate(
        dplyr::across(
          .cols = -c(N_class,df),
          .fns  = ~ round(.x, 2)
        )
      )
    aic_vals <- sort(unique(df$AIC))
    bic_vals <- sort(unique(df$BIC))
    ent_vals <- sort(unique(df$entropy), decreasing = TRUE)
    apcp_vals  <- sort(unique(df$Av_Prob), decreasing = TRUE)

    DT::datatable(df,extensions = 'Buttons',
                  options = list(scrollX = TRUE, dom = 'B',
                                 buttons = list(
                                   list(
                                     extend = 'csv',
                                     text = 'Export CSV',
                                     filename = paste0('Fit Comparison LCA')  
                                   ),
                                   list(
                                     extend = 'excel',
                                     text = 'Export Excel',
                                     filename = paste0('Fit Comparison LCA')
                                   ))), 
                  rownames = FALSE) %>% 
      formatStyle('AIC',
                  backgroundColor = styleEqual(
                    c(aic_vals[1], aic_vals[2]),
                    c('lightgreen', 'khaki')
                  ),
                  fontWeight = styleEqual(aic_vals[1], 'bold')
      ) %>%
      formatStyle('BIC',
                  backgroundColor = styleEqual(
                    c(bic_vals[1], bic_vals[2]),
                    c('lightgreen', 'khaki')
                  ),
                  fontWeight = styleEqual(bic_vals[1], 'bold')
      ) %>%
      formatStyle('entropy',
                  backgroundColor = styleInterval(c(0.7, 0.8), c('lightcoral', 'khaki', 'lightgreen')),
                  fontWeight      = styleInterval(c(0.7, 0.8), c('bold', 'bold',''))
      ) %>%
      formatStyle('Av_Prob',
                    backgroundColor = styleInterval(c(0.8, 0.9), c('lightcoral', 'khaki', 'lightgreen')),
                    fontWeight      = styleInterval(c(0.8, 0.9), c('bold', 'bold',''))
      ) %>% 
      formatStyle(
        'p_Gsq',
        backgroundColor = styleInterval(0.05, c('lightcoral', 'lightgreen')),
        fontWeight      = styleInterval(0.05, c('bold', ''))
      ) %>%
      formatStyle(
        'p_Deviance',
        backgroundColor = styleInterval(0.05, c('lightgreen', 'lightcoral')),
        fontWeight      = styleInterval(0.05, c('', 'bold'))
      ) %>% 
      formatStyle(
        'Smallest_class_size',
        backgroundColor = styleInterval(c(5, 7), c('lightcoral', 'khaki', 'lightgreen')),
        fontWeight      = styleInterval(c(5, 7), c('bold', 'bold',''))
      )
}, server = FALSE)

# ==== Fit Plot (AIC/BIC) ====
fit_plot_lca_reactive <- reactive({
  req(lca_models())
  fit <- fit_lca() %>% dplyr::select(N_class, BIC, AIC)
  fit_long <- fit %>% pivot_longer(-N_class, names_to="Index", values_to="Value")
  ggplot(fit_long, aes(x=N_class, y=Value, color=Index, group=Index)) +
    geom_line(size = 1.2) + geom_point(size = 3) +
    geom_text(aes(label = round(Value, 2)), vjust = -0.6, size = 3) +
    labs(title = "BIC & AIC Comparison", x = "Number of Class", y = "Fit Index") +
    theme_minimal(base_size = 14) + 
    theme(legend.position = "bottom", axis.line = element_line(color = "black"))
})
# Render Plot
output$fit_plot_lca <- renderPlot({
  fit_plot_lca_reactive()
})

# ==== Download Buttons ====
output$download_plot_AicBic_LCA <- make_download_plot(
  plot_reactive = fit_plot_lca_reactive,
  filename_prefix = "Fit BIC & AIC Plot_LCA"
)

# ==== Entropy Plot  ====
entropy_plot_lca_reactive <- reactive({
  req(lca_models())
  
  fit <- fit_lca() %>%
    dplyr::select(N_class, entropy, Av_Prob) %>%
    tidyr::pivot_longer(-N_class, names_to = "Index", values_to = "Value")
  
  ggplot(
    fit,
    aes(
      N_class, Value,
      color = Index, linetype = Index, shape = Index, group = Index
    )
  ) +
    geom_line(size = 1.2) +
    geom_point(size = 3) +
    geom_text(aes(label = round(Value, 2)), vjust = -0.6, size = 3, show.legend = FALSE) +
    scale_color_manual(values = c(entropy = "black", Av_Prob = "blue")) +
    scale_shape_manual(values = c(entropy = 17, Av_Prob = 15)) +
    scale_linetype_manual(values = c(entropy = "solid", Av_Prob = "dashed")) +
    labs(
      title = "Entropy & Average Posterior Probability",
      x = "Number of Class",
      y = "Value"
    ) +
    theme_minimal(base_size = 14) +
    theme(legend.position = "bottom", axis.line = element_line(color = "black"))
})

# Render Plot
output$entropy_plot_lca <- renderPlot({
  entropy_plot_lca_reactive()
})

# ==== Download Buttons ====
output$download_plot_entropy <- make_download_plot(
  plot_reactive = entropy_plot_lca_reactive,
  filename_prefix = "Entropy & Av. Posterior Probability Plot_LCA"
)

observeEvent(input$best_class_lca, {
  req(input$best_class_lca)
  best_c_r(as.numeric(input$best_class_lca))
}, ignoreInit = TRUE)

# ==== Input nama class ====
output$class_name_inputs_lca <- renderUI({
  req(input$best_class_lca)
  lapply(1:input$best_class_lca, function(i) {
    textInput(paste0("class_name_", i), paste("Class", i, "Name:"), value = paste("Class", i))
  })
})

# === 1. Fungsi pembuat plot ===
build_best_model_plot_lca <- function(lca_models, k, vars_lca, class_names, interactive = TRUE) {
  model_k <- lca_models[[as.character(k)]]
  probs <- model_k$probs
  
  prob <- data.frame(matrix("", ncol = 2 + as.numeric(k), nrow = 0))
  names(prob) <- c("Var", "Level", paste0("Prob.class", 1:as.numeric(k)))
  
  for (j in seq_along(vars_lca)) {
    a <- data.frame(
      Var = vars_lca[j],
      Level = colnames(probs[[j]]),
      t(probs[[j]])
    )
    a <- setNames(a, names(prob))
    prob <- rbind(prob, a)
  }
  
  prob_gathered <- prob %>%
    tidyr::gather(key = "class", value = "probability", 3:ncol(prob)) %>%
    dplyr::mutate(
      class = gsub("Prob.", "", class),
      probability = round(as.numeric(probability), 3),
      Var = factor(Var, levels = vars_lca)
    )# %>%
    # dplyr::arrange(Var)
  
  prob_gathered$class <- factor(
    prob_gathered$class,
    levels = paste0("class", 1:as.numeric(k)),
    labels = class_names
  )
  
  if (interactive) {
    gg <- ggplot(prob_gathered,
                 aes(
                   x = Var, y = probability, fill = Level,
                   tooltip = paste0(
                     "Class: ", class,
                     "<br>Variable: ", Var,
                     "<br>Level: ", Level,
                     "<br>Probability: ", probability
                   ),
                   data_id = class
                 )) +
      geom_bar_interactive(stat = "identity", position = "stack")
  } else {
    gg <- ggplot(prob_gathered,
                 aes(x = Var, y = probability, fill = Level)) +
      geom_bar(stat = "identity", position = "stack")
  }
  
  gg +
    facet_grid(~ class) +
    ylab("Probability") + xlab("") +
    theme_minimal(base_size = 13) +
    theme(
      axis.text.x = element_text(angle = 90, hjust = 1, vjust = 0.5, size = 8),
      panel.grid.major.x = element_blank(),
      panel.grid.minor.x = element_blank(),
      strip.text = element_text(face = "bold", size = 11)
    )
}

# === 2. Reactive untuk plot interaktif ===
best_model_plot_lca_reactive <- reactive({
  req(lca_models(), input$best_class_lca, input$vars_lca)
  k <- input$best_class_lca
  class_names <- sapply(1:k, function(i) input[[paste0("class_name_", i)]])
  if (length(class_names) == 0) class_names <- paste("Class", 1:k)
  
  build_best_model_plot_lca(lca_models()$polca, k, input$vars_lca, class_names, interactive = TRUE)
})

# === 3. Render plot interaktif ===
output$best_model_plot_lca <- renderGirafe({
  k <- input$best_class_lca
  filename_plot <- paste0("LCA_best_model_", k, "class")
  
  ggiraph::girafe(
    ggobj = best_model_plot_lca_reactive(),
    width_svg = 9, height_svg = 5,
    options = list(
      opts_hover(css = "fill-opacity:1;"),
      opts_tooltip(css = "background-color:white;color:black;
                    border:1px solid #ccc;padding:6px;
                    font-size:11px;border-radius:5px;"),
      opts_toolbar(saveaspng = TRUE, pngname = filename_plot)
    )
  )
})

# === 4. Reactive versi statis untuk download ===
best_model_plot_lca_static <- reactive({
  req(lca_models(), input$best_class_lca, input$vars_lca)
  k <- input$best_class_lca
  class_names <- sapply(1:k, function(i) input[[paste0("class_name_", i)]])
  if (length(class_names) == 0) class_names <- paste("Class", 1:k)
  
  build_best_model_plot_lca(lca_models()$polca, k, input$vars_lca, class_names, interactive = FALSE)
})


# === 3. Download handler panggil plot statis ===
output$download_plot_best_LCA <- make_download_plot(
  plot_reactive = best_model_plot_lca_static,
  filename_prefix = paste0("LCA_best_model_", input$best_class_lca, "class")
)

# ==== Summary table ====
summary_data_lca <- reactive({
  req(lca_models(), input$best_class_lca, input$vars_lca)
  
  k <- as.numeric(input$best_class_lca)
  model_k <- lca_models()$polca[[as.character(k)]]
  req(model_k)
  
  # Ambil nama class dari input (fallback jika belum diisi)
  class_names <- sapply(1:k, function(i) {
    nm <- input[[paste0("class_name_", i)]]
    if (is.null(nm) || nm == "") paste0("Class ", i) else nm
  })
  
  # === 1. Ukuran kelas ===
  class_size <- data.frame(
    base::table(model_k$predclass)) %>%
    dplyr::rename(Class = Var1, N = Freq) %>%
    dplyr::mutate(Percent=round(100*N/sum(N),2))
  class_size$Class <- class_names[class_size$Class]
  
  # === 2. Probabilitas kategori (adaptasi dari plot Girafe) ===
  probs <- model_k$probs
  listvar <- input$vars_lca
  
  prob <- data.frame(matrix("", ncol = 2 + k, nrow = 0))
  names(prob) <- c("Variable", "Category", paste0("Prob.", class_names))
  
  for (j in seq_along(listvar)) {
    a <- data.frame(
      Variable = listvar[j],
      Category = colnames(probs[[j]]),
      t(probs[[j]])
    )
    names(a) <- names(prob)
    prob <- rbind(prob, a)
  }
  
  prob <- prob %>%
    mutate(across(starts_with("Prob."), as.numeric)) %>%
    arrange(Variable)
  
  # Gabungkan class size + probabilitas (tidak merge, tapi list agar bisa dua tabel)
  list(
    class_size = class_size,
    probability = prob
  )
})


# ==== Render table ====
output$summary_table_lca <- renderUI({
  req(summary_data_lca())
  dat <- summary_data_lca()
  
  tagList(
    tags$h5("Class Size"),
    DTOutput("class_size_table_lca"),
    tags$h5("Item-Categories Probabilities"),
    DTOutput("probability_table_lca")
  )
})

output$class_size_table_lca <- renderDT({
  df <- summary_data_lca()$class_size
  DT::datatable(df,extensions = 'Buttons',
                options = list(scrollX = TRUE, dom = 'B',
                               buttons = list(
                                 list(
                                   extend = 'csv',
                                   text = 'Export CSV',
                                   filename = paste0('Class Size')
                                 ),
                                 list(
                                   extend = 'excel',
                                   text = 'Export Excel',
                                   filename = paste0('Class Size')
                                 ))),
                rownames = FALSE)
})

output$probability_table_lca <- renderDT({
  df <- summary_data_lca()$probability
  numeric_cols <- which(sapply(df, function(x) is.numeric(x)))
  DT::datatable(df,extensions = 'Buttons',
                options = list(scrollX = TRUE, pageLength=25,
                               dom = 'Brtp',
                               buttons = list(
                                 list(
                                   extend = 'csv',
                                   text = 'Export CSV',
                                   filename = paste0('Class Probability')
                                 ),
                                 list(
                                   extend = 'excel',
                                   text = 'Export Excel',
                                   filename = paste0('Class Probability')
                                 ))),
                rownames = FALSE) %>%
    formatRound(columns = numeric_cols, digits = 2)
})

# df_out <- reactive({
#   req(lca_models(), input$best_class_lca)
#   k <- as.numeric(input$best_class_lca)
#   # Ambil nama class dari input (fallback jika belum diisi)
#   class_names <- sapply(1:k, function(i) {
#     nm <- input[[paste0("class_name_", i)]]
#     if (is.null(nm) || nm == "") paste0("Class ", i) else nm
#   })
#   model_k <- lca_models()$polca[[as.character(input$best_class_lca)]]
#   req(model_k)
#   
#   df <- data_lca()
#   df$Class <- model_k$predclass
#   df$Class <- class_names[df$Class]
#   df$Respon <- df %>%
#     tidyr::unite("Respon", all_of(input$vars_lca), sep = "-", remove = FALSE) %>%
#     dplyr::pull(Respon)
#   
#   # --- Posterior probabilities ---
#   posterior_df <- as.data.frame(round(model_k$posterior,2))
#   colnames(posterior_df) <- paste0("Posterior_Prob_", class_names)
#   numeric_cols <- which(sapply(posterior_df, is.numeric))
#   
#   # Gabungkan: ID + vars + posterior + Class
#   df <- cbind(
#     df,
#     posterior_df
#   )
#   df
#   
# })
df_out <- reactive({
  req(lca_models(), input$best_class_lca)
  
  k <- as.numeric(input$best_class_lca)
  
  # --- Class names ---
  class_names <- sapply(1:k, function(i) {
    nm <- input[[paste0("class_name_", i)]]
    if (is.null(nm) || nm == "") paste0("Class ", i) else nm
  })
  
  model_k <- lca_models()$polca[[as.character(input$best_class_lca)]]
  req(model_k)
  
  df <- data_lca()
  n <- nrow(df)
  
  # --- Tentukan baris yang benar-benar dipakai LCA ---
  vars_used <- input$vars_lca
  valid_idx <- complete.cases(df[, vars_used])
  
  # --- Inisialisasi kolom hasil (panjang = data asli) ---
  df$Class <- NA_character_
  
  # Isi hanya untuk baris valid
  df$Class[valid_idx] <- class_names[model_k$predclass]
  
  # --- Respon pattern (aman walau ada NA) ---
  df$Respon <- df |>
    tidyr::unite("Respon", all_of(vars_used), sep = "-", remove = FALSE) |>
    dplyr::pull(Respon)
  
  # --- Posterior probabilities ---
  posterior_mat <- model_k$posterior
  posterior_df <- matrix(NA, nrow = n, ncol = k)
  posterior_df[valid_idx, ] <- round(posterior_mat, 2)
  
  posterior_df <- as.data.frame(posterior_df)
  colnames(posterior_df) <- paste0("Posterior_Prob_", class_names)
  
  # --- Gabungkan ke data asli ---
  df <- cbind(df, posterior_df)
  
  df
})

# Data Summary Respon_With Class
output$data_summary_lca_with_class <- DT::renderDT({
  req(df_out(), input$best_class_lca)
  df <- df_out()
  # Tabel frekuensi
  freq_table <- df %>%
    dplyr::count(Respon,Class, name = "Freq") %>%
    dplyr::mutate(
      Percent = round(100 * Freq / sum(Freq), 2)
    ) %>%
    dplyr::arrange(dplyr::desc(Freq))
  
  DT::datatable(freq_table,extensions = 'Buttons',
                options = list(scrollX = TRUE, pageLength=25,
                               dom = 'Brtp',
                               buttons = list(
                                 list(
                                   extend = 'csv',
                                   text = 'Export CSV',
                                   filename = paste0('Data LCA')  
                                 ),
                                 list(
                                   extend = 'excel',
                                   text = 'Export Excel',
                                   filename = paste0('Data LCA')
                                 ))),
                rownames = TRUE)
}, server = FALSE)


# ==== Summary Table & Class per ID ====
output$profile_table_lca <- renderDT({
  req(lca_models(), input$best_class_lca)
  df <- df_out()
  DT::datatable(df,extensions = 'Buttons',
                options = list(scrollX = TRUE, pageLength=10,
                               dom = 'Brtp',
                               buttons = list(
                                 list(
                                   extend = 'csv',
                                   text = 'Export CSV',
                                   filename = paste0('LCA Result with ', input$best_class_lca, ' Classes')
                                 ),
                                 list(
                                   extend = 'excel',
                                   text = 'Export Excel',
                                   filename = paste0('LCA Result with', input$best_class_lca, ' Classes')
                                 ))),
                rownames = TRUE)
  
}, server = FALSE)

output$var_x_ui <- renderUI({
  req(df_out())
  selectInput(
    "var_x",
    label = "X Absis",
    choices = names(df_out()),
    selected = "Class",
    multiple = FALSE 
  )
})
output$var_y_ui <- renderUI({
  req(df_out())
  selectInput(
    "var_y",
    label = "Y Absis",
    choices = setdiff(names(df_out()), "Class"),
    selected = "",
    multiple = TRUE
  )
})
output$cross_latent <- renderPlot({
  req(lca_models(), input$best_class_lca, input$var_x, input$var_y)
  df <- df_out()
  # =========================
  # 1. RINGKAS DATA
  # =========================
  Summary_long <- df %>%
    dplyr::select(all_of(c(input$var_x, input$var_y))) %>%
    tidyr::pivot_longer(
      cols = all_of(input$var_y),
      names_to = "Variable",
      values_to = "Value"
    ) %>%
    dplyr::group_by(
      Class  = .data[[input$var_x]],
      Variable
    ) %>%
    dplyr::summarise(
      Mean = mean(Value, na.rm = TRUE),
      .groups = "drop"
    ) %>% 
    dplyr::mutate(
      Variable = factor(Variable, levels = input$var_y)
    )
  
  # =========================
  # 2. BASE PLOT
  # =========================
  p <- ggplot(
    Summary_long,
    aes(
      x = Variable,
      y = Mean,
      fill = Class,
      color = Class,
      group = Class
    )
  )
  # =========================
  # 3. SWITCH BAR / LINE
  # =========================
  if (input$plot_type == "bar") {
    
    p <- p +
      geom_col(
        position = position_dodge(width = 0.7),
        width = 0.6
      )
  } else {
    p <- p +
      geom_line(
        aes(linetype = Class),
        size = 1.2
      ) +
      geom_point(
        aes(shape = Class),
        size = 3
      )
  }
  
  # =========================
  # 4. FINAL STYLE
  # =========================
  p +
    geom_text(
      aes(label = round(Mean, 2)),
      position = position_dodge(width = 0.7),
      vjust = -0.7,
      size = 4.5,
      show.legend = FALSE
    ) +
    labs(
      title = paste0("Class Plot for ", input$best_class_lca, " Classes"),
      x = "Variable",
      y = "Mean Value"
    ) +
    ylim(0, max(Summary_long$Mean, na.rm = TRUE) * 1.18) +
    
    theme_minimal(base_size = 10) +
    theme(
      axis.text.x = element_text(angle = 90, hjust = 1, vjust = 0.5, size = 9),
      legend.position = "bottom",
      axis.line = element_line(color = "black")
    )
  })

output$reg_lca_summary <- renderTable ({
  req(lca_models(), input$best_class_lca, input$use_cov_lca)
  
  model_k <- lca_models()$polca[[as.character(input$best_class_lca)]]
  req(model_k$coeff)
  
  coef_mat <- model_k$coeff
  coef_df <- as.data.frame(coef_mat)
  coef_df$Covariate <- rownames(coef_df)
  
  coef_df <- coef_df %>%
    relocate(Covariate) %>%
    mutate(across(where(is.numeric), round, 3))
  coef_df
})
output$reg_lca_plot <- renderPlot({
  req(
    lca_models(),
    input$best_class_lca,
    input$use_cov_lca,
    input$cov_lca
  )
  
  # === HANYA UNTUK 2 KELAS ===
  if (as.numeric(input$best_class_lca) != 2) {
    return(NULL)
  }
  
  model_k <- lca_models()$polca[[as.character(input$best_class_lca)]]
  beta <- model_k$coeff
  coef_names <- rownames(beta)
  
  # === Pilih kovariat ===
  cov_name <- input$cov_lca[1]
  cov_vals <- sort(unique(data_lca()[[cov_name]]))
  
  # === Design matrix X ===
  X <- data.frame(matrix(0, nrow = length(cov_vals), ncol = length(coef_names)))
  colnames(X) <- coef_names
  
  if ("(Intercept)" %in% coef_names) {
    X[["(Intercept)"]] <- 1
  }
  if (cov_name %in% coef_names) {
    X[[cov_name]] <- cov_vals
  }
  
  # === Probabilities ===
  eta <- as.matrix(X) %*% beta
  p_class2 <- as.vector(exp(eta) / (1 + exp(eta)))
  p_class1 <- 1 - p_class2
  
  prob_df <- data.frame(
    Covariate = cov_vals,
    `Class 1` = p_class1,
    `Class 2` = p_class2
  )
  
  prob_long <- tidyr::pivot_longer(
    prob_df,
    cols = c("Class 1", "Class 2"),
    names_to = "Class",
    values_to = "Probability"
  )
  
  # === Plot ===
  ggplot(prob_long, aes(Covariate, Probability, color = Class)) +
    geom_line(size = 1.2) +
    geom_point(size = 3) +
    scale_y_continuous(limits = c(0, 1)) +
    labs(
      x = cov_name,
      y = "Predicted Class Membership Probability",
      title = "Effect of Covariate on Latent Class Membership (2-Class Model)"
    ) +
    theme_minimal(base_size = 14) +
    theme(
      legend.position = "bottom",
      axis.line = element_line(color = "black")
    )
})
output$anova_lca_ui <- renderUI({
  req(lca_models(), input$best_class_lca, input$var_x, input$var_y)
  
  if (isTRUE(input$check_anova)) {
    tagList(
      tags$h5(
        paste0(
          "Latent Class Differences on ",
          paste(input$var_y, collapse = ", ")
        )
      ),
      tags$h6("Analysis of Variance"),
      tableOutput("anova_lca_table"),
      uiOutput("posthoc_lca_ui")
    )
  }
})
output$anova_lca_table <- renderTable({
  req(df_out(), input$var_x, input$var_y)
  
  df <- df_out()
  x  <- input$var_x
  ys <- input$var_y
  
  # =========================
  # CASE 1: ONE-WAY ANOVA
  # =========================
  if (length(ys) == 1) {
    
    fml <- as.formula(paste(ys, "~", x))
    fit <- aov(fml, data = df)
    tbl <- summary(fit)[[1]]
    
    pval <- tbl["Pr(>F)"][1,1]
    
    out <- data.frame(
      Source = rownames(tbl),
      round(tbl, 4),
      row.names = NULL
    )
    
    out$Interpretation <- c(
      ifelse(
        pval < 0.05,
        "Significant difference",
        "No significant difference"
      ),
      rep("", nrow(out) - 1)
    )
    
    out
    
  } else {
    
    # =========================
    # CASE 2: MANOVA
    # =========================
    
    fml <- as.formula(
      paste(
        "cbind(",
        paste(ys, collapse = ","),
        ") ~ ",
        x
      )
    )
    
    fit <- manova(fml, data = df)
    tbl <- summary(fit, test = "Wilks")$stats
    pval <- tbl[1, "Pr(>F)"]
    
    out <- data.frame(
      Effect = rownames(tbl),
      round(tbl, 4),
      row.names = NULL
    )
    
    out$Interpretation <- ifelse(
      out$Effect == x,
      ifelse(
        pval < 0.05,
        "Significant difference",
        "No significant difference"
      ),
      ""
    )
    
    out
  }
})
output$posthoc_lca_ui <- renderUI({
  req(df_out(), input$var_x, input$var_y)
  
  if (length(input$var_y) == 1) {
    tagList(
      tags$h6("Post-hoc Test (Tukey HSD)"),
      tableOutput("posthoc_lca_table")
    )
  }
})
output$posthoc_lca_table <- renderTable({
  req(df_out(), input$var_x, input$var_y)
  
  # Post-hoc hanya untuk ANOVA
  if (length(input$var_y) != 1) return(NULL)
  
  df <- df_out()
  x  <- input$var_x
  y  <- input$var_y
  
  fml <- as.formula(paste(y, "~", x))
  fit <- aov(fml, data = df)
  
  # p-value ANOVA utama
  p_main <- summary(fit)[[1]]["Pr(>F)"][1, 1]
  
  # Jika tidak signifikan → tampilkan pesan
  if (is.na(p_main) || p_main >= 0.05) {
    return(
      data.frame(
        Note = "Post-hoc test was not conducted because the ANOVA result was not statistically significant."
      )
    )
  }
  
  tuk <- TukeyHSD(fit)[[1]]
  tuk <- round(as.data.frame(tuk), 4)
  
  data.frame(
    Comparison = rownames(tuk),
    tuk,
    Interpretation = ifelse(
      tuk$`p adj` < 0.05,
      "Significant difference",
      "Not significant"
    ),
    row.names = NULL
  )
})
output$reg_lca_ui <- renderUI({
  req(input$use_cov_lca)
  
  if (isTRUE(input$use_cov_lca) && length(input$cov_lca) > 0) {
    tagList(
      tags$h5("Covariate Effects on Class Membership"),
      tableOutput("reg_lca_summary"),
      br(),
      plotOutput("reg_lca_plot", height = "400px")
    )
  }
})

# CrossTab====
output$crosstab_ui <- renderUI({
  req(input$crosstab_result)
  if (!input$crosstab_result) return(NULL)
  
  req(df_out())
  vars <- setdiff(names(df_out()), "Class")
  fluidRow(
    column(
      12,
      h5("Cross Tabulation Settings"),
      selectInput(
        "crosstab_vars",
        "Select Variables:",
        choices = vars,
        multiple = TRUE
      )
    ),
    column(
      6,
      h5("Cross Tabulation For Categorical Variables"),
      tableOutput("crosstab_cat"),
      plotOutput("crosstab_cat_plot")
    ),
    column(
      6,
      h5("Mean & SD for Continuous Variables"),
      tableOutput("crosstab_cont"),
      plotOutput("crosstab_cont_plot")
    ),
    br(), br(), br()
  )
})
output$crosstab_cat <- renderTable({
  req(input$crosstab_result, input$crosstab_vars)
  if (!input$crosstab_result) return(NULL)
  
  df <- df_out()
  
  vars_cat <- input$crosstab_vars[
    sapply(df[input$crosstab_vars], function(x)
      is.factor(x) || is.character(x))
  ]
  
  if (length(vars_cat) == 0) return(NULL)
  
  do.call(rbind, lapply(vars_cat, function(v) {
    tab <- as.data.frame(
      table(Class = df$Class, Category = df[[v]])
    )
    
    tab <- tab |>
      dplyr::group_by(Category) |>
      dplyr::mutate(
        Percent = round(Freq / sum(Freq) * 100, 2)
      ) |>
      dplyr::ungroup()
    
    data.frame(
      Class  = tab$Class,
      Variable = v,
      Category = tab$Category,
      N        = tab$Freq,
      Percent  = tab$Percent
    )
  }))
})
output$crosstab_cont <- renderTable({
  req(df_out(), input$crosstab_vars)
  
  df <- df_out()
  
  vars_num <- input$crosstab_vars[
    sapply(df[input$crosstab_vars], is.numeric)
  ]
  
  if (length(vars_num) == 0) return(NULL)
  
  res <- lapply(vars_num, function(v) {
    aggregate(
      df[[v]],
      by = list(Class = df$Class),
      FUN = function(x)
        paste0(round(mean(x, na.rm = TRUE), 2),
               " (", round(sd(x, na.rm = TRUE), 2), ")")
    ) |>
      setNames(c("Class", v))
  })
  
  Reduce(function(x, y) merge(x, y, by = "Class"), res)
})
output$crosstab_cat_plot <- renderPlot({
  req(input$crosstab_result, input$crosstab_vars)
  if (!input$crosstab_result) return(NULL)
  
  df <- df_out()
  
  vars_cat <- input$crosstab_vars[
    sapply(df[input$crosstab_vars], function(x)
      is.factor(x) || is.character(x))
  ]
  
  if (length(vars_cat) == 0) return(NULL)
  
  plot_data <- do.call(rbind, lapply(vars_cat, function(v) {
    tab <- as.data.frame(table(Class = df$Class, Category = df[[v]]))
    
    tab <- tab |>
      dplyr::group_by(Category) |>
      dplyr::mutate(
        Percent = round(Freq / sum(Freq) * 100, 2)
      ) |>
      dplyr::ungroup()
    
    tab$Variable <- v
    tab
  }))
  
  ggplot(plot_data,
         aes(x = Category, y = Percent, fill = Class)) +
    geom_col(
      width = 0.6,
      position = position_dodge(width = 0.7)
    ) +
    geom_text(
      aes(label = paste0(Percent, "%")),
      position = position_dodge(width = 0.7),
      vjust = -0.25,
      size = 3
    ) +
    facet_wrap(~ Variable, scales = "free_x") +
    labs(
      title = "Categorical Cross-Tabulation by Class",
      y = "Percentage (%)",
      x = NULL
    ) +
    theme_minimal() +
    theme(
      axis.text.x = element_text(angle = 90, hjust = 1, vjust = 0.5, size = 9),
      legend.position = "bottom",
      axis.line = element_line(color = "black")
    )
})
output$crosstab_cont_plot <- renderPlot({
  req(input$crosstab_result, input$crosstab_vars)
  if (!input$crosstab_result) return(NULL)
  
  df <- df_out()
  
  vars_num <- input$crosstab_vars[
    sapply(df[input$crosstab_vars], is.numeric)
  ]
  
  if (length(vars_num) == 0) return(NULL)
  
  plot_data <- df |>
    dplyr::select(Class, dplyr::all_of(vars_num)) |>
    tidyr::pivot_longer(
      cols = -Class,
      names_to = "Variable",
      values_to = "Value"
    ) |>
    dplyr::group_by(Class, Variable) |>
    dplyr::summarise(
      Mean = mean(Value, na.rm = TRUE),
      SD   = sd(Value, na.rm = TRUE),
      .groups = "drop"
    )
  
  ggplot(plot_data, aes(x = Class, y = Mean, fill = Class)) +
    geom_col(width = 0.6) +
    geom_errorbar(
      aes(ymin = Mean - SD, ymax = Mean + SD),
      width = 0.2
    ) +
    facet_wrap(~ Variable, scales = "free_y") +
    labs(
      title = "Continuous Variables by Class",
      y = "Mean (± SD)",
      x = NULL
    ) +
    theme_minimal() +
    theme(
      axis.text.x = element_text(angle = 90, hjust = 1, vjust = 0.5, size = 9),
      legend.position = "none",
      axis.line = element_line(color = "black")
    )
})

}

# Home module ----
home_mod <- function(id){
  ns <- NS(id)
  homepage_ui()
  }

# LCA module ----
lca_mod <- function(id, project){
  ns <- NS(id)
  lca_ui(project)
}

# LPA module ----
lpa_mod <- function(id, project){
  ns <- NS(id)
  lpa_ui(project)
}
# LPT module ----
lta_mod <- function(id, project){
  ns <- NS(id)
  lta_ui(project)
}
# EFA module ----
efa_mod <- function(id, project){
  ns <- NS(id)
  efa_ui(project)
}
# CFA module ----
cfa_mod <- function(id, project){
  ns <- NS(id)
  cfa_ui(project)
}

# --- Project Latent Structure Analysis (Project LSA)  ---
# Author: Hasan Djidu

# ==== Load UI & Server Components ====
source("ui_module.R")
source("homepage_ui.R")
source("lca_ui.R")
source("lpa_ui.R")
source("efa_ui.R")
source("cfa_ui.R")
source("lta_ui.R")
source("plotinfose.R")
source("lta_info_vis.R")

# ==== Load Logic / Server Modules ====
source("serverEFA.R")
source("serverCFA.R")
source("serverLCA.R")
source("serverLPA.R")
source("serverLTA.R")

# ==== Misc utilities ====
source("reference_list.R")
source("simDataDesc.R")
source("downloadPlot.R")
source("styleCSS.R")

# ==== Main Library ======
library(shiny)
library(shinyWidgets)
library(DT)
library(readxl)
library(dplyr)
library(ggplot2)
library(stringr)
library(tidyverse)
library(plotly)
library(tidyr)
options(shiny.maxRequestSize = 500 * 1024^2) # 300 MB

# ===== UI =====
ui <- fluidPage(
  styleCSS,
  uiOutput("mainUI")
)
# ==== Server =====
server <- function(input, output, session) {
  observeEvent(TRUE,
    {
      showModal(modalDialog(
        title = NULL,
        div(
          style = "text-align:center; line-height:1.6;",
          tags$img(src = "logoProjectLSA.png", height = "70px"),
          tags$h3("Welcome to projectLSA"),
          tags$p(
            "projectLSA is an interactive R Shiny application designed to support ",
            "latent structure analysis for research & teaching purposes."
          ),
          HTML(
            "
         If you use this application in academic or research work, please cite it as follows:<br>
         <strong>In-text citation:</strong>
         <span style='color:#2563eb;'>(Djidu et al., 2026)</span><br><br>
         <strong>Reference:</strong><br>
         Djidu, H., Retnawati, H., Hadi, S., &amp; Haryanto. (2026).
         <em>projectLSA: A Shiny Application for Integrated Latent Structure Analysis</em>.
         Applied Psychological Measurement.
         <a href='https://doi.org/10.1177/01466216261446305' target='_blank'>
         https://doi.org/10.1177/01466216261446305</a><br><br>
         Djidu, H., Retnawati, H., Hadi, S., Haryanto (2026). <em>projectLSA: R Shiny application for latent structure analysis with a graphical user interface</em>.
         <a href='https://doi.org/10.32614/CRAN.package.projectLSA' target='_blank'>https://doi.org/10.32614/CRAN.package.projectLSA</a>. R package."
          )
        ),
        footer = tagList(
          actionButton("btn_donate_home_modal", "Support Us (Donate)", icon = icon("heart"), class = "btn btn-danger btn-sm btn-glow", onclick = "$(this).removeClass('btn-glow');"),
          modalButton("START")
        ),
        easyClose = TRUE,
        size = "l"
      ))
    },
    once = TRUE
  )

  # Download RIS handler for the homepage
  output$download_ris_home <- downloadHandler(
    filename = function() {
      "projectLSA_citation.ris"
    },
    content = function(file) {
      ris_text <- c(
        "TY  - JOUR",
        "AU  - Djidu, Hasan",
        "AU  - Retnawati, Heri",
        "AU  - Hadi, Samsul",
        "AU  - Haryanto",
        "PY  - 2026",
        "DA  - 2026/04/20",
        "TI  - projectLSA: A Shiny Application for Integrated Latent Structure Analysis",
        "JO  - Applied Psychological Measurement",
        "DO  - 10.1177/01466216261446305",
        "UR  - https://doi.org/10.1177/01466216261446305",
        "ER  - "
      )
      writeLines(ris_text, file)
    }
  )

  # Handlers for donation buttons on homepage / welcome modal
  show_donate_modal <- function() {
    showModal(modalDialog(
      title = "Support projectLSA",
      tags$div(
        style = "text-align: center;",
        tags$h4("Help Us Keep Growing!"),
        tags$p("If you find projectLSA helpful for your research, consider supporting its continuous development."),
        tags$div(
          style = "background-color: #f8f9fa; border: 2px dashed #17a2b8; padding: 20px; border-radius: 10px; margin-bottom: 20px;",
          tags$strong("Support via Bank BNI:", style = "color: #17a2b8; font-size: 16px;"), tags$br(),
          tags$span("0331355050", style = "font-size: 24px; font-weight: bold; color: #d32f2f; letter-spacing: 2px;"), tags$br(),
          tags$em("Account Name: Hasan Djidu"), tags$br(), tags$br(),
          tags$strong("International Transfer (SWIFT):", style = "color: #17a2b8; font-size: 14px;"), tags$br(),
          tags$span("BNINIDJA", style = "font-size: 16px; font-weight: bold; color: #d32f2f;")
        )
      ),
      footer = modalButton("Close"),
      easyClose = TRUE
    ))
  }

  observeEvent(input$btn_donate_home, {
    show_donate_modal()
  })
  observeEvent(input$btn_donate_home_modal, {
    show_donate_modal()
  })

  # === reactive value to save current project ===
  project <- reactiveVal("home")

  # === observe event from homepage ===
  observeEvent(input$go_lca, {
    project("lca")
  })
  observeEvent(input$go_lpa, {
    project("lpa")
  })
  observeEvent(input$go_lta, {
    project("lta")
  })
  observeEvent(input$go_efa, {
    project("efa")
  })
  observeEvent(input$go_cfa, {
    project("cfa")
  })
  observeEvent(input$go_home, {
    project("home")
  })

  # === Render main UI ===
  output$mainUI <- renderUI({
    req(project())
    fluidPage(
      switch(project(),
        "lca" = lca_mod("lca"),
        "lpa" = lpa_mod("lpa"),
        "lta" = lta_mod("lta"),
        "efa" = efa_mod("efa"),
        "cfa" = cfa_mod("cfa"),
        home_mod("home")
      )
    )
  })
  # === Logic server: jalankan modul sesuai project aktif ===
  observeEvent(project(), {
    current <- project()
    switch(current,
      "lpa" = server_lpa(input, output, session),
      "efa" = server_efa(input, output, session),
      "cfa" = server_cfa(input, output, session),
      "lca" = server_lca(input, output, session),
      "lta" = server_lta(input, output, session),
      NULL
    )
  })
  # LPA
  output$package_references_lpa <- renderUI({
    render_package_refs(c("shiny", "tidyverse", "tidyLPA", "DT", "readxl", "dplyr", "ggplot2"))
  })
  # LCA
  output$package_references_lca <- renderUI({
    render_package_refs(c("shiny", "tidyverse", "poLCA", "glca", "DT", "readxl", "dplyr", "ggplot2", "ggiraph"))
  })
  # LTA / IRT
  output$package_references_lta <- renderUI({
    lta_reference <- list(
      Desjardins_Bulut_2018 = "Desjardins, C. D., & Bulut, O. (2018). <em>Handbook of educational measurement and psychometrics using R</em> (1st ed.). Chapman & Hall/CRC."
    )
    render_package_refs(c("shiny", "tidyverse", "mirt", "DT", "readxl", "dplyr", "ggplot2"),
      manual_refs = lta_reference
    )
  })
  # EFA
  output$package_references_efa <- renderUI({
    render_package_refs(c("shiny", "tidyverse", "psych", "DT", "readxl", "dplyr", "ggplot2"))
  })
  # CFA References
  output$package_references_cfa <- renderUI({
    # Define manual references
    manual_refs <- list(
      Alamer_2025 = "Alamer, A. (2025). Structural equation modeling (SEM) in L2 writing research: Simple tutorial and useful recommendations. <em>Research Methods in Applied Linguistics</em>, 4(2), 100202. <a href='https://doi.org/10.1016/j.rmal.2025.100202' target='_blank'>https://doi.org/10.1016/j.rmal.2025.100202</a>",
      Hair_2019 = "Hair, J. F., Black, W. C., Babin, B. J., & Anderson, R. E. (2019). <em>Multivariate data analysis</em> (Eighth edition). Cengage.",
      Hu_1999 = "Hu, L., & Bentler, P. M. (1999). Cutoff criteria for fit indexes in covariance structure analysis: Conventional criteria versus new alternatives. <em>Structural Equation Modeling: A Multidisciplinary Journal</em>, 6(1), 1–55. <a href='https://doi.org/10.1080/10705519909540118' target='_blank'>https://doi.org/10.1080/10705519909540118</a>",
      Schumacker_2008 = "Schumacker, R. E., & Lomax, R. G. (2008). <em>A beginner's guide to structural equation modeling</em> (2nd ed.). Psychology Press.",
      Kline_2016 = "Kline, R. B. (2016). <em>Principles and practice of structural equation modeling</em> (4th ed.). Guilford Press.",
      Brown_2015 = "Brown, T. A. (2015). <em>Confirmatory factor analysis for applied research</em> (2nd ed.). Guilford Press."
    )

    render_package_refs(
      pkgs = c("shiny", "tidyverse", "psych", "lavaan", "DT", "readxl", "dplyr", "semPlot", "semptools", "semTools"),
      manual_refs = manual_refs
    )
  })
}

shinyApp(ui, server)

library(shiny)
# ==== styleCSS.R ====
styleCSS <- tags$head(
  # Import FontAwesome untuk ikon
  tags$link(
    rel = "stylesheet",
    href = "https://cdnjs.cloudflare.com/ajax/libs/font-awesome/6.5.0/css/all.min.css"
  ),
  
  tags$style(HTML("
.container-fluid {
    padding-left: 5px !important;
    padding-right: 5px !important;
  }
  /* ================================
     STYLE UNTUK DATATABLE
  ================================ */
  table.dataTable {
    font-size: 12px !important;
    line-height: 0.7 !important;
    letter-spacing: -0.3px !important;
    border-collapse: collapse !important;
  }

  table.dataTable th,
  table.dataTable td {
    padding: 4px 6px !important;
  }

  table.dataTable thead th {
    background-color: darkcyan !important;
    color: white !important;
    height: 0.7 !important;
    font-weight: bold !important;
  }

  table.dataTable tbody tr:nth-child(even) {
    background-color: #f2f6fa !important;
  }

  /* ================================
     STYLE UNTUK TABEL BIASA
  ================================ */
  table, th, td {
    font-size: 12px !important;
    line-height: 0.7 !important;
    letter-spacing: -0.3px !important;
    border-collapse: collapse !important;
  }

  th {
    background-color: darkcyan !important;
    color: white !important;
    font-weight: bold !important;
  }

  tr:nth-child(even) {
    background-color: #f2f6fa !important;
  }

  td, th {
    padding: 4px 6px !important;
  }

  /* ================================
     STYLE UNTUK TOMBOL (BUTTON)
  ================================ */
  .btn {
    width: auto !important;
    padding: 5px 10px !important;
    font-size: 12px !important;
    line-height: 1 !important;
    border-radius: 8px !important;
    font-weight: 700 !important;
    transition: all 0.1s ease-in-out !important;
    display: block;
    margin: 0 auto;
  }

  .btn i {
    margin-right: 6px !important;
  }

  .btn:hover {
    box-shadow: 0 2px 6px rgba(0,0,0,0.2) !important;
  }

  .btn:active {
    transform: scale(0.97) !important;
    box-shadow: 0 1px 3px rgba(0,0,0,0.3) !important;
  }
  
  @keyframes pulse-glow {
    0% { box-shadow: 0 0 0 0 rgba(255, 193, 7, 0.7); }
    70% { box-shadow: 0 0 0 10px rgba(255, 193, 7, 0); }
    100% { box-shadow: 0 0 0 0 rgba(255, 193, 7, 0); }
  }
  
  .btn-glow {
    animation: pulse-glow 2s infinite !important;
  }

  /* Warna Tombol Khusus */
  .btn-primary {
    background-color: #0077dd !important;
    border: none !important;
  }
  .btn-primary:hover {
    background-color: #0056b3 !important;
  }

  .btn-danger {
    background-color: #fd4084 !important;
    border: none !important;
  }
  .btn-danger:hover {
    background-color: #a71d2a !important;
  }

  .btn-success {
    background-color: #28a745 !important;
    border: none !important;
  }
  .btn-success:hover {
    background-color: #1e7e34 !important;
  }
  /* =======================================
     STYLE   DT BUTTONS
   ======================================= */

.dt-button {
  padding: 4px 10px !important;    
  font-size: 9px !important;        
  border-radius: 4px !important; 
  background-color: black !important;
  color: white !important;
  border: none !important;
  font-weight: 600 !important;
  margin: 3px !important;
  line-height: 1 !important;
}

/* Hover */
.dt-button:hover {
  background-color: #005bb0 !important;
}

/* Active */
.dt-button:active {
  transform: scale(0.97) !important;
}

.dataTables_wrapper .dataTables_paginate .paginate_button {
    font-size: 10px !important; 
    padding: 2px 6px !important;     
    border-radius: 6px !important;  
  }

  .dataTables_wrapper .dataTables_paginate {
    margin-top: 2px !important;      
  }

  /* ================================
     STYLE UNTUK INPUT (select, numeric, file)
  ================================ */
  select.form-control,
  input.form-control,
  .selectize-input,
  .selectize-dropdown-content {
    border-radius: 12px !important;
    font-size: 12px !important;
    line-height: 1.2 !important;
    border: 1px solid #ddd !important;
  }

  .selectize-input {
    padding: 6px 10px !important;
    min-height: 32px !important;
  }

  .selectize-dropdown-content {
    font-size: 12px !important;
  }

  /* ================================
     STYLE UNTUK HOMEPAGE / LANDING PAGE
  ================================ */
  body {
    background: #fff;
    font-family: 'Poppins', sans-serif;
    margin: 0;
    padding: 0;
  }

  .landing {
    text-align: center;
    margin-top: 25px;
  }

  .hero {
    display: flex;
    flex-direction: column;
    align-items: center;
    gap: 5px;
  }

  .welcome {
    font-size: 16px;
    font-weight: 800;
    color: #444;
    letter-spacing: 1px;
  }

.app-name { font-size:36px; font-weight:900; margin-top:1px; color:transparent; -webkit-text-fill-color:black; -webkit-background-clip:text; background:none; }
.app-name { text-shadow:-2px -2px 3px #fff,2px -2px 3px #fff,-2px 2px 3px #fff,2px 2px 3px #fff,0 0 5px rgba(0,0,0,.45),0 0 10px rgba(0,0,0,.35),0 0 15px rgba(0,0,0,.25); }

  .subtitle{
    font-size: 13px;
    font-weight: 600;
    color: #444;
  }

  h4.page-subtitle {
    margin-top: 20px;
    color: #333;
  }

  .quad-container {
    position: relative;
    z-index: 0;
    width: 90vw;
    height: 60vh;
    margin: 10px auto;
    display: grid;
    grid-template-columns: 1fr 1fr;
    grid-template-rows: 1fr 1fr;
    gap: 10px;
    justify-items: center;
    align-items: center;
  }

  .quad-container::before,
  .quad-container::after {
    content: '';
    position: absolute;
    z-index: 1;
  }

  
.quad-container::before{
  top:50%;left:0;width:100%;height:3px;
  background:linear-gradient(90deg,blue,red);
}

.quad-container::after{
  left:50%;top:0;width:3px;height:100%;
  background:linear-gradient(180deg,green,red);
}

  .axis-label {
    position: absolute;
    font-weight: bold;
    color: black;
    font-size: clamp(12px, 2vw, 14px);
    background: white;
    border-radius: 6px;
    z-index: 5;
  }

  .axis-label.x-top {
    top: 0;
    left: 50%;
    transform: translateX(-50%);
  }

  .axis-label.x-bottom {
    bottom: 0;
    left: 50%;
    transform: translateX(-50%);
  }

  .axis-label.y-left {
    left: 0;
    top: 50%;
    transform: translateY(-50%);
  }

  .axis-label.y-right {
    right: 0;
    top: 50%;
    transform: translateY(-50%);
  }

  .center-logo {
    position: absolute;
    top: 50%;
    left: 50%;
    transform: translate(-50%, -50%);
    z-index: 6;
    padding: 6px;
  }

  .center-logo img {
    width: clamp(100px, 2vw, 200px);
    height: auto;
    border-radius: 50%;
  }

  .quad-card {
    width: 90%;
    height: 80%;
    border-radius: 21px;
    box-shadow: 0 4px 10px rgba(0,0,0,0.15);
    padding: 1.5vw;
    display: flex;
    flex-direction: column;
    justify-content: space-between;
    align-items: center;
    text-align: center;
    z-index: 2;
    transition: transform .5s;
  }

  .quad-card:hover {
    transform: scale(1.05);
  }
#card_lca { background: #eef3ff; border: 2px solid #c7d4ff; }   /* Blue */
#card_lpa { background: #edf7f6; border: 2px solid #c6e6e2; }   /* Teal */
#card_fa  { background: #f3effa; border: 2px solid #d9cff0; }   /* Purple */
#card_lta { background: #fbeef2; border: 2px solid #f2cdd8; }   /* Rose */
/* Warna & tema masing-masing card */
#card_lpa .project-icon { background: #dff1ee; color: #138f87; } /* Teal */
#card_lca .project-icon { background: #dde6ff; color: #2a55e6; } /* Blue */
#card_fa  .project-icon { background: #e6e0f5; color: #6b4cc2; } /* Purple */
#card_lta .project-icon { background: #f6dde6; color: #c23b63; } /* Rose */

#card_lca .btn-pill { background: #2a55e6; }  /* Blue */
#card_lpa .btn-pill { background: #138f87; }  /* Teal */
#card_fa  .btn-pill { background: #6b4cc2; }  /* Purple */
#card_lta .btn-pill { background: #c23b63; }  /* Rose */

  .project-title {
    font-size: clamp(12px, 1.5vw, 27px);
    font-weight: 600;
    margin-bottom: 4px;
  }

  .project-desc {
    color: #444;
    font-size: clamp(12px, 1vw, 120px);
    line-height: 1.3em;
    padding: 0 .5vw;
  }

  .project-icon {
    font-size: clamp(25px, 2vw, 40px);
    width: clamp(40px, 4vw, 60px);
    height: clamp(40px, 4vw, 60px);
    display: flex;
    align-items: center;
    justify-content: center;
    border-radius: 12px;
  }



  .btn-pill {
    display: block !important;
    width: 100% !important;
    padding: clamp(5px, .8vw, 10px) 0;
    border-radius: 8px;
    text-align: center;
    text-decoration: none;
    margin-top: 3px;
    color: #fff;
    font-size: clamp(9px, 1vw, 18px);
    transition: .1s;
  }

  .btn-pill:hover {
    opacity: .85;
    transform: scale(1.001);
  }
  
   /* Styling untuk labels */
         .shiny-input-container label {
           font-size: 12px !important;
           font-weight: normal !important;
           color: #555555 !important;
           margin-bottom: 3px !important;
         }
         
         /* Styling untuk input fields */
         .form-control {
           font-size: 12px !important;
           height: 30px !important;
           padding: 4px 8px !important;
         }
         
         /* Styling khusus untuk numeric input */
         input[type='number'] {
           font-size: 12px !important;
           height: 28px !important;

         }
         
         /* Styling untuk select input */
         select.form-control {
           font-size: 12px !important;
           height: 30px !important;
           padding: 4px 8px !important;
         }
/* === Style khusus untuk selectInput besar (itemtype) === */
.select-large .selectize-input {
  font-size: 18px !important;
  font-weight: bold !important;
  text-align: center !important;
  padding: 12px 16px !important;
  border-radius: 14px !important;
  background-color: lightgreen !important;
  border: 2px solid #cfcfcf !important;
  box-shadow: 0 3px 6px rgba(0,0,0,0.1) !important;
  transition: all 0.2s ease-in-out !important;
}

.select-large .selectize-input:hover {
  border-color: #007bff !important;
  box-shadow: 0 0 8px rgba(0,123,255,0.3) !important;
}

.select-large .selectize-input.focus {
  border-color: #0056b3 !important;
  box-shadow: 0 0 0 0.2rem rgba(0,123,255,0.25) !important;
}

.select-large .option {
  text-align: center !important;
  font-size: 16px !important;
  font-weight: bold !important;
}

/* === Style selectInput mini tanpa highlight pilihan (versi tinggi lebih pendek) === */
.select-mini .selectize-input {
  font-size: 12px !important;
  font-weight: 600 !important;
  text-align: center !important;
  justify-content: center !important;

  padding: 1px 6px !important;          
  line-height: 12px !important;  
  min-height: 18px !important;     
  height: 18px !important;
  
  border-radius: 8px !important;
  background: lightgreen !important;
  border: 1.5px solid #b8bced !important;
  box-shadow: 0 2px 3px rgba(0,0,0,0.07) !important;
  transition: all 0.2s ease-in-out !important;
}

.select-mini .selectize-input:hover {
  border-color: #756bff !important;
  box-shadow: 0 0 5px rgba(117,107,255,0.33) !important;
}

.select-mini .selectize-input.focus {
  border-color: #574cff !important;
  box-shadow: 0 0 0 0.15rem rgba(87,76,255,0.22) !important;
}

.select-mini .option {
  text-align: center !important;
  font-size: 12px !important;
  font-weight: 600 !important;
}

/* Hilangkan highlight */
.select-mini .selectize-dropdown .highlight {
  background: none !important;
  color: inherit !important;
  box-shadow: none !important;
}


         /* Styling untuk text input */
         input[type='text'] {
           font-size: 12px !important;
         }
         
         /* Styling untuk checkbox labels */
         .checkbox label {
           font-size: 12px !important;
           font-weight: normal !important;
           color: #555555 !important;
         }
  
  
  .well {
        background-color: #f8f9fa;
        border: 1px solid #dee2e6;
      }
      .shiny-output-error { color: #dc3545; }
      .shiny-output-error:before { content: 'Error: '; }
      .small-input { max-width: 100px; }

  @media (max-width: 700px) {
    .quad-container {
      transform: scale(1);
      transform-origin: center top;
    }
  }
  
.badge-warning{
  background:#fee2e2;
  border-left:4px solid #ef4444;
  color:#7f1d1d;
  padding:8px 12px;
  border-radius:5px;
  font-size:12px;
  line-height:1.4;
}

.badge-info{
  background:#e0e7ff;
  border-left:4px solid #3b82f6;
  color:#1e3a8a;  
  padding:10px 14px;
  border-radius:4px;
  font-size:13px;
  line-height:1.6;
}

.badge-appinfo{
  background:#fffef9;            /* ivory lembut */
  border-left:4px solid #334155; /* slate blue soft */
  color:#374151;                 /* dark gray (BUKAN hitam) */
  padding:10px 14px;
  border-radius:4px;
  font-size:13px;
  line-height:1.6;
}


/* Background + border modal */
.modal-content{
  background:#fffdf7;        /* ivory */
  border:2px solid #1e293b;  /* biru kehitaman */
  border-radius:16px;
  box-shadow:0 12px 30px rgba(15,23,42,.25);
}

/* Header & footer lebih bersih */
.modal-header, .modal-footer{
  border:none;
}

.modal.fade .modal-dialog {
  transform: translateY(-30px) scale(1);
  transition: transform 1.3s cubic-bezier(0.22, 1, 0.36, 1);
}

.modal.show .modal-dialog {
  transform: translateY(0) scale(1);
}

  "))
)

# # --- Fungsi utilitas untuk membuat package references ---
# render_package_refs <- function(pkgs) {
#   refs_df <- purrr::map_df(pkgs, function(pkg) {
#     cit <- tryCatch(utils::citation(pkg), error = function(e) NULL)
#     if (is.null(cit)) {
#       return(tibble(pkg = pkg, ref = paste0(pkg, ": citation not available")))
#     }
#     
#     txt <- paste0(format(cit, style = "text"), collapse = " ")
#     
#     # Ubah penanda markdown menjadi HTML italic
#     txt <- gsub("_([^_]+)_", "<em>\\1</em>", txt, perl = TRUE)
#     txt <- gsub("\\*([^*]+)\\*", "<em>\\1</em>", txt, perl = TRUE)
#     
#     # Ubah DOI menjadi link
#     txt <- gsub("(doi:\\s*\\S+)", "<a href='https://doi.org/\\1' target='_blank'>\\1</a>", txt, perl = TRUE)
#     
#     # Bersihkan format
#     txt <- gsub("\\s+,", ",", txt)
#     txt <- gsub(",\\s*$", "", txt)
#     txt <- trimws(txt)
#     
#     tibble(pkg = pkg, ref = txt)
#   })
#   
#   # Sorting berdasarkan nama author pertama
#   refs_df <- refs_df %>%
#     mutate(first_word = sub("\\s.*", "", ref)) %>%
#     arrange(first_word)
#   
#   # Buat HTML ordered list
#   refs_html <- paste0(
#     "<ol style='line-height:1.6;'>",
#     paste0(
#       purrr::map_chr(refs_df$ref, ~paste0("<li style='margin-bottom:6px;'>", ., "</li>")),
#       collapse = ""
#     ),
#     "</ol>"
#   )
#   
#   HTML(refs_html)
# }

# --- Fungsi utilitas untuk membuat package references dan referensi manual ---
render_package_refs <- function(pkgs, manual_refs = NULL) {
  # Process package citations
  refs_df <- purrr::map_df(pkgs, function(pkg) {
    cit <- tryCatch(utils::citation(pkg), error = function(e) NULL)
    if (is.null(cit)) {
      return(tibble(pkg = pkg, ref = paste0(pkg, ": citation not available"), type = "package"))
    }
    
    txt <- paste0(format(cit, style = "text"), collapse = " ")
    
    # Ubah penanda markdown menjadi HTML italic
    txt <- gsub("_([^_]+)_", "<em>\\1</em>", txt, perl = TRUE)
    txt <- gsub("\\*([^*]+)\\*", "<em>\\1</em>", txt, perl = TRUE)
    
    # Ubah DOI menjadi link
    txt <- gsub("(doi:\\s*\\S+)", "<a href='https://doi.org/\\1' target='_blank'>\\1</a>", txt, perl = TRUE)
    
    # Bersihkan format
    txt <- gsub("\\s+,", ",", txt)
    txt <- gsub(",\\s*$", "", txt)
    txt <- trimws(txt)
    
    tibble(pkg = pkg, ref = txt, type = "package")
  })
  
  # Add manual references jika ada
  if (!is.null(manual_refs)) {
    manual_df <- tibble(
      pkg = names(manual_refs) %||% paste0("ref_", seq_along(manual_refs)),
      ref = unlist(manual_refs),
      type = "manual"
    )
    refs_df <- bind_rows(refs_df, manual_df)
  }
  
  # Sorting berdasarkan nama author pertama
  refs_df <- refs_df %>%
    mutate(first_word = sub("\\s.*", "", ref)) %>%
    arrange(first_word)
  
  # Buat HTML ordered list
  refs_html <- paste0(
    "<ol style='line-height:1.6;'>",
    paste0(
      purrr::map_chr(refs_df$ref, ~paste0("<li style='margin-bottom:6px;'>", ., "</li>")),
      collapse = ""
    ),
    "</ol>"
  )
  
  HTML(refs_html)
}

# === Custom Test Information & SE Plot ===
plot_info_se <- function(mod) {
  # Create a theta grid
  theta <- seq(-10, 10, length = 500)
  
  # Compute test information and SE
  info <- testinfo(mod, theta)
  SE <- 1 / sqrt(info)
  
  # Combine into a data frame
  tinfo <- data.frame(theta = theta, info = info, SE = SE)
  
  # Find points where Information ≈ 1
  tinfo$diff <- abs(tinfo$info - 1)
  
  # Left point (negative theta)
  idx_left <- which.min(tinfo$diff[tinfo$theta < 0])
  theta_left <- tinfo$theta[tinfo$theta < 0][idx_left]
  info_left <- tinfo$info[tinfo$theta < 0][idx_left]
  
  # Right point (positive theta)
  idx_right <- which.min(tinfo$diff[tinfo$theta > 0])
  theta_right <- tinfo$theta[tinfo$theta > 0][idx_right]
  info_right <- tinfo$info[tinfo$theta > 0][idx_right]
  
  # ===== Plot base =====
  plot(tinfo$theta, tinfo$info, type = "n",
       ylim = c(0, max(tinfo$info) + 0.08*max(tinfo$info) ),
       xlim = c(theta_left-1, theta_right+1 ),
       xlab = expression(theta),
       ylab = "Information / SE",
       main = "Test Information and Standard Error")
  
  # ===== Shaded regions =====
  # Red = less reliable, Blue = reliable region
  polygon(c(tinfo$theta[tinfo$theta <= theta_left],
            rev(tinfo$theta[tinfo$theta <= theta_left])),
          c(tinfo$info[tinfo$theta <= theta_left],
            rep(0, sum(tinfo$theta <= theta_left))),
          col = rgb(1, 0, 0, 0.2), border = NA)
  
  polygon(c(tinfo$theta[tinfo$theta >= theta_right],
            rev(tinfo$theta[tinfo$theta >= theta_right])),
          c(tinfo$info[tinfo$theta >= theta_right],
            rep(0, sum(tinfo$theta >= theta_right))),
          col = rgb(1, 0, 0, 0.2), border = NA)
  
  polygon(c(tinfo$theta[tinfo$theta >= theta_left & tinfo$theta <= theta_right],
            rev(tinfo$theta[tinfo$theta >= theta_left & tinfo$theta <= theta_right])),
          c(tinfo$info[tinfo$theta >= theta_left & tinfo$theta <= theta_right],
            rep(0, sum(tinfo$theta >= theta_left & tinfo$theta <= theta_right))),
          col = rgb(0, 0, 1, 0.2), border = NA)
  
  # ===== Lines =====
  lines(tinfo$theta, tinfo$info, lwd = 2, col = "blue")
  lines(tinfo$theta, tinfo$SE, lwd = 2, col = "red", lty = 2)
  
  # ===== Reference lines and markers =====
  abline(h = 1, lty = 3, col = 'gray')
  abline(v = theta_left, lty = 2, col = 'gray')
  abline(v = theta_right, lty = 2, col = 'gray')
  
  points(theta_left, info_left, col = 'black', pch = 19, cex = 1.5)
  points(theta_right, info_right, col = 'black', pch = 19, cex = 1.5)
  
  text(theta_left, 1.7*info_left,
       labels = bquote(theta == .(round(theta_left, 2))),
       col = "black", pos = 2)
  
  text(theta_right, 1.7*info_right,
       labels = bquote(theta == .(round(theta_right, 2))),
       col = "black", pos = 4)
  
  # ===== Legend =====
  legend("topright", 
         legend = c("Test Information", "Standard Error", 
                    "Reliable Region", "Less Reliable Region"),
         col = c("blue", "red", rgb(0, 0, 1, 0.2), rgb(1, 0, 0, 0.2)),
         lty = c(1, 2, NA, NA), lwd = c(2, 2, NA, NA),
         pch = c(NA, NA, 15, 15), pt.cex = 2, bty = "n")
}

# ==== HELPER FUNCTIONS (GLOBAL IN SERVER) ====
# CR AVE & HTMT ====
calc_ave_cr <- function(fit) {
  std <- lavaan::standardizedSolution(fit)
  
  loading <- std[std$op == "=~", c("lhs","rhs","est.std")]
  loading$err <- 1 - loading$est.std^2
  
  result <- lapply(split(loading, loading$lhs), function(x) {
    lambda2 <- x$est.std^2
    theta   <- x$err
    
    AVE <- sum(lambda2) / (sum(lambda2) + sum(theta))
    CR  <- (sum(x$est.std))^2 /
      ((sum(x$est.std))^2 + sum(theta))
    
    c(AVE = AVE, CR = CR)
  })
  
  as.data.frame(do.call(rbind, result))
}
calc_htmt <- function(data, model_syntax,
                      missing = "listwise",
                      ordered = NULL,
                      absolute = TRUE) {
  
  ## -------------------------------------------------
  ## 1. Parse lavaan model syntax
  ## -------------------------------------------------
  lines <- unlist(strsplit(model_syntax, "\n"))
  fac_lines <- lines[grepl("=~", lines)]
  
  blocks <- lapply(fac_lines, function(x) {
    trimws(unlist(strsplit(strsplit(x, "=~")[[1]][2], "\\+")))
  })
  names(blocks) <- trimws(sub("=~.*", "", fac_lines))
  
  # Merge items for factors defined across multiple lines (duplicate factor names)
  unique_factors <- unique(names(blocks))
  merged_blocks <- lapply(unique_factors, function(f) {
    unique(unlist(blocks[names(blocks) == f]))
  })
  names(merged_blocks) <- unique_factors
  blocks <- merged_blocks
  
  ## -------------------------------------------------
  ## 2. Lavcor
  ## -------------------------------------------------
  R <- lavaan::lavCor(
    data,
    missing = missing,
    ordered = ordered,
    output  = "cor"
  )
  
  if (absolute) R <- abs(R)
  
  ## -------------------------------------------------
  ## 3. Indicator check 
  ## -------------------------------------------------
  blocks <- lapply(blocks, function(x) intersect(x, colnames(R)))
  
  ## -------------------------------------------------
  ## 4. calculate HTMT2 (GEOMETRIC MEAN)
  ## -------------------------------------------------
  k <- length(blocks)
  htmt <- matrix(NA, k, k, dimnames = list(names(blocks), names(blocks)))
  
  for (i in seq_len(k)) {
    for (j in seq_len(k)) {
      
      if (i == j) {
        htmt[i, j] <- 1
        next
      }
      
      Xi <- blocks[[i]]
      Xj <- blocks[[j]]
      
      # heterotrait correlations
      r_het <- R[Xi, Xj]
      
      # monotrait correlations
      r_mono_i <- R[Xi, Xi][lower.tri(R[Xi, Xi])]
      r_mono_j <- R[Xj, Xj][lower.tri(R[Xj, Xj])]
      
      htmt[i, j] <-
        exp(mean(log(r_het))) /
        sqrt(
          exp(mean(log(r_mono_i))) *
            exp(mean(log(r_mono_j)))
        )
    }
  }
  
  htmt
}
# ====END of CR AVE and HTMT ====
server_cfa <- function(input, output, session) {
  library(lavaan)
  library(semPlot)
  library(readxl)
  library(psych)
  library(tibble)
  library(semptools)
  # library(semTools)
  library(data.table)
  library(kableExtra)
  
  data_user <- reactive({
    if (input$data_source == "bfi") {
      df <- psych::bfi %>% dplyr::select(A1:O5) %>% tibble::rownames_to_column("id_auto")
    }
    if (input$data_source == "HolzingerSwineford1939") {
      df <- lavaan::HolzingerSwineford1939 %>% dplyr::select(x1:x9) %>% tibble::rownames_to_column("id_auto")
    }
    
    if (input$data_source == "upload") {
      req(input$datafile)
      ext <- tolower(tools::file_ext(input$datafile$name))
      showModal(modalDialog(title = NULL, "Reading Your File, Please wait...", footer = NULL, easyClose = FALSE))
      df <- switch(
        ext,
        "csv"  = data.table::fread(input$datafile$datapath,data.table = FALSE),
        "xls"  = readxl::read_excel(input$datafile$datapath),
        "xlsx" = readxl::read_excel(input$datafile$datapath),
        "sav"  = haven::read_sav(input$datafile$datapath),
        "rds"  = readRDS(input$datafile$datapath),
        stop("Unsupported file type. Please upload CSV, Excel, SPSS (.sav), or RDS file.")
      )
      removeModal()
      df <- df %>% mutate(across(everything(), ~ifelse(.x == "", NA, .x)),
                          id_auto = paste0("id_", sprintf("%04d", 1:n())))
    }
    return(df)
  })
  
  output$id_select_ui <- renderUI({
    req(data_user())
    selectizeInput("id_cols", "ID columns (optional)", choices = names(data_user()), multiple = TRUE,
                   selected = names(data_user())[grepl("id", names(data_user()), ignore.case = TRUE)])
  })
  
  output$var_select_ui <- renderUI({
    req(data_user())
    all_vars <- names(data_user())
    id_cols <- input$id_cols
    choices <- setdiff(all_vars, id_cols)
    selectInput("selected_vars", "Select Variables (used for CFA):", choices = choices, multiple = TRUE, selected = choices)
  })
  
  output$data_preview <- renderDT({
    req(data_user(), input$selected_vars)
    df <- data_user()[, input$selected_vars, drop = FALSE]
    # Identifikasi kolom numeric 
    numeric_cols <- which(sapply(df, is.numeric))
    #df <- df %>% mutate(across(where(is.numeric), ~ round(.x, 3)))
    datatable(df,
              options = list(scrollX = TRUE, dom = 'rtp', pageLength = 25), 
              rownames = T) %>% 
      formatRound(columns = numeric_cols, digits = 2)
  }, server = TRUE)
  
  
  # ===== CFA MODEL =====
  output$cfa_model_ui <- renderUI({
    model_text <- switch(input$data_source,
                         "bfi" = "A =~ A1 + A2 + A3 + A4 + A5\nC =~ C1 + C2 + C3 + C4 + C5\nE =~ E1 + E2 + E3 + E4 + E5\nN =~ N1 + N2 + N3 + N4 + N5\nO =~ O1 + O2 + O3 + O4 + O5",
                         "HolzingerSwineford1939" = "Visual =~ x1 + x2 + x3\nTextual =~ x4 + x5 + x6\nSpeed =~ x7 + x8 + x9\nG =~ Visual+Textual+Speed",
                         "# Enter your lavaan model syntax here\n# Example:\n# factor1 =~ item1 + item2 + item3\n# factor2 =~ item4 + item5 + item6"
    )
    
    tags$textarea(id = "cfa_model_text", rows = 8, style = "width:100%; font-family: monospace; font-size:11.5px", model_text)
  })
  
  # ---- CFA storage ----
  cfa_fit_list <- reactiveVal(list())
  cfa_current_id <- reactiveVal(NULL)
  cfa_model_counter <- reactiveVal(0)
  run_lavaan <- function(model_text, data, estimator = "ML", missing = "listwise") {
    tryCatch({
      data_numeric <- data %>% dplyr::select(where(is.numeric))
      
      if (estimator %in% c("DWLS", "WLSMV","MLR","GLS", "ULS" )) 
      {
        lavaan::cfa(model = model_text, data = data_numeric, estimator = estimator, ov.order = "model",
                    missing = missing)
      } else {
        if (missing == "fiml") {
          lavaan::cfa(model = model_text, data = data_numeric, estimator = estimator, ov.order = "model", missing = "fiml")
        } else if (missing == "pairwise") {
          lavaan::cfa(model = model_text, data = data_numeric, estimator = estimator, ov.order = "model", missing = "pairwise")
        } else {
          lavaan::cfa(model = model_text, data = data_numeric, estimator = estimator, ov.order = "model", missing = "listwise")
        }
      }
    }, error = function(e) {
      showNotification(paste("Model error:", e$message), type = "error")
      return(NULL)
    })
  }
  observeEvent(input$run_cfa, {
    req(data_user(), input$cfa_model_text, input$selected_vars)
    updateTabsetPanel(session, "main_tab_cfa", selected = "fit_tab_cfa")
  })
  # Main run CFA
  observeEvent(input$run_cfa, {
    req(data_user(), input$cfa_model_text, input$selected_vars)
    
    if (nchar(trimws(input$cfa_model_text)) == 0) {
      showNotification("Please enter a CFA model", type = "error")
      return()
    }
    
    df <- data_user()[, input$selected_vars, drop = FALSE]
    df <- df %>% mutate(across(everything(), as.numeric))
    
    showModal(modalDialog("Running CFA ...", footer = NULL))
    
    fit <- run_lavaan(
      model_text = input$cfa_model_text, 
      data = df, 
      estimator = input$cfa_estimator, 
      missing = input$cfa_missing
    )
    
    removeModal()
    
    if (is.null(fit)) return()
    
    measures <- tryCatch({
      lavaan::fitMeasures(fit, c("chisq","df","pvalue","rmsea","cfi","tli","srmr", "gfi", "agfi", "nfi", "nnfi"))
    }, error = function(e) {
      c(chisq = NA, df = NA, pvalue = NA, rmsea = NA, cfi = NA, tli = NA, srmr = NA, gfi = NA, agfi = NA, nfi = NA, nnfi = NA)
    })
    
    
    # AVE <- tryCatch({
    #   semTools::AVE(object = fit)
    #   }, error = function(e) {
    #   '<NA>'
    # })
    # RelSEM <- tryCatch({
    #   semTools::compRelSEM(object = fit)
    # }, error = function(e) {
    #   '<NA>'
    # })
    # HTMT <- tryCatch({
    #   semTools::htmt(model = input$cfa_model_text, data = df)
    # }, error = function(e) {
    #   '<NA>'
    # })
    
    ave_cr <- tryCatch({
      calc_ave_cr(fit)
    }, error = function(e) {
      NA
    })
    AVE    <- setNames(ave_cr$AVE, rownames(ave_cr))
    RelSEM <- setNames(ave_cr$CR,  rownames(ave_cr))
    HTMT <- tryCatch({
      calc_htmt(df, input$cfa_model_text)
    }, error = function(e) {
      NA
    })
    
    scoreCfa <- tryCatch({
      lavaan::predict(fit)
    }, error = function(e) {
      '<NA>'
    })
    
    current_count <- cfa_model_counter() + 1
    cfa_model_counter(current_count)
    new_id <- paste0("Model_", current_count)
    
    cur <- cfa_fit_list()
    
    # Auto-generate modification note
    auto_note <- ""
    if (length(cur) == 0) {
      auto_note <- "Initial Model"
    } else {
      last_model <- cur[[length(cur)]]
      old_syntax <- last_model$spec
      new_syntax <- input$cfa_model_text
      
      old_lines <- trimws(unlist(strsplit(old_syntax, "\n")))
      new_lines <- trimws(unlist(strsplit(new_syntax, "\n")))
      old_lines <- old_lines[old_lines != ""]
      new_lines <- new_lines[new_lines != ""]
      
      added <- setdiff(new_lines, old_lines)
      removed <- setdiff(old_lines, new_lines)
      
      changes <- c()
      if (length(added) > 0) changes <- c(changes, paste("Added:", paste(added, collapse = ", ")))
      if (length(removed) > 0) changes <- c(changes, paste("Removed:", paste(removed, collapse = ", ")))
      
      if (length(changes) == 0) {
        auto_note <- "Modified formatting only"
      } else {
        auto_note <- paste(changes, collapse = " | ")
      }
    }
    
    cur[[new_id]] <- list(
      id = new_id, 
      spec = input$cfa_model_text, 
      fit = fit, 
      measures = measures, 
      ave = AVE,
      relsem = RelSEM,
      htmt = HTMT,
      scoreCfa = as.data.frame(scoreCfa),
      note = auto_note,
      time = Sys.time()
    )
    
    cfa_fit_list(cur)
    cfa_current_id(new_id)
    showNotification(paste0("CFA finished: ", new_id), type = "message")
  })
  
  # ====== Fit Comparison Table =======
  output$fit_comparison <- renderUI({
    req(cfa_fit_list())
    lst <- cfa_fit_list()
    if (length(lst) == 0) return(tags$p("No models run yet. Run CFA to see results."))
    df <- do.call(rbind, lapply(lst, function(x){
      m <- x$measures
      status_info <- get_fit_status(m["chisq"], m["df"], m["pvalue"], 
                                    m["rmsea"], m["cfi"], m["srmr"], m["gfi"])
      data.frame(
        Model = x$id,
        Note = ifelse(is.null(x$note) || x$note == "", "-", x$note),
        chisq = round(m["chisq"], 2),
        df = m["df"],
        chisq_df_ratio = ifelse(is.na(m["chisq"]), NA, sprintf("%.2f",m["chisq"]/m["df"])),
        pvalue = ifelse(is.na(m["pvalue"]), NA,
                        sprintf("%.3f", m["pvalue"])),
        RMSEA = ifelse(is.na(m["rmsea"]), NA, sprintf("%.3f", m["rmsea"])),
        CFI = ifelse(is.na(m["cfi"]), NA, sprintf("%.3f", m["cfi"])),
        TLI = ifelse(is.na(m["tli"]), NA, sprintf("%.3f", m["tli"])),
        GFI = ifelse(is.na(m["gfi"]), NA, sprintf("%.3f", m["gfi"])),
        SRMR = ifelse(is.na(m["srmr"]), NA, sprintf("%.3f", m["srmr"])),
        NFI = ifelse(is.na(m["nfi"]), NA, sprintf("%.3f", m["nfi"])),
        Status = status_info$status,
        
        StatusColor = status_info$color,  # tetap: warna background status
        stringsAsFactors = FALSE
      )
    }))
    
    # Add Action column
    df$Action <- sapply(seq_len(nrow(df)), function(i) {
      as.character(
        actionButton(
          inputId = paste0("del_model_btn_", i),
          label = icon("trash"),
          class = "btn-danger btn-sm",
          style = "font-size: 11px !important; padding: 2px 10px !important;",
          onclick = paste0('Shiny.setInputValue("del_model_selected", "', df$Model[i], '", {priority: "event"})')
        )
      )
    })
    
    # Buat tabel HTML manual
    column(12,
           tags$div(
             style = "margin-top: 0px; font-size: 13px;",
             tags$b("Fit Model Comparison")),
           # ---Render table DT =====
           datatable(df[, c("Model", "Note", "chisq", "df", "chisq_df_ratio", "pvalue", "RMSEA", "CFI", "TLI", "GFI", "SRMR", "NFI", "Status", "Action")],
                     rownames = FALSE,
                     escape = FALSE,
                     extensions = 'Buttons',
                     options = list(
                       dom = 'Brt',
                       buttons = list(
                         list(
                           extend = 'csv',
                           text = 'Export CSV',
                           filename = paste0('Fit Model Comparison')
                         ),
                         list(
                           extend = 'excel',
                           text = 'Export Excel',
                           filename = paste0('Fit Model Comparison')
                         )),
                       pageLength = 30,
                       columnDefs = list(list(className = 'dt-center', targets = 2:11))
                     )) %>%
             # --- Tetap: style kolom Status dengan warna ---
             formatStyle('chisq_df_ratio', color = styleInterval(c(2,5), c('green','orange','red')), fontWeight = 'bold', textAlign = 'center') %>% 
             # formatStyle('df', color = styleInterval(c(2,5), c('green','orange','red'))[df$chi_sq_ratio], fontWeight = 'bold', textAlign = 'center') %>% 
             formatStyle('pvalue', color = styleInterval(c(0.05,0.1), c('red','green','green')), fontWeight = 'bold', textAlign = 'center') %>%
             formatStyle('RMSEA', color = styleInterval(c(0.08,0.1), c('green','orange','red')), fontWeight = 'bold', textAlign = 'center') %>% 
             formatStyle('CFI', color = styleInterval(c(0.88,0.9), c('red','orange','green')), fontWeight = 'bold', textAlign = 'center') %>% 
             formatStyle('TLI', color = styleInterval(c(0.88,0.9), c('red','orange','green')), fontWeight = 'bold', textAlign = 'center') %>%
             formatStyle('GFI', color = styleInterval(c(0.88,0.9), c('red','orange','green')), fontWeight = 'bold', textAlign = 'center') %>% 
             formatStyle('SRMR', color = styleInterval(c(0.05,0.08), c('green','orange','red')), fontWeight = 'bold', textAlign = 'center') %>% 
             formatStyle('NFI', color = styleInterval(c(0.88,0.9), c('red','orange','green')), fontWeight = 'bold', textAlign = 'center') %>% 
             formatStyle('Status', backgroundColor = styleEqual(df$Status, df$StatusColor), fontWeight = 'bold', color = 'black', textAlign = 'center'),
           tags$div(
             style = "margin-top: 0px; font-size: 12px; color: #6c757d;",
             tags$b("Interpretation Guidelines:"),
             tags$ul(
               tags$li(tags$span(style = "color: green;", "Good:"),"χ² < 2 df; pvalue ≥ 0.05; RMSEA < 0.08; CFI/TLI/NFI ≥ 0.9; SRMR ≤ 0.05;"),
               tags$li(tags$span(style = "color: orange;", "Acceptable:"), "2 df ≤ χ² < 5 df; RMSEA < 0.1; CFI/TLI/NFI ≥ 0.89; SRMR ≤ 0.08;"), 
               tags$li(tags$span(style = "color: red;", "Poor:"), "Outside acceptable ranges")
             )
           ),
           tags$hr(),
           column(6,
                  tags$div(
                    style = "margin-top: 0px; font-size: 13px;",
                    tags$b("Average Variance Explained (AVE):"),
                    DTOutput("ave_table")  # Table AVE
                  )), 
           column(6,
                  tags$div(
                    style = "margin-top: 1px; font-size: 13px;",
                    tags$b("Composite Reliability (CR):"),
                    DTOutput("reliability_table"),  # Table Reliability
                  )),
           tags$div(
             style = "margin-top: 0px; font-size: 12px; color: #6c757d;",
             tags$b("Interpretation Guidelines:"),
             tags$ul(
               tags$li(tags$span(style = "color: green;", "Good:"),"AVE ≥ 0.5; CR ≥ 0.7"),
               tags$li(tags$span(style = "color: orange;", "Acceptable:"), "AVE ≥ 0.45; CR ≥ 0.6"), 
               tags$li(tags$span(style = "color: red;", "Poor:"), "Outside acceptable ranges")
             ),
             tags$p(tags$span(style = "color: blue;", "References:"), "Alamer (2025); Hu & Bentler (1999); Kline (2015); & Schumacker & Lomax (2016)")
           ),
           
           conditionalPanel(
             condition = "input.htmt_opt == true",
             tags$hr(),
             tags$div(
               style = "margin-top: 0px; font-size: 13px;",
               tags$b('Heterotrait–Monotrait Ratio (HTMT) for Current Model')), 
             DTOutput("htmt_table"),  
             tags$div(
               style = "margin-top: 0px; font-size: 12px; color: #6c757d;",
               tags$b("Interpretation Guidelines:"),
               tags$ul(
                 tags$li(tags$span(style = "color: green;", "Good discriminant validity:"),"HTMT < 0.7 "),
                 tags$li(tags$span(style = "color: orange;", "Acceptable discriminant validity:"), "HTMT < 0.8"), 
                 tags$li(tags$span(style = "color: red;", "Potential discriminant validity issues:"), "HTMT ≥ 0.8")
               )
             )
           ),
           tags$hr(),
           column(12,
                  tags$div(
                    style = "margin-top: 0px; font-size: 13px;",
                    tags$b('Modification Indices')),    
                  DTOutput("mi_table")
           )
    )
    
  })
  
  # Handle model deletion
  observeEvent(input$del_model_selected, {
    req(input$del_model_selected)
    
    cur_list <- cfa_fit_list()
    model_id <- input$del_model_selected
    
    if (model_id %in% names(cur_list)) {
      cur_list[[model_id]] <- NULL
      cfa_fit_list(cur_list)
      
      # Update current ID
      if (length(cur_list) > 0) {
        cfa_current_id(names(cur_list)[length(cur_list)])
      } else {
        cfa_current_id(NULL)
      }
      
      showNotification(paste("Deleted", model_id), type = "message")
    }
  })
  
  # === Output: HTMT Table =====
  output$htmt_table <- renderDT({
    lst <- cfa_fit_list()
    if (length(lst) == 0) return(NULL)
    
    # Ambil model terakhir
    last_model <- lst[[length(lst)]]
    
    if (!is.null(last_model$htmt) && !identical(last_model$htmt, '<NA>')) {
      # Konversi HTMT matrix ke data frame
      htmt_matrix <- as.matrix(last_model$htmt)
      htmt_df <- as.data.frame(htmt_matrix)
      
      # Tambahkan nama baris sebagai kolom pertama
      htmt_df <- cbind(Variable = rownames(htmt_df), htmt_df)
      rownames(htmt_df) <- NULL
      
      # Identifikasi kolom numeric (HTMT values)
      numeric_cols <- which(sapply(htmt_df, is.numeric))
      
      datatable(htmt_df, rownames = FALSE, extensions = 'Buttons',
                options = list(
                  dom = 'Bt',
                  pageLength = nrow(htmt_df),
                  buttons = list(
                    list(
                      extend = 'csv',
                      text = 'Export CSV',
                      filename = paste0('HTMT')
                    ),
                    list(
                      extend = 'excel',
                      text = 'Export Excel',
                      filename = paste0('HTMT')
                    ))
                )) %>%
        formatStyle(columns = numeric_cols,
                    color = styleInterval(c(0.7, 0.8), c('green', 'orange', 'red'))) %>%
        formatStyle(columns = 'Variable',
                    backgroundColor = '#f8f9fa',
                    fontWeight = 'bold') %>%
        formatRound(columns = numeric_cols, digits = 3)
    } else {
      retu(data.frame(Message = "HTMT not available for the last model"))
    }
  })
  
  # ===== TABEL AVE =====
  output$ave_table <- renderDT({
    lst <- cfa_fit_list()
    if (length(lst) == 0) return(datatable(data.frame(Message = "No models available")))
    
    # Pastikan dplyr tersedia
    if (!require(dplyr, quietly = TRUE)) {
      stop("Package dplyr is required")
    }
    
    ave_data <- lapply(lst, function(x) {
      if (!is.null(x$ave) && !identical(x$ave, '<NA>')) {
        # Konversi AVE ke data frame
        ave_df <- as.data.frame(as.list(round(x$ave, 3)))
        ave_df$Model <- x$id
        return(ave_df)
      } else {
        data.frame(Model = x$id, Note = "AVE not available")
      }
    })
    
    # Gabungkan dengan full_join berantai
    combined_data <- Reduce(function(x, y) {
      dplyr::full_join(x, y, by = intersect(names(x), names(y)))
    }, ave_data)
    
    # Pastikan Model ada di kolom pertama
    if ("Model" %in% names(combined_data)) {
      model_col <- combined_data$Model
      combined_data$Model <- NULL
      combined_data <- data.frame(Model = model_col, combined_data)
    }
    
    datatable(combined_data, extensions = "Buttons",
              rownames = FALSE, 
             
              options = list( pageLength = 15,
                              dom = 'Btp',
                              buttons = list(
                               list(
                                 extend = 'csv',
                                 text = 'Export CSV',
                                 filename = paste0('AVE')
                               ),
                               list(
                                 extend = 'excel',
                                 text = 'Export Excel',
                                 filename = paste0('AVE')
                               )))) %>%
      formatStyle(
        columns = which(!names(combined_data) %in% c("Model", "Note")),
        color = styleInterval(c(0.45, 0.5), c('red', 'orange', 'green'))
      ) %>%
      formatRound(columns = which(sapply(combined_data, is.numeric)), digits = 3)
  })
  
  # ===== Table  Reliability =====
  output$reliability_table <- renderDT({
    lst <- cfa_fit_list()
    if (length(lst) == 0) return(datatable(data.frame(Message = "No models available")))
    
    rel_data <- lapply(lst, function(x) {
      if (!is.null(x$relsem) && !identical(x$relsem, '<NA>')) {
        rel_df <- as.data.frame(as.list(round(x$relsem, 3)), stringsAsFactors = FALSE)
        rel_df$Model <- x$id
        return(rel_df)
      } else {
        data.frame(Model = x$id, Note = "Reliability not available", stringsAsFactors = FALSE)
      }
    })
    
    combined_data <- Reduce(function(x, y) {
      common_cols <- intersect(names(x), names(y))
      if (length(common_cols) > 0) {
        dplyr::full_join(x, y, by = common_cols)
      } else {
        # Jika tidak ada kolom common, gabungkan secara manual
        cbind(x, y[, setdiff(names(y), names(x)), drop = FALSE])
      }
    }, rel_data)
    
    # Pastikan Model ada di kolom pertama
    if ("Model" %in% names(combined_data)) {
      model_col <- combined_data$Model
      combined_data$Model <- NULL
      combined_data <- data.frame(Model = model_col, combined_data, stringsAsFactors = FALSE)
    }
    
    # Identifikasi kolom numeric (Reliability values)
    numeric_cols <- which(sapply(combined_data, is.numeric))
    
    datatable(combined_data, rownames = FALSE, extensions = 'Buttons',
              options = list( pageLength = 15,
                              dom = 'Btp',
                              buttons = list(
                               list(
                                 extend = 'csv',
                                 text = 'Export CSV',
                                 filename = paste0('Composite Realiability')
                               ),
                               list(
                                 extend = 'excel',
                                 text = 'Export Excel',
                                 filename = paste0('Composite Reliability')
                               ))
              )) %>%
      formatStyle(columns = numeric_cols,
                  color = styleInterval(c(0.6, 0.7), c('red', 'orange', 'green'))) %>%
      formatStyle(columns = 'Model',
                  backgroundColor = '#f8f9fa') %>%
      formatRound(columns = numeric_cols, digits = 3)
  })
  
  # ===== Table  FSCORES =====
  output$fscores_cfa <- renderDT({
    req(input$run_cfa)
    lst <- cfa_fit_list()
    cur_id <- cfa_current_id()
    if (is.null(cur_id)) return(NULL)
    scoreCfa <- cfa_fit_list()[[cur_id]]$scoreCfa
    numeric_cols <- which(sapply(scoreCfa, is.numeric))
    datatable(scoreCfa, rownames = TRUE, extensions = 'Buttons',
              options = list(dom = 'Brtp',
                             buttons = list(
                               list(
                                 extend = 'csv',
                                 text = 'Export CSV',
                                 filename = paste0('Factor Scores')
                               ),
                               list(
                                 extend = 'excel',
                                 text = 'Export Excel',
                                 filename = paste0('Factor Scores')
                               )))) %>%
      formatRound(columns = numeric_cols, digits = 3)
  }, server = FALSE)
  
  # Helper function untuk menentukan status fit ======
  get_fit_status <- function(chisq,df, pvalue, rmsea, cfi, srmr,gfi) {
    if (is.na(chisq)||is.na(rmsea) || is.na(cfi) || is.na(srmr)|| is.na(gfi)) {
      return(list(status = "Unknown", color = "#e2e3e5"))
    }
    if (chisq/df<2 && rmsea < 0.08 && cfi >= 0.9 && gfi >= 0.9 && srmr <= 0.05) {
      return(list(status = "Excellent", color = 'lightgreen'))
    }  else if (pvalue >=0.05 && rmsea < 0.08 && cfi >= 0.9 && gfi >= 0.9 && srmr <= 0.05) {
      return(list(status = "Excellent", color = 'lightgreen'))
    } else if (pvalue < 0.05 && rmsea < 0.08 && cfi >= 0.9 && gfi >= 0.9 && srmr <= 0.05) {
      return(list(status = "Good", color = "#d4edda"))
    } else if  (
      sum(c(chisq/df<2, rmsea < 0.08,cfi >= 0.9, gfi >= 0.9,srmr <= 0.05,pvalue >= 0.05)) >= 3 ||
      sum(c(chisq/df<2, rmsea < 0.08,cfi >= 0.9, gfi >= 0.9,srmr <= 0.05)) >= 3
    ) {
      return(list(status = "Acceptable", color = "#fff3cd"))
    } else {
      return(list(status = "Poor", color = "#f8d7da"))
    }
  }
  
  
  # Loadings table
  output$loadings_table <- renderDT({
    cur_id <- cfa_current_id()
    if (is.null(cur_id)) return(NULL)
    
    fit <- cfa_fit_list()[[cur_id]]$fit
    
    if (input$cfa_std_est) {
      sol <- tryCatch(lavaan::standardizedSolution(fit), error = function(e) NULL)
    } else {
      sol <- tryCatch(lavaan::parameterEstimates(fit), error = function(e) NULL)
    }
    
    if (is.null(sol)) return(datatable(data.frame(Message = "No parameter estimates available")))
    
    loadings <- sol[sol$op == "=~", ]
    
    if (nrow(loadings) == 0) return(datatable(data.frame(Message = "No factor loadings found")))
    
    if (input$cfa_std_est) {
      display_df <- loadings[, c("lhs","rhs","est.std","pvalue")]
      colnames(display_df) <- c("Factor","Item","StdLoading","pvalue")
      display_df$StdLoading <- round(display_df$StdLoading, 3)
    } else {
      display_df <- loadings[, c("lhs","rhs","est","se","pvalue")]
      colnames(display_df) <- c("Factor","Item","Estimate","SE","pvalue")
      display_df$Estimate <- round(display_df$Estimate, 3)
      display_df$SE <- round(display_df$SE, 3)
    }
    
    display_df$pvalue <- format.pval(display_df$pvalue, digits = 3)
    
    datatable(display_df, extensions = 'Buttons',
              options = list(pageLength = 15, scrollX = TRUE,dom = 'Brtp',
                             buttons = list(
                               list(
                                 extend = 'csv',
                                 text = 'Export CSV',
                                 filename = paste0('Factor Loading')
                               ),
                               list(
                                 extend = 'excel',
                                 text = 'Export Excel',
                                 filename = paste0('Factor Loading')
                               )))) %>% 
      formatStyle(columns = 'StdLoading',
                  fontWeight = 'bold',
                  color = styleInterval(c(0.3, 0.5, 0.7), c('red', 'orange', 'blue', 'green')) )
  }, server = FALSE)
  
  # Variances table
  output$variances_table <- renderDT({
    cur_id <- cfa_current_id()
    if (is.null(cur_id)) return(NULL)
    
    fit <- cfa_fit_list()[[cur_id]]$fit
    
    if (input$cfa_std_est) {
      sol <- tryCatch(lavaan::standardizedSolution(fit), error = function(e) NULL)
    } else {
      sol <- tryCatch(lavaan::parameterEstimates(fit), error = function(e) NULL)
    }
    
    if (is.null(sol)) return(datatable(data.frame(Message = "No parameter estimates available")))
    
    variances <- sol[sol$op == "~~" & sol$lhs == sol$rhs, ]
    
    if (nrow(variances) == 0) return(datatable(data.frame(Message = "No variances found")))
    
    if (input$cfa_std_est) {
      display_df <- variances[, c("lhs", "est.std", "se", "pvalue")]
      colnames(display_df) <- c("Variable", "StdVariance", "SE", "pvalue")
      display_df$StdVariance <- round(display_df$StdVariance, 3)
      display_df$SE <- round(display_df$SE, 3)
    } else {
      display_df <- variances[, c("lhs", "est", "se", "pvalue")]
      colnames(display_df) <- c("Variable", "Variance", "SE", "pvalue")
      display_df$Variance <- round(display_df$Variance, 3)
      display_df$SE <- round(display_df$SE, 3)
    }
    
    display_df$pvalue <- format.pval(display_df$pvalue, digits = 3)
    
    datatable(display_df, extensions = 'Buttons',
              options = list(pageLength = 15, scrollX = TRUE, dom = 'Brtp',
                             buttons = list(
                               list(extend = 'csv', text = 'Export CSV', filename = 'Variances'),
                               list(extend = 'excel', text = 'Export Excel', filename = 'Variances')
                             )))
  }, server = FALSE)
  
  # Variance warning
  output$variance_warning <- renderUI({
    cur_id <- cfa_current_id()
    if (is.null(cur_id)) return(NULL)
    
    fit <- cfa_fit_list()[[cur_id]]$fit
    sol <- tryCatch(lavaan::parameterEstimates(fit), error = function(e) NULL)
    
    if (is.null(sol)) return(NULL)
    
    variances <- sol[sol$op == "~~" & sol$lhs == sol$rhs, ]
    has_negative <- any(variances$est < 0, na.rm = TRUE)
    
    if (has_negative) {
      tags$div(
        class = "alert alert-danger",
        icon("exclamation-triangle"),
        tags$b(" Warning: Heywood case detected! One or more variances are negative. This may indicate a problem with the model or data.")
      )
    } else {
      NULL
    }
  })

  # HTML Report Download Handler
  # Modal for HTML export
  observeEvent(input$btn_export_modal, {
    showModal(modalDialog(
      title = "Export HTML Report",
      tags$div(
        style = "text-align: center;",
        tags$h4("Your report is ready to download!"),
        tags$p("If you find projectLSA helpful for your research, consider supporting its continuous development. Your support helps us keep the project alive!"),
        tags$div(
          style = "background-color: #f8f9fa; border: 2px dashed #17a2b8; padding: 20px; border-radius: 10px; margin-bottom: 20px;",
          tags$strong("Support via Bank BNI:", style="color: #17a2b8; font-size: 16px;"), tags$br(),
          tags$span("0331355050", style="font-size: 24px; font-weight: bold; color: #d32f2f; letter-spacing: 2px;"), tags$br(),
          tags$em("Account Name: Hasan Djidu"), tags$br(), tags$br(),
          tags$strong("International Transfer (SWIFT):", style="color: #17a2b8; font-size: 14px;"), tags$br(),
          tags$span("BNINIDJA", style="font-size: 16px; font-weight: bold; color: #d32f2f;")
        ),
        downloadButton("download_report_html", "Download Report Now", class = "btn btn-success btn-lg")
      ),
      footer = modalButton("Close"),
      easyClose = TRUE
    ))
  })
  
  output$download_report_html <- downloadHandler(
    filename = function() {
      cur_id <- cfa_current_id()
      if(is.null(cur_id)) return(paste0("Report_", Sys.Date(), ".html"))
      
      fit_obj <- cfa_fit_list()[[cur_id]]$fit
      pe <- lavaan::parameterEstimates(fit_obj)
      
      # Detect if SEM
      is_sem <- any(pe$op == "~")
      prefix <- if (is_sem) "SEM_Report_" else "CFA_Report_"
      
      # Extract main latent name
      latents <- unique(pe$lhs[pe$op == "=~"])
      latent_name <- if(length(latents) > 0) latents[length(latents)] else "Model"
      
      paste0(prefix, latent_name, "_", Sys.Date(), ".html")
    },
    content = function(file) {
      req(cfa_current_id())
      cur_id <- cfa_current_id()
      fit_info <- cfa_fit_list()[[cur_id]]
      
      report_path <- file.path(system.file("app", package = "projectLSA"), "cfa_report.Rmd")
      if (report_path == "" || !file.exists(report_path)) {
        report_path <- "cfa_report.Rmd"
      }
      
      tempReport <- file.path(tempdir(), "cfa_report.Rmd")
      file.copy(report_path, tempReport, overwrite = TRUE)
      
      # Get plot colors safely for the params
      if (input$plot_color_scheme == "Custom") {
        man_col <- input$mancolour
        lat_col <- input$latcolour
      } else {
        colors <- get_color_scheme(input$plot_color_scheme)
        man_col <- colors$man
        lat_col <- colors$lat
      }
      
      # Get all models for history
      lst <- cfa_fit_list()
      fit_initial <- lst[[1]]$fit
      
      model_history_df <- do.call(rbind, lapply(lst, function(x){
        m <- x$measures
        data.frame(
          Model = x$id,
          Note = ifelse(is.null(x$note) || x$note == "", "-", x$note),
          chisq = m["chisq"],
          df = m["df"],
          pvalue = m["pvalue"],
          RMSEA = m["rmsea"],
          CFI = m["cfi"],
          TLI = m["tli"],
          SRMR = m["srmr"],
          stringsAsFactors = FALSE
        )
      }))
      
      # Render to a temporary html file first to ensure pandoc handles extensions correctly
      out_html <- file.path(tempdir(), "cfa_report_out.html")
      
      # Detect model type
      pe <- lavaan::parameterEstimates(fit_info$fit)
      model_type <- if(any(pe$op == "~")) "SEM" else "CFA"
      
      showModal(modalDialog("Generating Report...", footer = NULL))
      tryCatch({
        rmarkdown::render(tempReport, output_file = out_html,
          params = list(
            model_type = model_type,
            fit_final = fit_info$fit,
            fit_initial = fit_initial,
            model_history_df = model_history_df,
            dec = input$cfa_dec_sep,
            ave = fit_info$ave,
            relsem = fit_info$relsem,
            htmt = fit_info$htmt,
            plot_standardized = input$plot_standardized,
            plot_style = input$plot_style,
            plot_layout = input$plot_layout,
            plot_residuals = input$plot_residuals,
            plot_exoCov = input$plot_exoCov,
            plot_edge_label_size = input$plot_edge_label_size,
            plot_nodesize_lat = input$plot_nodesize_lat,
            plot_nodesize_man = input$plot_nodesize_man,
            plot_nodesize_man2 = input$plot_nodesize_man2,
            plot_rotation = input$plot_rotation,
            man_col = man_col,
            lat_col = lat_col,
            bifactor = input$bifactor,
            edgewidth = input$edgewidth,
            plotwidth = input$plotwidth,
            plotheight = input$plotheight
          ),
          envir = new.env(parent = globalenv())
        )
        
        # Copy to the Shiny-provided 'file' path
        file.copy(out_html, file, overwrite = TRUE)
        
      }, error = function(e) {
        showNotification(paste("Error generating report:", e$message), type = "error")
      }, finally = {
        removeModal()
      })
    },
    contentType = "text/html"
  )
  # Modification indices
  mi_df <- reactive({
    cur_id <- cfa_current_id()
    if (is.null(cur_id)) return(NULL)
    
    fit <- cfa_fit_list()[[cur_id]]$fit
    mi <- tryCatch(lavaan::modindices(fit, sort. = TRUE, minimum.value = 1), error = function(e) NULL)
    
    if (is.null(mi) || nrow(mi) == 0) return(NULL)
    
    if (nrow(mi) > 20) mi <- mi[1:20, ]
    
    mi
  })
  
  output$mi_table <- renderDT({
    mi <- mi_df()
    #mi <- mi %>% dplyr::mutate(across(where(is.numeric), ~ round(.x, 3)))
    if (is.null(mi)) return(datatable(data.frame(Message = "No modification indices available")))
    
    mi$Action <- sapply(1:nrow(mi), function(i) {
      as.character(
        actionButton(
          class = "btn-primary sm",
          style = "font-size: 11px !important; padding: 2px 10px !important;",  
          inputId = paste0("mi_btn_", i),
          label = "Use This",
          onclick = paste0('Shiny.setInputValue("mi_selected", "', i, '", {priority: "event"})')
        )
      )
    })
    
    display_mi <- mi[, c("lhs", "op", "rhs", "mi", "epc", "Action")]
    colnames(display_mi) <- c("From", "Operator", "To", "MI", "EPC", "Action")
    
    datatable(display_mi, rownames = FALSE, escape = FALSE, 
              options = list(pageLength = 10, scrollX = TRUE,dom = 'rtp',
                             # Atur ukuran font dan styling
                             initComplete = JS(
                               "function(settings, json) {",
                               "$(this.api().table().header()).css({'font-size': '5px', 'font-weight': 'bold'});",
                               "$(this.api().table().body()).css({'font-size': '5px', 'line-height': '1'});",
                               "}"
                             )))    %>% 
      formatRound(columns = which(sapply(display_mi, is.numeric)), digits = 3)
  }, server = FALSE)
  
  # Handle MI button clicks
  observeEvent(input$mi_selected, {
    req(input$mi_selected)
    
    mi <- mi_df()
    if (is.null(mi)) return()
    
    idx <- as.numeric(input$mi_selected)
    if (idx > nrow(mi)) return()
    
    row <- mi[idx, ]
    
    if (row$op == "=~") {
      new_line <- paste(row$lhs, "=~", row$rhs)
    } else if (row$op == "~~") {
      new_line <- paste(row$lhs, "~~", row$rhs)
    } else if (row$op == "~") {
      new_line <- paste(row$lhs, "~", row$rhs)
    } else {
      new_line <- paste(row$lhs, row$op, row$rhs)
    }
    
    current_model <- input$cfa_model_text
    updated_model <- paste(current_model, "\n", new_line)
    
    updateTextAreaInput(session, "cfa_model_text", value = updated_model)
    
    showNotification(paste("Added MI suggestion to model:", new_line), type = "message")
  })
  
  # Update fungsi get_color_scheme
  get_color_scheme <- function(scheme_name, custom_man = NULL, custom_lat = NULL) {
    if (scheme_name == "Custom" && !is.null(custom_man) && !is.null(custom_lat)) {
      return(list(man = custom_man, lat = custom_lat))
    }
    
    switch(scheme_name,
           "Blue-Yellow" = list(
             man = "#A1E3F9",
             lat = c("#FFFFBA")
           ),
           "Ocean" = list(
             man = "#B3E5FC",
             lat = c("#81D4FA", "#4FC3F7", "#29B6F6")
           ),
           "Forest" = list(
             man = "#C8E6C9",
             lat = c("#A5D6A7", "#81C784", "#4CAF50")
           ),
           "Rainbow" = list(
             man = "#B3E5FC", 
             lat = c("#FF8A65", "#81D4FA", "#AED581", "#CE93D8","#FFF176" )
           ),
           "Pastel" = list(
             man = "#F8ECE5",
             lat = c("#E2E2E2", "#E8EAF6", "#F3E5F5", "#E0F2F1")
           ),
           "Greyscale" = list(
             man = "#EEEEEE",
             lat = c("#DDDDDD", "#CCCCCC", "#BBBBBB", "#AAAAAA")
           ),
           "Earth" = list(
             man = "#D7CCC8",
             lat = c("#BCAAA4", "#A1887F", "#8D6E63", "#795548")
           ),
           "Vibrant" = list(
             man = "#FFE0B2",
             lat = c("#FF5252", "#E040FB", "#7C4DFF", "#536DFE")
           ),
           "Monochrome" = list(
             man = "#E0E0E0",
             lat = c("#9E9E9E", "#757575", "#616161", "#424242")
           ),
           "Sunset" = list(
             man = "#FFCCBC",
             lat = c("#FF8A65", "#FF7043", "#F4511E", "#E64A19")
           ),
           "Rose" = list(
             man = "#F8BBD0",
             lat = c("#F06292", "#EC407A", "#E91E63", "#D81B60")
           ),
           "Mint" = list(
             man = "#B2DFDB",
             lat = c("#4DB6AC", "#26A69A", "#009688", "#00897B")
           )
    )
  }
  
  
  # Get factor names safely
  get_factor_names <- function(fit_obj) {
    tryCatch({
      if (inherits(fit_obj, "lavaan")) {
        params <- lavaan::parameterEstimates(fit_obj)
        factors <- unique(params$lhs[params$op == "=~"])
        if (length(factors) > 0) return(factors)
        
        model_text <- fit_obj@Model@model.syntax
        if (is.character(model_text)) {
          lines <- strsplit(model_text, "\n")[[1]]
          factor_lines <- grep("=~", lines, value = TRUE)
          factors <- sapply(strsplit(factor_lines, "=~"), function(x) trimws(x[1]))
          return(factors)
        }
      }
      return("Unknown")
    }, error = function(e) {
      return("Error getting factors")
    })
  }
  
  # Helper to generate the fit text
  get_fit_text <- function(m, selected, dec_sep = ".") {
    all_indices <- c("chisq","df","pvalue","rmsea","cfi","gfi","srmr","tli","nfi")
    if ("SELECT ALL" %in% selected) selected <- all_indices
    selected <- intersect(all_indices, selected)
    labels <- c(chisq="χ²", df="df", pvalue="p", rmsea="RMSEA", cfi="CFI", gfi="GFI", srmr="SRMR", tli="TLI", nfi="NFI")
    format_val <- function(name, val) {
      if (is.na(val)) return("NA")
      fmt_with_sep <- function(x) {
        if(dec_sep == ",") gsub("\\.", ",", x) else x
      }
      if (name == "pvalue") return(fmt_with_sep(format.pval(val, digits = 3)))
      if (name == "chisq")  return(fmt_with_sep(as.character(round(val, 2))))
      if (name == "df")     return(val)
      return(fmt_with_sep(sprintf("%.3f", val)))
    }
    paste(sapply(selected, function(n) paste0(labels[n], "=", format_val(n, m[n]))), collapse = "; ")
  }

  render_cfa_plot_impl <- function() {
    cur_id <- cfa_current_id()
    if (is.null(cur_id)) {
      plot(1, 1, type = "n", xlab = "", ylab = "", axes = FALSE, main = "No CFA model available")
      text(1, 1, "Please run a CFA model first\n\n1. Select 'Built-in: HolzingerSwineford1939'\n2. Keep the example model\n3. Click 'Run CFA'", cex = 1.2, col = "darkred")
      return(invisible(NULL))
    }
    
    obj <- cfa_fit_list()[[cur_id]]
    fit_obj <- obj$fit
    m <- obj$measures
    
    if (input$plot_color_scheme == "Custom") {
      colors <- list(man = input$mancolour, lat = input$latcolour)
    } else {
      colors <- get_color_scheme(input$plot_color_scheme)
    }
    
    # Fix bottom margin so plot expands upwards
    b_margin <- 6
    A <- semPlot::semPaths(
      fit_obj,
      what = ifelse(input$plot_standardized, "std", "est"),
      style = input$plot_style,
      layout = input$plot_layout,
      residuals = input$plot_residuals,
      exoCov = input$plot_exoCov,
      edge.label.cex = input$plot_edge_label_size,
      nCharNodes = 6,
      nCharEdges = 6,
      sizeLat = input$plot_nodesize_lat,
      sizeMan = input$plot_nodesize_man,
      sizeMan2 = input$plot_nodesize_man2,
      rotation = input$plot_rotation,
      color = colors,
      label.color = "black",
      mar = c(b_margin, 15-input$plotwidth, 20-input$plotheight, 15-input$plotwidth),
      bifactor = input$bifactor,
      esize = input$edgewidth,
      edge.color = "black",       
      freeStyle = c(1, "black"),
      fixedStyle = c(2, "black"),
      fade = FALSE,
      DoNotPlot = TRUE
    )
    
    # Add significance
    A <- semptools::mark_sig(semPaths_plot = A, object = fit_obj,
                             alphas = c("*" = 0.05, "**" = 0.01, "***" = 0.001))
    
    # Apply decimal separator to edge labels
    if (input$cfa_dec_sep == ",") {
      A$graphAttributes$Edges$labels <- gsub("\\.", ",", A$graphAttributes$Edges$labels)
    }
    
    # Plot the graph
    plot(A)
    
    # Add fit info inside a box
    factor_names <- get_factor_names(fit_obj)
    fit_text_raw <- get_fit_text(m, input$fit_indices_selected, dec_sep = input$cfa_dec_sep)
    wrapped_fit <- paste(strwrap(paste0("Fit indices: ", fit_text_raw), width = 110), collapse = "\n")
    
    estimator_text <- paste0("Estimator = ", input$cfa_estimator, " | Factors: ", paste(factor_names, collapse = ", "))
    sig_sign <- '*** : p ≤ 0.001", "** : p ≤ 0.01", "* : p ≤ 0.05'
    
    legend_text <- c(wrapped_fit, estimator_text, sig_sign)
    text_colors <- c("#2c3e50", "darkblue", "#6c757d")
    
    # Get legend coordinates without plotting
    leg_info <- legend("bottom", legend = legend_text, plot = FALSE, inset = c(0, 0), cex = 0.9, y.intersp = 1.0)
    
    # Draw empty legend box with correct width
    max_w <- max(strwidth(legend_text, cex = 0.9))
    legend("bottom", legend = rep("", length(legend_text)), bg = "#f8f9fa", box.col = "#e2e8f0", 
           inset = c(0, 0), cex = 0.9, xpd = TRUE, y.intersp = 1.0, text.width = max_w)
           
    # Draw text centered
    center_x <- leg_info$rect$left + leg_info$rect$w / 2
    y_coords <- leg_info$text$y
    for(i in seq_along(legend_text)) {
      text(x = center_x, y = y_coords[i], labels = legend_text[i], col = text_colors[i], cex = 0.9, xpd = TRUE, adj = c(0.5, 0.5))
    }
    # (Watermark has been removed as requested)
  }

  output$path_plot <- renderPlot({
    tryCatch({
      render_cfa_plot_impl()
    }, error = function(e) {
      plot(1, 1, type = "n", xlab = "", ylab = "", axes = FALSE, main = "Plot Error")
      text(1, 1, paste("Error creating plot:\n", e$message), cex = 1.2, col = "red")
    })
  })
  
  output$download_cfa_plot <- downloadHandler(
    filename = function() {
      paste0("CFA_Plot_", Sys.Date(), ".png")
    },
    content = function(file) {
      png(file, width = 12, height = 10, units = "in", res = 300)
      tryCatch({
        render_cfa_plot_impl()
      }, finally = {
        dev.off()
      })
    }
  )
  
  observeEvent(input$fit_indices_selected, {
    sel <- input$fit_indices_selected
    if ("SELECT ALL" %in% sel && length(sel) > 1)
      updateSelectInput(session, "fit_indices_selected",
                        selected = if (tail(sel,1)=="SELECT ALL") "SELECT ALL"
                        else setdiff(sel,"SELECT ALL"))
  })
  
  # Fit information below plot
  output$plot_fit_info <- renderUI({
    cur_id <- cfa_current_id()
    if (is.null(cur_id)) return(NULL)
    
    obj <- cfa_fit_list()[[cur_id]]
    m <- obj$measures
    fit_obj <- obj$fit
    
    # Get factor names safely
    factor_names <- get_factor_names(fit_obj)
    
    # fit_text <- paste0(
    #   "χ² = ", round(m["chisq"], 2), "; ",
    #   "df = ", m["df"], "; ",
    #   "p = ", ifelse(is.na(m["pvalue"]), "NA", format.pval(m["pvalue"], digits = 3)), "; ",
    #   "RMSEA = ", ifelse(is.na(m["rmsea"]), "NA", sprintf("%.3f", m["rmsea"])), "; ",
    #   "CFI = ", ifelse(is.na(m["cfi"]), "NA", sprintf("%.3f", m["cfi"])), "; ",
    #   "GFI = ", ifelse(is.na(m["gfi"]), "NA", sprintf("%.3f", m["gfi"])), "; ",
    #   "SRMR = ", ifelse(is.na(m["srmr"]), "NA", sprintf("%.3f", m["srmr"])),"; ",
    #   "TLI = ", ifelse(is.na(m["tli"]), "NA", sprintf("%.3f", m["tli"])),"; ",
    #   "NFI = ", ifelse(is.na(m["nfi"]), "NA", sprintf("%.3f", m["nfi"]))
    # 
    # )
    selected <- input$fit_indices_selected
    fit_text <- get_fit_text(m, selected, dec_sep = input$cfa_dec_sep)
    
    
    
    
    estimator_text <- paste0(
      "Estimator = ", input$cfa_estimator, 
      " | Factors: ", paste(factor_names, collapse = ", ")
    )
    sig_sign <- paste0(
      '*** : p ≤ 0.001", "** : p ≤ 0.01", "* : p ≤ 0.05'
    )
    
    tagList(
      tags$div(style = "margin-top: 0px; padding: 5px; background-color: #f8f9fa; border-radius: 5px; text-align: center;",
               tags$p(style = "margin: 0; font-size: 11px; color: #2c3e50;", 
                      tags$b("Fit indices: "), fit_text),
               tags$p(style = "margin: 0; font-size: 11px; color: darkblue;", 
                      estimator_text),
               tags$p(style = "margin: 0; font-size: 10px; color: #6c757d;", 
                      sig_sign)
      )
    )
  })
  
  # Download handlers
  output$download_ris <- downloadHandler(
    filename = function() {
      "projectLSA_citation.ris"
    },
    content = function(file) {
      ris_text <- c(
        "TY  - JOUR",
        "AU  - Djidu, Hasan",
        "AU  - Retnawati, Heri",
        "AU  - Hadi, Samsul",
        "AU  - Haryanto",
        "PY  - 2026",
        "DA  - 2026/04/20",
        "TI  - projectLSA: A Shiny Application for Integrated Latent Structure Analysis",
        "JO  - Applied Psychological Measurement",
        "DO  - 10.1177/01466216261446305",
        "UR  - https://doi.org/10.1177/01466216261446305",
        "ER  - "
      )
      writeLines(ris_text, file)
    }
  )

  
  
}

#' Latent Structure Analysis Toolkit
#'
#' Provides an interactive Shiny-based environment for performing Latent Profile
#' Analysis (LPA), Latent Class Analysis (LCA), Latent Trait Analysis (LTA; IRT),
#' Exploratory Factor Analysis (EFA), Confirmatory Factor Analysis (CFA), and
#' Structural Equation Modeling (SEM). All analyses are conducted through a
#' graphical user interface without requiring users to write code.
#'
#' @section Main Function:
#' The primary entry point is \code{\link{run_projectLSA}}, which launches the
#' full interactive application.
#'
#' @keywords internal
"_PACKAGE"

homepage_ui <- function() {
  
  fluidPage(

    div(class="landing",
        div(class="hero",
            div(class="welcome","WELCOME TO"),
            div( class = "app-name", "projectLSA" ),            
            div(

              HTML(
                '<div style="text-align: left; padding: 8px; background-color: rgba(255,255,255,0.1); border-left: 3px solid #ccc; margin-top: 5px; font-size: 10px; color: #555;">
                 <strong>How to Cite:</strong><br>
                 Djidu, H., Retnawati, H., Hadi, S., &amp; Haryanto. (2026). <em>projectLSA: A Shiny Application for Integrated Latent Structure Analysis</em>. Applied Psychological Measurement. 
                 <a href="https://doi.org/10.1177/01466216261446305" target="_blank" style="color: #0077dd;">https://doi.org/10.1177/01466216261446305</a><br>
                 Djidu, H., Retnawati, H., Hadi, S., Haryanto (2026). <em>projectLSA: R Shiny application for latent structure analysis with a graphical user interface</em>. 
                 <a href="https://doi.org/10.32614/CRAN.package.projectLSA" target="_blank" style="color: #0077dd;">https://doi.org/10.32614/CRAN.package.projectLSA</a>. R package.
                 </div>'
              ),
              br(),
              div(style = "display: flex; gap: 8px; justify-content: center; align-items: center; flex-wrap: wrap; margin-top: 5px;",
                  downloadButton("download_ris_home", "Download RIS", class = "btn btn-info btn-sm", style = "margin: 0 !important;"),
                  actionButton("btn_donate_home", "Support projectLSA", icon = icon("heart"), class = "btn btn-danger btn-sm btn-glow", style = "margin: 0 !important;", onclick="$(this).removeClass('btn-glow');")
              )
            )
            
        ),
        br(),
        div(class="subtitle","Choose one projectLSA you want to work on:")
    ),
    br(),
    div(class="quad-container",
        div(class="axis-label x-top","Latent = categorical"),
        div(class="axis-label x-bottom","Latent = scale"),
        div(class="axis-label y-left","Observed = scale <<"),
        div(class="axis-label y-right",">> Observed = categorical"),
        div(class="center-logo", tags$img(src="logoProjectLSA.png")),
        
        div(class="quad-card", id="card_lpa", onclick="$('#go_lpa').click();",
            div(class="project-icon", tags$i(class="fa-solid fa-chart-simple")),
            div(class="project-title","LPA - Latent Profile Analysis"),
            div(class="project-desc","Identify latent profile (unobserved subgroups) within your sample using continuous data."),
            actionButton("go_lpa", label="OPEN LPA", class="btn-pill")
        ),
        div(class="quad-card", id="card_lca", onclick="$('#go_lca').click();",
            div(class="project-icon", tags$i(class="fa-solid fa-layer-group")),
            div(class="project-title","LCA - Latent Class Analysis"),
            div(class="project-desc","Identify latent class (unobserved subgroups) within your sample using categorical data."),
            actionButton("go_lca", label="OPEN LCA", class="btn-pill")
        ),
        div(class="quad-card", id="card_fa",
            div(class="project-icon", tags$i(class="fa-solid fa-sitemap")),
            div(class="project-title","FA - Factor Analysis"),
            div(class="project-desc","Explore latent constructs with exploratory or confirmatory FA using continuous data."),
            
            div(style="display: flex; gap: 10px; justify-content: center; margin-top: 10px; width: 100%;",
                actionButton("go_efa", label="OPEN EFA", width = '50%', class="btn-pill btn-primary"),
                actionButton("go_cfa", label="OPEN CFA", width = '50%', class="btn-pill btn-secondary")
            )
            ),
        
        div(class="quad-card", id="card_lta", onclick="$('#go_lta').click();",
            div(class="project-icon", tags$i(class="fa-solid fa-brain")),
            div(class="project-title","LTA - Latent Trait Analysis"),
            div(class="project-desc","Estimate item and person parameters using LTA approaches using categorical data."),
            actionButton("go_lta", label="OPEN LTA", class="btn-pill")
        )
    ),
    div(
      style = "text-align:center;",
      tags$p(
        style = "font-size:13px; color:#777;",
        format(Sys.Date(), "%Y"), 
        "projectLSA. All rights reserved."
      )
    )
  )
}

# Fungsi untuk menu LCA
lca_ui <- function(project) {
  tabsetPanel(
    id = "main_tab_lca",
    
    # --- TAB 1: Prepare Data & Model ----
    tabPanel(
      title = tagList(icon("upload"), "Prepare Data & Model"),
      sidebarLayout(
        sidebarPanel(
          width = 3,
          actionButton("go_home", 
                       label = tagList(icon("home"), "Main Menu"), 
                       class = "btn btn-danger btn-block",
                       style = "width: 100% !important;"),
          br(),
          selectInput(
            "data_source_lca", 
            "Select Data Source:",
            choices = c("Upload Data" = "upload",
                        "Cheating (Dayton, 1998)" = "cheat",
                        "Lazarsfeld & Henry (1968)" = "lazar",
                        "General Social Survey (McCutcheon, 1987)" = "gss82"
            ),
            selected = "upload"
          ),
          conditionalPanel(
            condition = "input.data_source_lca == 'upload'",
            fileInput("datafile_lca", "Upload Data (csv/xlsx)", accept = c(".csv", ".xlsx"))
          ),
          uiOutput("id_select_ui_lca"),
          uiOutput("var_select_ui_lca"),
          numericInput("min_class_lca", "Min. Number of Class:", 1, min = 1),
          numericInput("max_class_lca", "Max. Number of Class:", 6, min = 3),
          uiOutput("cov_lca_ui"),
          
          br(),
          
          actionButton("run_lca", 
                       label = tagList(icon("play"), "Run LCA"), 
                       class = "btn btn-success btn-block",
                       style = "width: 100% !important;")
        ),
        mainPanel(
          width = 9,
          h5(icon("table"), "Data Preview"),
          DTOutput("data_preview_lca"),
          DTOutput("data_summary_lca"),
          br(),
          uiOutput("data_description"),
          
          br(),
          
          
        )
      )
    ),
    
    # --- TAB 2: Fit Model Comparison ----
    tabPanel(
      title = tagList(icon("chart-line"), "Fit Model Comparison"),
      value = "fit_tab_lca",  
      
      fluidRow(
        column(12,
               div(
                 style = "text-align:center;",  
                 div(
                   style = "display:inline-block; font-size:11px; line-height:0.9; padding:0; margin:0;",
                   h5(icon("info-circle"), "Model Fit Statistics"), 
                   DTOutput("fit_table_lca")
                 )
               ),
               br(),
               tags$div(
                 style = "margin-top: 0px; font-size: 12px; color: #6c757d;",
                 class = "badge-info",
                
                 tags$b("Note:"),
                 tags$ul(
                   tags$li(tags$span(style = "color: blue;", "AIC:"), 
                           "Akaike Information Criterion; smaller values indicate better model fit."),
                   tags$li(tags$span(style = "color: blue;", "BIC:"), 
                           "Bayesian Information Criterion; smaller values indicate better and more parsimonious model fit."),
                   tags$li(tags$span(style = "color: blue;", "Entropy:"), 
                           "A measure of classification quality (values closer to 1 indicate clearer class separation)."),
                   tags$li(tags$span(style = "color: blue;", "Gsq:"), 
                           "Likelihood ratio statistic (G²) for assessing absolute model fit."),
                   tags$li(tags$span(style = "color: blue;", "p_Gsq:"), 
                           HTML("Bootstrap p-value for G². <b>H₀:</b> the model with K classes adequately fits the data. p ≥ 0.05 indicates acceptable absolute fit.")
                   ),
                   tags$li(tags$span(style = "color: blue;", "Deviance:"), 
                           "Deviance statistic comparing two nested models with K and K+1 classes."),
                   tags$li(tags$span(style = "color: blue;", "p_Deviance:"), 
                           HTML("Bootstrap p-value from the deviance test reported for the <b>K-class model</b>. 
     <b>H₀:</b> the (K−1)-class model is not significantly worse than the K-class model. 
     p < 0.05 indicates that adding one class (from K−1 to K) significantly improves model fit.")
                   ),
                   tags$li(tags$span(style = "color: blue;", "Av_Prob:"), 
                           "Average posterior classification probability by class, indicating classification accuracy.")
                 )
               )
        ),
        column(6, h5(icon("chart-bar"), "AIC & BIC Comparison"), 
               downloadButton("download_plot_AicBic_LCA", "Download Plot AIC/BIC (.png)"),
               plotOutput("fit_plot_lca")
               ),
        column(6, h5(icon("chart-bar"), "Entropy & Average Posterior Probability Comparison"), 
               downloadButton("download_plot_entropy", "Download Plot Entropy & Av.Probs (.png)"),
               plotOutput("entropy_plot_lca")
        ), br(), br(),
      )
    ),
    
    # --- TAB 3: Best Model ----
    tabPanel(
      title = tagList(icon("star"), "Best Model"),
      sidebarLayout(
        sidebarPanel(
          width = 2, 
          numericInput("best_class_lca",label =  "Select the Best Number of Class:",value = 3, min = 1),
          uiOutput("class_name_inputs_lca")
        ),
        mainPanel(
          width = 10, 
          h5(icon("project-diagram"), "Class Plot of the Best Model"),
          downloadButton("download_plot_best_LCA", "Download Plot Best Model (.png)"),
          tags$b("Item-Categories Probabilities Plot"),
          ggiraph::girafeOutput("best_model_plot_lca", width = '100%', height = 'auto'),
          h5(icon("table"), "Class Size and Item-Category Probabilities"),
          tableOutput("summary_table_lca"),
          h5(icon("table"), "Latent Class Response Patterns"),
          DT::DTOutput("data_summary_lca_with_class"),
          br(),
          br(),
          
        )
      )
    ),
    
    # --- TAB 4: Summary & Report ----
    tabPanel(
      title = tagList(icon("file-alt"), "Summary & Report"),
      h4("Class Classification Data"),
      DTOutput("profile_table_lca"),
      h4("Compare Class"),
      column(12,
        column(3, uiOutput("var_x_ui")),
        column(4, uiOutput("var_y_ui")),
        column(2,
               selectInput(
                 "plot_type", "Plot Type:",
                 choices = c("Line" = "line", "Bar" = "bar"),
                 selected = "bar"
               )
        ),
        column(3,
               checkboxInput(
                 "check_anova",
                 label = "Compare Classes",
                 value = FALSE
               )
        )
      ),
      br(),

      column(12,
        column(6, plotOutput("cross_latent", height = "420px")),
        column(6, uiOutput("anova_lca_ui"))
      ),
      column(12, uiOutput("reg_lca_ui")),
      column(12,
             h4("Explore Other Variables by Class"),
             checkboxInput(
               "crosstab_result",
               label = "Explore variable",
               value = FALSE
             ),
             column(12, uiOutput("crosstab_ui")),
             br(),
             br(),
             br()
      ) 
    ),
    # --- TAB 5: About ----
    tabPanel(
      title = tagList(icon("info-circle"), "About"),
      fluidRow(
        column(
          width = 8, offset = 2,
          br(),
          div(
            style = "text-align:center;",
            tags$hr(),
            tags$h5("projectLSA Was Developed By:"),
            tags$p(
              tags$a(
                href = "https://scholar.google.com/citations?user=PSAwkTYAAAAJ&hl=id",
                target = "_blank",
                "Dr. Hasan Djidu, M.Pd."),
              tags$br(),
              "Universitas Sembilanbelas November Kolaka"
            ),
            tags$h5("Supervised By:"),
            tags$p(
              tags$a(
                href = "https://scholar.google.com/citations?user=7CzPTYIAAAAJ&hl=id",
                target = "_blank",
                "Prof. Dr. Heri Retnawati, M.Pd."), 
              tags$br(),
              "Universitas Negeri Yogyakarta"
            ),
            
            tags$p(tags$a(
              href = "https://scholar.google.com/citations?hl=id&user=VGKeBm0AAAAJ",
              target = "_blank",
              "Prof. Dr. Samsul Hadi"), 
              tags$br(),
              "Universitas Negeri Yogyakarta"
            ),
            tags$p(tags$a(
              href = "https://scholar.google.com/citations?hl=id&user=k4MA8XgAAAAJ",
              target = "_blank",
              "Dr. Drs. Ir. Haryanto, M.Pd., M.T."), 
              tags$br(),
              "Universitas Negeri Yogyakarta"
            ),
            tags$b("Contact:"),
            tags$a("hasandjidu@gmail.com"),
            tags$hr()
          )
        ),
        column(
          width = 8, offset = 2,
          h4("References (R Packages)"),
          uiOutput("package_references_lca"),
          br(),
          div(
            style = "text-align:center;",
            tags$p(
              style = "font-size:13px; color:#777;",
             format(Sys.Date(), "%Y"), 
              "projectLSA. All rights reserved."
            )
          )
        )
      )
    )
  )
}

# --- Project Latent Structure Analysis (Project LSA)  ---
# Author: Hasan Djidu

# ==== Load UI & Server Components ====
source("ui_module.R")
source("homepage_ui.R")
source("lca_ui.R")
source("lpa_ui.R")
source("efa_ui.R")
source("cfa_ui.R")
source("lta_ui.R")
source("plotinfose.R")
source("lta_info_vis.R")

# ==== Load Logic / Server Modules ====
source("serverEFA.R")
source("serverCFA.R")
source("serverLCA.R")
source("serverLPA.R")
source("serverLTA.R")

# ==== Misc utilities ====
source("reference_list.R")
source("simDataDesc.R")
source("downloadPlot.R")
source("report_utils.R")
source("styleCSS.R")

# ==== AI Assistant ====
source("ai_helper.R")
source("ai_widget_ui.R")
source("serverAIWidget.R")
source("console_widget_ui.R")

# ==== Main Library ======
library(shiny)
library(shinyWidgets)
library(httr)
library(jsonlite)
library(DT)
library(readxl)
library(dplyr)
library(ggplot2)
library(stringr)
library(plotly)
library(tidyr)
library(shinyBS)
library(haven)

options(shiny.maxRequestSize = 500 * 1024^2) # 300 MB

# ===== UI =====
ui <- fluidPage(
  styleCSS,
  uiOutput("mainUI"),
  ai_widget_ui(),
  console_widget_ui()
)
# ==== Server =====
server <- function(input, output, session) {

  # Global AI context: filled by each analysis module with its latest results
  ai_context <- reactiveValues(results_text = "", module = "", ai_report_text = NULL)

  # Global AI widget logic (chat, summary, settings)
  server_ai_widget(input, output, session, ai_context)

  # Global R console context
  console_context <- reactiveValues(text = "No R output yet.")
  output$global_console_output <- renderPrint({
    cat(console_context$text)
  })

  # === AI Settings Logic ===
  observeEvent(input$ai_provider, {
    providers <- ai_providers()
    selected <- input$ai_provider
    if (selected %in% names(providers)) {
      updateTextInput(session, "ai_model", value = providers[[selected]]$default_model)
    }
  })

  observeEvent(input$test_ai_connection, {
    req(input$ai_provider, input$ai_api_key)
    output$ai_connection_status_ui <- renderUI({
      tags$span(style = "color: blue;", "Testing...")
    })

    res <- ask_ai(
      prompt = "Hello! Please reply with exactly 'Connection OK'.",
      provider = input$ai_provider,
      model = input$ai_model,
      api_key = input$ai_api_key
    )

    if (grepl("Error", res, ignore.case = TRUE) || grepl("HTTP", res)) {
      output$ai_connection_status_ui <- renderUI({
        tags$span(style = "color: red;", res)
      })
    } else {
      output$ai_connection_status_ui <- renderUI({
        tags$span(style = "color: green; font-weight: bold;", icon("check"), " Connection OK")
      })
    }
  })

  observeEvent(TRUE,
    {
      showModal(modalDialog(
        title = NULL,
        div(
          style = "text-align:center; line-height:1.6;",
          tags$img(src = "logoProjectLSA.png", height = "70px"),
          tags$h3("Welcome to projectLSA"),
          tags$p(
            "projectLSA is an interactive R Shiny application designed to support ",
            "latent structure analysis for research & teaching purposes."
          ),
          HTML(
            "
         If you use this application in academic or research work, please cite it as follows:<br>
         <strong>In-text citation:</strong>
         <span style='color:#2563eb;'>(Djidu et al., 2026)</span><br><br>
         <strong>Reference:</strong><br>
         Djidu, H., Retnawati, H., Hadi, S., &amp; Haryanto. (2026).
         <em>projectLSA: A Shiny Application for Integrated Latent Structure Analysis</em>.
         Applied Psychological Measurement.
         <a href='https://doi.org/10.1177/01466216261446305' target='_blank'>
         https://doi.org/10.1177/01466216261446305</a><br><br>
         Djidu, H., Retnawati, H., Hadi, S., Haryanto (2026). <em>projectLSA: R Shiny application for latent structure analysis with a graphical user interface</em>.
         <a href='https://doi.org/10.32614/CRAN.package.projectLSA' target='_blank'>https://doi.org/10.32614/CRAN.package.projectLSA</a>. R package."
          )
        ),
        footer = tagList(
          actionButton("btn_donate_home_modal", "Support Us (Donate)", icon = icon("heart"), class = "btn btn-danger btn-sm btn-glow", onclick = "$(this).removeClass('btn-glow');"),
          modalButton("START")
        ),
        easyClose = TRUE,
        size = "l"
      ))
    },
    once = TRUE
  )

  # Download RIS handler for the homepage
  output$download_ris_home <- downloadHandler(
    filename = function() {
      "projectLSA_citation.ris"
    },
    content = function(file) {
      ris_text <- c(
        "TY  - JOUR",
        "AU  - Djidu, Hasan",
        "AU  - Retnawati, Heri",
        "AU  - Hadi, Samsul",
        "AU  - Haryanto",
        "PY  - 2026",
        "DA  - 2026/04/20",
        "TI  - projectLSA: A Shiny Application for Integrated Latent Structure Analysis",
        "JO  - Applied Psychological Measurement",
        "DO  - 10.1177/01466216261446305",
        "UR  - https://doi.org/10.1177/01466216261446305",
        "ER  - "
      )
      writeLines(ris_text, file)
    }
  )

  # Handlers for donation buttons on homepage / welcome modal
  show_donate_modal <- function() {
    showModal(modalDialog(
      title = "Support projectLSA",
      tags$div(
        style = "text-align: center;",
        tags$h4("Help Us Keep Growing!"),
        tags$p("If you find projectLSA helpful for your research, consider supporting its continuous development."),
        tags$div(
          style = "background-color: #f8f9fa; border: 2px dashed #17a2b8; padding: 20px; border-radius: 10px; margin-bottom: 20px;",
          tags$strong("Support via Bank BNI:", style = "color: #17a2b8; font-size: 16px;"), tags$br(),
          tags$span("0331355050", style = "font-size: 24px; font-weight: bold; color: #d32f2f; letter-spacing: 2px;"), tags$br(),
          tags$em("Account Name: Hasan Djidu"), tags$br(), tags$br(),
          tags$strong("International Transfer (SWIFT):", style = "color: #17a2b8; font-size: 14px;"), tags$br(),
          tags$span("BNINIDJA", style = "font-size: 16px; font-weight: bold; color: #d32f2f;")
        )
      ),
      footer = modalButton("Close"),
      easyClose = TRUE
    ))
  }

  observeEvent(input$btn_donate_home, {
    show_donate_modal()
  })
  observeEvent(input$btn_donate_home_modal, {
    show_donate_modal()
  })

  # === reactive value to save current project ===
  project <- reactiveVal("home")

  # === observe event from homepage ===
  observeEvent(input$go_lca, {
    project("lca")
  })
  observeEvent(input$go_lpa, {
    project("lpa")
  })
  observeEvent(input$go_lta, {
    project("lta")
  })
  observeEvent(input$go_efa, {
    project("efa")
  })
  observeEvent(input$go_cfa, {
    project("cfa")
  })
  observeEvent(input$go_home, {
    project("home")
  })

  # === Render main UI ===
  output$mainUI <- renderUI({
    req(project())
    fluidPage(
      switch(project(),
        "lca" = lca_mod("lca"),
        "lpa" = lpa_mod("lpa"),
        "lta" = lta_mod("lta"),
        "efa" = efa_mod("efa"),
        "cfa" = cfa_mod("cfa"),
        home_mod("home")
      )
    )
  })
  # === Logic server: jalankan modul sesuai project aktif ===
  observeEvent(project(), {
    current <- project()
    # Reset the shared AI/console context when switching modules so the
    # assistant never reasons over results from a previous analysis.
    if (current != "home") {
      ai_context$results_text <- ""
      ai_context$module <- ""
      console_context$text <- "No R output yet."
    }
    switch(current,
      "lpa" = server_lpa(input, output, session, ai_context, console_context),
      "efa" = server_efa(input, output, session, ai_context, console_context),
      "cfa" = server_cfa(input, output, session, ai_context, console_context),
      "lca" = server_lca(input, output, session, ai_context, console_context),
      "lta" = server_lta(input, output, session, ai_context, console_context),
      NULL
    )
  })
  # LPA
  output$package_references_lpa <- renderUI({
    render_package_refs(c("shiny", "tidyverse", "tidyLPA", "DT", "readxl", "dplyr", "ggplot2"))
  })
  # LCA
  output$package_references_lca <- renderUI({
    render_package_refs(c("shiny", "tidyverse", "poLCA", "glca", "DT", "readxl", "dplyr", "ggplot2", "ggiraph"))
  })
  # LTA / IRT
  output$package_references_lta <- renderUI({
    lta_reference <- list(
      Desjardins_Bulut_2018 = "Desjardins, C. D., & Bulut, O. (2018). <em>Handbook of educational measurement and psychometrics using R</em> (1st ed.). Chapman & Hall/CRC."
    )
    render_package_refs(c("shiny", "tidyverse", "mirt", "DT", "readxl", "dplyr", "ggplot2"),
      manual_refs = lta_reference
    )
  })
  # EFA
  output$package_references_efa <- renderUI({
    render_package_refs(c("shiny", "tidyverse", "psych", "DT", "readxl", "dplyr", "ggplot2"))
  })
  # CFA References
  output$package_references_cfa <- renderUI({
    # Define manual references
    manual_refs <- list(
      Alamer_2025 = "Alamer, A. (2025). Structural equation modeling (SEM) in L2 writing research: Simple tutorial and useful recommendations. <em>Research Methods in Applied Linguistics</em>, 4(2), 100202. <a href='https://doi.org/10.1016/j.rmal.2025.100202' target='_blank'>https://doi.org/10.1016/j.rmal.2025.100202</a>",
      Hair_2019 = "Hair, J. F., Black, W. C., Babin, B. J., & Anderson, R. E. (2019). <em>Multivariate data analysis</em> (Eighth edition). Cengage.",
      Hu_1999 = "Hu, L., & Bentler, P. M. (1999). Cutoff criteria for fit indexes in covariance structure analysis: Conventional criteria versus new alternatives. <em>Structural Equation Modeling: A Multidisciplinary Journal</em>, 6(1), 1–55. <a href='https://doi.org/10.1080/10705519909540118' target='_blank'>https://doi.org/10.1080/10705519909540118</a>",
      Schumacker_2008 = "Schumacker, R. E., & Lomax, R. G. (2008). <em>A beginner's guide to structural equation modeling</em> (2nd ed.). Psychology Press.",
      Kline_2016 = "Kline, R. B. (2016). <em>Principles and practice of structural equation modeling</em> (4th ed.). Guilford Press.",
      Brown_2015 = "Brown, T. A. (2015). <em>Confirmatory factor analysis for applied research</em> (2nd ed.). Guilford Press."
    )

    render_package_refs(
      pkgs = c("shiny", "tidyverse", "psych", "lavaan", "DT", "readxl", "dplyr", "semPlot", "semptools", "semTools"),
      manual_refs = manual_refs
    )
  })
}

shinyApp(ui, server)

#' projectLSA: package-level imports
#'
#' This file records package-level imports for roxygen2 so that NAMESPACE
#' will include the appropriate import directives.
#'
#' @import shiny
#' @importFrom colourpicker colourInput
#' @importFrom data.table data.table
#' @importFrom dplyr select
#' @importFrom DT datatable
#' @importFrom flextable flextable
#' @importFrom ggiraph girafe
#' @importFrom ggplot2 ggplot
#' @importFrom glca glca
#' @importFrom haven read_sav
#' @importFrom httr POST
#' @importFrom jsonlite fromJSON
#' @importFrom kableExtra kbl
#' @importFrom knitr knit
#' @importFrom lavaan sem
#' @importFrom magick image_read
#' @importFrom mclust Mclust
#' @importFrom mirt mirt
#' @importFrom officer read_docx
#' @importFrom plotly plot_ly
#' @importFrom poLCA poLCA
#' @importFrom psych fa
#' @importFrom purrr map
#' @importFrom readr read_csv
#' @importFrom readxl read_excel
#' @importFrom rlang .data
#' @importFrom rmarkdown render
#' @importFrom scales percent
#' @importFrom semPlot semPaths
#' @importFrom semptools mark_sig
#' @importFrom shinyBS bsTooltip
#' @importFrom shinycssloaders withSpinner
#' @importFrom shinyWidgets pickerInput
#' @importFrom stats sd
#' @importFrom stringr str_detect
#' @importFrom tibble tibble
#' @importFrom tidyLPA estimate_profiles
#' @importFrom tidyr pivot_longer
#' @importFrom viridisLite viridis
#' @importFrom writexl write_xlsx
#' @keywords internal
"_PACKAGE"

server_efa <- function(input, output, session) {
  library(psych)
  library(lavaan)
  
  # ==== LOAD DATA ====
  data_user <- reactive({
    if (input$data_source == "bfi") return(psych::bfi %>% dplyr::select(A1:O5)%>% rownames_to_column("id_auto"))
    if (input$data_source == "HolzingerSwineford1939") return(lavaan::HolzingerSwineford1939%>% dplyr::select(x1:x9)%>% rownames_to_column("id_auto"))
    if (input$data_source == "upload") {
      req(input$datafile)
      ext <- tolower(tools::file_ext(input$datafile$name))
      showModal(modalDialog(title = NULL, "Reading Your File, Please wait...", footer = NULL, easyClose = FALSE))
      df <- switch(
        ext,
        "csv"  = data.table::fread(input$datafile$datapath,data.table = FALSE),
        "xls"  = readxl::read_excel(input$datafile$datapath),
        "xlsx" = readxl::read_excel(input$datafile$datapath),
        "sav"  = haven::read_sav(input$datafile$datapath),
        "rds"  = readRDS(input$datafile$datapath),
        stop("Unsupported file type. Please upload CSV, Excel, SPSS (.sav), or RDS file.")
      )
      removeModal()
      df <- df %>% mutate(across(everything(), ~ifelse(.x == "", NA, .x)),
                          id_auto = paste0("id_", sprintf("%04d", 1:n())))
    }
    return(df)
  })
  

  # ==== Pilih ID ====
  output$id_select_ui_fa <- renderUI({
    req(data_user())
    selectizeInput(
      "id_lca",
      label = "Select ID Columns (Optional):",
      choices = names(data_user()),
      selected = names(data_user())[str_detect(names(data_user()), "id")],
      multiple = TRUE,
      options = list(placeholder = 'Choose one or more ID columns')
    )
  })
  
  # ==== Pilih Variabel (selain ID) ====
  output$var_select_ui <- renderUI({
    req(data_user())
    all_vars <- names(data_user())
    id_cols <- input$id_lca
    
    # Hilangkan kolom ID dari pilihan variabel
    available_vars <- setdiff(all_vars, id_cols)
    selectInput(
      "selected_vars",
      label = "Select Variables:",
      choices = available_vars,
      selected = available_vars,  # default pilih semua yang tersisa
      multiple = TRUE
    )
  })

  output$data_preview_fa <- renderDT({
    req(data_user(),input$selected_vars)
    df <- data_user()[,input$selected_vars]
    df <- df %>%
      dplyr::mutate(across(where(is.numeric), ~ round(.x, 3)))
    
    DT::datatable(
      head(df, 50),extensions = 'Buttons',
      options = list(dom='Brtp',scrollX = TRUE, pageLength = 25,  
                     buttons = list(
                       list(extend = 'csv',
                            text = 'Export CSV',
                            filename = 'Data EFA'
                       ),
                       list(extend = 'excel',
                            text = 'Export Excel',
                            filename = 'Data EFA'
                       ))),
      rownames = FALSE
    )
  }, server = FALSE)
  
  
  # === Ketika user memilih CFA ===
  observeEvent(input$fa_type, {
    if (input$fa_type == "cfa") {
      message("Running CFA server...")
      cfa_server(input, output, session, data_user = data_user)
    }
  })
  
  
  
  
  # ==== Dynamic input nama faktor ====
  output$factor_names_ui <- renderUI({
    req(input$n_factors)
    n <- input$n_factors
    div(
      style = "
      display: flex;
      justify-content: center;
      flex-wrap: wrap;
      gap: 8px;
      margin-top: 5px;
    ",
      
      lapply(1:n, function(i) {
        textInput(
          inputId = paste0("factor_name_", i),
          label = NULL,
          value = paste0("Factor ", i),
          width = "100px"
        )
      }),
      br(),
    )
  })
  
  # ==== EFA ====
  efa_result <- eventReactive(c(input$run_efa, input$n_factors), {
    req(data_user(), input$selected_vars)
    
    withProgress(message = "Running Exploratory Factor Analysis (EFA)...", value = 0, {
      incProgress(0.1, detail = "Preparing data...")
      
      df <- as.data.frame(data_user())[ , input$selected_vars, drop = FALSE]
      
      # ensure numeric columns
      nonnum <- which(!sapply(df, is.numeric))
      if (length(nonnum) > 0) {
        df[nonnum] <- lapply(df[nonnum], function(x) as.numeric(as.character(x)))
      }
      
      # handle missing
      if (input$missing_method_efa == "mean") {
        for (j in seq_along(df)) df[is.na(df[, j]), j] <- mean(df[, j], na.rm = TRUE)
        missing_arg <- "no"
      } else if (input$missing_method_efa == "pairwise") {
        missing_arg <- "pairwise"
      } else {
        df <- na.omit(df)
        missing_arg <- "listwise"
      }
      
      incProgress(0.3, detail = "Computing KMO and Bartlett tests...")
      # compute KMO and Bartlett
      kmo_res <- tryCatch(psych::KMO(df), error = function(e) NULL)
      bartlett_res <- tryCatch(
        psych::cortest.bartlett(cor(df, use = "pairwise.complete.obs"), n = nrow(df)),
        error = function(e) NULL
      )
      
      incProgress(0.6, detail = "Performing Parallel Analysis...")
      par <- tryCatch(
        psych::fa.parallel(df, n.iter = 20, main = "Parallel Analysis (fa.parallel)"),
        error = function(e) NULL
      )
      
      incProgress(0.9, detail = paste("Extracting", input$n_factors, "factors..."))
      fa_res <- tryCatch(
        psych::fa(df, nfactors = input$n_factors, rotate = input$rotation_method),
        error = function(e) NULL
      )
      
      incProgress(1, detail = "Finalizing results...")
      
      efa_out <- list(
        kmo = kmo_res,
        bartlett = bartlett_res,
        parallel = par,
        fa = fa_res,
        loadings = if (!is.null(fa_res)) fa_res$loadings else NULL,
        df_used = df,
        scores = as.data.frame(psych::factor.scores(df, fa_res, Phi = NULL,rho=NULL,missing=FALSE,impute="mean")$scores)
      )
      
      efa_out
    })
  })

  
  # ==== Pindah ke tab hasil EFA setelah dijalankan ====
  observeEvent(input$run_efa, {
    updateTabsetPanel(session, "main_tab_fa", selected = "KMO & Bartlett Tests")
  })
  
  
  # ==== Tampil hasil tes EFA dengan gaya HTML ====
  output$efa_tests <- renderUI({
    req(efa_result())
    out <- efa_result()
    n_fact <- if (!is.null(out$parallel$nfact)) out$parallel$nfact else NA
    n_comp <- if (!is.null(out$parallel$ncomp)) out$parallel$ncomp else NA
    bart <- out$bartlett
    kmo <- out$kmo
    
    # ==== Format hasil KMO ====
    if (!is.null(kmo)) {
      MSA_overall <- round(kmo$MSA, 3)
      interpret <- if (MSA_overall >= 0.90) {
        "Excellent sampling adequacy — the data are highly suitable for factor analysis."
      } else if (MSA_overall >= 0.80) {
        "Meritorious sampling adequacy — the data are suitable for factor analysis."
      } else if (MSA_overall >= 0.70) {
        "Middling sampling adequacy — acceptable for factor analysis."
      } else if (MSA_overall >= 0.60) {
        "Mediocre sampling adequacy — factor analysis may still be appropriate."
      } else if (MSA_overall >= 0.50) {
        "Miserable sampling adequacy — consider improving data quality or sample size."
      } else {
        "Unacceptable sampling adequacy — factor analysis is not recommended."
      }
      
      kmo_html <- paste0(
        "<ul style='margin-top:4px;'>",
        "<li><b>Overall MSA:</b> ", MSA_overall, "</li>",
        "</ul>",
        "<p style='margin-left:10px; color:#2c3e50;'>", interpret, "</p>"
      )
    } else {
      kmo_html <- "<p style='color:#999;'>KMO test could not be computed (check data).</p>"
    }
    
    # ==== Format hasil Bartlett ====
    if (!is.null(bart)) {
      pval <- bart$p.value
      signif_text <- if (pval < 0.05) {
        "<span style='color:#2ecc71;font-weight:500;'>Significant (p < 0.05)</span> — the correlation matrix is <b>not an identity matrix</b>, indicating that factor analysis is appropriate."
      } else {
        "<span style='color:#e74c3c;font-weight:500;'>Not significant (p ≥ 0.05)</span> — the correlation matrix is <b>close to an identity matrix</b>, suggesting that factor analysis may <b>not be suitable</b> for this data."
      }
      
      bartlett_html <- paste0(
        "<ul style='margin-top:4px;'>",
        "<li><b>Chi-Square:</b> ", round(bart$chisq, 2), "</li>",
        "<li><b>df:</b> ", bart$df, "</li>",
        "<li><b>p-value:</b> ", format.pval(pval, digits = 3), "</li>",
        "</ul>",
        "<p style='margin-left:10px;'>", signif_text, "</p>"
      )
    } else {
      bartlett_html <- "<p style='color:#999;'>Bartlett test could not be computed (check data).</p>"
    }
    
    # ==== Gabungkan semua hasil ====
    HTML(paste0(
      "<div style='font-family:Segoe UI, sans-serif; line-height:1.6; background:#f9f9f9;
               border-radius:12px; padding:12px 18px; border:1px solid #ddd;'>",
      
      # --- Parallel Analysis ---
      "<h4 style='font-weight:bold; color:#2c3e50; margin-bottom:6px;'>📊 Parallel Analysis Recommendation</h4>",
      "<p style='margin-left:10px;'>
       <a>Suggested Number of Factors =</a> ", n_fact, "<br>
       <a>Suggested Number of Components =</a> ", n_comp, "
     </p>",
      
      "<hr style='border: none; border-top: 1px solid #ccc;'>",
      
      # --- KMO and MSA ---
      "<h4 style='font-weight:bold; color:#2c3e50; margin-bottom:6px;'>🧮 Kaiser-Meyer-Olkin (KMO) Measure</h4>",
      kmo_html,
      
      "<hr style='border: none; border-top: 1px solid #ccc;'>",
      
      # --- Bartlett's Test ---
      "<h4 style='font-weight:bold; color:#2c3e50; margin-bottom:6px;'>🧪 Bartlett's Test of Sphericity</h4>",
      bartlett_html,
      
      "</div>"
    ))
  })
  
  
  # ==== Plot KMO per Item (Lollipop Chart) ====
  output$kmo_item <- renderPlot({
    req(efa_result())
    out <- efa_result()
    kmo_res <- out$kmo
    req(!is.null(kmo_res))
    # Ambil nilai MSA per item (measure of sampling adequacy)
    kmo_values <- kmo_res$MSAi
    MSA_all <- data.frame(Item = 'Overall', KMO= kmo_res$MSA)
    kmo_data <- data.frame(
      Item = names(kmo_values),
      KMO = as.numeric(kmo_values)
    ) %>% rbind(MSA_all)
    
    # Tambahkan kategori kualitas KMO
    kmo_data <- kmo_data %>%
      dplyr::mutate(Kategori = dplyr::case_when(
        KMO < 0.5 ~ "Unacceptable",
        KMO < 0.6 ~ "Miserable",
        KMO < 0.7 ~ "Mediocre",
        KMO < 0.8 ~ "Meritorious",
        TRUE ~ "Marvelous"
      ))
    
    # Skema warna kategori
    kategori_colors <- c(
      "Unacceptable" = "#d73027",
      "Miserable"    = "#fc8d59",
      "Mediocre"     = "#fee08b",
      "Meritorious"  = "#4575b4",
      "Marvelous"    = "blue"
    )
    kmo_data$Item <- factor(kmo_data$Item, levels = kmo_data$Item)
    
    # Plot lollipop
    ggplot(kmo_data, aes(x = KMO, y = Item)) +
      geom_segment(aes(x = 0, xend = KMO, yend = Item, color = Kategori), linewidth = 1.5) +
      geom_point(aes(color = Kategori), size = 4) +
      # === Tambahkan label nilai di ujung ===
      geom_text(aes(label = round(KMO, 2)), 
                hjust = -0.3,  # geser sedikit ke kanan
                size = 3, 
                color = "black") +
      geom_vline(xintercept = 0.5, linetype = "dashed", color = "red", linewidth = 1) +
      scale_color_manual(values = kategori_colors) +
      labs(
        title = "Kaiser-Meyer-Olkin (KMO) per Item",
        subtitle = "Measure of Sampling Adequacy (MSA)",
        x = "MSA Value",
        y = "Item",
        color = "Category"
      ) +
      xlim(0, 1) +
      theme_minimal(base_size = 13) +
      theme(
        plot.title = element_text(face = "bold", size = 15),
        plot.subtitle = element_text(size = 11),
        axis.text.y = element_text(face = "bold"),
        legend.position = "bottom"
      )
  })
  
  # Scree plot behaviour:
  output$scree_plot <- renderPlot({
    req(efa_result())
    out <- efa_result()
    df <- out$df_used
    invisible(capture.output(
      psych::fa.parallel(df, fm = "ml", n.iter = 20, main = "Parallel Analysis (fa.parallel)")
    ))    
  })

  # ==== Output Ringkasan Hasil FA (Centered) ====
  output$efa_summary <- renderUI({
    req(efa_result())
    out <- efa_result()
    
    if (is.null(out$fa)) {
      return(tags$p("No factor analysis result available.", style = "color: #999; font-style: italic; text-align:center;"))
    }
    
    fa <- out$fa
    n_factors <- fa$factors
    method <- fa$method
    rotation <- fa$rotation
    total_var <- sum(fa$Vaccounted["Proportion Var", ]) * 100
    
    # --- Ambil nama faktor dari input teks ---
    factor_names <- sapply(1:n_factors, function(i) {
      input[[paste0("factor_name_", i)]] %||% paste0("Factor ", i)
    })
    
    # --- Matriks loading ---
    L <- as.matrix(fa$loadings)
    if (is.null(colnames(L))) colnames(L) <- paste0("Factor", seq_len(ncol(L)))
    if (length(factor_names) == ncol(L)) colnames(L) <- factor_names
    rown <- rownames(L)
    
    # Tentukan loading terbesar per item
    absmat <- abs(L)
    max.idx <- apply(absmat, 1, function(x) if (all(is.na(x))) NA_integer_ else which.max(x))
    
    # --- Buat tabel loading HTML ---
    hdr <- paste0(
      "<tr><th style='text-align:left;padding:6px;border:1px solid #ccc;background:#f8f8f8;'>Item</th>",
      paste0("<th style='padding:6px;border:1px solid #ccc;background:#f8f8f8;'>", colnames(L), "</th>", collapse = ""),
      "</tr>"
    )
    
    rows_html <- vapply(seq_len(nrow(L)), FUN.VALUE = character(1), function(i) {
      cells <- vapply(seq_len(ncol(L)), FUN.VALUE = character(1), function(j) {
        val <- L[i, j]
        if (is.na(val)) txt <- "" else txt <- format(round(val, 3), nsmall = 3)
        if (!is.na(max.idx[i]) && j == max.idx[i] && abs(L[i, j]) > 0) {
          paste0("<td style='background:#b2f0b2;padding:6px;border:1px solid #ccc;text-align:center;font-weight:600;'>", txt, "</td>")
        } else {
          paste0("<td style='padding:6px;border:1px solid #ccc;text-align:center;'>", txt, "</td>")
        }
      }, USE.NAMES = FALSE)
      paste0("<tr><td style='text-align:left;padding:6px;border:1px solid #ccc;'>", rown[i], "</td>", paste(cells, collapse = ""), "</tr>")
    })
    
    tbl_html <- paste0("<table style='margin:auto;border-collapse:collapse;width:auto;font-size:13px;font-family:Arial,Helvetica,sans-serif;'>",
                       hdr, paste(rows_html, collapse = ""), "</table>")
    
    # --- Variance explained ---
    variance_df <- as.data.frame(round(fa$Vaccounted * 100, 2))
    variance_df <- tibble::rownames_to_column(variance_df, var = "Metric")
    if (length(factor_names) == ncol(variance_df) - 1)
      colnames(variance_df)[-1] <- factor_names
    
    # Gunakan table HTML agar tetap center
    variance_tbl <- paste0(
      "<table style='margin:auto;border-collapse:collapse;width:auto;font-size:13px;font-family:Arial,Helvetica,sans-serif;'>",
      paste0(
        "<tr><th style='padding:6px;border:1px solid #ccc;background:#f0f0f0;'>", 
        paste(colnames(variance_df), collapse = "</th><th style='padding:6px;border:1px solid #ccc;background:#f0f0f0;'>"), 
        "</th></tr>"
      ),
      paste(
        apply(variance_df, 1, function(row) {
          paste0("<tr>", paste0("<td style='text-align:center;padding:6px;border:1px solid #ccc;'>", row, "</td>", collapse = ""), "</tr>")
        }),
        collapse = ""
      ),
      "</table>"
    )
    
    tagList(
      div(
        style = "text-align:center;",
        tags$h4("Exploratory Factor Analysis Summary", style = "font-weight:bold;color:#2c3e50;"),
        tags$p(HTML(paste0(
          "<b>Extraction method:</b> ", method, "<br>",
          "<b>Rotation:</b> ", rotation, "<br>",
          "<b>Number of factors extracted:</b> ", n_factors, "<br>",
          "<b>Total variance explained:</b> ", sprintf("%.2f%%", total_var)
        ))),
        tags$h5("Factor Loadings", style = "margin-top:15px;font-weight:bold;"),
        HTML(tbl_html),
        tags$h5("Variance Explained by Each Factor", style = "margin-top:20px;font-weight:bold;"),
        HTML(variance_tbl)
      )
    )
  })
  
  # =====Factor Scores EFA =====
  output$efa_scores_table <- DT::renderDataTable({
    req(efa_result())
    out <- efa_result()
    
    if (is.null(out$fa)) {
      return(tags$p("No factor analysis result available.", style = "color: #999; font-style: italic; text-align:center;"))
    }
    n_factors <- out$fa$factors
    
    # --- Ambil nama faktor dari input teks ---
    factor_names <- sapply(1:n_factors, function(i) {
      input[[paste0("factor_name_", i)]] %||% paste0("Factor ", i)
    })
    
    scores <- efa_result()$scores
    if (length(factor_names) == ncol(scores))
      colnames(scores) <- factor_names
    datatable(
      round(as.data.frame(scores), 3),extensions = 'Buttons',
      options = list(dom='Brtp',scrollX = TRUE, pageLength = 25,  
                     buttons = list(
                       list(extend = 'csv',
                            text = 'Export CSV',
                            filename = 'Factor Scores EFA'
                       ),
                       list(extend = 'excel',
                            text = 'Export Excel',
                            filename = 'Factor Scores EFA'
                       ))),
      caption = "Estimated Factor Scores for Each Observation"
    )
  })
  # ==== 3. DOWNLOAD BUTTONS ====
  output$download_loading <- downloadHandler(
    filename = function() {
      paste0("efa_loadings_", Sys.Date(), ".csv")
    },
    content = function(file) {
      res <- efa_result()
      load_tab <- as.data.frame(round(res$fa$loadings[, ], 3))
      load_tab$Item <- rownames(res$fa$loadings)
      load_tab <- load_tab[, c(ncol(load_tab), 1:(ncol(load_tab)-1))]
      write.csv(load_tab, file, row.names = FALSE)
    }
  )
  
  output$download_scores <- downloadHandler(
    filename = function() {
      paste0("efa_factor_scores_", Sys.Date(), ".csv")
    },
    content = function(file) {
      scores <- efa_result()$scores
      write.csv(round(as.data.frame(scores), 3), file, row.names = FALSE)
    }
  )
}

#' Launch the projectLSA Shiny Application
#'
#' This function starts the Shiny app included in the projectLSA package.
#'
#' @return Launches a Shiny application (no return value)
#' @examples
#' if (interactive()) {
#'   projectLSA::run_projectLSA()
#' }
#'
#' @export
run_projectLSA <- function() {
  app_dir <- system.file("app", package = "projectLSA")
  if (app_dir == "") {
    stop("Could not find Shiny app. Try reinstalling the projectLSA package.", call. = FALSE)
  }
  shiny::runApp(app_dir, launch.browser = TRUE)
}

# ==== server_lpa.R ====
server_lpa <- function(input, output, session) {
  library(tidyLPA)
  library(mclust)
  library(ggplot2)
  library(stats)
  library(haven)
  library(data.table)
  set.seed(100)

  # --- Upload data ----
  data_user <- reactive({
    if (input$data_source == "pisaUSA15") {
      df <- tidyLPA::pisaUSA15[1:500,]%>% mutate(id_auto = paste0("id_", sprintf("%04d", 1:n())))
    } else if (input$data_source == "curry_mac") {
      df <- tidyLPA::curry_mac %>% mutate(id_auto = paste0("id_", sprintf("%04d", 1:n())))
      
    } else if (input$data_source == "id_edu") {
      df <- tidyLPA::id_edu %>% mutate(id_auto = paste0("id_", sprintf("%04d", 1:n())))
    } else {
      req(input$datafile)
      ext <- tolower(tools::file_ext(input$datafile$name))
      showModal(modalDialog(title = NULL, "Reading Your File, Please wait...", footer = NULL, easyClose = FALSE))
      df <- switch(
        ext,
        "csv"  = data.table::fread(input$datafile$datapath,data.table = FALSE),
        "xls"  = readxl::read_excel(input$datafile$datapath),
        "xlsx" = readxl::read_excel(input$datafile$datapath),
        "sav"  = haven::read_sav(input$datafile$datapath),
        "rds"  = readRDS(input$datafile$datapath),
        stop("Unsupported file type. Please upload CSV, Excel, SPSS (.sav), or RDS file.")
      )
      removeModal()
      df <- df %>% mutate(across(everything(), ~ifelse(.x == "", NA, .x)),
                          id_auto = paste0("id_", sprintf("%04d", 1:n())))
    }
    return(df)
  })
  observeEvent(input$data_source, {
    if (input$data_source != "upload") {
      showNotification(paste("Using built-in dataset:", input$data_source), type = "message")
    }
  })
  output$data_description <- renderUI({
    desc_html <- if (input$data_source == "pisaUSA15") {
      desc_pisa
    } else if (input$data_source == "curry_mac") {
      desc_curry_text
    } else if (input$data_source == "id_edu") {
      desc_id_edu
    } else { ""
    }
    
    div(
      class = "badge-info",
      HTML(desc_html)
    )
  })
  # ---  ID ---
  output$id_select_ui <- renderUI({
    req(data_user())
    selectInput(
      "id_vars",
      label = "Select ID Columns (Optional):",
      choices = names(data_user()),
      selected = names(data_user())[str_detect(names(data_user()), "id")],
      multiple = TRUE
      )
  })
  
  # --- Pilih variabel ---
  output$var_select_ui <- renderUI({
    req(data_user())
    selectInput(
      "selected_vars",
      label = "Select Variables for LPA:",
      choices = names(data_user()),
      selected = names(data_user())[1:min(3, ncol(data_user()))],
      multiple = TRUE 
      )
  })
  
  observeEvent(c(input$data_source, input$datafile), {
    updateSelectInput(session,"selected_vars",selected = "") 
  }, ignoreInit = TRUE)
  
  # --- Data preview ----
  output$data_preview <- renderDT({
    req(data_user(), input$selected_vars)
    df <- data_user() %>% 
      dplyr::select(input$selected_vars)
    numeric_cols <- which(sapply(df, function(x) is.numeric(x)))
    datatable(df,
              extensions = 'Buttons',
              options = list(dom='Brtp',scrollX = TRUE, #pageLength = nrow(df),
                             buttons = list(
                               list(extend = 'csv',
                                    text = 'Export CSV',
                                    filename = 'Data LPA'
                               ),
                               list(extend = 'excel',
                                    text = 'Export Excel',
                                    filename = 'Data LPA'
                               ))),
      rownames = T) %>% 
      formatRound(columns = numeric_cols, digits = 2)
  }, server = TRUE)
  
  output$data_summary_lpa <- DT::renderDT({
    req(data_user(), input$selected_vars)
    
    df <- data_user() %>% 
      dplyr::select(all_of(input$selected_vars))
    
    table_summary <- data.frame(
      N        = sapply(df, function(x) sum(!is.na(x))),
      Missing  = sapply(df, function(x) sum(is.na(x))),
      Mean     = sapply(df, function(x) mean(x, na.rm = TRUE)),
      SD       = sapply(df, function(x) sd(x, na.rm = TRUE)),
      Variance = sapply(df, function(x) var(x, na.rm = TRUE)),
      Min      = sapply(df, function(x) min(x, na.rm = TRUE)),
      Max      = sapply(df, function(x) max(x, na.rm = TRUE))
    )
    
    # Transpose supaya Stat = baris, Variabel = kolom
    table_summary <- t(table_summary)
    # Round Mean, SD, Variance ke 2 desimal
    table_summary[c("Mean", "SD", "Variance","Min", "Max"), ] <- round(table_summary[c("Mean", "SD", "Variance", "Min", "Max"), ], 2)
    
    DT::datatable(
      table_summary,
      extensions = "Buttons",
      options = list(
        scrollX = TRUE,
        dom = "Bt",
        buttons = list(
          list(
            extend = "csv",
            text = "Export CSV",
            filename = "Data_LPA_Summary"
          ),
          list(
            extend = "excel",
            text = "Export Excel",
            filename = "Data_LPA_Summary"
          )
        )
      ),
      rownames = TRUE
    )
  }, server = FALSE)
 
  
  # --- Jalankan LPA ----
  observeEvent(input$run_lpa, {
    req(data_user(), input$selected_vars)
    updateTabsetPanel(session, "main_tab_lpa", selected = "fit_tab")
  })
  fitcompare <- eventReactive(input$run_lpa, {
    req(data_user(), input$selected_vars, input$model_to_run)
    showModal(modalDialog(title = NULL, "Please wait, running LPA...", footer = NULL, easyClose = FALSE))
    df <- data_user() %>% dplyr::select(input$selected_vars)
    df <- df %>% mutate(across(everything(), as.numeric))
    min_k <- input$min_profiles
    max_k <- input$max_profiles
    
    model_spec <- tibble::tibble(
      model = c(1, 2, 3, 6),
      variances = c("equal", "varying", "equal", "varying"),
      covariances = c("zero", "zero", "equal", "varying")
    )

    model_to_run <- input$model_to_run
    
    spec_selected <- model_spec %>%
      dplyr::filter(model %in% model_to_run) %>%
      dplyr::arrange(match(model, model_to_run))
    
    variances   <- spec_selected$variances
    covariances <- spec_selected$covariances
    
    withProgress(message = "Running...", {
      incProgress(1/(max_k-min_k+1), detail = paste("Compare LPA results"))
      fitcompare <- df %>%
        tidyLPA::single_imputation() %>%   # ← ini kuncinya
        tidyLPA::estimate_profiles(
          n_profiles = min_k:max_k,
          variances  = variances,
          covariances = covariances
          #model = model_to_run
        ) %>%
        tidyLPA::get_fit()
      fitcompare <- as.data.frame(fitcompare) %>% dplyr::rename(n_profiles=Classes)
      removeModal()
      showNotification("LPA completed successfully!", type = "message")
    })
    fitcompare
  })
  
  models <- eventReactive(c(input$best_k, input$model_type), {
    req(data_user(), fitcompare(),input$selected_vars, input$best_k)
    df <- data_user()[, input$selected_vars]
    df <- df %>% mutate(across(everything(), as.numeric))
    min_k <- input$min_profiles
    max_k <- input$max_profiles
    model_type <- input$model_type
    variances <- ifelse(input$model_type %in% c(1, 3), "equal", "varying")
    covariances <- ifelse(input$model_type %in% c(1,2), "zero",
                          ifelse(input$model_type %in% c(3), "equal", "varying"))
    model_list <- list()
    for (k in input$best_k) {
      withProgress(message = "Running...", {
      incProgress(1/(max_k-min_k+1), detail = paste("Estimate LPA Model for ", k, " Profiles"))
      model_list[[as.character(k)]] <- df %>%
        tidyLPA::single_imputation() %>%   # ← ini kuncinya
        tidyLPA::estimate_profiles(
          n_profiles = k,
          variances = variances,
          covariances = covariances
        )
      showNotification("Estimation completed successfully!", type = "message")
      })
    }
    model_list
   
  })
  
  # --- Fit table ----
  output$fit_table <- renderDT({
    req(fitcompare())
    fitcompare <- fitcompare() %>%
      dplyr::mutate(across(where(is.numeric), ~round(.x, 2))) %>%
      dplyr::select(-c(AWE, CAIC, CLC, KIC, SABIC, ICL))
    prob_min_vals <- sort(unique(fitcompare$prob_min), decreasing = TRUE)
    prob_max_vals <- sort(unique(fitcompare$prob_max),decreasing = TRUE)
    aic_vals <- sort(unique(fitcompare$AIC))
    bic_vals <- sort(unique(fitcompare$BIC))
    ent_vals <- sort(unique(fitcompare$Entropy), decreasing = TRUE)
    
    datatable(fitcompare, extensions = 'Buttons',
              options = list(dom='B', scrollX = TRUE, pageLength = 40,
                             buttons = list(
                               list(extend='csv', text='Export CSV', filename='FIT Comparison LPA'),
                               list(extend='excel', text='Export Excel', filename='FIT Comparison LPA')
                             )),
              rownames = FALSE) %>%
      formatStyle('AIC',
                  backgroundColor = styleEqual(
                    c(aic_vals[1], aic_vals[2]),
                    c('lightgreen', 'khaki')
                  ),
                  fontWeight = styleEqual(aic_vals[1], 'bold')
      ) %>%
      formatStyle('BIC',
                  backgroundColor = styleEqual(
                    c(bic_vals[1], bic_vals[2]),
                    c('lightgreen', 'khaki')
                  ),
                  fontWeight = styleEqual(bic_vals[1], 'bold')
      ) %>%
      formatStyle('Entropy',
                  backgroundColor = styleInterval(c(0.7, 0.8), c('lightcoral', 'khaki', 'lightgreen')),
                  fontWeight      = styleInterval(c(0.7, 0.8), c('bold', 'bold',''))
      ) %>%
      formatStyle('prob_min',
                  backgroundColor = styleEqual(
                    c(prob_min_vals[1], prob_min_vals[2]),
                    c('lightgreen', 'khaki')
                  ),
                  fontWeight = styleEqual(ent_vals[1], 'bold')
      ) %>%
      formatStyle('prob_max',
                  backgroundColor = styleEqual(
                    c(prob_max_vals[1], prob_max_vals[2]),
                    c('lightgreen', 'khaki')
                  ),
                  fontWeight = styleEqual(ent_vals[1], 'bold')
      ) %>%
      formatStyle('n_min',
                  backgroundColor = styleInterval(
                    c(0.05, 0.07),
                    c('lightcoral', 'khaki', 'lightgreen'))
                  ) %>%
      formatStyle('BLRT_p',
                  backgroundColor = styleInterval(0.05, c('lightgreen', 'lightcoral')),
                  fontWeight = styleInterval(0.05, c('bold', ''))
      )
  }, server = FALSE)
  fit_bic_reactive <- reactive({
    req(fitcompare())
    fit_bic <- fitcompare() %>%
      dplyr::select(Model, n_profiles, BIC) %>%
      dplyr::mutate(n_profiles = factor(n_profiles), Model = factor(Model))
    
    ggplot(fit_bic, aes(n_profiles, BIC, group = Model, shape = Model, color = Model)) +
      geom_line(size = 1.2, linetype = 4) + geom_point(size = 3) +
      geom_text(aes(label = round(BIC, 2)), vjust = -0.6, size = 4, show.legend = FALSE) +
      labs(subtitle  = "BIC Comparison Across Models", x = "Number of Profiles", y = "BIC") +
      theme_minimal(base_size = 14) +
      theme(legend.position = "bottom", axis.line = element_line(color = "black"))
  })
  fit_aic_reactive <- reactive({
    req(fitcompare())
    fit_aic <- fitcompare() %>%
      dplyr::select(Model, n_profiles, AIC) %>%
      dplyr::mutate(n_profiles = factor(n_profiles), Model = factor(Model))
    
    ggplot(fit_aic, aes(n_profiles, AIC, group = Model, shape = Model, color = Model)) +
      geom_line(size = 1.2, linetype = 5) + geom_point(size = 3) +
      geom_text(aes(label = round(AIC, 2)), vjust = -0.6, size = 4, show.legend = FALSE) +
      labs(subtitle = "AIC Comparison Across Models", x = "Number of Profiles", y = "AIC") +
      theme_minimal(base_size = 14) +
      theme(legend.position = "bottom", axis.line = element_line(color = "black"))
  })
  fit_entropy_reactive <- reactive({
    req(fitcompare())
    fit_entropy <- fitcompare() %>%
      dplyr::select(Model, n_profiles, Entropy) %>%
      dplyr::mutate(n_profiles = factor(n_profiles), Model = factor(Model))
    
    ggplot(fit_entropy, aes(n_profiles, Entropy, group = Model, shape = Model,color = Model)) +
      geom_line(size = 1.2, linetype = 2) + geom_point(size = 3) +
      geom_text(aes(label = round(Entropy, 2)), vjust = -0.6, size = 4, show.legend = FALSE) +
      labs(subtitle = "Entropy Comparison Across Models", x = "Number of Profiles", y = "Entropy") +
      theme_minimal(base_size = 14) +
      theme(legend.position = "bottom", axis.line = element_line(color = "black"))
  })
  fit_class_size_reactive <- reactive({
    req(fitcompare())
    fit_class_size <- fitcompare() %>%
      dplyr::select(Model, n_profiles, n_min) %>%
      dplyr::mutate(n_profiles = factor(n_profiles), Model = factor(Model))
    
    ggplot(fit_class_size, aes(n_profiles, n_min, group = Model, shape = Model, color = Model)) +
      geom_line(size = 1.2, linetype = 1) + geom_point(size = 4) +
      geom_text(aes(label = round(n_min, 2)), vjust = -0.6, size = 4, show.legend = FALSE) +
      labs(subtitle = "Min Class Size Comparison Across Models", x = "Number of Profiles", y = "Minimum Class Size") +
      theme_minimal(base_size = 14) +
      theme(legend.position = "bottom", axis.line = element_line(color = "black"))
  })
  
  # ==== Render Plot ====
  output$fit_bic <- renderPlot({
    fit_bic_reactive()
  })
  output$fit_aic <- renderPlot({
    fit_aic_reactive()
  })
  output$fit_entropy <- renderPlot({
    fit_entropy_reactive()
  })
  output$fit_class_size <- renderPlot({
    fit_class_size_reactive()
  })

  # ==== Download Buttons ====
  # output$download_plot_AicBic_LPA <- make_download_plot(
  #   plot_reactive = fit_plot_reactive,
  #   filename_prefix = "Fit AIC BIC Plot_LPA"
  # )
  # 
  # output$download_plot_entropy_LPA <- make_download_plot(
  #   plot_reactive = entropy_plot_reactive,
  #   filename_prefix = "EntropyPlot_LPA"
  # )
  # --- Input nama profil ---
  output$profile_name_inputs <- renderUI({
    req(input$best_k)
    lapply(1:input$best_k, function(i) {
      textInput(paste0("profile_name_", i), paste("Profile", i, "Name:"), value = paste("Profile", i))
    })
  })
  best_model_plot_reactive <- reactive({
    req(models(), input$best_k, input$selected_vars)
    
    k <- as.character(input$best_k)
    model_k <- models()[[k]]
    params  <- tidyLPA::get_estimates(model_k)
    
    # === Profile names (AMAN) ===
    profile_names <- sapply(1:input$best_k, function(i) {
      nm <- input[[paste0("profile_name_", i)]]
      if (is.null(nm) || nm == "") paste0("Profile ", i) else as.character(nm)
    })
    
    # === Ambil Means + SE ===
    plot_df <- params %>%
      dplyr::filter(
        Category == "Means",
        Parameter %in% input$selected_vars
      ) %>%
      dplyr::mutate(
        Profile = factor(
          Class,
          levels = seq_along(profile_names),
          labels = profile_names
        ),
        Variable = factor(Parameter, levels = input$selected_vars),
        Mean = Estimate,
        SE   = se
      )
    
    # === Plot ===
    ggplot(plot_df,
           aes(x = Variable, y = Mean,
               group = Profile,
               color = Profile,
               linetype = Profile,
               shape = Profile)) +
      geom_line(linewidth = 1.2) +
      geom_point(size = 3) +
      
      # === Error bar (Mean ± SE) ===
      geom_errorbar(
        aes(ymin = Mean - SE, ymax = Mean + SE),
        width = 0.15,
        linewidth = 0.8
      ) +
      
      geom_text(
        aes(label = round(Mean, 2)),
        vjust = -0.9,
        size = 4,
        show.legend = FALSE
      ) +
      
      labs(
        title = paste0("Profile Plot Based on Model Estimates (K = ", k, ")"),
        x = "Indicator",
        y = "Estimated Mean (± SE)"
      ) +
      
      theme_minimal(base_size = 14) +
      theme(
        legend.position = "bottom",
        axis.line = element_line(color = "black"),
        panel.grid.minor = element_blank()
      )
  })
  
  # RenderP lot
  output$best_model_plot <- renderPlot({
    best_model_plot_reactive()
  })
  
  # ==== Download Buttons ====
  output$download_plot_best_LPA <- make_download_plot(
    plot_reactive = best_model_plot_reactive,
    filename_prefix = "BestPlot_LPA"
  )
  #=======
  summary_data <- reactive({
    req(models(), input$best_k)
    
    model_k <- models()[[as.character(input$best_k)]]
    df_cls  <- get_data(model_k)
    params  <- tidyLPA::get_estimates(model_k)
  
    profile_names <- sapply(1:input$best_k, function(i) input[[paste0("profile_name_", i)]])
    df_cls$Profile <- factor(df_cls$Class, labels = profile_names)
    class_size <- df_cls %>%
          dplyr::count(Profile) %>%
          dplyr::mutate(Percent = round(100 * n / sum(n), 2))
    
    # === Mean & Variance (model-based) ===
    mean_var_tbl <- params %>%
      dplyr::filter(Category %in% c("Means", "Variances")) %>%
      dplyr::mutate(Profile = factor(Class, labels = profile_names)) %>%
      dplyr::select(Profile, Variable = Parameter, Category, Estimate) %>%
      tidyr::pivot_wider(
        names_from  = Category,
        values_from = Estimate
      ) %>%
      dplyr::mutate(
        MeanVar = sprintf("%.2f (%.2f)", Means, Variances)
      ) %>%
      dplyr::select(Profile, Variable, MeanVar)
    
    # === Bentuk tabel ringkas ===
    profile_summary <- mean_var_tbl %>%
      tidyr::pivot_wider(
        names_from  = Variable,
        values_from = MeanVar
      )
    
    # === Gabungkan dengan class size ===
    summary_table <- class_size %>%
      dplyr::left_join(profile_summary, by = "Profile")
    
    summary_table
  })
  # --- Render tabel ke UI ---
  output$summary_table <- renderDT({
    req(summary_data())
    df <- summary_data()
    DT::datatable(df,extensions = 'Buttons',
                  options = list(scrollX = TRUE, dom = 'B',
                                 buttons = list(
                                   list(
                                     extend = 'csv',
                                     text = 'Export CSV',
                                     filename = paste0('Summary LPA')
                                   ),
                                   list(
                                     extend = 'excel',
                                     text = 'Export Excel',
                                     filename = paste0('Summary LPA')
                                   ))),
                  rownames = FALSE)
  })

  
  df_out <- reactive({
    req(models(), input$best_k)
    df <- data_user()
    id_cols <- if (is.null(input$id_vars) || length(input$id_vars) == 0) "id_auto" else input$id_vars
    model_k <- models()[[as.character(input$best_k)]]
    cluster <- get_data(model_k)$Class
    profile_names <- sapply(1:input$best_k, function(i) input[[paste0("profile_name_", i)]])
    df$Profile <- factor(cluster, labels = profile_names)
    #df_out <- df[, c(id_cols, input$selected_vars, "Profile")]
    numeric_cols <- which(sapply(df_out, is.numeric))
    df
  })
  # --- Data profil per ID ---
  output$profile_table <- renderDT({
    req(models(), input$best_k)
    df <- df_out()
    id_cols <- if (is.null(input$id_vars) || length(input$id_vars) == 0) "id_auto" else input$id_vars
    df <- df[, c(id_cols, input$selected_vars, "Profile")]
    numeric_cols <- which(sapply(df, is.numeric))

    datatable(df,
              extensions = 'Buttons',
              options = list(dom='Brtp',scrollX = TRUE, pageLength = 10,
                             buttons = list(
                               list(extend = 'csv',
                                    text = 'Export CSV',
                                    filename = 'Data Profle LPA'
                               ),
                               list(extend = 'excel',
                                    text = 'Export Excel',
                                    filename = 'Data Profile LPA'
                               ))),
              rownames = T) %>%
      formatRound(columns = numeric_cols, digits = 2)
  }, server = FALSE)
  output$var_x_ui <- renderUI({
    req(df_out())
    selectInput(
      "var_x",
      label = "X Absis",
      choices = names(df_out()),
      selected = "Profile",
      multiple = FALSE 
    )
  })
  output$var_y_ui <- renderUI({
    req(df_out())
    selectInput(
      "var_y",
      label = "Y Absis",
      choices = setdiff(names(df_out()), "Profile"),
      selected = "",
      multiple = TRUE
    )
  })
  output$cross_latent <- renderPlot({
    req(models(), input$best_k, input$var_x, input$var_y, input$plot_type)
    df <- df_out()
    
    # =========================
    # 1. RINGKAS DATA
    # =========================
    Summary_long <- df %>%
      dplyr::select(all_of(c(input$var_x, input$var_y))) %>%
      tidyr::pivot_longer(
        cols = all_of(input$var_y),
        names_to = "Variable",
        values_to = "Value"
      ) %>%
      dplyr::group_by(
        Profile  = .data[[input$var_x]],
        Variable
      ) %>%
      dplyr::summarise(
        Mean = mean(Value, na.rm = TRUE),
        .groups = "drop"
      ) %>% 
      dplyr::mutate(
        Variable = factor(Variable, levels = input$var_y)
      )
    
    # =========================
    # 2. BASE PLOT
    # =========================
    p <- ggplot(
      Summary_long,
      aes(
        x = Variable,
        y = Mean,
        fill = Profile,
        color = Profile,
        group = Profile
      )
    )
    
    # =========================
    # 3. SWITCH BAR / LINE
    # =========================
    if (input$plot_type == "bar") {
      
      p <- p +
        geom_col(
          position = position_dodge(width = 0.7),
          width = 0.6
        )
      
    } else {
      
      p <- p +
        geom_line(
          aes(linetype = Profile),
          size = 1.2
        ) +
        geom_point(
          aes(shape = Profile),
          size = 3
        )
    }
    
    # =========================
    # 4. FINAL STYLE
    # =========================
    p +
      geom_text(
        aes(label = round(Mean, 2)),
        position = position_dodge(width = 0.7),
        vjust = -0.7,
        size = 4.5,
        show.legend = FALSE
      ) +
      labs(
        subtitle = paste0("Profile Plot for ", input$best_k, " Profiles"),
        x = "Variable",
        y = "Mean Value"
      ) +
      theme_minimal(base_size = 12) +
      theme(
        axis.text.x = element_text(angle = 90, hjust = 1, vjust = 0.5, size = 9),
        legend.position = "bottom",
        axis.line = element_line(color = "black")
      )
  })
  output$anova_lpa_ui <- renderUI({
    req(models(), input$best_k, input$var_x, input$var_y)
    
    if (isTRUE(input$check_anova)) {
      tagList(
        tags$h5(
          paste0(
            "Latent Profile Differences on ",
            paste(input$var_y, collapse = ", ")
          )
        ),
        tags$h6("Analysis of Variance"),
        tableOutput("anova_lpa_table"),
        uiOutput("posthoc_lpa_ui")
      )
    }
  })
  output$anova_lpa_table <- renderTable({
    req(df_out(), input$var_x, input$var_y)
    
    df <- df_out()
    x  <- input$var_x
    ys <- input$var_y
    
    # =========================
    # CASE 1: ONE-WAY ANOVA
    # =========================
    if (length(ys) == 1) {
      
      fml <- as.formula(paste(ys, "~", x))
      fit <- aov(fml, data = df)
      tbl <- summary(fit)[[1]]
      
      pval <- tbl["Pr(>F)"][1,1]
      
      out <- data.frame(
        Source = rownames(tbl),
        round(tbl, 4),
        row.names = NULL
      )
      
      out$Interpretation <- c(
        ifelse(
          pval < 0.05,
          "Significant difference",
          "No significant difference"
        ),
        rep("", nrow(out) - 1)
      )
      
      out
      
    } else {
      
      # =========================
      # CASE 2: MANOVA
      # =========================
      
      fml <- as.formula(
        paste(
          "cbind(",
          paste(ys, collapse = ","),
          ") ~ ",
          x
        )
      )
      
      fit <- manova(fml, data = df)
      tbl <- summary(fit, test = "Wilks")$stats
      pval <- tbl[1, "Pr(>F)"]
      
      out <- data.frame(
        Effect = rownames(tbl),
        round(tbl, 4),
        row.names = NULL
      )
      
      out$Interpretation <- ifelse(
        out$Effect == x,
        ifelse(
          pval < 0.05,
          "Significant difference",
          "No significant difference"
        ),
        ""
      )
      
      out
    }
  })
  output$posthoc_lpa_ui <- renderUI({
    req(df_out(), input$var_x, input$var_y)
    
    if (length(input$var_y) == 1) {
      tagList(
        tags$h6("Post-hoc Test (Tukey HSD)"),
        tableOutput("posthoc_lpa_table")
      )
    }
  })
  output$posthoc_lpa_table <- renderTable({
    req(df_out(), input$var_x, input$var_y)
    
    # Post-hoc hanya untuk ANOVA
    if (length(input$var_y) != 1) return(NULL)
    
    df <- df_out()
    x  <- input$var_x
    y  <- input$var_y
    
    fml <- as.formula(paste(y, "~", x))
    fit <- aov(fml, data = df)
    
    # p-value ANOVA utama
    p_main <- summary(fit)[[1]]["Pr(>F)"][1, 1]
    
    # Jika tidak signifikan → tampilkan pesan
    if (is.na(p_main) || p_main >= 0.05) {
      return(
        data.frame(
          Note = "Post-hoc test was not conducted because the ANOVA result was not statistically significant."
        )
      )
    }
    
    tuk <- TukeyHSD(fit)[[1]]
    tuk <- round(as.data.frame(tuk), 4)
    
    data.frame(
      Comparison = rownames(tuk),
      tuk,
      Interpretation = ifelse(
        tuk$`p adj` < 0.05,
        "Significant difference",
        "Not significant"
      ),
      row.names = NULL
    )
  })
  output$crosstab_ui <- renderUI({
    req(input$crosstab_result)
    if (!input$crosstab_result) return(NULL)
    
    req(df_out())
    vars <- setdiff(names(df_out()), "Profile")
    fluidRow(
      column(
        12,
        h5("Cross Tabulation Settings"),
        selectInput(
          "crosstab_vars",
          "Select Variables:",
          choices = vars,
          multiple = TRUE
        )
      ),
      column(
        6,
        h5("Cross Tabulation For Categorical Variables"),
        tableOutput("crosstab_cat"),
        plotOutput("crosstab_cat_plot")
      ),
      column(
        6,
        h5("Mean & SD for Continuous Variables"),
        tableOutput("crosstab_cont"),
        plotOutput("crosstab_cont_plot")
      ),
      br(), br(), br()
    )
  })
  output$crosstab_cat <- renderTable({
    req(input$crosstab_result, input$crosstab_vars)
    if (!input$crosstab_result) return(NULL)
    
    df <- df_out()
    
    vars_cat <- input$crosstab_vars[
      sapply(df[input$crosstab_vars], function(x)
        is.factor(x) || is.character(x))
    ]
    
    if (length(vars_cat) == 0) return(NULL)
    
    do.call(rbind, lapply(vars_cat, function(v) {
      tab <- as.data.frame(
        table(Profile = df$Profile, Category = df[[v]])
      )
      
      tab <- tab |>
        dplyr::group_by(Category) |>
        dplyr::mutate(
          Percent = round(Freq / sum(Freq) * 100, 2)
        ) |>
        dplyr::ungroup()
      
      data.frame(
        Profile  = tab$Profile,
        Variable = v,
        Category = tab$Category,
        N        = tab$Freq,
        Percent  = tab$Percent
      )
    }))
  })
  output$crosstab_cont <- renderTable({
    req(df_out(), input$crosstab_vars)
    
    df <- df_out()
    
    vars_num <- input$crosstab_vars[
      sapply(df[input$crosstab_vars], is.numeric)
    ]
    
    if (length(vars_num) == 0) return(NULL)
    
    res <- lapply(vars_num, function(v) {
      aggregate(
        df[[v]],
        by = list(Profile = df$Profile),
        FUN = function(x)
          paste0(round(mean(x, na.rm = TRUE), 2),
                 " (", round(sd(x, na.rm = TRUE), 2), ")")
      ) |>
        setNames(c("Profile", v))
    })
    
    Reduce(function(x, y) merge(x, y, by = "Profile"), res)
  })
  output$crosstab_cat_plot <- renderPlot({
    req(input$crosstab_result, input$crosstab_vars)
    if (!input$crosstab_result) return(NULL)
    
    df <- df_out()
    
    vars_cat <- input$crosstab_vars[
      sapply(df[input$crosstab_vars], function(x)
        is.factor(x) || is.character(x))
    ]
    
    if (length(vars_cat) == 0) return(NULL)
    
    plot_data <- do.call(rbind, lapply(vars_cat, function(v) {
      tab <- as.data.frame(table(Profile = df$Profile, Category = df[[v]]))
      
      tab <- tab |>
        dplyr::group_by(Category) |>
        dplyr::mutate(
          Percent = round(Freq / sum(Freq) * 100, 2)
        ) |>
        dplyr::ungroup()
      
      tab$Variable <- v
      tab
    }))
    
    ggplot(plot_data,
           aes(x = Category, y = Percent, fill = Profile)) +
      geom_col(
        width = 0.6,
        position = position_dodge(width = 0.7)
      ) +
      geom_text(
        aes(label = paste0(Percent, "%")),
        position = position_dodge(width = 0.7),
        vjust = -0.25,
        size = 3
      ) +
      facet_wrap(~ Variable, scales = "free_x") +
      labs(
        title = "Categorical Cross-Tabulation by Profile",
        y = "Percentage (%)",
        x = NULL
      ) +
      theme_minimal() +
      theme(
        axis.text.x = element_text(angle = 90, hjust = 1, vjust = 0.5, size = 9),
        legend.position = "bottom",
        axis.line = element_line(color = "black")
      )
  })
  output$crosstab_cont_plot <- renderPlot({
    req(input$crosstab_result, input$crosstab_vars)
    if (!input$crosstab_result) return(NULL)
    
    df <- df_out()
    
    vars_num <- input$crosstab_vars[
      sapply(df[input$crosstab_vars], is.numeric)
    ]
    
    if (length(vars_num) == 0) return(NULL)
    
    plot_data <- df |>
      dplyr::select(Profile, dplyr::all_of(vars_num)) |>
      tidyr::pivot_longer(
        cols = -Profile,
        names_to = "Variable",
        values_to = "Value"
      ) |>
      dplyr::group_by(Profile, Variable) |>
      dplyr::summarise(
        Mean = mean(Value, na.rm = TRUE),
        SD   = sd(Value, na.rm = TRUE),
        .groups = "drop"
      )
    
    ggplot(plot_data, aes(x = Profile, y = Mean, fill = Profile)) +
      geom_col(width = 0.6) +
      geom_errorbar(
        aes(ymin = Mean - SD, ymax = Mean + SD),
        width = 0.2
      ) +
      facet_wrap(~ Variable, scales = "free_y") +
      labs(
        title = "Continuous Variables by Profile",
        y = "Mean (± SD)",
        x = NULL
      ) +
      theme_minimal() +
      theme(
        axis.text.x = element_text(angle = 90, hjust = 1, vjust = 0.5, size = 9),
        legend.position = "none",
        axis.line = element_line(color = "black")
      )
  })
  
  
  
  
  
  
  
  
  
  
  
}

# serverAIWidget.R

server_ai_widget <- function(input, output, session, ai_context) {
  
  # Chat History State
  ai_chat_history <- reactiveVal(character(0))
  
  # Summary Result State
  ai_summary_res <- reactiveVal("")
  
  # Helper to format chat history as HTML
  output$ai_global_chat_history <- renderUI({
    hist <- ai_chat_history()
    if (length(hist) == 0) {
      return(HTML("<div style='color: #888; text-align: center; margin-top: 20px;'>No conversation yet.</div>"))
    }
    HTML(paste(hist, collapse = "<br><br>"))
  })
  
  # Render Summary
  output$ai_global_summary_result <- renderUI({
    res <- ai_summary_res()
    if (res == "") {
      return(HTML("<div style='color: #888; text-align: center;'>The summary of your analysis results will appear here.</div>"))
    }
    HTML(res)
  })
  
  # Chat Send Logic
  observeEvent(input$ai_global_chat_send, {
    prompt <- trimws(input$ai_global_chat_input)
    if (prompt == "") return()
    
    # Update input to empty
    updateTextInput(session, "ai_global_chat_input", value = "")
    
    # Add User message
    current_chat <- ai_chat_history()
    user_msg <- paste0("<b>You:</b><br>", htmltools::htmlEscape(prompt))
    current_chat <- c(current_chat, user_msg)
    ai_chat_history(current_chat)
    
    session$sendCustomMessage("scroll_ai_chat", "scroll")
    
    # We could show a thinking indicator or just append it
    current_chat <- c(current_chat, "<div id='ai_thinking' style='color:#666;'><i>AI is thinking...</i></div>")
    ai_chat_history(current_chat)
    session$sendCustomMessage("scroll_ai_chat", "scroll")
    
    # Construct full prompt with context
    ctx_text <- ai_context$results_text
    sys_prompt <- "You are an AI assistant for projectLSA. You are in a conversational Q&A mode. The user's latest analysis results are provided as Context. ONLY answer the user's specific Question. DO NOT generate a full summary of the Context unless explicitly asked by the user in the Question."
    full_prompt <- paste0(sys_prompt, "\n\nContext:\n", ctx_text, "\n\nQuestion:\n", prompt)
    
    # Call AI
    tryCatch({
      response <- ask_ai(full_prompt, input$ai_provider, input$ai_model, input$ai_api_key)
      response_html <- gsub("\n", "<br>", response)
      
      # Remove thinking and add response
      current_chat <- current_chat[-length(current_chat)]
      ai_msg <- paste0("<b>AI:</b><br>", response_html)
      current_chat <- c(current_chat, ai_msg)
      ai_chat_history(current_chat)
      
    }, error = function(e) {
      current_chat <- current_chat[-length(current_chat)]
      err_msg <- paste0("<b>AI Error:</b><br><span style='color:red;'>", e$message, "</span>")
      current_chat <- c(current_chat, err_msg)
      ai_chat_history(current_chat)
    })
    
    session$sendCustomMessage("scroll_ai_chat", "scroll")
  })
  
  # Clear Chat
  observeEvent(input$ai_global_chat_clear, {
    ai_chat_history(character(0))
  })
  
  # Generate Summary Logic
  observeEvent(input$ai_global_summary_generate, {
    ctx_text <- ai_context$results_text
    
    if (is.null(ctx_text) || trimws(ctx_text) == "") {
      ai_summary_res("<span style='color:red;'>No active analysis results. Please run an analysis first.</span>")
      return()
    }
    
    ai_summary_res("<i>Generating summary...</i>")
    
    format <- input$ai_summary_format
    lang <- input$ai_summary_lang
    
    # Parse context
    context_str <- ""
    if (!is.null(input$ai_context_text) && trimws(input$ai_context_text) != "") {
      context_str <- trimws(input$ai_context_text)
    }
    
    if (!is.null(input$ai_context_file)) {
      file_path <- input$ai_context_file$datapath
      ext <- tools::file_ext(input$ai_context_file$name)
      file_content <- tryCatch({
        if (tolower(ext) == "txt") {
          paste(readLines(file_path, warn = FALSE), collapse = "\n")
        } else if (tolower(ext) == "pdf" && requireNamespace("pdftools", quietly = TRUE)) {
          paste(pdftools::pdf_text(file_path), collapse = "\n")
        } else if (tolower(ext) == "docx" && requireNamespace("officer", quietly = TRUE)) {
          doc <- officer::read_docx(file_path)
          content <- officer::docx_summary(doc)
          paste(content$text[!is.na(content$text)], collapse = "\n")
        } else {
          ""
        }
      }, error = function(e) { paste("Error reading file:", e$message) })
      
      if (file_content != "") {
        context_str <- paste0(context_str, "\n\n[Context from uploaded document:]\n", file_content)
      }
    }
    
    context_prompt_part <- ""
    if (context_str != "") {
      context_prompt_part <- paste0("TAKE THE FOLLOWING RESEARCH CONTEXT INTO ACCOUNT (use it to tailor your interpretation):\n", context_str, "\n\n")
    }

    # Manuscript format specific instruction
    manuscript_instruction <- ""
    if (format == "Manuscript") {
      manuscript_instruction <- paste0(
        "Because the requested format is 'Manuscript (Narrative)', you MUST write a journal-article style report. ",
        "Compose complete, flowing, professional paragraphs. ",
        "You MUST refer to tables and figures within the text (e.g. 'As presented in Table 1...' or 'Based on Figure 1...'). ",
        "You MUST insert a placeholder immediately after the paragraph that refers to a table/figure, using the format `[Insert Table/Figure X here]`. ",
        "Do not build the table itself; only write the placeholder so the user can paste the actual table from the application later.\n"
      )
    }

    prompt <- paste0(
      "Your ONLY task is to produce a comprehensive summary of the analysis results below. ",
      "Do NOT add chat greetings, small talk, or any conversational text.\n",
      context_prompt_part,
      "IMPORTANT: If the results include a history of several models (such as Model_1, Model_2, etc.), ",
      "describe the condition of the initial model, the reason for and details of the modifications applied to the subsequent models, ",
      "and how those modifications improved the final result (model comparison).\n",
      manuscript_instruction,
      "REQUIRED: Always include in-text citations for any standard or cut-off criterion you invoke (e.g. Hu & Bentler, 1999; Hair et al., 2010; Schreiber et al., 2006; Nylund et al., 2007). ",
      "In addition, you MUST add a dedicated section titled 'References' at the end of your summary containing the full reference list for every citation you used, so that the references are exported together with the summary.\n\n",
      "Requested format: ", format, "\n",
      "Write the summary in this language: ", lang, "\n\n",
      "Analysis Results:\n", ctx_text
    )
    
    tryCatch({
      response <- ask_ai(prompt, input$ai_provider, input$ai_model, input$ai_api_key)
      response_html <- gsub("\n", "<br>", response)
      ai_summary_res(response_html)
    }, error = function(e) {
      ai_summary_res(paste0("<span style='color:red;'>Error: ", e$message, "</span>"))
    })
  })

  # Add to report
  observeEvent(input$ai_add_to_report, {
    res <- ai_summary_res()
    if (res != "" && !grepl("<i>Generating summary...</i>", res)) {
      ai_context$ai_report_text <- res
      showNotification("AI Summary added! It will be included when you export HTML report.", type = "message", duration = 5)
    } else {
      showNotification("Please generate an AI summary first.", type = "warning")
    }
  })

  
  # Save API Key from Widget
  observeEvent(input$save_ai_widget_api_key_chat, {
    key <- trimws(input$ai_widget_api_key_chat)
    if (key != "") {
      updateSelectInput(session, "ai_provider", selected = input$ai_widget_provider_chat)
      updateTextInput(session, "ai_model", value = input$ai_widget_model_chat)
      updateTextInput(session, "ai_api_key", value = key)
    }
  })
  
  observeEvent(input$save_ai_widget_api_key_sum, {
    key <- trimws(input$ai_widget_api_key_sum)
    if (key != "") {
      updateSelectInput(session, "ai_provider", selected = input$ai_widget_provider_sum)
      updateTextInput(session, "ai_model", value = input$ai_widget_model_sum)
      updateTextInput(session, "ai_api_key", value = key)
    }
  })
}

# fa_ui.R (revisi)
cfa_ui <- function(project) {
  fluidRow(
    column(width = 3,
           
           wellPanel(
             actionButton("go_home", 
                        label = tagList(icon("home"), "Main Menu"), 
                        class = "btn btn-danger btn-block",
                        style = "width: 100% !important;"),
             br(),
             selectInput("data_source", "Select Data Source:",
                         choices = c("Upload Data" = "upload",
                                     "Built-in: bfi" = "bfi",
                                     "Built-in: HolzingerSwineford1939" = "HolzingerSwineford1939"),
                         selected = "upload"),
             conditionalPanel(condition = "input.data_source == 'upload'",
                              fileInput("datafile", "Upload Data (csv/xlsx)", accept = c(".csv", ".xlsx"))),
             uiOutput("id_select_ui"),
             uiOutput("var_select_ui"),
             tags$label("Model (Lavaan syntax):"),
             uiOutput("cfa_model_ui"),
            
             selectInput("cfa_estimator", "Estimator:",
                         choices = c("ML" = "ML", "MLR" = "MLR", "WLSMV" = "WLSMV", 
                                     "DWLS" = "DWLS", "GLS" = "GLS", "ULS" = "ULS"),
                         selected = "ML"),
             selectInput("cfa_missing", "Missing data:",
                         choices = c("Listwise" = "listwise", "Pairwise" = "pairwise", "FIML" = "fiml"),
                         selected = "listwise"),
             radioButtons("cfa_dec_sep", "Decimal Separator:", 
                          choices = c("English (.)" = ".", "Indonesian (,)" = ","),
                          selected = ".", inline = TRUE),
             checkboxInput("cfa_std_est", "Standardized estimates", TRUE),
             checkboxInput("htmt_opt", "Heterotrait–Monotrait Ratio (HTMT)", FALSE),
             
             actionButton("run_cfa", label = tagList(icon("play"), "Run CFA"),
                          class = "btn btn-success btn-block",
                          style = "width: 100% !important;"),
             br(),
             actionButton("btn_export_modal", "Export Report (HTML)", class = "btn btn-warning btn-block btn-glow", style = "width: 100%; font-weight: bold;", onclick="$(this).removeClass('btn-glow');")
           )
    ), 
    column(width = 9,
           tabsetPanel(
             id = "main_tab_cfa",
             
             tabPanel(
               title = tagList(icon("upload"), "Data Preview"),
               DTOutput("data_preview")),
             # ====Model Summary =====
             tabPanel(
               title = tagList(icon("chart-line"), "Model Summary"),
                      value = "fit_tab_cfa", 
                      br(),
                      fluidRow(
                        uiOutput("fit_comparison")

                      )),
             tabPanel(
               title = tagList(icon("table"), "Loadings & Params"),
               br(), DTOutput("loadings_table")
               ),
             tabPanel(
               title = tagList(icon("chart-bar"), "Variances"),
               br(), 
               uiOutput("variance_warning"),
               DTOutput("variances_table")
               ),
             tabPanel(
               title = tagList(icon("calculator"), "Factor Scores"),
               br(),
               DTOutput("fscores_cfa")
               ),
            # ====Plot =====
             tabPanel(
               title = tagList(icon("project-diagram"), "Path Plot"),
                      br(),
                      fluidRow(
                        column(3,
                               tags$h5("Plot Settings", style = "color: #2c3e50;"),
                               
                               wellPanel(
                                 style = "padding: 10px; margin-bottom: 10px;",
                                 tags$strong("1. General & Layout"),
                                 fluidRow(
                                   column(6, selectInput("plot_style", "Style:", choices = c("lisrel", "ram", "mx", "OpenMx"), selected = "lisrel")),
                                   column(6, selectInput("plot_layout", "Layout:", choices = c("tree", "tree2", "tree3", "spring", "circle", "circle2"), selected = "tree2"))
                                 ),
                                 fluidRow(
                                   column(6, numericInput("plotwidth", "Width:", value = 5, min = 1, max = 15, step = 0.3)),
                                   column(6, numericInput("plotheight", "Height:", value = 5, min = 1, max = 20, step = 0.3))
                                 ),
                                 fluidRow(
                                   column(6, numericInput("plot_rotation", "Rotation:", value = 4, min = 1, max = 4, step = 1)),
                                   column(6, textInput("bifactor", "Bfactor:", value = NULL, placeholder = "Gen. Factor"))
                                 )
                               ),
                               
                               wellPanel(
                                 style = "padding: 10px; margin-bottom: 10px;",
                                 tags$strong("2. Node & Edge Sizes"),
                                 fluidRow(
                                   column(4, numericInput("plot_nodesize_lat", "Latent:", value = 8, min = 3, max = 15, step = 0.3)),
                                   column(4, numericInput("plot_nodesize_man", "Obs(W):", value = 8, min = 2, max = 15, step = 0.3)),
                                   column(4, numericInput("plot_nodesize_man2", "Obs(H):", value = 4, min = 1, max = 10, step = 0.3))
                                 ),
                                 fluidRow(
                                   column(6, numericInput("plot_edge_label_size", "Label Size:", value = 0.75, min = 0.5, max = 2, step = 0.1)),
                                   column(6, numericInput("edgewidth", "Edge Width:", value = 0.3, min = 0.3, max = 5, step = 0.1))
                                 )
                               ),
                               
                               wellPanel(
                                 style = "padding: 10px; margin-bottom: 10px;",
                                 tags$strong("3. Colors & Display Options"),
                                 selectInput("plot_color_scheme", "Color scheme:",
                                             choices = c("Blue-Yellow", "Ocean", "Forest", "Rainbow", "Pastel", "Greyscale", "Earth", "Vibrant", "Monochrome", "Sunset", "Rose", "Mint", "Custom"),
                                             selected = "Blue-Yellow"),
                                 conditionalPanel(
                                   condition = "input.plot_color_scheme == 'Custom'",
                                   fluidRow(
                                     column(6, colourpicker::colourInput("mancolour", "Manifest:", value = "#A1E3F9", showColour = "background", palette = "square")),
                                     column(6, colourpicker::colourInput("latcolour", "Latent:", value = "#FFFFBA", showColour = "background", palette = "square"))
                                   )
                                 ),
                                 checkboxInput("plot_standardized", "Standardized estimates", TRUE),
                                 checkboxInput("plot_residuals", "Show residuals", FALSE),
                                 checkboxInput("plot_exoCov", "Show exogenous covariances", FALSE)
                               )
                        ),
                        column(9, 
                               plotOutput("path_plot", height = "700px"),
                               br(),
                               div(style = "text-align: center; margin-top: 10px; background-color: #f8f9fa; padding: 15px; border-radius: 8px; border: 1px solid #e2e8f0;",
                                   tags$strong("4. Fit Indices & Export", style = "display: block; margin-bottom: 10px;"),
                                   checkboxGroupInput(
                                     "fit_indices_selected",
                                     label = NULL,
                                     choices = c("Chi-Square (χ²)" = "chisq", "df" = "df", "p-value" = "pvalue", "RMSEA" = "rmsea", "CFI" = "cfi", "GFI" = "gfi", "SRMR" = "srmr", "TLI" = "tli", "NFI" = "nfi"),
                                     selected = c("chisq", "df", "pvalue", "rmsea", "cfi", "srmr"),
                                     inline = TRUE
                                   ),
                                   br(),
                                   downloadButton("download_cfa_plot", "Download Plot (PNG)", class = "btn btn-primary")
                               )
                        )
                      )),
             # --- About ----
            tabPanel(
              title = tagList(icon("info-circle"), "About"),
              fluidRow(
                column(
                  width = 8, offset = 2,
                  br(),
                  div(
                    style = "text-align:center;",
                    tags$hr(),
                    tags$h5("projectLSA Was Developed By:"),
                    tags$p(
                      tags$a(
                        href = "https://scholar.google.com/citations?user=PSAwkTYAAAAJ&hl=id",
                        target = "_blank",
                        "Dr. Hasan Djidu, M.Pd."),
                      tags$br(),
                      "Universitas Sembilanbelas November Kolaka"
                    ),
                    tags$h5("Supervised By:"),
                    tags$p(
                      tags$a(
                      href = "https://scholar.google.com/citations?user=7CzPTYIAAAAJ&hl=id",
                      target = "_blank",
                      "Prof. Dr. Heri Retnawati, M.Pd."), 
                      tags$br(),
                      "Universitas Negeri Yogyakarta"
                    ),
                    
                    tags$p(tags$a(
                      href = "https://scholar.google.com/citations?hl=id&user=VGKeBm0AAAAJ",
                      target = "_blank",
                      "Prof. Dr. Samsul Hadi"), 
                      tags$br(),
                      "Universitas Negeri Yogyakarta"
                    ),
                    tags$p(tags$a(
                      href = "https://scholar.google.com/citations?hl=id&user=k4MA8XgAAAAJ",
                      target = "_blank",
                      "Dr. Drs. Ir. Haryanto, M.Pd., M.T."), 
                      tags$br(),
                      "Universitas Negeri Yogyakarta"
                    ),
                    tags$b("Contact:"),
                    tags$a("hasandjidu@gmail.com"),
                    tags$hr()
                   )
                 ),
                 column(
                   width = 8, offset = 2,
                   h4("References (R Packages)"),
                   uiOutput("package_references_cfa"),
                   br(),
                   div(
                     style = "text-align:center;",
                     tags$p(
                       style = "font-size:13px; color:#777;",
                       format(Sys.Date(), "%Y"), 
                       "projectLSA. All rights reserved."
                     )
                   )
                 )
               )
             )
             
           )
    )
  )  
}

# IRT -----
# ==== Reactive Data IRT ====
server_lta <- function(input, output, session, ai_context, console_context) {
  library(mirt)
  library(plotly)
  library(psych)
  library(shinycssloaders)

  # Set while a saved workspace is being loaded, so the "reset results when the
  # data changes" observer below does not wipe the models we just restored.
  lta_restoring <- reactiveVal(FALSE)

  data_user <- reactive({
    req(input$dimension, input$data_source_lta)
    if (input$dimension == "uni" && input$data_source_lta == "diko") {
      a <- matrix(runif(15, 0.8, 2), ncol = 1)      # discrimination 1 dimension
      d <- matrix(rnorm(15, 0, 1), ncol = 1)       # difficulty
      theta <- matrix(rnorm(750), ncol = 1)       # ability 1 dimension
      resp <- simdata(a = a, d = d, itemtype = "2PL", Theta = theta)
      df <- as.data.frame(resp) %>% tidyr::drop_na()
      df$Group <- sample(c("Group_A", "Group_B"), nrow(df), replace = TRUE)
      
    } else if (input$dimension == "uni" && input$data_source_lta == "poli") {
      a <- matrix(rlnorm(20, .2, .3))  
      diffs <- t(apply(matrix(runif(20*4, .3, 1), 20), 1, cumsum))
      diffs <- -(diffs - rowMeans(diffs))
      d <- diffs + rnorm(20)
      resp <- simdata(a, d, 300, itemtype = 'graded')
      df <- as.data.frame(resp) %>% tidyr::drop_na()
      df$Group <- sample(c("Group_A", "Group_B"), nrow(df), replace = TRUE)
      
    } else if (input$dimension == "multi" && input$data_source_lta == "diko") {
      N <- 750     
      n_items <- 15
      n_factors <- 2
      a <- matrix(0, nrow = n_items, ncol = n_factors)
      a[1:8,1] <- runif(8, 0.5, 1.5)   # loading ke F1
      a[9:15,2] <- runif(7, 0.5, 1.5)  # loading ke F2
      d <- matrix(rnorm(n_items, 0, 1), ncol = 1)
      sigma <- matrix(c(1,0.3,0.3,1), nrow = 2)
      resp <- simdata(a = a, d = d, N = N, itemtype = '2PL', sigma = sigma)
      df <- as.data.frame(resp) %>% tidyr::drop_na()
      df$Group <- sample(c("Group_A", "Group_B"), nrow(df), replace = TRUE)
      
    } else if (input$dimension == "multi" && input$data_source_lta == "poli") {
      df <- as.data.frame(psych::bfi) %>% dplyr::select(A1:E2, gender) %>% tidyr::drop_na()
      df$gender <- factor(df$gender, levels = c(1, 2), labels = c("Male", "Female"))
      colnames(df)[ncol(df)] <- "Gender"
    } else {
      req(input$datafile_lta)
      ext <- tolower(tools::file_ext(input$datafile_lta$name))
      showModal(modalDialog(title = NULL, "Reading Your File, Please wait...", footer = NULL, easyClose = FALSE))
      if (ext == "rds") {
        res <- readRDS(input$datafile_lta$datapath)
        if (is.list(res) && !is.data.frame(res) && identical(res$type, "projectLSA_workspace")) {
          if (identical(res$module, "LTA")) {
            # Restore Workspace State
            lta_restoring(TRUE)
            lta_fit_list(res$lta_fit_list)
            lta_fit_compare(res$lta_fit_compare)
            lta_current_id(res$lta_current_id)
            df <- res$raw_data
            
            # Restore UI Inputs
            if (!is.null(res$input_state)) {
              updateSelectInput(session, "data_source_lta", selected = res$input_state$data_source_lta)
              updateSelectInput(session, "datatype", selected = res$input_state$datatype)
              updateSelectInput(session, "dimension", selected = res$input_state$dimension)
              updateSelectInput(session, "fit_stats", selected = res$input_state$fit_stats)
              if (!is.null(res$input_state$selected_vars)) {
                 updateSelectInput(session, "selected_vars", selected = res$input_state$selected_vars)
              }
              if (!is.null(res$input_state$itemtype)) {
                 updateSelectInput(session, "itemtype", selected = res$input_state$itemtype)
              }
              if (!is.null(res$input_state$lta_model_text)) {
                 updateTextAreaInput(session, "lta_model_text", value = res$input_state$lta_model_text)
              }
            }
            
            showNotification("LTA Workspace restored successfully!", type = "message")
          } else {
            stop("Uploaded workspace belongs to a different module: ", res$module)
          }
        } else if (inherits(res, "SingleGroupClass") || inherits(res, "MultipleGroupClass")) {
          lta_fit_list(list(list(id = "model_1", model = res, type = "Uploaded Model", timestamp = format(Sys.time(), "%Y-%m-%d %H:%M:%S"))))
          lta_current_id("model_1")
          df <- as.data.frame(res@Data$data)
          showModal(modalDialog("LTA Model uploaded successfully! Please navigate to the Summary or Plot tab.", easyClose = TRUE))
        } else {
          df <- res
        }
      } else {
        df <- switch(
          ext,
          "csv"  = data.table::fread(input$datafile_lta$datapath, data.table = FALSE),
          "xls"  = readxl::read_excel(input$datafile_lta$datapath),
          "xlsx" = readxl::read_excel(input$datafile_lta$datapath),
          "sav"  = haven::read_sav(input$datafile_lta$datapath),
          stop("Unsupported file type. Please upload CSV, Excel, SPSS (.sav), or RDS file.")
        )
      }
      removeModal()
      
      if (!"id_auto" %in% names(df)) {
        df <- df %>% mutate(across(everything(), ~ifelse(.x == "", NA, .x)),
                            id_auto = paste0("id_", sprintf("%04d", 1:n())))
      }
    }
    return(df)
  })
  output$id_select_ui <- renderUI({
    req(data_user(), input$dimension, input$data_source_lta)
    selectizeInput("id_cols", "ID columns (optional)", choices = names(data_user()), multiple = TRUE,
                   selected = names(data_user())[grepl("id", names(data_user()), ignore.case = TRUE)])
  })
  output$var_select_ui <- renderUI({
    req(data_user(), input$dimension, input$data_source_lta)
    all_vars <- names(data_user())
    id_cols <- input$id_cols
    choices <- setdiff(all_vars, id_cols)
    selectInput("selected_vars", "Select Variables (used for IRT):", choices = choices, multiple = TRUE, selected = choices)
  })
  
  data_lta <- reactive({
    req(input$dimension, input$data_source_lta, data_user(), input$selected_vars, input$fit_stats)
    df <- data_user() %>% dplyr::select(all_of(input$selected_vars))
    # Exclude non-numeric columns for IRT model fitting
    df %>% dplyr::select(where(is.numeric))
  })
  
  # ==== Preview Data ====
  output$data_preview_lta <- DT::renderDT({
    req(input$dimension, input$data_source_lta, data_user(), input$selected_vars, input$fit_stats,data_lta())
    df <- data_lta() 
    numeric_cols <- which(sapply(df, is.numeric))
    DT::datatable(df, extensions = 'Buttons',
                  options = list(dom='Brtp',scrollX = TRUE, pageLength = 25,  
                                 buttons = list(
                                   list(extend = 'csv',
                                        text = 'Export CSV',
                                        filename = 'Data LTA'
                                   ),
                                   list(extend = 'excel',
                                        text = 'Export Excel',
                                        filename = 'Data LTA'
                                   ))),
                  rownames = TRUE) %>% 
      formatRound(columns = numeric_cols, digits = 0)
  }, server = FALSE)
  output$itemtype_ui <- renderUI({
    req(input$datatype, input$data_source_lta)
    source_type <- if(input$data_source_lta == "upload") {
      input$datatype
    } else {
      input$data_source_lta
    }
    choices <- switch(source_type,
                      "diko" = c("Rasch" = "Rasch",
                                 "2-Parameter Logistic (2-PL)" = "2PL",
                                 "3-Parameter Logistic (3-PL)" = "3PL"),
                      "poli" = c("Partial Credit Model (PCM)" = "Rasch",
                                 "Graded Response Model (GRM)" = "graded",
                                 "Generalized PCM (GPCM)" = "gpcm")
    )
    
    div(
      class = "select-large", # styling selector
      selectInput(
        inputId = "itemtype",
        label = NULL,
        choices = choices,
        selected = choices[[1]],
        width = "100%"
      )
    )
  })
  
  # ==== Reset all stored IRT results when data_user changes ====
  observeEvent(data_user(), {
    # ...unless the new data came from a restored workspace, which carries its
    # own models with it.
    if (isTRUE(lta_restoring())) {
      lta_restoring(FALSE)
      return(invisible(NULL))
    }
    # Reset stored models and current selection
    lta_fit_list(list())
    lta_fit_compare(NULL)
    lta_current_id(NULL)
  })
  # ===== IRT / MIRT MODEL =====
  output$lta_model_ui <- renderUI({
    req(input$data_source_lta, input$dimension=="multi")
    model_text <- switch(input$data_source_lta,
                         # === 1. Multidimensional - Dichotomous ===
                         "diko" = "
# Example 1: Multidimensional - Dichotomous (2 dimensions)
model <- '
F1 = 1-5
F2 = 6-10
'
",
# === 2. Multidimensional - Polytomous ===
"poli" = "
# Example 2: Multidimensional - Polytomous (2 dimensions)
model <- '
F1 = 1-5
F2 = 6-10 
'
"
    )
    tags$textarea(
      id = "lta_model_text",
      rows = 10,
      style = "width:100%; font-family: monospace; font-size:11px;",
      model_text
    )
  })
  
  # ==== IRT storage ====
  lta_fit_list <- reactiveVal(list())
  lta_fit_compare <- reactiveVal(NULL)
  lta_current_id <- reactiveVal(NULL)
  
  # ==== Fungsi untuk menjalankan model ====
  run_mirt_model <- function(data, model_def, itemtype) {
    tryCatch({
      mirt(data, model = model_def, itemtype = itemtype, verbose = FALSE)
    }, error = function(e) {
      showNotification(paste("Model", itemtype, "error:", e$message), type = "error")
      return(NULL)
    })
  }
  
  # ==== IRT Main Run ====
  observeEvent(input$run_lta, {
    req(data_lta(), input$selected_vars, input$fit_stats)
    # Tambahkan kondisi khusus untuk upload
    if (input$data_source_lta == "upload") {
      req(input$datatype)
    }
    updateTabsetPanel(session, "main_tab_lta", selected = "summary_tab")
    df <- data_lta()
    # Model
    source_type <- if(input$data_source_lta == "upload") {
      req(input$datatype)   # pastikan user sudah pilih
      input$datatype
    } else {
      input$data_source_lta
    }
    
    itemtypes <- switch(source_type,
                        "diko" = c("Rasch","2PL","3PL"),
                        "poli" = c("Rasch","graded","gpcm"))
    # Define Model Syntax
    model_def <- if (input$dimension == "multi") {
      mirt.model(input$lta_model_text,itemnames = df)
    }  else {1}
    showModal(modalDialog("Running Item Response Theory ...", footer = NULL))
    
    fit_results <- list()
    for (item in itemtypes) {
      mod <- run_mirt_model(df, model_def, item)
      if (is.null(mod)) next
      measures <- tryCatch({
        as.data.frame(anova(mod))
      }, error = function(e) data.frame())
      
      N_itemfit <- tryCatch({
        itemfit_df <- as.data.frame(mirt::itemfit(mod, fit_stats=input$fit_stats))
        p_cols <- grep("^p\\.", names(itemfit_df), value = TRUE)
        sum(itemfit_df[[p_cols]] >= 0.05, na.rm = TRUE)
      }, error = function(e) 0)
      
      LD <- tryCatch({
        Q3_mat <- residuals(mod, type = "Q3", verbose = FALSE)
        # ambil item names
        items <- colnames(Q3_mat)
        # ekstrak upper triangle
        idx <- which(upper.tri(Q3_mat), arr.ind = TRUE)
        q3_long <- data.frame(
          item  = items[idx[,2]],
          item2 = items[idx[,1]],
          Q3    = Q3_mat[upper.tri(Q3_mat)]
        )
        violations <- q3_long %>% 
          dplyr::filter(abs(Q3) > 0.2)
        unique_items <- unique(c(violations$item, violations$item2))
        N_LD <- length(unique_items)
        list(N_LocalDependency = N_LD,
             item_LocalDep = unique_items
        )
      }, error = function(e) {
        list(N_violations = NULL, itemLD = NA)}
      )
      fit_results[[item]] <- list(
        id = item,
        model_type = item,
        fit = mod,
        measures = measures,
        N_itemfit = N_itemfit,
        LD = LD,
        time = Sys.time()
      )
    }
    # ---- Model comparison (if possible) ----
    models_for_anova <- lapply(fit_results, function(x) x$fit)
    models_for_anova <- models_for_anova[!sapply(models_for_anova, is.null)]
    lta_fit_compare(
      switch(source_type,
             "diko" = as.data.frame(anova(models_for_anova[['Rasch']],
                                          models_for_anova[['2PL']],
                                          models_for_anova[['3PL']])),
             "poli" = rbind(
               as.data.frame(anova(models_for_anova[['Rasch']],
                                   models_for_anova[['graded']]
               )),
               as.data.frame(anova(models_for_anova[['Rasch']],
                                   models_for_anova[['gpcm']]))[2,]
             )
      ))
    removeModal()
    
    cur <- lta_fit_list()
    cur <- modifyList(cur, fit_results)
    best_name <- input$itemtype
    
    all_model_types <- sapply(cur, function(x) x$model_type)
    match_idx <- which(tolower(all_model_types) == tolower(best_name))
    
    if (length(match_idx) == 0) {
      best_id <- names(cur)[1]
    } else {
      best_id <- names(cur)[match_idx[1]]
    }
    lta_fit_list(cur)
    lta_current_id(best_id)
    
    showNotification(paste0("LTA finished. Selected model: ", best_name), type = "message")
    
  })
  
  output$fit_comparison <- renderUI({
    req(data_lta(), input$selected_vars, input$fit_stats, lta_fit_list())
    source_type <- if(input$data_source_lta == "upload") {
      input$datatype
    } else {
      input$data_source_lta
    }
    all_measures <- do.call(rbind, lapply(lta_fit_list(), function(x) x$measures)) %>% 
      tibble::rownames_to_column("Model") %>%
      dplyr::mutate(
        Model = dplyr::case_when(
          source_type == 'poli' & Model == 'Rasch'   ~ 'PCM',
          source_type == 'poli' & Model == 'graded'  ~ 'GRM',
          source_type == 'poli' & Model == 'gpcm'    ~ 'GPCM',
          TRUE                                    ~ Model
        )
      ) %>% 
      dplyr::select(Model)
    
    N_itemfit <- do.call(rbind, lapply(lta_fit_list(), function(x) x$N_itemfit))
    N_itemLD <- do.call(rbind, lapply(lta_fit_list(), function(x) x$LD$N_LocalDependency))
    item_LocalDep <- do.call(rbind, lapply(lta_fit_list(), function(x) x$LD$item_LocalDep))
    comparison <- cbind(all_measures,lta_fit_compare(),
                        N_itemfit, N_itemLD) %>% dplyr::rename(`p (χ²)`=p, 'χ²'=X2)
    comparison <- comparison %>% 
      dplyr::mutate(
        AIC   = round(AIC, 2),
        SABIC = round(SABIC, 2),
        BIC   = round(BIC, 2),
        HQ    = round(HQ, 2),
        logLik = round(logLik,2),
        `χ²` = round(`χ²`,2),
        df=df,
        `p (χ²)` = case_when(
          is.na(p (`p (χ²)`)) ~ NA_character_,
          `p (χ²)` < 0.01 ~ "p <.01",
          `p (χ²)` < 0.05  ~ "p <.05",
          TRUE      ~ "p >.05"
        ),
        N_itemfit =  N_itemfit,
        N_itemLD = N_itemLD     
      )
    min_AIC   <- min(comparison$AIC, na.rm = TRUE)
    min_SABIC <- min(comparison$SABIC, na.rm = TRUE)
    min_BIC   <- min(comparison$BIC, na.rm = TRUE)
    min_HQ    <- min(comparison$HQ, na.rm = TRUE)
    max_Nfit  <- max(comparison$N_itemfit , na.rm = TRUE)
    minLD  <- min(comparison$N_itemLD, na.rm = TRUE)
    column(12,
           DT::datatable(comparison,extensions = 'Buttons',
                         options = list(scrollX = TRUE, pageLength = 10,
                                        dom = 'B',
                                        buttons = list(
                                          list(
                                            extend = 'csv',
                                            text = 'Export CSV',
                                            filename = 'Fit Comparison'  # <-- nama file CSV
                                          ),
                                          list(
                                            extend = 'excel',
                                            text = 'Export Excel',
                                            filename = 'Fit Comparison'  # <-- nama file Excel
                                          ))),
                         rownames = FALSE) %>%
             # === Highlight ===
             formatStyle(
               columns = 'AIC',
               color = styleEqual(min_AIC, 'black'),
               backgroundColor = styleEqual(min_AIC, 'lightgreen'),
               fontWeight = styleEqual(min_AIC, 'bold')
             ) %>%
             formatStyle(
               columns = 'BIC',
               color = styleEqual(min_BIC, 'black'),
               backgroundColor = styleEqual(min_BIC, 'lightgreen'),
               fontWeight = styleEqual(min_BIC, 'bold')
             ) %>%
             formatStyle(
               columns = 'SABIC',
               color = styleEqual(min_SABIC, 'black'),
               backgroundColor = styleEqual(min_SABIC, 'lightgreen'),
               fontWeight = styleEqual(min_SABIC, 'bold')
             ) %>%
             formatStyle(
               columns = 'HQ',
               color = styleEqual(min_HQ, 'black'),
               backgroundColor = styleEqual(min_HQ, 'lightgreen'),
               fontWeight = styleEqual(min_HQ, 'bold')
             ) %>%
             # === Highlight untuk kolom N_itemfit (nilai terbesar) ===
             formatStyle(
               columns = 'N_itemfit',
               color = styleEqual(max_Nfit, 'black'),
               backgroundColor = styleEqual(max_Nfit, 'lightgreen'),
               fontWeight = styleEqual(max_Nfit, 'bold')
             )  %>% 
             formatStyle(
               columns = 'N_itemLD',
               color = styleEqual(minLD, 'black'),
               backgroundColor = styleEqual(minLD, 'lightgreen'),
               fontWeight = styleEqual(minLD, 'bold')
             ) %>% 
             formatStyle(
               columns = 'p (χ²)',
               backgroundColor = styleEqual(
                 c('p <.01', 'p <.05'),
                 c('lightgreen', 'lightgreen')
               )
             ),
           tags$div(
             style = "margin-top: 0px; font-size: 12px; color: #6c757d;",
             tags$b("Interpretation Guidelines:"),
             tags$ul(
               tags$li(tags$span(style = "color: green;", "Best Model:"),"Smaller AIC/BIC/SABIC/HQ/N_ItemLD; Higher N_itemfit"),
               tags$li(tags$span(style = "color: red;", "ItemLD (Violation of Local Independency Assumption):"),  "|Q3| > 0.2 (Yen, 1984)"),
               tags$li(tags$span(style = "color: green;", "p.value <.05 / p.value <.01:"), 
                       "The more complex model fits better than the simpler model (Desjardin & Bulut, 2018)")
               
             ),
             tags$hr()
           )
    )
  })
  # === Helper: Get the currently selected model based on input$itemtype ===
  selected_fit <- reactive({
    req(lta_fit_list(), input$itemtype)
    all_models <- lta_fit_list()
    match_idx <- which(
      tolower(sapply(all_models, function(x) x$model_type)) == tolower(input$itemtype)
    )
    if (length(match_idx) == 0) {
      showNotification(paste0("Model '", input$itemtype, "' not found."), type = "warning")
      return(NULL)
    }
    all_models[[match_idx[1]]]$fit
  })
  
  # === Item Summary (reactive to selected model) ===
  output$item_summary <- renderUI({
    req(selected_fit())
    itemparam <- as.data.frame(mirt::coef(selected_fit(), IRTpars = TRUE, simplify = TRUE)$items)
    itemfit <- as.data.frame(mirt::itemfit(selected_fit(), fit_stats = input$fit_stats)) %>% 
      dplyr::select (paste0(input$fit_stats), paste0('p.',input$fit_stats)) %>% 
      dplyr::rename(`χ²_itemfit` = paste0(input$fit_stats), `p(χ²)_itemfit` = paste0('p.',input$fit_stats))
    
    item_summary <- cbind(itemparam,itemfit) %>% tibble::rownames_to_column("item")
    numeric_cols <- which(sapply(item_summary, is.numeric))
    
    DT::datatable(item_summary,extensions = 'Buttons',
                  options = list(scrollX = TRUE, dom = 'Brtp',
                                 buttons = list(
                                   list(
                                     extend = 'csv',
                                     text = 'Export CSV',
                                     filename = paste0('Item Summary-',input$itemtype)  
                                   ),
                                   list(
                                     extend = 'excel',
                                     text = 'Export Excel',
                                     filename = paste0('Item Summary-',input$itemtype)
                                   ))), 
                  rownames = FALSE) %>%
      formatRound(columns = numeric_cols, digits = 2) %>% 
      DT::formatStyle(columns = "p(χ²)_itemfit",
                      fontWeight = 'bold',
                      color = styleInterval(0.04999999, c('red', 'green'))
      )
  })
  
  
  output$icc_info <- renderUI({
    req(input$dimension, selected_fit() )
    nfactors <- selected_fit()@Model$nfact
    if (input$dimension =='uni')
    {
      column(12,
             column(6,
                    plotOutput('icc'),
                    uiOutput('select_icc'),    
             ),
             column(6, plotOutput('infose')))
    } else if (input$dimension =='multi' && nfactors == 2 ) {
      column(12,
             plotOutput('icc'),
             uiOutput('select_icc'),  
             uiOutput("info_ui_multi")
      )
    }
    else if (input$dimension =='multi' && nfactors >= 3 )
    {
      column(12,
             uiOutput("info_ui_multi")
      )
    }
    else{return(NULL)}
  })
  
  output$select_icc <- renderUI({
    req(data_lta())
    choices <- c('SELECT ALL',names(data_lta()))
    div(class = "select-mini", 
        selectInput("item_select_icc", NULL, choices = choices, multiple = TRUE, selected = 'SELECT ALL', width = '100%'))
  }) 
  observeEvent(input$item_select_icc, {
    req(input$item_select_icc)
    selected <- input$item_select_icc
    # Jika "All_Item" dipilih bersama dengan yang lain, sisakan hanya "All_Item"
    if ("SELECT ALL" %in% selected && length(selected) > 1) {
      # Jika "All_Item" baru saja diklik, hapus item lain
      if (tail(selected, 1) == "SELECT ALL") {
        updateSelectInput(session, "item_select_icc", selected = "SELECT ALL")
      } else {
        # Jika item lain baru diklik saat All_Item aktif, hapus All_Item
        updateSelectInput(session, "item_select_icc", selected = setdiff(selected, "SELECT ALL"))
      }
    }
  })
  
  # === ICC Plot (reactive to itemtype) ===
  output$icc <- renderPlot({
    req(selected_fit())
    req(input$item_select_icc)
    
    all_names <- names(data_lta())         # urutan nama sesuai choices
    if ("SELECT ALL" %in% input$item_select_icc) {
      which_idx <- seq_along(all_names)
    } else {
      # ambil indeks dari nama yang dipilih (preserve urutan as in all_names)
      which_idx <- which(all_names %in% input$item_select_icc)
      # optional: jika pengguna memilih nama yang tidak ada (safety), bail out
      if (length(which_idx) == 0) return()
    }
    # plot mirt: gunakan which.items untuk menyebutkan indeks item
    mirt::plot(selected_fit(), type = "trace", which.items = which_idx,
               main = paste("ICC -", input$itemtype))
  })
  
  # === IIF Plot (reactive to itemtype, using custom function) ===
  output$infose <- renderPlot({
    req(selected_fit(), input$dimension == "uni")
    plot_info_se(selected_fit())
  })
  
  observeEvent(input$dimension, {
    output$info_ui_multi <- renderUI({
      if (input$dimension == "multi") {
        infoVisUI("info1")
      } else {
        NULL
      }
    })
  })
  
  # Jalankan modul server hanya jika dimensi = multi
  observeEvent(input$dimension, {
    req(input$dimension == "multi")
    infoVisServer("info1", model_reactive = reactive({ selected_fit() }))
  })
  
  # ==== Factor Score ====
  fscore_data <- reactive({
    req(selected_fit())
    
    tryCatch({
      as.data.frame(fscores(selected_fit(), method = "EAP", full.scores = TRUE))
    }, error = function(e) data.frame())
  })
  
  output$fscoreLTA <- renderUI({
    df <- fscore_data()
    req(nrow(df) > 0)
    
    numeric_cols <- which(sapply(df, is.numeric))
    
    tagList(
      column(
        6,
        DTOutput("fscore_table")
      ),
      
      column(
        6,
        br(),
        numericInput("n_kat", "Number of Factor Score Category:", value = 3, min = 2, max = 10),
        uiOutput("cutoff_inputs"),
        lapply(numeric_cols, function(i) {
          plotname <- paste0("fscore_donut_", i)
          div(
            style = "margin-bottom: 20px;",   # jarak antar-plot
            plotOutput(plotname, height = 280)
          )
        })
      )
    )
  })
  output$fscore_table <- DT::renderDT({
    df <- fscore_data()
    req(nrow(df) > 0)
    
    numeric_cols <- which(sapply(df, is.numeric))
    
    DT::datatable(
      df,
      extensions = "Buttons",
      options = list(
        scrollX = TRUE,
        pageLength = nrow(df),
        dom = "Brtp",
        buttons = list(
          list(
            extend = "csv",
            text = "Export CSV",
            filename = paste0("Factor Scores-", input$itemtype),
            exportOptions = list(modifier = list(page = "all"))
          ),
          list(
            extend = "excel",
            text = "Export Excel",
            filename = paste0("Factor Scores-", input$itemtype),
            exportOptions = list(modifier = list(page = "all"))
          )
        )
      ),
      rownames = TRUE
    ) %>%
      formatRound(columns = numeric_cols, digits = 2) %>%
      DT::formatStyle(
        columns = numeric_cols,
        fontWeight = "bold"      )
  }, server = FALSE)
  output$cutoff_inputs <- renderUI({
    req(input$n_kat)
    
    n_cut <- input$n_kat - 1
    
    tagList(
      lapply(1:n_cut, function(i) {
        
        div(
          style = "display: inline-block; margin-right: 10px;",
          numericInput(
            paste0("cut_", i),
            label = paste("Cut-off", i),
            value = NA,
            step = 0.01,
            width = "70px"   # small width
          )
        )
        
      })
    )
  })
  observe({
    df <- fscore_data()
    req(nrow(df) > 0)
    
    numeric_cols <- which(sapply(df, is.numeric))
    
    n_kat <- input$n_kat   # default 3 kategori
    
    lapply(seq_along(numeric_cols), function(idx) {
      local({
        col_index <- numeric_cols[idx]
        plotname  <- paste0("fscore_donut_", col_index)
        output[[plotname]] <- renderPlot({
          
          scores <- df[[col_index]]
          n_kat <- input$n_kat
          n_cut <- n_kat - 1
          
          # Ambil cut-off manual
          cutoffs <- numeric(0)
          for (i in 1:n_cut) {
            val <- input[[paste0("cut_", i)]]
            if (!is.null(val) && !is.na(val)) cutoffs <- c(cutoffs, val)
          }
          
          # Jika cut-off belum lengkap → fallback quantile
          if (length(cutoffs) != n_cut) {
            cuts <- quantile(scores, probs = seq(0, 1, length.out = n_kat + 1))
          } else {
            cuts <- c(min(scores), sort(cutoffs), max(scores))
          }
          
          # Kategorisasi
          kategori <- cut(scores, breaks = cuts, include.lowest = TRUE, labels = FALSE)
          
          # Frekuensi
          freq <- table(kategori)
          df_plot <- data.frame(
            kategori = factor(names(freq)),
            freq = as.numeric(freq)
          )
          
          df_plot$percent <- df_plot$freq / sum(df_plot$freq) * 100
          df_plot$label <- paste0(df_plot$freq, " (", round(df_plot$percent), "%)")
          
          # ===== Legend label (θ ranges) =====
          legend_labels <- c()
          for (i in 1:n_kat) {
            low  <- cuts[i]
            high <- cuts[i+1]
            
            if (i == 1) {
              legend_labels[i] <- paste0("θ ≤ ", round(high, 2))
            } else if (i == n_kat) {
              legend_labels[i] <- paste0("θ > ", round(low, 2))
            } else {
              legend_labels[i] <- paste0(round(low, 2), " < θ ≤ ", round(high, 2))
            }
          }
          
          # Warna kategori
          colors <- scales::hue_pal()(n_kat)
          
          # ====== PLOT DONUT ======
          ggplot(df_plot, aes(x = 2, y = freq, fill = kategori)) +
            
            # Donut bar
            geom_col(width = 1, color = "white") +
            
            # ===== LABEL DI LUAR DONUT =====
          geom_text(
            aes(
              x = 2.35,   # posisi label lebih keluar dari donut
              label = label
            ),
            position = position_stack(vjust = 0.5),
            size = 4,
            fontface = "bold"
          ) +
            
            coord_polar(theta = "y") +
            scale_fill_manual(
              values = colors,
              labels = legend_labels,
              name = "θ Range"
            ) +
            # ===== LABEL DI TENGAH DONUT =====
          annotate(
            "text",
            x = 0.5, 
            y = 0,
            label = paste0("F", which(numeric_cols == col_index), "-Score"),
            size = 6,
            fontface = "bold"
          ) +
            # Maintain the strcuture
            xlim(0.5, 2.5) +
            theme_void() +
            ggtitle(paste0("Factor Score : ", colnames(df)[col_index])) +
            theme(
              plot.title = element_text(hjust = 0.5),
              legend.position = "bottom"
            )
        })
        
      })
    })
  })
  
  # ====== INFORMATION & RELIABILITY TAB ======
  output$info_rel_ui <- renderUI({
    req(selected_fit())
    mod <- selected_fit()
    
    # Calculate Marginal Reliability
    marg_rel <- tryCatch({
      mirt::marginal_rxx(mod)
    }, error = function(e) NA)
    
    # Generate TIF table
    theta_seq <- matrix(seq(-4, 4, by = 0.5))
    tif_vals <- tryCatch({
      mirt::testinfo(mod, theta_seq)
    }, error = function(e) rep(NA, length(theta_seq)))
    
    tif_df <- data.frame(
      Theta = as.numeric(theta_seq),
      Information = tif_vals
    )
    
    # Render
    tagList(
      h4(icon("check-circle"), "Marginal Reliability"),
      tags$p("Marginal reliability estimate(s) for the factors in this model:"),
      verbatimTextOutput("marg_rel_out"),
      tags$hr(),
      h4(icon("table"), "Test Information Function (TIF) Table"),
      tags$p("Test information values across different levels of the latent trait (\u03B8):"),
      DTOutput("tif_table")
    )
  })
  
  output$marg_rel_out <- renderPrint({
    req(selected_fit())
    mod <- selected_fit()
    mirt::marginal_rxx(mod)
  })
  
  output$tif_table <- renderDT({
    req(selected_fit())
    mod <- selected_fit()
    theta_seq <- matrix(seq(-4, 4, by = 0.5))
    tif_vals <- mirt::testinfo(mod, theta_seq)
    
    df <- data.frame(
      Theta = as.numeric(theta_seq),
      Information = round(tif_vals, 3)
    )
    
    datatable(df, options = list(pageLength = 10, dom = 'rtip'))
  })
  
  # ====== DIF ANALYSIS TAB ======
  output$dif_ui <- renderUI({
    req(data_user())
    
    # Find categorical variables for grouping
    df <- data_user()
    cat_vars <- names(df)[sapply(df, function(x) is.factor(x) || is.character(x) || length(unique(x)) <= 5)]
    
    if (length(cat_vars) == 0) {
      return(tags$p("No suitable grouping variable found in the dataset for DIF analysis. Please ensure your dataset includes a categorical column (e.g., Gender, Region)."))
    }
    
    tagList(
      fluidRow(
        column(4,
               h4(icon("cogs"), "DIF Settings"),
               selectInput("dif_group_var", "Select Grouping Variable:", choices = cat_vars),
               selectInput("dif_anchor_items", "Anchor Items (optional):", 
                           choices = c("None", input$selected_vars), selected = "None", multiple = TRUE),
               actionButton("run_dif", "Run DIF Analysis", class = "btn-primary", icon = icon("play"))
        ),
        column(8,
               h4(icon("table"), "DIF Results (mirt)"),
               withSpinner(DTOutput("dif_results_table"))
        )
      )
    )
  })
  
  dif_results <- eventReactive(input$run_dif, {
    req(selected_fit(), input$dif_group_var, data_user())
    
    df <- data_user()
    group_var <- df[[input$dif_group_var]]
    if (length(unique(na.omit(group_var))) < 2) {
      showNotification("Grouping variable must have at least 2 categories.", type = "error")
      return(NULL)
    }
    
    # The selected_fit() model provides the variable names
    mod <- selected_fit()
    vars <- mirt::extract.mirt(mod, "itemnames")
    mod_syntax <- NULL # If custom, wait we don't have model_syntax easily here. Let's just fit standard multiple group.
    
    # We need to fit a multiple group model for DIF
    # Use mirt::multipleGroup
    item_data <- df[, vars, drop = FALSE]
    
    # Simplified approach: If we can't easily refit, we might just use Wald test or likelihood ratio.
    # To use mirt::DIF, we first fit a multipleGroup model where all items are constrained to be equal across groups.
    showNotification("Fitting multiple group model... this may take a while.", id = "dif_msg", duration = NULL)
    
    tryCatch({
      # Base constrained model
      if (is.null(mod_syntax) || mod_syntax == "") {
         mg_model <- mirt::multipleGroup(item_data, 1, group = as.character(group_var), invariance = c('free_means', 'free_var', 'slopes', 'intercepts'))
      } else {
         model_def <- mirt::mirt.model(mod_syntax)
         mg_model <- mirt::multipleGroup(item_data, model_def, group = as.character(group_var), invariance = c('free_means', 'free_var', 'slopes', 'intercepts'))
      }
      
      items2test <- vars
      if (!is.null(input$dif_anchor_items) && !("None" %in% input$dif_anchor_items)) {
        items2test <- setdiff(vars, input$dif_anchor_items)
      }
      
      dif_res <- mirt::DIF(mg_model, which.par = c('a1', 'd'), items2test = items2test, p.adjust = "fdr")
      removeNotification("dif_msg")
      return(dif_res)
    }, error = function(e) {
      removeNotification("dif_msg")
      showNotification(paste("DIF Error:", e$message), type = "error")
      return(NULL)
    })
  })
  
  output$dif_results_table <- renderDT({
    req(dif_results())
    res <- dif_results()
    
    if (is.null(res)) return(NULL)
    
    df_res <- as.data.frame(res)
    df_res$Item <- rownames(df_res)
    df_res <- df_res[, c("Item", setdiff(names(df_res), "Item"))]
    
    datatable(df_res, options = list(pageLength = 15, scrollX = TRUE)) %>%
      formatRound(columns = 2:ncol(df_res), digits = 3) %>%
      formatStyle(
        'p',
        color = styleInterval(0.05, c('red', 'black')),
        fontWeight = styleInterval(0.05, c('bold', 'normal'))
      )
  })

  # ==== R Console Output & Model Export ====
  observeEvent(selected_fit(), {
    req(selected_fit())
    out <- paste(capture.output(print(selected_fit())), collapse = "\n")
    console_context$text <- out
  })
  
  output$export_lta_rds <- downloadHandler(
    filename = function() { paste0("LTA_Workspace_", Sys.Date(), ".rds") },
    content = function(file) {
      req(selected_fit())
      workspace <- list(
        type = "projectLSA_workspace",
        module = "LTA",
        raw_data = data_user(),
        lta_fit_list = lta_fit_list(),
        lta_fit_compare = lta_fit_compare(),
        lta_current_id = lta_current_id(),
        input_state = list(
          data_source_lta = input$data_source_lta,
          datatype = input$datatype,
          dimension = input$dimension,
          fit_stats = input$fit_stats,
          selected_vars = input$selected_vars,
          itemtype = input$itemtype,
          lta_model_text = input$lta_model_text
        )
      )
      saveRDS(workspace, file)
    }
  )

  # ==== Score New Data ====
  output$download_lta_template <- downloadHandler(
    filename = function() { "LTA_template.xlsx" },
    content = function(file) {
      req(selected_fit())
      items <- mirt::extract.mirt(selected_fit(), "itemnames")
      df <- data.frame(matrix(ncol = length(items), nrow = 0))
      colnames(df) <- items
      writexl::write_xlsx(df, file)
    }
  )

  lta_newscores_reactive <- eventReactive(input$lta_score_newdata_btn, {
    req(selected_fit(), input$lta_newdata)
    ext <- tools::file_ext(input$lta_newdata$name)
    df <- switch(
      ext,
      "csv" = read.csv(input$lta_newdata$datapath),
      "xlsx" = readxl::read_excel(input$lta_newdata$datapath),
      "xls" = readxl::read_excel(input$lta_newdata$datapath),
      stop("Invalid file format")
    )
    
    # ensure only columns from model are used
    items <- mirt::extract.mirt(selected_fit(), "itemnames")
    df_used <- df[, intersect(colnames(df), items), drop = FALSE]
    
    # Calculate scores
    scores <- mirt::fscores(selected_fit(), response.pattern = df_used)
    as.data.frame(scores)
  })

  output$lta_newscores_table <- DT::renderDataTable({
    req(lta_newscores_reactive())
    datatable(round(lta_newscores_reactive(), 3), options = list(scrollX = TRUE))
  })

  output$download_lta_newscores <- downloadHandler(
    filename = function() { "LTA_New_Scores.csv" },
    content = function(file) {
      req(lta_newscores_reactive())
      write.csv(lta_newscores_reactive(), file, row.names = FALSE)
    }
  )

  # ==== AI Assistant ====
  # Update global AI context whenever results change
  observe({
    res_text <- ""
    if (length(lta_fit_list()) > 0 && !is.null(input$itemtype)) {
      fit <- try(selected_fit(), silent = TRUE)
      if (!inherits(fit, "try-error") && !is.null(fit)) {
        res_text <- paste0(
          "=== MODEL COMPARISON ===\n",
          paste(capture.output(print(lta_fit_compare())), collapse = "\n"), "\n\n",
          "=== SELECTED MODEL (", input$itemtype, ") ===\n",
          paste(capture.output(summary(fit)), collapse = "\n"), "\n\n"
        )
        itempar <- try(
          as.data.frame(mirt::coef(fit, IRTpars = TRUE, simplify = TRUE)$items),
          silent = TRUE
        )
        if (!inherits(itempar, "try-error")) {
          res_text <- paste0(
            res_text,
            "=== ITEM PARAMETERS (IRT metric) ===\n",
            paste(capture.output(print(round(itempar, 3))), collapse = "\n"), "\n\n"
          )
        }
        itemfit <- try(
          as.data.frame(mirt::itemfit(fit, fit_stats = input$fit_stats)),
          silent = TRUE
        )
        if (!inherits(itemfit, "try-error")) {
          res_text <- paste0(
            res_text,
            "=== ITEM FIT (", input$fit_stats, ") ===\n",
            paste(capture.output(print(itemfit)), collapse = "\n"), "\n"
          )
        }
      }
    }
    ai_context$results_text <- res_text
    ai_context$module <- "Latent Trait Analysis (LTA / IRT)"
  })

  # Per-session directory, so concurrent users never share a report file
  lta_report_res <- session_report_dir(session, "lta")
  lta_report_path <- reactiveVal(NULL)
  
  observeEvent(input$lta_generate_preview, {
    req(selected_fit())
    
    report_path <- file.path(system.file("app", package = "projectLSA"), "lta_report.Rmd")
    if (report_path == "" || !file.exists(report_path)) {
      report_path <- "lta_report.Rmd"
    }
    
    tempReport <- file.path(lta_report_res$path, "lta_report.Rmd")
    file.copy(report_path, tempReport, overwrite = TRUE)
    
    out_html <- file.path(lta_report_res$path, "lta_report_out.html")
    
    showModal(modalDialog("Generating Report Preview...", footer = NULL))
    tryCatch({
      rmarkdown::render(tempReport, output_file = out_html,
        params = list(
          lta_res = selected_fit(),
          console_out = console_context$text,
          ai_summary = if (is.null(ai_context$ai_report_text)) "" else ai_context$ai_report_text
        )
      )
      lta_report_path(out_html)
    }, error = function(e) {
      showNotification(paste("Error rendering report:", e$message), type = "error")
    }, finally = {
      removeModal()
    })
  })
  
  output$lta_report_preview_frame <- renderUI({
    req(lta_report_path())
    tags$iframe(src = paste0(lta_report_res$prefix, "/lta_report_out.html?v=", as.integer(Sys.time())), width = "100%", height = "800px", style = "border: none;")
  })

  output$download_report_lta <- downloadHandler(
    filename = function() {
      paste0("LTA_Report_", Sys.Date(), ".html")
    },
    content = function(file) {
      req(lta_report_path())
      file.copy(lta_report_path(), file, overwrite = TRUE)
    }
  )
}

# Fungsi untuk menu LCA
lca_ui <- function(project) {
  tabsetPanel(
    id = "main_tab_lca",
    
    # --- TAB 1: Prepare Data & Model ----
    tabPanel(
      title = tagList(icon("upload"), "Prepare Data & Model"),
      sidebarLayout(
        sidebarPanel(
          width = 3,
          actionButton("go_home", 
                       label = tagList(icon("home"), "Main Menu"), 
                       class = "btn btn-danger btn-block",
                       style = "width: 100% !important;"),
          br(),
          selectInput(
            "data_source_lca", 
            "Select Data Source:",
            choices = c("Upload Data" = "upload",
                        "Cheating (Dayton, 1998)" = "cheat",
                        "Lazarsfeld & Henry (1968)" = "lazar",
                        "General Social Survey (McCutcheon, 1987)" = "gss82"
            ),
            selected = "upload"
          ),
          conditionalPanel(
            condition = "input.data_source_lca == 'upload'",
            fileInput("datafile_lca", "Upload Data/Workspace (csv/xlsx/sav/rds)", accept = c(".csv", ".xlsx", ".sav", ".rds"))
          ),
          uiOutput("id_select_ui_lca"),
          uiOutput("var_select_ui_lca"),
          numericInput("min_class_lca", "Min. Number of Class:", 1, min = 1),
          numericInput("max_class_lca", "Max. Number of Class:", 6, min = 3),
          uiOutput("cov_lca_ui"),
          
          br(),
          
          actionButton("run_lca",
                       label = tagList(icon("play"), "Run LCA"),
                       class = "btn btn-success btn-block",
                       style = "width: 100% !important; margin-bottom: 10px;"),
          downloadButton("export_lca_rds", "Save Project (.rds)",
                         class = "btn btn-primary",
                         style = "width: 100% !important;")
        ),
        mainPanel(
          width = 9,
          h5(icon("table"), "Data Preview"),
          DTOutput("data_preview_lca"),
          DTOutput("data_summary_lca"),
          br(),
          uiOutput("data_description"),
          
          br(),
          
          
        )
      )
    ),
    
    # --- TAB 2: Fit Model Comparison ----
    tabPanel(
      title = tagList(icon("chart-line"), "Fit Model Comparison"),
      value = "fit_tab_lca",  
      
      fluidRow(
        column(12,
               div(
                 style = "text-align:center;",  
                 div(
                   style = "display:inline-block; font-size:11px; line-height:0.9; padding:0; margin:0;",
                   h5(icon("info-circle"), "Model Fit Statistics"), 
                   DTOutput("fit_table_lca")
                 )
               ),
               br(),
               tags$div(
                 style = "margin-top: 0px; font-size: 12px; color: #6c757d;",
                 class = "badge-info",
                
                 tags$b("Note:"),
                 tags$ul(
                   tags$li(tags$span(style = "color: blue;", "AIC:"), 
                           "Akaike Information Criterion; smaller values indicate better model fit."),
                   tags$li(tags$span(style = "color: blue;", "BIC:"), 
                           "Bayesian Information Criterion; smaller values indicate better and more parsimonious model fit."),
                   tags$li(tags$span(style = "color: blue;", "Entropy:"), 
                           "A measure of classification quality (values closer to 1 indicate clearer class separation)."),
                   tags$li(tags$span(style = "color: blue;", "Gsq:"), 
                           "Likelihood ratio statistic (G²) for assessing absolute model fit."),
                   tags$li(tags$span(style = "color: blue;", "p_Gsq:"), 
                           HTML("Bootstrap p-value for G². <b>H₀:</b> the model with K classes adequately fits the data. p ≥ 0.05 indicates acceptable absolute fit.")
                   ),
                   tags$li(tags$span(style = "color: blue;", "Deviance:"), 
                           "Deviance statistic comparing two nested models with K and K+1 classes."),
                   tags$li(tags$span(style = "color: blue;", "p_Deviance:"), 
                           HTML("Bootstrap p-value from the deviance test reported for the <b>K-class model</b>. 
     <b>H₀:</b> the (K−1)-class model is not significantly worse than the K-class model. 
     p < 0.05 indicates that adding one class (from K−1 to K) significantly improves model fit.")
                   ),
                   tags$li(tags$span(style = "color: blue;", "Av_Prob:"), 
                           "Average posterior classification probability by class, indicating classification accuracy.")
                 )
               )
        ),
        column(6, h5(icon("chart-bar"), "AIC & BIC Comparison"), 
               downloadButton("download_plot_AicBic_LCA", "Download Plot AIC/BIC (.png)"),
               plotOutput("fit_plot_lca")
               ),
        column(6, h5(icon("chart-bar"), "Entropy & Average Posterior Probability Comparison"), 
               downloadButton("download_plot_entropy", "Download Plot Entropy & Av.Probs (.png)"),
               plotOutput("entropy_plot_lca")
        ), br(), br(),
      )
    ),
    
    # --- TAB 3: Best Model ----
    tabPanel(
      title = tagList(icon("star"), "Best Model"),
      sidebarLayout(
        sidebarPanel(
          width = 2, 
          numericInput("best_class_lca",label =  "Select the Best Number of Class:",value = 3, min = 1),
          uiOutput("class_name_inputs_lca")
        ),
        mainPanel(
          width = 10, 
          h5(icon("project-diagram"), "Class Plot of the Best Model"),
          downloadButton("download_plot_best_LCA", "Download Plot Best Model (.png)"),
          tags$b("Item-Categories Probabilities Plot"),
          ggiraph::girafeOutput("best_model_plot_lca", width = '100%', height = 'auto'),
          h5(icon("table"), "Class Size and Item-Category Probabilities"),
          tableOutput("summary_table_lca"),
          h5(icon("table"), "Latent Class Response Patterns"),
          DT::DTOutput("data_summary_lca_with_class"),
          br(),
          br(),
          
        )
      )
    ),
    
    # --- TAB 4: Summary & Report ----
    tabPanel(
      title = tagList(icon("file-alt"), "Summary & Report"),
      h4("Class Classification Data"),
      DTOutput("profile_table_lca"),
      h4("Compare Class"),
      column(12,
        column(3, uiOutput("var_x_ui")),
        column(4, uiOutput("var_y_ui")),
        column(2,
               selectInput(
                 "plot_type", "Plot Type:",
                 choices = c("Line" = "line", "Bar" = "bar"),
                 selected = "bar"
               )
        ),
        column(3,
               checkboxInput(
                 "check_anova",
                 label = "Compare Classes",
                 value = FALSE
               )
        )
      ),
      br(),

      column(12,
        column(6, plotOutput("cross_latent", height = "420px")),
        column(6, uiOutput("anova_lca_ui"))
      ),
      column(12, uiOutput("reg_lca_ui")),
      column(12,
             h4("Explore Other Variables by Class"),
             checkboxInput(
               "crosstab_result",
               label = "Explore variable",
               value = FALSE
             ),
             column(12, uiOutput("crosstab_ui")),
             br(),
             br(),
             br()
      )
    ),

    # --- Report Preview ----
    tabPanel(
      title = tagList(icon("file-alt"), "Report Preview"),
      column(
        12,
        br(),
        div(style = "display: flex; gap: 10px; margin-bottom: 15px;",
            actionButton("lca_generate_preview", tagList(icon("sync"), " Generate Report Preview"), class = "btn btn-success"),
            downloadButton("download_report_lca", "Download HTML Report", class = "btn btn-primary")
        ),
        div(
          style = "border: 1px solid #ddd; border-radius: 4px; padding: 5px; background: #f9f9f9;",
          uiOutput("lca_report_preview_frame")
        )
      )
    ),

    # --- TAB 5: About ----
    tabPanel(
      title = tagList(icon("info-circle"), "About"),
      fluidRow(
        column(
          width = 8, offset = 2,
          br(),
          div(
            style = "text-align:center;",
            tags$hr(),
            tags$h5("projectLSA Was Developed By:"),
            tags$p(
              tags$a(
                href = "https://scholar.google.com/citations?user=PSAwkTYAAAAJ&hl=id",
                target = "_blank",
                "Dr. Hasan Djidu, M.Pd."),
              tags$br(),
              "Universitas Sembilanbelas November Kolaka"
            ),
            tags$h5("Supervised By:"),
            tags$p(
              tags$a(
                href = "https://scholar.google.com/citations?user=7CzPTYIAAAAJ&hl=id",
                target = "_blank",
                "Prof. Dr. Heri Retnawati, M.Pd."), 
              tags$br(),
              "Universitas Negeri Yogyakarta"
            ),
            
            tags$p(tags$a(
              href = "https://scholar.google.com/citations?hl=id&user=VGKeBm0AAAAJ",
              target = "_blank",
              "Prof. Dr. Samsul Hadi"), 
              tags$br(),
              "Universitas Negeri Yogyakarta"
            ),
            tags$p(tags$a(
              href = "https://scholar.google.com/citations?hl=id&user=k4MA8XgAAAAJ",
              target = "_blank",
              "Dr. Drs. Ir. Haryanto, M.Pd., M.T."), 
              tags$br(),
              "Universitas Negeri Yogyakarta"
            ),
            tags$b("Contact:"),
            tags$a("hasandjidu@gmail.com"),
            tags$hr()
          )
        ),
        column(
          width = 8, offset = 2,
          h4("References (R Packages)"),
          uiOutput("package_references_lca"),
          br(),
          div(
            style = "text-align:center;",
            tags$p(
              style = "font-size:13px; color:#777;",
             format(Sys.Date(), "%Y"), 
              "projectLSA. All rights reserved."
            )
          )
        )
      )
    )
  )
}

library(viridisLite)
# ---- Helper functions ----
# Create Theta grid for visualization
make_theta_grid <- function(nfactors, dim_x = 1, dim_y = 2, seq_x = seq(-4,4,0.25), seq_y = seq(-4,4,0.25), fixed_vals = NULL) {
  # nfactors: total factors in model
  # dim_x, dim_y: indices of the two dimensions to visualise
  # fixed_vals: named numeric vector for other dims (length nfactors - 2), default 0
  if (nfactors == 1) {
    Theta <- matrix(seq_x, ncol = 1)
    colnames(Theta) <- paste0("F", 1)
    list(Theta = Theta, grid_x = seq_x, grid_y = NULL)
  } else if (nfactors == 2) {
    g <- expand.grid(seq_x, seq_y)
    colnames(g) <- paste0("F", c(dim_x, dim_y))
    Theta <- as.matrix(g)
    colnames(Theta) <- paste0("F", 1:2)
    list(Theta = Theta, grid_x = seq_x, grid_y = seq_y)
  } else {
    # generate grid for dim_x and dim_y; fix others
    g <- expand.grid(seq_x, seq_y)
    # build full Theta matrix with nfactors cols
    Theta_full <- matrix(0, nrow = nrow(g), ncol = nfactors)
    colnames(Theta_full) <- paste0("F", 1:nfactors)
    Theta_full[, dim_x] <- g[[1]]
    Theta_full[, dim_y] <- g[[2]]
    # set fixed values
    if (!is.null(fixed_vals)) {
      for (i in seq_len(nfactors)) {
        if (i != dim_x && i != dim_y) {
          if (!is.null(fixed_vals[paste0('F', i)])) Theta_full[, i] <- fixed_vals[paste0('F', i)]
        }
      }
    }
    list(Theta = Theta_full, grid_x = seq_x, grid_y = seq_y)
  }
}

# compute testinfo and SE given a mirt model and Theta matrix
# --- Helper: compute test information and SE (auto degrees) ---
compute_info_se <- function(mod, Theta_mat) {
  # deteksi jumlah dimensi laten dari model
  n_dim <- tryCatch(extract.mirt(mod, "nfact"), error = function(e) 1)
  
  # buat vektor degrees otomatis sepanjang jumlah dimensi
  degrees <- rep(0, n_dim)
  
  # hitung test information
  tinfo <- testinfo(mod, Theta = Theta_mat, degrees = degrees)
  
  # hitung standard error (hindari pembagian 0)
  se <- 1 / sqrt(pmax(tinfo, .Machine$double.eps))
  
  list(info = tinfo, se = se)
}

# reshape vector of info into matrix matching grid_x x grid_y (row-major by grid_x)
vec_to_matrix <- function(vec, len_x, len_y) {
  # mirt returns values in the order of expand.grid(seq_x, seq_y) => first varies X then Y?
  # In R, expand.grid(seq_x, seq_y) returns first column varying slowest: seq_x repeats for each seq_y.
  # We used expand.grid(seq_x, seq_y) meaning rows correspond to combinations (x1,y1),(x2,y1)...? To be safe, reshape by length(seq_x)
  matrix(vec, nrow = len_x, ncol = len_y, byrow = FALSE)
}

# ---- Shiny module UI ----
infoVisUI <- function(id) {
  ns <- NS(id)
  tagList(
    fluidRow(
      # === 5️⃣ Info Function Multi-dim ===
      column(3,
             selectInput(ns('dim_x'), 'X dimension', choices = NULL),
             selectInput(ns('dim_y'), 'Y dimension', choices = NULL),
             numericInput(ns('grid_res'), 'Grid resolution (step)', value = 0.25, min = 0.05, step = 0.05),
             uiOutput(ns('fix_other_ui')),
             actionButton(ns('update_plot'), 'Update')
      ),
      column(9,
             tabsetPanel(
               tabPanel('Surface / Heatmap',
                        plotlyOutput(ns('surface_plot'), height = '500px'),
                        br(),
                        plotOutput(ns('heatmap_plot'), height = '300px')
               ),
               tabPanel('SE Surface',
                        plotlyOutput(ns('se_surface'), height = '500px'),
                        br(),
                        plotOutput(ns('se_heatmap'), height = '300px')
               ),
               tabPanel('Item Info',
                        selectInput(ns('which_item'), 'Which item (optional)', choices = NULL),
                        plotOutput(ns('item_info_plot'), height = '400px')
               ),
               tabPanel('Interpretation',
                        verbatimTextOutput(ns('interpretation'))
               )
             )
      )
    )
  )
}

# ---- Shiny module server ----
infoVisServer <- function(id, model_reactive) {
  moduleServer(id, function(input, output, session) {
    ns <- session$ns
    observeEvent(model_reactive(), {
      mod <- model_reactive()
      req(mod)
      nf <- mod@Data$ncol
      # number of latent dimensions
      nfactors <- mod@Model$nfact
      dims <- paste0('F', seq_len(nfactors))
      updateSelectInput(session, 'dim_x', choices = dims, selected = dims[1])
      if (nfactors >= 2) updateSelectInput(session, 'dim_y', choices = dims, selected = dims[2])
      updateSelectInput(session, 'which_item', choices = c('All testinfo', rownames(as.data.frame(coef(mod, simplify=TRUE)$items))))
    })
    
    output$fix_other_ui <- renderUI({
      mod <- model_reactive()
      req(mod)
      nfactors <- mod@Model$nfact
      dims <- paste0('F', seq_len(nfactors))
      if (nfactors <= 2) return(NULL)
      # create sliders for other dimensions
      others <- setdiff(dims, c(input$dim_x, input$dim_y))
      lapply(others, function(d) {
        sliderInput(ns(paste0('fix_', d)), label = paste('Fix', d), min = -4, max = 4, value = 0, step = 0.5)
      })
    })
    
    # gather fixed vals
    fixed_vals_reactive <- reactive({
      mod <- model_reactive()
      req(mod)
      nfactors <- mod@Model$nfact
      dims <- paste0('F', seq_len(nfactors))
      fixed <- numeric(0)
      if (nfactors > 2) {
        others <- setdiff(dims, c(input$dim_x, input$dim_y))
        for (d in others) {
          val <- input[[paste0('fix_', d)]]
          if (is.null(val)) val <- 0
          fixed[paste0(d)] <- val
        }
      }
      fixed
    })
    
    # build grid and compute info when user clicks update or model changes
    grid_info <- eventReactive(list(input$update_plot, model_reactive()), {
      mod <- model_reactive()
      req(mod)
      nfactors <- mod@Model$nfact
      seq_step <- input$grid_res
      seq_x <- seq(-4, 4, by = seq_step)
      seq_y <- seq(-4, 4, by = seq_step)
      dx <- as.integer(sub('F', '', input$dim_x))
      dy <- if (!is.null(input$dim_y)) as.integer(sub('F', '', input$dim_y)) else dx
      fixed_vals <- fixed_vals_reactive()
      g <- make_theta_grid(nfactors = nfactors, dim_x = dx, dim_y = dy, seq_x = seq_x, seq_y = seq_y, fixed_vals = fixed_vals)
      res <- compute_info_se(mod, g$Theta)
      list(grid = g, info = res$info, se = res$se, nfactors = nfactors)
    }, ignoreNULL = FALSE)
    
    output$surface_plot <- renderPlotly({
      gi <- grid_info()
      req(gi)
      g <- gi$grid
      len_x <- length(g$grid_x)
      len_y <- ifelse(is.null(g$grid_y), 1, length(g$grid_y))
      zmat <- vec_to_matrix(gi$info, len_x, len_y)
      plot_ly(x = g$grid_x, y = g$grid_y, z = zmat, type = 'surface') %>%
        layout(scene = list(xaxis = list(title = input$dim_x), yaxis = list(title = input$dim_y), zaxis = list(title = 'Information')))
    })
    
    output$heatmap_plot <- renderPlot({
      gi <- grid_info()
      req(gi)
      g <- gi$grid
      df_plot <- if (is.null(g$grid_y)) {
        tibble::tibble(X = g$grid_x, info = gi$info)
      } else {
        # reconstruct expand.grid order
        d <- expand.grid(g$grid_x, g$grid_y)
        tibble::tibble(X = d[[1]], Y = d[[2]], info = gi$info)
      }
      if ('Y' %in% names(df_plot)) {
        ggplot(df_plot, aes(X, Y, fill = info)) +
          geom_raster() +
          scale_fill_viridis_c() +
          labs(x = input$dim_x, y = input$dim_y, fill = 'Information') +
          theme_minimal()
      } else {
        ggplot(df_plot, aes(X, info)) + geom_line() + geom_point() + theme_minimal() + labs(x = input$dim_x, y = 'Information')
      }
    })
    
    output$se_surface <- renderPlotly({
      gi <- grid_info()
      req(gi)
      g <- gi$grid
      len_x <- length(g$grid_x)
      len_y <- ifelse(is.null(g$grid_y), 1, length(g$grid_y))
      zmat <- vec_to_matrix(gi$se, len_x, len_y)
      plot_ly(x = g$grid_x, y = g$grid_y, z = zmat, type = 'surface') %>%
        layout(scene = list(xaxis = list(title = input$dim_x), yaxis = list(title = input$dim_y), zaxis = list(title = 'SE')))
    })
    
    output$se_heatmap <- renderPlot({
      gi <- grid_info()
      req(gi)
      g <- gi$grid
      df_plot <- if (is.null(g$grid_y)) {
        tibble::tibble(X = g$grid_x, se = gi$se)
      } else {
        d <- expand.grid(g$grid_x, g$grid_y)
        tibble::tibble(X = d[[1]], Y = d[[2]], se = gi$se)
      }
      if ('Y' %in% names(df_plot)) {
        ggplot(df_plot, aes(X, Y, fill = se)) +
          geom_raster() +
          scale_fill_viridis_c(direction = -1) +
          labs(x = input$dim_x, y = input$dim_y, fill = 'SE') +
          theme_minimal()
      } else {
        ggplot(df_plot, aes(X, se)) + geom_line() + geom_point() + theme_minimal() + labs(x = input$dim_x, y = 'SE')
      }
    })
    
    output$item_info_plot <- renderPlot({
      mod <- model_reactive()
      req(mod)
      which_item <- input$which_item
      if (is.null(which_item) || which_item == 'All testinfo') {
        plot(mod, type = 'info')
      } else {
        it <- which(rownames(as.data.frame(coef(mod, simplify=TRUE)$items)) == which_item)
        if (length(it) == 0) {
          plot(mod, type = 'info')
        } else {
          # compute iteminfo over a reasonable grid (unidim projection)
          nfactors <- mod@Model$nfact
          if (nfactors == 1) {
            Theta <- matrix(seq(-4,4,0.1), ncol=1)
            item_info <- iteminfo(extract.item(mod, it), Theta)
            plot(seq(-4,4,0.1), item_info, type='l', xlab='Theta', ylab='Item Information')
          } else {
            # For multidim, compute item info on slice fixing others at 0
            g <- make_theta_grid(nfactors = mod@Model$nfact, dim_x = 1, dim_y = 2, 
                                 seq_x = seq(-4,4,0.25), seq_y = seq(-4,4,0.25), fixed_vals = NULL)
            # build degrees vector with correct length:
            n_dim <- mod@Model$nfact
            # If you want a single angle replicated across dims (safe default 0)
            degrees_vec <- rep(45, n_dim)
            # iteminfo expects Theta matrix matching nfactors
            ii <- iteminfo(extract.item(mod, it), g$Theta,degrees = degrees_vec)
            mat <- vec_to_matrix(ii, length(g$grid_x), length(g$grid_y))
            image(x = g$grid_x, y = g$grid_y, z = t(mat), xlab='F1', ylab='F2', main = paste('Item info -', which_item))
          }
        }
      }
    })
    output$interpretation <- renderText({
      gi <- grid_info()
      req(gi)
      g <- gi$grid
      # find peak info
      idx <- which.max(gi$info)
      if (gi$nfactors == 1) {
        peak_x <- g$grid_x[idx]
        paste0('Peak information at ', input$dim_x, ' = ', round(peak_x, 2), '. SE at that point = ', round(gi$se[idx], 3))
      } else {
        # map idx to coordinates
        dfx <- expand.grid(g$grid_x, g$grid_y)
        peak_x <- dfx[[1]][idx]
        peak_y <- dfx[[2]][idx]
        paste0('Peak test information found at ', input$dim_x, ' ≈ ', round(peak_x,2), ' and ', input$dim_y, ' ≈ ', round(peak_y,2),
               '. Standard error there ≈ ', round(gi$se[idx], 3),
               '\nInterpretation: the test is most precise for respondents with this combination of latent traits. Areas far from this point show lower information and higher SE.')
      }
    })
  })
}

# Fungsi untuk menu Siswa
lpa_ui <- function(project) {
  tabsetPanel(
    id = "main_tab_lpa",
    
    # --- TAB 1: Prepare Data & Model ----
    tabPanel(
      title = tagList(icon("upload"), "Prepare Data & Model"),
      sidebarLayout(
        sidebarPanel(
          width = 3,
          # Tombol kembali ke halaman utama
            actionButton("go_home",
                         label = tagList(icon("home"), "Main Menu"), 
                         class = "btn btn-danger btn-block",
                         style = "width: 100% !important;"),
            br(),
          selectInput(
            "data_source", 
            "Select Data Source:",
            choices = c("Upload Data" = "upload",
                        "Built-in dataset: pisaUSA15" = "pisaUSA15",
                        "Built-in dataset: curry_mac" = "curry_mac",
                        "Built-in dataset: id_edu" = "id_edu"
                        ),
            selected = "upload"
          ),
          conditionalPanel(
            condition = "input.data_source == 'upload'",
            fileInput("datafile", "Upload Data (csv/xlsx)", accept = c(".csv", ".xlsx"))
          ),
          #fileInput("datafile", "Upload Data (csv/xlsx)", accept = c(".csv", ".xlsx")),
          uiOutput("id_select_ui"),
          uiOutput("var_select_ui"),
          numericInput("min_profiles", "Min Number of Profiles:", 2, min = 1),
          numericInput("max_profiles", "Max. Number of Profiles:", 6, min = 2),
          
          selectInput(
            "model_to_run", 
            "Select Model Type:",
            choices = list(
              "Model 1: Equal variances, covariances fixed to 0" = 1,
              "Model 2: Varying variances, covariances fixed to 0" = 2,
              "Model 3: Equal variances & covariances" = 3,
              "Model 6: Varying variances & covariances" = 6
            ),
            selected = 1,
            multiple = TRUE
          ),
          actionButton("run_lpa", 
                       label = tagList(icon("play"), "Run LPA"), 
                       class = "btn btn-success btn-block",
                       style = "width: 100% !important;")
          
          
        ),
        mainPanel(
          width = 9,
          
          h5(icon("table"), "Data Preview"),
          DTOutput("data_preview"),
          br(),
          DTOutput("data_summary_lpa"),
          br(),
          uiOutput("data_description"),
          
          br(),
        )
      )
    ),
    
    # --- TAB 2: Fit Model Comparison ----
    tabPanel(
      title = tagList(icon("chart-line"), "Fit Model Comparison"),
      value = "fit_tab",  # <--- tambahkan ini juga
      
      column(12,
        div(
          style = "text-align:center;",  # pusatkan kontainer
          div(
            style = "display:inline-block; font-size:11px; line-height:0.9; padding:0; margin:0;",
            
            column(12, h5(icon("info-circle"), "Model Fit Statistics"), 
                   DTOutput("fit_table")
                   )
        )),
        br(),
        tags$div(
          style = "margin-top: 0px; font-size: 11px; color: #6c757d;",
          class = "badge-info",
          tags$b("Note:"),
          tags$ul(
            tags$li(tags$span(style="color:blue;","Model selection: "),
                    "The selected LPA model defines the variance–covariance structure across profiles and influences interpretability and classification quality."),
            tags$li(tags$span(style="color:blue;","Model 1: "),
                    "Equal variances, covariances fixed to zero (most parsimonious)."),
            tags$li(tags$span(style="color:blue;","Model 2: "),
                    "Varying variances, covariances fixed to zero (allows different variability)."),
            tags$li(tags$span(style="color:blue;","Model 3: "),
                    "Equal variances and covariances across profiles."),
            tags$li(tags$span(style="color:blue;","Model 6: "),
                    "Varying variances and covariances (most flexible, higher risk of overfitting)."),
            tags$li(tags$span(style="color:blue;","BIC / AIC: "),
                    "Lower values indicate better relative model fit."),
            tags$li(tags$span(style="color:blue;","Entropy: "),
                    "Classification accuracy (values close to 1 indicate clear profiles)."),
            tags$li(tags$span(style="color:blue;","prob_min / prob_max: "),
                    "Minimum and maximum average posterior probabilities for assigned profiles."),
            tags$li(tags$span(style="color:blue;","n_min / n_max: "),
                    "Proportion of the sample in the smallest and largest profiles."),
            tags$li(tags$span(style="color:blue;","BLRT: "),
                    "Significant results favor the k-profile model over the k−1 model.")
          )
        ),
      column(3, #h5(icon("chart-bar"), "BIC Comparison"), 
             plotOutput("fit_bic"), height = "40px"),
      column(3, #h5(icon("chart-bar"), "AIC Comparison"), 
             plotOutput("fit_aic"),height = "40px"),
      column(3, #h5(icon("chart-bar"), "Entropy Comparison"), 
             plotOutput("fit_entropy"),height = "40px"),
      column(3, #h5(icon("chart-bar"), "Min. Class Size Comparison"), 
             plotOutput("fit_class_size"),height = "40px"),
      br(),br(), br()
      )
      
    ),
    
    # --- TAB 3: Best Model ----
    tabPanel(
      title = tagList(icon("star"), "Best Model"),
      sidebarLayout(
        sidebarPanel(
          width = 3, 
          selectInput(
            "model_type", 
            "Select Model Type:",
            choices = list(
              "Model 1: Equal variances, covariances fixed to 0" = 1,
              "Model 2: Varying variances, covariances fixed to 0" = 2,
              "Model 3: Equal variances & covariances" = 3,
              "Model 6: Varying variances & covariances" = 6
            ),
            selected = 1
          ),
          numericInput("best_k", label = "Select the Best Number of Profiles:",value = 3, min = 1),
          uiOutput("profile_name_inputs")
        ),
        mainPanel(
          width = 9, 
          br(),
    
          h5(icon("project-diagram"), "Profile Plot of the Best Model"),
          downloadButton("download_plot_best_LPA", "Download Plot Best Model (.png)"),
          plotOutput("best_model_plot", height = "450px"),
          br(),
          
          div(
            style = "text-align:center;",  # pusatkan kontainer
            div(
              style = "display:inline-block; font-size:11px; line-height:0.9; padding:0; margin:0;",
              h5(icon("info-circle"), "Estimated Profile Parameters: Class Size & Variabel Means (Variances)"), 
              DTOutput("summary_table") 
              )
          )
        )
      )
    ),
    
    # --- TAB 4: Summary & Report ----
    tabPanel(
      title = tagList(icon("file-alt"), "Summary & Report"),
      h4("Profile Classification Data"),
      DTOutput("profile_table"),
      br(),
      column(12,
        h4("Compare Profiles"),
        column(3, uiOutput("var_x_ui")),
        column(3, uiOutput("var_y_ui")),
        column(3,
               selectInput(
                 "plot_type", "Plot Type:",
                 choices = c("Line" = "line", "Bar" = "bar"),
                 selected = "bar"
               )
        ),
        column(3,
               checkboxInput(
                 "check_anova",
                 label = "Compare Profiles",
                 value = FALSE
               )
        )
      ),
      br(),
      # =========================
      # ROW 2: MAIN OUTPUT
      # =========================
      column(12,
        column(6, plotOutput("cross_latent", height = "420px")),
        column(6, uiOutput("anova_lpa_ui")),
        br()
      ),
      column(12,
             h4("Explore Other Variables by Profile"),
             checkboxInput(
               "crosstab_result",
               label = "Explore variable",
               value = FALSE
             ),
             column(12, uiOutput("crosstab_ui")),
             br(),
             br(),
             br()
      )
    ),
    tabPanel(
      title = tagList(icon("info-circle"), "About"),
      fluidRow(
        column(
          width = 8, offset = 2,
          br(),
          div(
            style = "text-align:center;",
            tags$hr(),
            tags$h5("projectLSA Was Developed By:"),
            tags$p(
              tags$a(
                href = "https://scholar.google.com/citations?user=PSAwkTYAAAAJ&hl=id",
                target = "_blank",
                "Dr. Hasan Djidu, M.Pd."),
              tags$br(),
              "Universitas Sembilanbelas November Kolaka"
            ),
            tags$h5("Supervised By:"),
            tags$p(
              tags$a(
                href = "https://scholar.google.com/citations?user=7CzPTYIAAAAJ&hl=id",
                target = "_blank",
                "Prof. Dr. Heri Retnawati, M.Pd."), 
              tags$br(),
              "Universitas Negeri Yogyakarta"
            ),
            
            tags$p(tags$a(
              href = "https://scholar.google.com/citations?hl=id&user=VGKeBm0AAAAJ",
              target = "_blank",
              "Prof. Dr. Samsul Hadi"), 
              tags$br(),
              "Universitas Negeri Yogyakarta"
            ),
            tags$p(tags$a(
              href = "https://scholar.google.com/citations?hl=id&user=k4MA8XgAAAAJ",
              target = "_blank",
              "Dr. Drs. Ir. Haryanto, M.Pd., M.T."), 
              tags$br(),
              "Universitas Negeri Yogyakarta"
            ),
            tags$b("Contact:"),
            tags$a("hasandjidu@gmail.com"),
            tags$hr()
          )
        ),
        column(
          width = 8, offset = 2,
          h4("References (R Packages)"),
          uiOutput("package_references_lpa"),
          br(),
          div(
            style = "text-align:center;",
            tags$p(
              style = "font-size:13px; color:#777;",
              format(Sys.Date(), "%Y"), 
              "projectLSA. All rights reserved."
            ) ))
      )
    )
  )
}

# LTA -----
# ==== Reactive Data LTA ====
server_lta <- function(input, output, session) {
  library(mirt)
  library(plotly)
  library(psych)
  data_user <- reactive({
    req(input$dimension, input$data_source_lta)
    set.seed(321)
    if (input$dimension == "uni" && input$data_source_lta == "diko") {
      a <- matrix(runif(15, 0.8, 2), ncol = 1)      # discrimination 1 dimension
      d <- matrix(rnorm(15, 0, 1), ncol = 1)       # difficulty
      theta <- matrix(rnorm(750), ncol = 1)       # ability 1 dimension
      resp <- simdata(a = a, d = d, itemtype = "2PL", Theta = theta)
      df <- as.data.frame(resp) %>% tidyr::drop_na()
      
    } else if (input$dimension == "uni" && input$data_source_lta == "poli") {
      a <- matrix(rlnorm(20, .2, .3))  
      diffs <- t(apply(matrix(runif(20*4, .3, 1), 20), 1, cumsum))
      diffs <- -(diffs - rowMeans(diffs))
      d <- diffs + rnorm(20)
      resp <- simdata(a, d, 300, itemtype = 'graded')
      df <- as.data.frame(resp) %>% tidyr::drop_na()
      
    } else if (input$dimension == "multi" && input$data_source_lta == "diko") {
      N <- 750     
      n_items <- 15
      n_factors <- 2
      a <- matrix(0, nrow = n_items, ncol = n_factors)
      a[1:8,1] <- runif(8, 0.5, 1.5)   # loading ke F1
      a[9:15,2] <- runif(7, 0.5, 1.5)  # loading ke F2
      d <- matrix(rnorm(n_items, 0, 1), ncol = 1)
      sigma <- matrix(c(1,0.3,0.3,1), nrow = 2)
      resp <- simdata(a = a, d = d, N = N, itemtype = '2PL', sigma = sigma)
      df <- as.data.frame(resp) %>% tidyr::drop_na()
      
    } else if (input$dimension == "multi" && input$data_source_lta == "poli") {
      df <- as.data.frame(psych::bfi) %>% dplyr::select(A1:E2) %>% tidyr::drop_na()
    } else {
      req(input$datafile_lta)
      ext <- tolower(tools::file_ext(input$datafile_lta$name))
      showModal(modalDialog(title = NULL, "Reading Your File, Please wait...", footer = NULL, easyClose = FALSE))
      df <- switch(
        ext,
        "csv"  = data.table::fread(input$datafile_lta$datapath,data.table = FALSE),
        "xls"  = readxl::read_excel(input$datafile_lta$datapath),
        "xlsx" = readxl::read_excel(input$datafile_lta$datapath),
        "sav"  = haven::read_sav(input$datafile_lta$datapath),
        "rds"  = readRDS(input$datafile_lta$datapath),
        stop("Unsupported file type. Please upload CSV, Excel, SPSS (.sav), or RDS file.")
      )
      removeModal()
      df <- df %>% mutate(across(everything(), ~ifelse(.x == "", NA, .x)),
                          id_auto = paste0("id_", sprintf("%04d", 1:n())))
    }
    return(df)
  })
  output$id_select_ui <- renderUI({
    req(data_user(), input$dimension, input$data_source_lta)
    selectizeInput("id_cols", "ID columns (optional)", choices = names(data_user()), multiple = TRUE,
                   selected = names(data_user())[grepl("id", names(data_user()), ignore.case = TRUE)])
  })
  output$var_select_ui <- renderUI({
    req(data_user(), input$dimension, input$data_source_lta)
    all_vars <- names(data_user())
    id_cols <- input$id_cols
    choices <- setdiff(all_vars, id_cols)
    selectInput("selected_vars", "Select Variables (used for LTA):", choices = choices, multiple = TRUE, selected = choices)
  })
  
  data_lta <- reactive({
    req(input$dimension, input$data_source_lta, data_user(), input$selected_vars, input$fit_stats)
    data_user() %>% dplyr::select(input$selected_vars)
  })
  
  # ==== Preview Data ====
  output$data_preview_lta <- DT::renderDT({
    req(input$dimension, input$data_source_lta, data_user(), input$selected_vars, input$fit_stats,data_lta())
    df <- data_lta() 
    numeric_cols <- which(sapply(df, is.numeric))
    DT::datatable(df, extensions = 'Buttons',
                  options = list(dom='Brtp',scrollX = TRUE, pageLength = 25,  
                                 buttons = list(
                                   list(extend = 'csv',
                                        text = 'Export CSV',
                                        filename = 'Data LTA'
                                   ),
                                   list(extend = 'excel',
                                        text = 'Export Excel',
                                        filename = 'Data LTA'
                                   ))),
                  rownames = TRUE) %>% 
      formatRound(columns = numeric_cols, digits = 0)
  }, server = FALSE)
  output$itemtype_ui <- renderUI({
    req(input$datatype, input$data_source_lta)
    source_type <- if(input$data_source_lta == "upload") {
      input$datatype
    } else {
      input$data_source_lta
    }
    choices <- switch(source_type,
                      "diko" = c("Rasch" = "Rasch",
                                 "2-Parameter Logistic (2-PL)" = "2PL",
                                 "3-Parameter Logistic (3-PL)" = "3PL"),
                      "poli" = c("Partial Credit Model (PCM)" = "Rasch",
                                 "Graded Response Model (GRM)" = "graded",
                                 "Generalized PCM (GPCM)" = "gpcm")
    )
    
    div(
      class = "select-large", # styling selector
      selectInput(
        inputId = "itemtype",
        label = NULL,
        choices = choices,
        selected = choices[[1]],
        width = "100%"
      )
    )
  })
  
  # ==== Reset all stored LTA results when data_user changes ====
  observeEvent(data_user(), {
    # Reset stored models and current selection
    lta_fit_list(list())
    lta_fit_compare(NULL)
    lta_current_id(NULL)
  })
  # ===== LTA / MIRT MODEL =====
  output$lta_model_ui <- renderUI({
    req(input$data_source_lta, input$dimension=="multi")
    model_text <- switch(input$data_source_lta,
                         # === 1. Multidimensional - Dichotomous ===
                         "diko" = "
# Example 1: Multidimensional - Dichotomous (2 dimensions)
model <- '
F1 = 1-5
F2 = 6-10
'
",
# === 2. Multidimensional - Polytomous ===
"poli" = "
# Example 2: Multidimensional - Polytomous (2 dimensions)
model <- '
F1 = 1-5
F2 = 6-10 
'
"
    )
    tags$textarea(
      id = "lta_model_text",
      rows = 10,
      style = "width:100%; font-family: monospace; font-size:11px;",
      model_text
    )
  })
  
  # ==== LTA storage ====
  lta_fit_list <- reactiveVal(list())
  lta_fit_compare <- reactiveVal(NULL)
  lta_current_id <- reactiveVal(NULL)
  
  # ==== Fungsi untuk menjalankan model ====
  run_mirt_model <- function(data, model_def, itemtype) {
    tryCatch({
      mirt(data, model = model_def, itemtype = itemtype, verbose = FALSE)
    }, error = function(e) {
      showNotification(paste("Model", itemtype, "error:", e$message), type = "error")
      return(NULL)
    })
  }
  
  # ==== LTA Main Run ====
  observeEvent(input$run_lta, {
    req(data_lta(), input$selected_vars, input$fit_stats)
    # Tambahkan kondisi khusus untuk upload
    if (input$data_source_lta == "upload") {
      req(input$datatype)
    }
    updateTabsetPanel(session, "main_tab_lta", selected = "summary_tab")
    df <- data_lta()
    # Model
    source_type <- if(input$data_source_lta == "upload") {
      req(input$datatype)   # pastikan user sudah pilih
      input$datatype
    } else {
      input$data_source_lta
    }
    
    itemtypes <- switch(source_type,
                        "diko" = c("Rasch","2PL","3PL"),
                        "poli" = c("Rasch","graded","gpcm"))
    # Define Model Syntax
    model_def <- if (input$dimension == "multi") {
      mirt.model(input$lta_model_text,itemnames = df)
    }  else {1}
    showModal(modalDialog("Running Latent Trait Analysis ...", footer = NULL))
    
    fit_results <- list()
    for (item in itemtypes) {
      mod <- run_mirt_model(df, model_def, item)
      if (is.null(mod)) next
      measures <- tryCatch({
        as.data.frame(anova(mod))
      }, error = function(e) data.frame())
      
      N_itemfit <- tryCatch({
        itemfit_df <- as.data.frame(mirt::itemfit(mod, fit_stats=input$fit_stats))
        p_cols <- grep("^p\\.", names(itemfit_df), value = TRUE)
        sum(itemfit_df[[p_cols]] >= 0.05, na.rm = TRUE)
      }, error = function(e) 0)
      
      LD <- tryCatch({
        Q3_mat <- residuals(mod, type = "Q3", verbose = FALSE)
        # ambil item names
        items <- colnames(Q3_mat)
        # ekstrak upper triangle
        idx <- which(upper.tri(Q3_mat), arr.ind = TRUE)
        q3_long <- data.frame(
          item  = items[idx[,2]],
          item2 = items[idx[,1]],
          Q3    = Q3_mat[upper.tri(Q3_mat)]
        )
        violations <- q3_long %>% 
          dplyr::filter(abs(Q3) > 0.2)
        unique_items <- unique(c(violations$item, violations$item2))
        N_LD <- length(unique_items)
        list(N_LocalDependency = N_LD,
             item_LocalDep = unique_items
        )
      }, error = function(e) {
        list(N_violations = NULL, itemLD = NA)}
      )
      fit_results[[item]] <- list(
        id = item,
        model_type = item,
        fit = mod,
        measures = measures,
        N_itemfit = N_itemfit,
        LD = LD,
        time = Sys.time()
      )
    }
    # ---- Model comparison (if possible) ----
    models_for_anova <- lapply(fit_results, function(x) x$fit)
    models_for_anova <- models_for_anova[!sapply(models_for_anova, is.null)]
    lta_fit_compare(
      switch(source_type,
             "diko" = as.data.frame(anova(models_for_anova[['Rasch']],
                                          models_for_anova[['2PL']],
                                          models_for_anova[['3PL']])),
             "poli" = rbind(
               as.data.frame(anova(models_for_anova[['Rasch']],
                                   models_for_anova[['graded']]
               )),
               as.data.frame(anova(models_for_anova[['Rasch']],
                                   models_for_anova[['gpcm']]))[2,]
             )
      ))
    removeModal()
    
    cur <- lta_fit_list()
    cur <- modifyList(cur, fit_results)
    best_name <- input$itemtype
    
    all_model_types <- sapply(cur, function(x) x$model_type)
    match_idx <- which(tolower(all_model_types) == tolower(best_name))
    
    if (length(match_idx) == 0) {
      best_id <- names(cur)[1]
    } else {
      best_id <- names(cur)[match_idx[1]]
    }
    lta_fit_list(cur)
    lta_current_id(best_id)
    
    showNotification(paste0("LTA finished. Selected model: ", best_name), type = "message")
    
  })
  
  output$fit_comparison <- renderUI({
    req(data_lta(), input$selected_vars, input$fit_stats, lta_fit_list())
    source_type <- if(input$data_source_lta == "upload") {
      input$datatype
    } else {
      input$data_source_lta
    }
    all_measures <- do.call(rbind, lapply(lta_fit_list(), function(x) x$measures)) %>% 
      tibble::rownames_to_column("Model") %>%
      dplyr::mutate(
        Model = dplyr::case_when(
          source_type == 'poli' & Model == 'Rasch'   ~ 'PCM',
          source_type == 'poli' & Model == 'graded'  ~ 'GRM',
          source_type == 'poli' & Model == 'gpcm'    ~ 'GPCM',
          TRUE                                    ~ Model
        )
      ) %>% 
      dplyr::select(Model)
    
    N_itemfit <- do.call(rbind, lapply(lta_fit_list(), function(x) x$N_itemfit))
    N_itemLD <- do.call(rbind, lapply(lta_fit_list(), function(x) x$LD$N_LocalDependency))
    item_LocalDep <- do.call(rbind, lapply(lta_fit_list(), function(x) x$LD$item_LocalDep))
    comparison <- cbind(all_measures,lta_fit_compare(),
                        N_itemfit, N_itemLD) %>% dplyr::rename(`p (χ²)`=p, 'χ²'=X2)
    comparison <- comparison %>% 
      dplyr::mutate(
        AIC   = round(AIC, 2),
        SABIC = round(SABIC, 2),
        BIC   = round(BIC, 2),
        HQ    = round(HQ, 2),
        logLik = round(logLik,2),
        `χ²` = round(`χ²`,2),
        df=df,
        `p (χ²)` = case_when(
          is.na(p (`p (χ²)`)) ~ NA_character_,
          `p (χ²)` < 0.01 ~ "p <.01",
          `p (χ²)` < 0.05  ~ "p <.05",
          TRUE      ~ "p >.05"
        ),
        N_itemfit =  N_itemfit,
        N_itemLD = N_itemLD     
      )
    min_AIC   <- min(comparison$AIC, na.rm = TRUE)
    min_SABIC <- min(comparison$SABIC, na.rm = TRUE)
    min_BIC   <- min(comparison$BIC, na.rm = TRUE)
    min_HQ    <- min(comparison$HQ, na.rm = TRUE)
    max_Nfit  <- max(comparison$N_itemfit , na.rm = TRUE)
    minLD  <- min(comparison$N_itemLD, na.rm = TRUE)
    column(12,
           DT::datatable(comparison,extensions = 'Buttons',
                         options = list(scrollX = TRUE, pageLength = 10,
                                        dom = 'B',
                                        buttons = list(
                                          list(
                                            extend = 'csv',
                                            text = 'Export CSV',
                                            filename = 'Fit Comparison'  # <-- nama file CSV
                                          ),
                                          list(
                                            extend = 'excel',
                                            text = 'Export Excel',
                                            filename = 'Fit Comparison'  # <-- nama file Excel
                                          ))),
                         rownames = FALSE) %>%
             # === Highlight ===
             formatStyle(
               columns = 'AIC',
               color = styleEqual(min_AIC, 'black'),
               backgroundColor = styleEqual(min_AIC, 'lightgreen'),
               fontWeight = styleEqual(min_AIC, 'bold')
             ) %>%
             formatStyle(
               columns = 'BIC',
               color = styleEqual(min_BIC, 'black'),
               backgroundColor = styleEqual(min_BIC, 'lightgreen'),
               fontWeight = styleEqual(min_BIC, 'bold')
             ) %>%
             formatStyle(
               columns = 'SABIC',
               color = styleEqual(min_SABIC, 'black'),
               backgroundColor = styleEqual(min_SABIC, 'lightgreen'),
               fontWeight = styleEqual(min_SABIC, 'bold')
             ) %>%
             formatStyle(
               columns = 'HQ',
               color = styleEqual(min_HQ, 'black'),
               backgroundColor = styleEqual(min_HQ, 'lightgreen'),
               fontWeight = styleEqual(min_HQ, 'bold')
             ) %>%
             # === Highlight untuk kolom N_itemfit (nilai terbesar) ===
             formatStyle(
               columns = 'N_itemfit',
               color = styleEqual(max_Nfit, 'black'),
               backgroundColor = styleEqual(max_Nfit, 'lightgreen'),
               fontWeight = styleEqual(max_Nfit, 'bold')
             )  %>% 
             formatStyle(
               columns = 'N_itemLD',
               color = styleEqual(minLD, 'black'),
               backgroundColor = styleEqual(minLD, 'lightgreen'),
               fontWeight = styleEqual(minLD, 'bold')
             ) %>% 
             formatStyle(
               columns = 'p (χ²)',
               backgroundColor = styleEqual(
                 c('p <.01', 'p <.05'),
                 c('lightgreen', 'lightgreen')
               )
             ),
           tags$div(
             style = "margin-top: 0px; font-size: 12px; color: #6c757d;",
             tags$b("Interpretation Guidelines:"),
             tags$ul(
               tags$li(tags$span(style = "color: green;", "Best Model:"),"Smaller AIC/BIC/SABIC/HQ/N_ItemLD; Higher N_itemfit"),
               tags$li(tags$span(style = "color: red;", "ItemLD (Violation of Local Independency Assumption):"),  "|Q3| > 0.2 (Yen, 1984)"),
               tags$li(tags$span(style = "color: green;", "p.value <.05 / p.value <.01:"), 
                       "The more complex model fits better than the simpler model (Desjardin & Bulut, 2018)")
               
             ),
             tags$hr()
           )
    )
  })
  # === Helper: Get the currently selected model based on input$itemtype ===
  selected_fit <- reactive({
    req(lta_fit_list(), input$itemtype)
    all_models <- lta_fit_list()
    match_idx <- which(
      tolower(sapply(all_models, function(x) x$model_type)) == tolower(input$itemtype)
    )
    if (length(match_idx) == 0) {
      showNotification(paste0("Model '", input$itemtype, "' not found."), type = "warning")
      return(NULL)
    }
    all_models[[match_idx[1]]]$fit
  })
  
  # === Item Summary (reactive to selected model) ===
  output$item_summary <- renderUI({
    req(selected_fit())
    itemparam <- as.data.frame(mirt::coef(selected_fit(), IRTpars = TRUE, simplify = TRUE)$items)
    itemfit <- as.data.frame(mirt::itemfit(selected_fit(), fit_stats = input$fit_stats)) %>% 
      dplyr::select (paste0(input$fit_stats), paste0('p.',input$fit_stats)) %>% 
      dplyr::rename(`χ²_itemfit` = paste0(input$fit_stats), `p(χ²)_itemfit` = paste0('p.',input$fit_stats))
    
    item_summary <- cbind(itemparam,itemfit) %>% tibble::rownames_to_column("item")
    numeric_cols <- which(sapply(item_summary, is.numeric))
    
    DT::datatable(item_summary,extensions = 'Buttons',
                  options = list(scrollX = TRUE, dom = 'Brtp',
                                 buttons = list(
                                   list(
                                     extend = 'csv',
                                     text = 'Export CSV',
                                     filename = paste0('Item Summary-',input$itemtype)  
                                   ),
                                   list(
                                     extend = 'excel',
                                     text = 'Export Excel',
                                     filename = paste0('Item Summary-',input$itemtype)
                                   ))), 
                  rownames = FALSE) %>%
      formatRound(columns = numeric_cols, digits = 2) %>% 
      DT::formatStyle(columns = "p(χ²)_itemfit",
                      fontWeight = 'bold',
                      color = styleInterval(0.04999999, c('red', 'green'))
      )
  })
  
  
  output$icc_info <- renderUI({
    req(input$dimension, selected_fit() )
    nfactors <- selected_fit()@Model$nfact
    if (input$dimension =='uni')
    {
      column(12,
             column(6,
                    plotOutput('icc'),
                    uiOutput('select_icc'),    
             ),
             column(6, plotOutput('infose')))
    } else if (input$dimension =='multi' && nfactors == 2 ) {
      column(12,
             plotOutput('icc'),
             uiOutput('select_icc'),  
             uiOutput("info_ui_multi")
      )
    }
    else if (input$dimension =='multi' && nfactors >= 3 )
    {
      column(12,
             uiOutput("info_ui_multi")
      )
    }
    else{return(NULL)}
  })
  
  output$select_icc <- renderUI({
    req(data_lta())
    choices <- c('SELECT ALL',names(data_lta()))
    div(class = "select-mini", 
        selectInput("item_select_icc", NULL, choices = choices, multiple = TRUE, selected = 'SELECT ALL', width = '100%'))
  }) 
  observeEvent(input$item_select_icc, {
    req(input$item_select_icc)
    selected <- input$item_select_icc
    # Jika "All_Item" dipilih bersama dengan yang lain, sisakan hanya "All_Item"
    if ("SELECT ALL" %in% selected && length(selected) > 1) {
      # Jika "All_Item" baru saja diklik, hapus item lain
      if (tail(selected, 1) == "SELECT ALL") {
        updateSelectInput(session, "item_select_icc", selected = "SELECT ALL")
      } else {
        # Jika item lain baru diklik saat All_Item aktif, hapus All_Item
        updateSelectInput(session, "item_select_icc", selected = setdiff(selected, "SELECT ALL"))
      }
    }
  })
  
  # === ICC Plot (reactive to itemtype) ===
  output$icc <- renderPlot({
    req(selected_fit())
    req(input$item_select_icc)
    
    all_names <- names(data_lta())         # urutan nama sesuai choices
    if ("SELECT ALL" %in% input$item_select_icc) {
      which_idx <- seq_along(all_names)
    } else {
      # ambil indeks dari nama yang dipilih (preserve urutan as in all_names)
      which_idx <- which(all_names %in% input$item_select_icc)
      # optional: jika pengguna memilih nama yang tidak ada (safety), bail out
      if (length(which_idx) == 0) return()
    }
    # plot mirt: gunakan which.items untuk menyebutkan indeks item
    mirt::plot(selected_fit(), type = "trace", which.items = which_idx,
               main = paste("ICC -", input$itemtype))
  })
  
  # === IIF Plot (reactive to itemtype, using custom function) ===
  output$infose <- renderPlot({
    req(selected_fit(), input$dimension == "uni")
    plot_info_se(selected_fit())
  })
  
  observeEvent(input$dimension, {
    output$info_ui_multi <- renderUI({
      if (input$dimension == "multi") {
        infoVisUI("info1")
      } else {
        NULL
      }
    })
  })
  
  # Jalankan modul server hanya jika dimensi = multi
  observeEvent(input$dimension, {
    req(input$dimension == "multi")
    infoVisServer("info1", model_reactive = reactive({ selected_fit() }))
  })
  
  # ==== Factor Score ====
  fscore_data <- reactive({
    req(selected_fit())
    
    tryCatch({
      as.data.frame(fscores(selected_fit(), method = "EAP", full.scores = TRUE))
    }, error = function(e) data.frame())
  })
  
  output$fscoreLTA <- renderUI({
    df <- fscore_data()
    req(nrow(df) > 0)
    
    numeric_cols <- which(sapply(df, is.numeric))
    
    tagList(
      column(
        6,
        DTOutput("fscore_table")
      ),
      
      column(
        6,
        br(),
        numericInput("n_kat", "Number of Factor Score Category:", value = 3, min = 2, max = 10),
        uiOutput("cutoff_inputs"),
        lapply(numeric_cols, function(i) {
          plotname <- paste0("fscore_donut_", i)
          div(
            style = "margin-bottom: 20px;",   # jarak antar-plot
            plotOutput(plotname, height = 280)
          )
        })
      )
    )
  })
  output$fscore_table <- DT::renderDT({
    df <- fscore_data()
    req(nrow(df) > 0)
    
    numeric_cols <- which(sapply(df, is.numeric))
    
    DT::datatable(
      df,
      extensions = "Buttons",
      options = list(
        scrollX = TRUE,
        pageLength = nrow(df),
        dom = "Brtp",
        buttons = list(
          list(
            extend = "csv",
            text = "Export CSV",
            filename = paste0("Factor Scores-", input$itemtype),
            exportOptions = list(modifier = list(page = "all"))
          ),
          list(
            extend = "excel",
            text = "Export Excel",
            filename = paste0("Factor Scores-", input$itemtype),
            exportOptions = list(modifier = list(page = "all"))
          )
        )
      ),
      rownames = TRUE
    ) %>%
      formatRound(columns = numeric_cols, digits = 2) %>%
      DT::formatStyle(
        columns = numeric_cols,
        fontWeight = "bold"      )
  }, server = FALSE)
  output$cutoff_inputs <- renderUI({
    req(input$n_kat)
    
    n_cut <- input$n_kat - 1
    
    tagList(
      lapply(1:n_cut, function(i) {
        
        div(
          style = "display: inline-block; margin-right: 10px;",
          numericInput(
            paste0("cut_", i),
            label = paste("Cut-off", i),
            value = NA,
            step = 0.01,
            width = "70px"   # small width
          )
        )
        
      })
    )
  })
  observe({
    df <- fscore_data()
    req(nrow(df) > 0)
    
    numeric_cols <- which(sapply(df, is.numeric))
    
    n_kat <- input$n_kat   # default 3 kategori
    
    lapply(seq_along(numeric_cols), function(idx) {
      local({
        col_index <- numeric_cols[idx]
        plotname  <- paste0("fscore_donut_", col_index)
        output[[plotname]] <- renderPlot({
          
          scores <- df[[col_index]]
          n_kat <- input$n_kat
          n_cut <- n_kat - 1
          
          # Ambil cut-off manual
          cutoffs <- numeric(0)
          for (i in 1:n_cut) {
            val <- input[[paste0("cut_", i)]]
            if (!is.null(val) && !is.na(val)) cutoffs <- c(cutoffs, val)
          }
          
          # Jika cut-off belum lengkap → fallback quantile
          if (length(cutoffs) != n_cut) {
            cuts <- quantile(scores, probs = seq(0, 1, length.out = n_kat + 1))
          } else {
            cuts <- c(min(scores), sort(cutoffs), max(scores))
          }
          
          # Kategorisasi
          kategori <- cut(scores, breaks = cuts, include.lowest = TRUE, labels = FALSE)
          
          # Frekuensi
          freq <- table(kategori)
          df_plot <- data.frame(
            kategori = factor(names(freq)),
            freq = as.numeric(freq)
          )
          
          df_plot$percent <- df_plot$freq / sum(df_plot$freq) * 100
          df_plot$label <- paste0(df_plot$freq, " (", round(df_plot$percent), "%)")
          
          # ===== Legend label (θ ranges) =====
          legend_labels <- c()
          for (i in 1:n_kat) {
            low  <- cuts[i]
            high <- cuts[i+1]
            
            if (i == 1) {
              legend_labels[i] <- paste0("θ ≤ ", round(high, 2))
            } else if (i == n_kat) {
              legend_labels[i] <- paste0("θ > ", round(low, 2))
            } else {
              legend_labels[i] <- paste0(round(low, 2), " < θ ≤ ", round(high, 2))
            }
          }
          
          # Warna kategori
          colors <- scales::hue_pal()(n_kat)
          
          # ====== PLOT DONUT ======
          ggplot(df_plot, aes(x = 2, y = freq, fill = kategori)) +
            
            # Donut bar
            geom_col(width = 1, color = "white") +
            
            # ===== LABEL DI LUAR DONUT =====
          geom_text(
            aes(
              x = 2.35,   # posisi label lebih keluar dari donut
              label = label
            ),
            position = position_stack(vjust = 0.5),
            size = 4,
            fontface = "bold"
          ) +
            
            coord_polar(theta = "y") +
            scale_fill_manual(
              values = colors,
              labels = legend_labels,
              name = "θ Range"
            ) +
            # ===== LABEL DI TENGAH DONUT =====
          annotate(
            "text",
            x = 0.5, 
            y = 0,
            label = paste0("F", which(numeric_cols == col_index), "-Score"),
            size = 6,
            fontface = "bold"
          ) +
            # Maintain the strcuture
            xlim(0.5, 2.5) +
            theme_void() +
            ggtitle(paste0("Factor Score : ", colnames(df)[col_index])) +
            theme(
              plot.title = element_text(hjust = 0.5),
              legend.position = "bottom"
            )
        })
        
      })
    })
  })
  
  
}

# ==== report_utils.R ====
# Per-session scratch space for generated HTML reports.
#
# tempdir() is shared by every Shiny session served by the same R process. When
# the application is hosted (rather than launched locally by a single user via
# run_projectLSA()), writing reports to a fixed path inside tempdir() means two
# concurrent users overwrite each other's file, and one user's report - which
# embeds their data, model output and any AI summary - can be served into
# another user's browser. addResourcePath() on the raw tempdir() also exposes
# every file in it over HTTP under a guessable name.
#
# Each session therefore gets its own subdirectory, published under its own
# resource prefix, and both are removed when the session ends.
#
# Returns a list with:
#   prefix - the resource prefix to use in a URL (e.g. "cfa_reports_<token>")
#   path   - the filesystem directory to render into
session_report_dir <- function(session, module) {
  token <- session$token
  if (is.null(token) || !nzchar(token)) {
    # testServer() and some non-browser sessions have no token; fall back to a
    # value that is still unique per session object.
    token <- sprintf("%s", format(as.numeric(Sys.time()) * 1000, scientific = FALSE))
    token <- gsub("[^0-9]", "", token)
  }

  prefix <- paste0(module, "_reports_", token)
  path <- file.path(tempdir(), prefix)

  if (!dir.exists(path)) {
    dir.create(path, showWarnings = FALSE, recursive = TRUE)
    addResourcePath(prefix, path)
    session$onSessionEnded(function() {
      try(removeResourcePath(prefix), silent = TRUE)
      unlink(path, recursive = TRUE)
    })
  }

  list(prefix = prefix, path = path)
}

# Fungsi untuk menu Siswa
lta_ui <- function(project) {
    tabsetPanel(
      id = "main_tab_lta",
      
      # --- TAB 1: Prepare Data & Model ----
      tabPanel(
        title = tagList(icon("upload"), "Prepare Data & Model"),
        sidebarLayout(
          sidebarPanel(
            width = 3,
            # Tombol kembali ke halaman utama
            actionButton("go_home", 
                         label = tagList(icon("home"), "Main Menu"), 
                         class = "btn btn-danger btn-block",
                         style = "width: 100% !important;"),
            br(),
            #uiOutput("data_source_lta_ui"),
            selectInput("data_source_lta", 
              "Select Data Source:",choices = c(
                "UPLOAD DATA" = "upload",
                "Built in Data: Dichotomous" = "diko",
                "Built in Data: Polytomous" = "poli"),
              selected = "upload"
            ),
            conditionalPanel(
              condition = "input.data_source_lta == 'upload'",
              fileInput("datafile_lta", "Upload Data (csv/xlsx)", accept = c(".csv", ".xlsx")),
              selectInput("datatype", 
                          "Select Data Type:",
                          choices = c("Dicotomous" = "diko",
                                      "Polytomous" = "poli"),
                          selected = 'poli')
            ),
            selectInput("dimension",
                        "Dimension:",
                        choices = c("Unidimensional" = "uni",
                                    "Multidimensional" = "multi"),
                        selected = "uni"),
            uiOutput("id_select_ui"),
            uiOutput("var_select_ui"),
            conditionalPanel(
              condition = "input.dimension == 'multi'",
              tags$label("Define Dimension Name & Item:"),
              uiOutput("lta_model_ui")
            ),
            selectInput("fit_stats", 
                        "Itemfit Stat:",
                        choices = c("S_X2 (complete cases only)" = "S_X2",
                                    "X2" = "X2",
                                    "X2*" = "X2*"),
                        selected = "S_X2"),
            
            actionButton("run_lta",
                         label = tagList(icon("play"), "Run LTA"), 
                         class = "btn btn-success btn-block",
                         style = "width: 100% !important;")

          ),
          mainPanel(
            width = 9,
            h5(icon("table"), "Data Preview"),
            DTOutput("data_preview_lta")
          )
        )
      ),
      # ==== Model Comparison
      tabPanel(
        title = tagList(icon("chart-line"), "Model Summary"),
        value = "summary_tab", 
        br(),
        
        column(
          12,
          # === 1️⃣ Model Comparison ===
          h4(icon("table"), "Model Fit Comparison"),
          uiOutput("fit_comparison"),
         
           tags$hr(style = "border-top: 2px solid #bbb;"),
          
          # === 2️⃣ Pilihan Model Terbaik ===
          column(4,
            h4(icon("cog"), "Select the Best Model"),
            p(
              "Choose the model you want to inspect in detail from the options below. ",
              "The best model can be determined by observing the smallest AIC/BIC and the highest item fit count above."),
              uiOutput('itemtype_ui')
            ),
          # === 3️⃣ Item Parameters & Fit ===
          column(8,
          h4(icon("chart-bar"), "Item Parameters & Item Fit Statistics"),
          uiOutput('item_summary')
          ),
          br(),
          br(),
          tags$hr(style = "border-top: 2px solid #bbb;"),
          column(12,
          h4(icon("chart-line"), "Item Characteristic Curve (ICC) and Information Function"),
          uiOutput('icc_info')
          ),
          br(),
          br(),
          br(),
        )
      ),
      tabPanel(
        title=tagList(icon("brain"), "Factor Scores"),
        column(12, uiOutput('fscoreLTA') )
        
      ),
      # ===== INFO ======
      tabPanel(
        title = tagList(icon("info-circle"), "About"),
        fluidRow(
          column(
            width = 8, offset = 2,
            br(),
            div(
              style = "text-align:center;",
              tags$hr(),
              tags$h5("projectLSA Was Developed By:"),
              tags$p(
                tags$a(
                  href = "https://scholar.google.com/citations?user=PSAwkTYAAAAJ&hl=id",
                  target = "_blank",
                  "Dr. Hasan Djidu, M.Pd."),
                tags$br(),
                "Universitas Sembilanbelas November Kolaka"
              ),
              tags$h5("Supervised By:"),
              tags$p(
                tags$a(
                  href = "https://scholar.google.com/citations?user=7CzPTYIAAAAJ&hl=id",
                  target = "_blank",
                  "Prof. Dr. Heri Retnawati, M.Pd."), 
                tags$br(),
                "Universitas Negeri Yogyakarta"
              ),
              
              tags$p(tags$a(
                href = "https://scholar.google.com/citations?hl=id&user=VGKeBm0AAAAJ",
                target = "_blank",
                "Prof. Dr. Samsul Hadi"), 
                tags$br(),
                "Universitas Negeri Yogyakarta"
              ),
              tags$p(tags$a(
                href = "https://scholar.google.com/citations?hl=id&user=k4MA8XgAAAAJ",
                target = "_blank",
                "Dr. Drs. Ir. Haryanto, M.Pd., M.T."), 
                tags$br(),
                "Universitas Negeri Yogyakarta"
              ),
              tags$b("Contact:"),
              tags$a("hasandjidu@gmail.com"),
              tags$hr()
            )
          ),
          column(
            width = 8, offset = 2,
            h4("References (R Packages)"),
            uiOutput("package_references_lta"),
            br(),
            div(
              style = "text-align:center;",
              tags$p(
                style = "font-size:13px; color:#777;",
                format(Sys.Date(), "%Y"), 
                "projectLSA. All rights reserved."
              ) ))
        )
      )
    )
  }

# Fungsi untuk menu Siswa
lpa_ui <- function(project) {
  tabsetPanel(
    id = "main_tab_lpa",
    
    # --- TAB 1: Prepare Data & Model ----
    tabPanel(
      title = tagList(icon("upload"), "Prepare Data & Model"),
      sidebarLayout(
        sidebarPanel(
          width = 3,
          # Tombol kembali ke halaman utama
            actionButton("go_home",
                         label = tagList(icon("home"), "Main Menu"), 
                         class = "btn btn-danger btn-block",
                         style = "width: 100% !important;"),
            br(),
          selectInput(
            "data_source", 
            "Select Data Source:",
            choices = c("Upload Data" = "upload",
                        "Built-in dataset: pisaUSA15" = "pisaUSA15",
                        "Built-in dataset: curry_mac" = "curry_mac",
                        "Built-in dataset: id_edu" = "id_edu"
                        ),
            selected = "upload"
          ),
          conditionalPanel(
            condition = "input.data_source == 'upload'",
            fileInput("datafile", "Upload Data/Workspace (csv/xlsx/sav/rds)", accept = c(".csv", ".xlsx", ".sav", ".rds"))
          ),
          #fileInput("datafile", "Upload Data (csv/xlsx)", accept = c(".csv", ".xlsx")),
          uiOutput("id_select_ui"),
          uiOutput("var_select_ui"),
          numericInput("min_profiles", "Min Number of Profiles:", 2, min = 1),
          numericInput("max_profiles", "Max. Number of Profiles:", 6, min = 2),
          
          selectInput(
            "model_to_run", 
            "Select Model Type:",
            choices = list(
              "Model 1: Equal variances, covariances fixed to 0" = 1,
              "Model 2: Varying variances, covariances fixed to 0" = 2,
              "Model 3: Equal variances & covariances" = 3,
              "Model 6: Varying variances & covariances" = 6
            ),
            selected = 1,
            multiple = TRUE
          ),
          actionButton("run_lpa",
                       label = tagList(icon("play"), "Run LPA"),
                       class = "btn btn-success btn-block",
                       style = "width: 100% !important; margin-bottom: 10px;"),
          downloadButton("export_lpa_rds", "Save Project (.rds)",
                         class = "btn btn-primary",
                         style = "width: 100% !important;")


        ),
        mainPanel(
          width = 9,
          
          h5(icon("table"), "Data Preview"),
          DTOutput("data_preview"),
          br(),
          DTOutput("data_summary_lpa"),
          br(),
          uiOutput("data_description"),
          
          br(),
        )
      )
    ),
    
    # --- TAB 2: Fit Model Comparison ----
    tabPanel(
      title = tagList(icon("chart-line"), "Fit Model Comparison"),
      value = "fit_tab",  # <--- tambahkan ini juga
      
      column(12,
        div(
          style = "text-align:center;",  # pusatkan kontainer
          div(
            style = "display:inline-block; font-size:11px; line-height:0.9; padding:0; margin:0;",
            
            column(12, h5(icon("info-circle"), "Model Fit Statistics"), 
                   DTOutput("fit_table")
                   )
        )),
        br(),
        tags$div(
          style = "margin-top: 0px; font-size: 11px; color: #6c757d;",
          class = "badge-info",
          tags$b("Note:"),
          tags$ul(
            tags$li(tags$span(style="color:blue;","Model selection: "),
                    "The selected LPA model defines the variance–covariance structure across profiles and influences interpretability and classification quality."),
            tags$li(tags$span(style="color:blue;","Model 1: "),
                    "Equal variances, covariances fixed to zero (most parsimonious)."),
            tags$li(tags$span(style="color:blue;","Model 2: "),
                    "Varying variances, covariances fixed to zero (allows different variability)."),
            tags$li(tags$span(style="color:blue;","Model 3: "),
                    "Equal variances and covariances across profiles."),
            tags$li(tags$span(style="color:blue;","Model 6: "),
                    "Varying variances and covariances (most flexible, higher risk of overfitting)."),
            tags$li(tags$span(style="color:blue;","BIC / AIC: "),
                    "Lower values indicate better relative model fit."),
            tags$li(tags$span(style="color:blue;","Entropy: "),
                    "Classification accuracy (values close to 1 indicate clear profiles)."),
            tags$li(tags$span(style="color:blue;","prob_min / prob_max: "),
                    "Minimum and maximum average posterior probabilities for assigned profiles."),
            tags$li(tags$span(style="color:blue;","n_min / n_max: "),
                    "Proportion of the sample in the smallest and largest profiles."),
            tags$li(tags$span(style="color:blue;","BLRT: "),
                    "Significant results favor the k-profile model over the k−1 model.")
          )
        ),
      column(3, #h5(icon("chart-bar"), "BIC Comparison"), 
             plotOutput("fit_bic"), height = "40px"),
      column(3, #h5(icon("chart-bar"), "AIC Comparison"), 
             plotOutput("fit_aic"),height = "40px"),
      column(3, #h5(icon("chart-bar"), "Entropy Comparison"), 
             plotOutput("fit_entropy"),height = "40px"),
      column(3, #h5(icon("chart-bar"), "Min. Class Size Comparison"), 
             plotOutput("fit_class_size"),height = "40px"),
      br(),br(), br()
      )
      
    ),
    
    # --- TAB 3: Best Model ----
    tabPanel(
      title = tagList(icon("star"), "Best Model"),
      sidebarLayout(
        sidebarPanel(
          width = 3, 
          selectInput(
            "model_type", 
            "Select Model Type:",
            choices = list(
              "Model 1: Equal variances, covariances fixed to 0" = 1,
              "Model 2: Varying variances, covariances fixed to 0" = 2,
              "Model 3: Equal variances & covariances" = 3,
              "Model 6: Varying variances & covariances" = 6
            ),
            selected = 1
          ),
          numericInput("best_k", label = "Select the Best Number of Profiles:",value = 3, min = 1),
          uiOutput("profile_name_inputs")
        ),
        mainPanel(
          width = 9, 
          br(),
    
          h5(icon("project-diagram"), "Profile Plot of the Best Model"),
          downloadButton("download_plot_best_LPA", "Download Plot Best Model (.png)"),
          plotOutput("best_model_plot", height = "450px"),
          br(),
          
          div(
            style = "text-align:center;",  # pusatkan kontainer
            div(
              style = "display:inline-block; font-size:11px; line-height:0.9; padding:0; margin:0;",
              h5(icon("info-circle"), "Estimated Profile Parameters: Class Size & Variabel Means (Variances)"), 
              DTOutput("summary_table") 
              )
          )
        )
      )
    ),
    
    # --- TAB 4: Summary & Report ----
    tabPanel(
      title = tagList(icon("file-alt"), "Summary & Report"),
      h4("Profile Classification Data"),
      DTOutput("profile_table"),
      br(),
      column(12,
        h4("Compare Profiles"),
        column(3, uiOutput("var_x_ui")),
        column(3, uiOutput("var_y_ui")),
        column(3,
               selectInput(
                 "plot_type", "Plot Type:",
                 choices = c("Line" = "line", "Bar" = "bar"),
                 selected = "bar"
               )
        ),
        column(3,
               checkboxInput(
                 "check_anova",
                 label = "Compare Profiles",
                 value = FALSE
               )
        )
      ),
      br(),
      # =========================
      # ROW 2: MAIN OUTPUT
      # =========================
      column(12,
        column(6, plotOutput("cross_latent", height = "420px")),
        column(6, uiOutput("anova_lpa_ui")),
        br()
      ),
      column(12,
             h4("Explore Other Variables by Profile"),
             checkboxInput(
               "crosstab_result",
               label = "Explore variable",
               value = FALSE
             ),
             column(12, uiOutput("crosstab_ui")),
             br(),
             br(),
             br()
      )
    ),

    # --- Report Preview ----
    tabPanel(
      title = tagList(icon("file-alt"), "Report Preview"),
      column(
        12,
        br(),
        div(style = "display: flex; gap: 10px; margin-bottom: 15px;",
            actionButton("lpa_generate_preview", tagList(icon("sync"), " Generate Report Preview"), class = "btn btn-success"),
            downloadButton("download_report_lpa", "Download HTML Report", class = "btn btn-primary")
        ),
        div(
          style = "border: 1px solid #ddd; border-radius: 4px; padding: 5px; background: #f9f9f9;",
          uiOutput("lpa_report_preview_frame")
        )
      )
    ),

    tabPanel(
      title = tagList(icon("info-circle"), "About"),
      fluidRow(
        column(
          width = 8, offset = 2,
          br(),
          div(
            style = "text-align:center;",
            tags$hr(),
            tags$h5("projectLSA Was Developed By:"),
            tags$p(
              tags$a(
                href = "https://scholar.google.com/citations?user=PSAwkTYAAAAJ&hl=id",
                target = "_blank",
                "Dr. Hasan Djidu, M.Pd."),
              tags$br(),
              "Universitas Sembilanbelas November Kolaka"
            ),
            tags$h5("Supervised By:"),
            tags$p(
              tags$a(
                href = "https://scholar.google.com/citations?user=7CzPTYIAAAAJ&hl=id",
                target = "_blank",
                "Prof. Dr. Heri Retnawati, M.Pd."), 
              tags$br(),
              "Universitas Negeri Yogyakarta"
            ),
            
            tags$p(tags$a(
              href = "https://scholar.google.com/citations?hl=id&user=VGKeBm0AAAAJ",
              target = "_blank",
              "Prof. Dr. Samsul Hadi"), 
              tags$br(),
              "Universitas Negeri Yogyakarta"
            ),
            tags$p(tags$a(
              href = "https://scholar.google.com/citations?hl=id&user=k4MA8XgAAAAJ",
              target = "_blank",
              "Dr. Drs. Ir. Haryanto, M.Pd., M.T."), 
              tags$br(),
              "Universitas Negeri Yogyakarta"
            ),
            tags$b("Contact:"),
            tags$a("hasandjidu@gmail.com"),
            tags$hr()
          )
        ),
        column(
          width = 8, offset = 2,
          h4("References (R Packages)"),
          uiOutput("package_references_lpa"),
          br(),
          div(
            style = "text-align:center;",
            tags$p(
              style = "font-size:13px; color:#777;",
              format(Sys.Date(), "%Y"), 
              "projectLSA. All rights reserved."
            ) ))
      )
    )
  )
}

# ==== Fungsi umum untuk download plot ====
make_download_plot <- function(plot_reactive, filename_prefix) {
  downloadHandler(
    filename = function() {
      paste0(filename_prefix, "_", Sys.Date(), ".png")
    },
    content = function(file) {
      ggsave(file, plot = plot_reactive(), width = 8, height = 5, dpi = 300)
    }
  )
}

desc_curry_text <- "
<b>Description</b><br>
This simulated dataset, based on Curry et al. (2019), contains data on moral relevance and judgment across the seven domains of the Morality As Cooperation scale.
<br><br>

<b>Details</b><br>
<table border='0' cellpadding='4'>
<tr><td><b>sex</b> (factor): Self-identified sex of participants (Male, Female, or Transgendered).</td></tr>
<tr><td><b>age_years</b> (numeric): Participants' age in years.</td></tr>
<tr><td><b>KinshipR</b> (numeric): Mean score of moral relevance, kinship subscale.</td></tr>
<tr><td><b>MutualismR</b> (numeric): Mean score of moral relevance, mutualism subscale.</td></tr>
<tr><td><b>ExchangeR</b> (numeric): Mean score of moral relevance, exchange subscale.</td></tr>
<tr><td><b>HawkR</b> (numeric): Mean score of moral relevance, hawk subscale.</td></tr>
<tr><td><b>DoveR</b> (numeric): Mean score of moral relevance, dove subscale.</td></tr>
<tr><td><b>DivisionR</b> (numeric): Mean score of moral relevance, division subscale.</td></tr>
<tr><td><b>PossessionR</b> (numeric): Mean score of moral relevance, possession subscale.</td></tr>
<tr><td><b>KinshipJ</b> (numeric): Mean score of moral judgment, kinship subscale.</td></tr>
<tr><td><b>MutualismJ</b> (numeric): Mean score of moral judgment, mutualism subscale.</td></tr>
<tr><td><b>ExchangeJ</b> (numeric): Mean score of moral judgment, exchange subscale.</td></tr>
<tr><td><b>HawkJ</b> (numeric): Mean score of moral judgment, hawk subscale.</td></tr>
<tr><td><b>DoveJ</b> (numeric): Mean score of moral judgment, dove subscale.</td></tr>
<tr><td><b>DivisionJ</b> (numeric): Mean score of moral judgment, division subscale.</td></tr>
<tr><td><b>PossessionJ</b> (numeric): Mean score of moral judgment, possession subscale.</td></tr>
</table><br>

<b>Reference</b><br>
Curry, O. S., Jones Chesters, M., & Van Lissa, C. J. (2019). 
<i>Mapping morality with a compass: Testing the theory of 'morality-as-cooperation' with a new questionnaire.</i> 
<em>Journal of Research in Personality, 78</em>, 106–124. 
<a href='https://doi.org/10.1016/j.jrp.2018.10.008' target='_blank'>https://doi.org/10.1016/j.jrp.2018.10.008</a>
"


desc_pisa <- "
<b>Description</b><br>
Student questionnaire data with four variables from the 2015 PISA survey for students in the United States.<br><br>

<b>Details</b><br>
<table border='0' cellpadding='4'>
<tr><td><b>broad_interest</b> (numeric): Composite measure of students' self-reported broad interest.</td></tr>
<tr><td><b>enjoyment</b> (numeric): Composite measure of students' self-reported enjoyment.</td></tr>
<tr><td><b>instrumental_mot</b> (numeric): Composite measure of students' self-reported instrumental motivation.</td></tr>
<tr><td><b>self_efficacy</b> (numeric): Composite measure of students' self-reported self-efficacy.</td></tr>
</table><br>

<b>Source</b><br>
<a href='http://www.oecd.org/pisa/data/' target='_blank'>OECD PISA Data Portal</a>
"


desc_id_edu <- "
<b>Description</b><br>
This simulated dataset, based on <i>Crocetti et al., 2014</i>, contains five annual assessments of adolescents' mean scores on the commitment, exploration (in depth), and reconsideration subscales of the Utrecht-Management of Identity Commitments Scale (<i>Crocetti et al., 2008</i>). 
The scores reported here reflect the <b>educational identity</b> subscales of this instrument. 
The first measurement wave occurred when adolescents were, on average, 14 years old, and the last one when they were 18 years old.<br><br>

<b>Details</b><br>
<table border='0' cellpadding='4'>
<tr><td><b>com1</b> (numeric): Mean score of educational commitment in wave 1.</td></tr>
<tr><td><b>exp1</b> (numeric): Mean score of educational exploration in wave 1.</td></tr>
<tr><td><b>rec1</b> (numeric): Mean score of educational reconsideration in wave 1.</td></tr>
<tr><td><b>com2</b> (numeric): Mean score of educational commitment in wave 2.</td></tr>
<tr><td><b>exp2</b> (numeric): Mean score of educational exploration in wave 2.</td></tr>
<tr><td><b>rec2</b> (numeric): Mean score of educational reconsideration in wave 2.</td></tr>
<tr><td><b>com3</b> (numeric): Mean score of educational commitment in wave 3.</td></tr>
<tr><td><b>exp3</b> (numeric): Mean score of educational exploration in wave 3.</td></tr>
<tr><td><b>rec3</b> (numeric): Mean score of educational reconsideration in wave 3.</td></tr>
<tr><td><b>com4</b> (numeric): Mean score of educational commitment in wave 4.</td></tr>
<tr><td><b>exp4</b> (numeric): Mean score of educational exploration in wave 4.</td></tr>
<tr><td><b>rec4</b> (numeric): Mean score of educational reconsideration in wave 4.</td></tr>
<tr><td><b>com5</b> (numeric): Mean score of educational commitment in wave 5.</td></tr>
<tr><td><b>exp5</b> (numeric): Mean score of educational exploration in wave 5.</td></tr>
<tr><td><b>rec5</b> (numeric): Mean score of educational reconsideration in wave 5.</td></tr>
<tr><td><b>sex</b> (factor): Adolescent sex; M = male, F = female.</td></tr>
</table><br>

<b>References</b><br>
Crocetti, E., Klimstra, T. A., Hale, W. W., Koot, H. M., & Meeus, W. (2013). 
<a href='https://doi.org/10.1007/s10964-013-9924-6' target='_blank'>
<i>Impact of early adolescent externalizing problem behaviors on identity development in middle to late adolescence: A prospective 7-year longitudinal study.</i></a> 
<i>Journal of Youth and Adolescence, 42(11)</i>, 1745–1758.
"

desc_cheating <- "
<b>Description</b><br>
This dataset contains dichotomous responses from 319 undergraduate students regarding academic cheating behaviors, along with their academic grade point average (GPA). 
The data are commonly used to illustrate latent class analysis of categorical variables.<br><br>

<b>Details</b><br>
<table border='0' cellpadding='4'>
<tr><td><b>LIEEXAM</b> (numeric): Whether the student ever lied to avoid taking an exam (1 = no, 2 = yes).</td></tr>
<tr><td><b>LIEPAPER</b> (numeric): Whether the student ever lied to avoid submitting a term paper on time (1 = no, 2 = yes).</td></tr>
<tr><td><b>FRAUD</b> (numeric): Whether the student ever purchased a term paper or obtained an exam copy prior to taking it (1 = no, 2 = yes).</td></tr>
<tr><td><b>COPYEXAM</b> (numeric): Whether the student ever copied answers during an exam from another student (1 = no, 2 = yes).</td></tr>
<tr><td><b>GPA</b> (factor): Academic grade point average, categorized into five groups:
(1) ≤ 2.99; 
(2) 3.00–3.25; 
(3) 3.26–3.50; 
(4) 3.51–3.75; 
(5) 3.76–4.00.</td></tr>
</table><br>

<b>Notes</b><br>
GPA data were not available for four students who reported never engaging in any cheating behavior.<br><br>

<b>Source</b><br>
Dayton, C. M. (1998). 
<i>Latent Class Scaling Analysis.</i> 
Thousand Oaks, CA: SAGE Publications.
"


desc_lazar <- "
<b>Description</b><br>
This dataset is a classical example introduced by Lazarsfeld and Henry (1968), consisting of response patterns from 1,000 subjects who attempted to solve three problems (A, B, and C). 
For each problem, responses are coded as (1) solved or (2) not solved. 
The data are provided in the form of a contingency table of response pattern frequencies and are widely used in the context of configural frequency analysis (CFA) and latent structure analysis.<br><br>

<b>Details</b><br>
<table border='0' cellpadding='4'>
<tr><td><b>V1</b> (numeric): Response to problem A (1 = solved, 2 = not solved).</td></tr>
<tr><td><b>V2</b> (numeric): Response to problem B (1 = solved, 2 = not solved).</td></tr>
<tr><td><b>V3</b> (numeric): Response to problem C (1 = solved, 2 = not solved).</td></tr>
<tr><td><b>Freq</b> (numeric): Frequency of each response pattern.</td></tr>
</table><br>

<b>Format</b><br>
The dataset is stored as a pattern frequency table with 8 rows (unique response patterns) and 4 columns. 
The total number of subjects represented in the table is 1,000.<br><br>

<b>References</b><br>
Lazarsfeld, P. F., & Henry, N. W. (1968). 
<i>Latent Structure Analysis.</i> 
Boston: Houghton Mifflin.<br><br>

"

desc_gss82 <- "
<b>Description</b><br>
This dataset contains survey responses from 1,202 white respondents to the 1982 General Social Survey (GSS), focusing on attitudes toward survey participation. 
It includes two dichotomous and two ordinal variables and is commonly used as a teaching and demonstration dataset for latent class analysis (LCA).<br><br>

<b>Details</b><br>
<table border='0' cellpadding='4'>

<tr><td><b>PURPOSE</b> (ordinal): Respondents’ views on the purpose of surveys, coded as 
1 = <i>Waste of time</i>, 
2 = <i>Depends</i>, 
3 = <i>Good</i>.</td></tr>

<tr><td><b>ACCURACY</b> (dichotomous): Respondents’ beliefs about the accuracy of survey results, coded as 
0 = <i>Not true</i>, 
1 = <i>Mostly true</i>.</td></tr>

<tr><td><b>UNDERSTA</b> (ordinal): Respondents’ self-reported understanding of survey questions, coded as 
1 = <i>Fair/Poor</i>, 
2 = <i>Good</i>.</td></tr>

<tr><td><b>COOPERAT</b> (ordinal): Respondents’ level of cooperation with the interviewer, coded as 
1 = <i>Impatient</i>, 
2 = <i>Cooperative</i>, 
3 = <i>Interested</i>.</td></tr>

</table><br>

<b>Notes</b><br>
Higher numeric values indicate more positive attitudes toward survey participation. 
The dataset appears in McCutcheon (1987, p. 30) and is intended for methodological illustration rather than population inference.<br><br>

<b>Source</b><br>
McCutcheon, A. L. (1987). <i>Latent Class Analysis.</i> Newbury Park, CA: SAGE Publications.
"

homepage_ui <- function() {
  
  fluidPage(

    div(class="landing",
        div(class="hero",
            div(class="welcome","WELCOME TO"),
            div( class = "app-name", "projectLSA" ),            
            div(

              HTML(
                '<div style="text-align: left; padding: 8px; background-color: rgba(255,255,255,0.1); border-left: 3px solid #ccc; margin-top: 5px; font-size: 10px; color: #555;">
                 <strong>How to Cite:</strong><br>
                 Djidu, H., Retnawati, H., Hadi, S., &amp; Haryanto. (2026). <em>projectLSA: A Shiny Application for Integrated Latent Structure Analysis</em>. Applied Psychological Measurement. 
                 <a href="https://doi.org/10.1177/01466216261446305" target="_blank" style="color: #0077dd;">https://doi.org/10.1177/01466216261446305</a><br>
                 Djidu, H., Retnawati, H., Hadi, S., Haryanto (2026). <em>projectLSA: R Shiny application for latent structure analysis with a graphical user interface</em>. 
                 <a href="https://doi.org/10.32614/CRAN.package.projectLSA" target="_blank" style="color: #0077dd;">https://doi.org/10.32614/CRAN.package.projectLSA</a>. R package.
                 </div>'
              ),
              br(),
              div(style = "display: flex; gap: 8px; justify-content: center; align-items: center; flex-wrap: wrap; margin-top: 5px;",
                  downloadButton("download_ris_home", "Download RIS", class = "btn btn-info btn-sm", style = "margin: 0 !important;"),
                  actionButton("btn_donate_home", "Support projectLSA", icon = icon("heart"), class = "btn btn-danger btn-sm btn-glow", style = "margin: 0 !important;", onclick="$(this).removeClass('btn-glow');")
              )
            )
            
        ),
        br(),
        div(
          style = "max-width: 720px; margin: 0 auto 10px auto; padding: 8px 14px; font-size: 11px; color: #374151; background: #f0f7fb; border-left: 4px solid #2563eb; border-radius: 4px; text-align: left;",
          HTML(
            "<strong>New in 0.1.1 &mdash; AI Assistant.</strong> Every module now has a floating
             <em>AI Assistant</em> (bottom right) that can answer questions about your results and
             draft a summary for your manuscript. Open its <em>Settings</em> tab and paste your own
             API key (Gemini, OpenAI, Groq, or OpenRouter) to enable it. The key is used
             only for your session and is never stored by the package; no data leaves your computer
             until you provide a key and send a request."
          )
        ),
        div(class="subtitle","Choose one projectLSA you want to work on:")
    ),
    br(),
    div(class="quad-container",
        div(class="axis-label x-top","Latent = categorical"),
        div(class="axis-label x-bottom","Latent = scale"),
        div(class="axis-label y-left","Observed = scale <<"),
        div(class="axis-label y-right",">> Observed = categorical"),
        div(class="center-logo", tags$img(src="logoProjectLSA.png")),
        
        div(class="quad-card", id="card_lpa", onclick="$('#go_lpa').click();",
            div(class="project-icon", tags$i(class="fa-solid fa-chart-simple")),
            div(class="project-title","LPA - Latent Profile Analysis"),
            div(class="project-desc","Identify latent profile (unobserved subgroups) within your sample using continuous data."),
            actionButton("go_lpa", label="OPEN LPA", class="btn-pill")
        ),
        div(class="quad-card", id="card_lca", onclick="$('#go_lca').click();",
            div(class="project-icon", tags$i(class="fa-solid fa-layer-group")),
            div(class="project-title","LCA - Latent Class Analysis"),
            div(class="project-desc","Identify latent class (unobserved subgroups) within your sample using categorical data."),
            actionButton("go_lca", label="OPEN LCA", class="btn-pill")
        ),
        div(class="quad-card", id="card_fa",
            div(class="project-icon", tags$i(class="fa-solid fa-sitemap")),
            div(class="project-title","FA - Factor Analysis"),
            div(class="project-desc","Explore latent constructs with exploratory or confirmatory FA using continuous data."),
            
            div(style="display: flex; gap: 10px; justify-content: center; margin-top: 10px; width: 100%;",
                actionButton("go_efa", label="OPEN EFA", width = '50%', class="btn-pill btn-primary"),
                actionButton("go_cfa", label="OPEN CFA", width = '50%', class="btn-pill btn-secondary")
            )
            ),
        
        div(class="quad-card", id="card_lta", onclick="$('#go_lta').click();",
            div(class="project-icon", tags$i(class="fa-solid fa-brain")),
            div(class="project-title","LTA - Latent Trait Analysis"),
            div(class="project-desc","Estimate item and person parameters using LTA approaches using categorical data."),
            actionButton("go_lta", label="OPEN LTA", class="btn-pill")
        )
    ),
    div(
      style = "text-align:center;",
      tags$p(
        style = "font-size:13px; color:#777;",
        format(Sys.Date(), "%Y"), 
        "projectLSA. All rights reserved."
      )
    )
  )
}

# fa_ui.R (revisi)
efa_ui <- function(project) {
  tabsetPanel(
    id = "main_tab_fa",
    
    # === TAB 1: Prepare Data & Model ====
    tabPanel(
      title = tagList(icon("upload"), "Prepare Data & Model"),
      sidebarLayout(
        sidebarPanel(
          width = 3,
          # Tombol kembali
          actionButton("go_home", 
                       label = tagList(icon("home"), "Main Menu"), 
                       class = "btn btn-danger btn-block",
                       style = "width: 100% !important;"),
          br(),
          
          # === Input sumber data ===
          selectInput("data_source", "Select Data Source:",
                      choices = c("Upload Data" = "upload",
                                  "Built-in: bfi" = "bfi",
                                  "Built-in: HolzingerSwineford1939" = "HolzingerSwineford1939"
                                  
                                 )),
          
          conditionalPanel(
            condition = "input.data_source == 'upload'",
            fileInput("datafile", "Upload Data (csv/xlsx)", accept = c(".csv", ".xlsx"))),
          uiOutput("id_select_ui_fa"),
          uiOutput("var_select_ui"),
          br(),
          selectInput("rotation_method", "Rotation Method:", 
                      choices = c("varimax", "oblimin", "promax", "none")),
          selectInput("missing_method_efa", "Handle Missing Data:",
                      choices = c("Listwise Deletion" = "listwise",
                                  "Pairwise Deletion" = "pairwise",
                                  "Mean Imputation" = "mean")),
          br(),
          actionButton("run_efa", label = tagList(icon("play"), "Run EFA"),
                       class = "btn btn-success btn-block",
                       style = "width: 100% !important;")
        ),
        
        mainPanel(
          width = 9,
          DTOutput("data_preview_fa")
        )
      )
    ),
    # ===== Pre-Analisis  ======
    tabPanel(
      "KMO & Bartlett Tests",
      
      fluidRow(
        column(12, uiOutput("efa_tests")),
        column(6, plotOutput("kmo_item" )),
        column(6, plotOutput("scree_plot"))
      )
      ),
    tabPanel(
      "EFA Results",
      fluidRow(
        
        # ==================== KIRI: EFA SUMMARY =====================
        column(
          6,
          div(
            class = "card shadow-sm p-3 mb-3 bg-white rounded",
            style = "height: 80vh; overflow-y: auto;",
            
            # ==== Header & Input ====
            div(
              style = "display: flex;flex-direction: column;align-items: center;justify-content: flex-start;text-align: center;width: 100%;padding: 10px;",
              # === Input Number of Factors (di tengah) ===
              div(
                style = "display: flex; flex-direction: column; align-items: center;",
                tags$h4(icon('chart-simple'), "Exploratory Factor Analysis Summary", style = "font-weight:bold;color:#2c3e50;"),
                tags$b("Number of Factors:"),
                numericInput("n_factors", label = NULL, value = 3, min = 1, max = 10, width = '120px')
              ),
              # === Dynamic factor names (juga di tengah) ===
              tags$b("Factors' Names:"),
              uiOutput("factor_names_ui")
            ),
            
            hr(),
            # ==== Tombol download di kanan ====
            div(
              style = "text-align: right; margin-bottom: 5px; width: 100%;",
              downloadButton(
                "download_loading",
                "Download Loadings (.csv)",
                class = "btn btn-primary btn-block btn-sm"
              )
            ),
            
            # ==== Output ringkasan ====
            uiOutput("efa_summary")
          )
        ),
        
        # ==================== KANAN: FACTOR SCORES =====================
        column(
          6,
          div(
            class = "card shadow-sm p-3 mb-3 bg-white rounded",
            style = "height: 80vh; overflow-y: auto;",
            div(
              style = "display: flex; flex-direction: column; align-items: center;justify-content: flex-start;text-align: center;width: 100%;padding: 10px;",
              tags$h4(icon("table"), "Factor Scores", style = "font-weight:bold;color:#2c3e50;"),
              tags$p("These are estimated latent factor scores for each observation.")
            ),
            div(
              class = "card shadow-sm p-3 mb-3 bg-white rounded",
              style = "height: 80vh; overflow-y: auto;",
              # ==== Tombol download skor ====
              div(
                style = "text-align: right; margin-bottom: 5px; width: 100%;",
                downloadButton(
                  "download_scores",
                  "Download Factor Scores (.csv)",
                  class = "btn btn-primary btn-block btn-sm"
                )
              ),
              
              # ==== Output skor faktor ====
              DTOutput ("efa_scores_table")
            )
          )
        )
      )
    ),
    tabPanel(
      title = tagList(icon("info-circle"), "About"),
      fluidRow(
        column(
          width = 8, offset = 2,
          br(),
          div(
            style = "text-align:center;",
            tags$hr(),
            tags$h5("projectLSA Was Developed By:"),
            tags$p(
              tags$a(
                href = "https://scholar.google.com/citations?user=PSAwkTYAAAAJ&hl=id",
                target = "_blank",
                "Dr. Hasan Djidu, M.Pd."),
              tags$br(),
              "Universitas Sembilanbelas November Kolaka"
            ),
            tags$h5("Supervised By:"),
            tags$p(
              tags$a(
                href = "https://scholar.google.com/citations?user=7CzPTYIAAAAJ&hl=id",
                target = "_blank",
                "Prof. Dr. Heri Retnawati, M.Pd."), 
              tags$br(),
              "Universitas Negeri Yogyakarta"
            ),
            
            tags$p(tags$a(
              href = "https://scholar.google.com/citations?hl=id&user=VGKeBm0AAAAJ",
              target = "_blank",
              "Prof. Dr. Samsul Hadi"), 
              tags$br(),
              "Universitas Negeri Yogyakarta"
            ),
            tags$p(tags$a(
              href = "https://scholar.google.com/citations?hl=id&user=k4MA8XgAAAAJ",
              target = "_blank",
              "Dr. Drs. Ir. Haryanto, M.Pd., M.T."), 
              tags$br(),
              "Universitas Negeri Yogyakarta"
            ),
            tags$b("Contact:"),
            tags$a("hasandjidu@gmail.com"),
            tags$hr()
          )
        ),
        column(
          width = 8, offset = 2,
          h4("References (R Packages)"),
          uiOutput("package_references_efa"),
          br(),
          div(
            style = "text-align:center;",
            tags$p(
              style = "font-size:13px; color:#777;",
              format(Sys.Date(), "%Y"), 
              "projectLSA. All rights reserved."
            )
          )
        )
      )
    )
  )
}

server_efa <- function(input, output, session, ai_context, console_context) {
  library(psych)
  library(lavaan)
  
  # ==== LOAD DATA ====
  raw_data_user <- reactive({
    req(input$data_source)
    if (input$data_source == "bfi") return(psych::bfi %>% dplyr::select(A1:O5)%>% rownames_to_column("id_auto"))
    if (input$data_source == "HolzingerSwineford1939") return(lavaan::HolzingerSwineford1939%>% dplyr::select(x1:x9)%>% rownames_to_column("id_auto"))
    if (input$data_source == "upload") {
      req(input$datafile)
      ext <- tolower(tools::file_ext(input$datafile$name))
      showModal(modalDialog(title = NULL, "Reading Your File, Please wait...", footer = NULL, easyClose = FALSE))
      if (ext == "rds") {
        res <- readRDS(input$datafile$datapath)
        if (is.list(res) && !is.data.frame(res) && identical(res$type, "projectLSA_workspace")) {
          if (identical(res$module, "EFA")) {
            # Restore Workspace State
            efa_aggregations(res$efa_aggregations)
            efa_result(res$efa_result)
            df <- res$raw_data
            showNotification("EFA Workspace restored successfully!", type = "message")
          } else {
            stop("Uploaded workspace belongs to a different module: ", res$module)
          }
        } else if (is.list(res) && "fa" %in% names(res) && inherits(res$fa, "fa")) {
          efa_result(res)
          vars <- rownames(res$fa$loadings)
          df <- as.data.frame(matrix(NA, nrow=1, ncol=length(vars)))
          colnames(df) <- vars
          showModal(modalDialog("EFA Model uploaded successfully! Please navigate to the Summary or Plot tab.", easyClose = TRUE))
        } else if (inherits(res, "fa")) {
          efa_result(list(fa = res, cor_mat = NULL, fit_stats = NULL, n_obs = NULL))
          vars <- rownames(res$loadings)
          df <- as.data.frame(matrix(NA, nrow=1, ncol=length(vars)))
          colnames(df) <- vars
          showModal(modalDialog("EFA Model uploaded successfully! Please navigate to the Summary or Plot tab.", easyClose = TRUE))
        } else {
          df <- res
        }
      } else {
        df <- switch(
          ext,
          "csv"  = data.table::fread(input$datafile$datapath, data.table = FALSE),
          "xls"  = readxl::read_excel(input$datafile$datapath),
          "xlsx" = readxl::read_excel(input$datafile$datapath),
          "sav"  = haven::read_sav(input$datafile$datapath),
          stop("Unsupported file type. Please upload CSV, Excel, SPSS (.sav), or RDS file.")
        )
      }
      removeModal()
      
      if (!"id_auto" %in% names(df)) {
        df <- df %>% mutate(across(everything(), ~ifelse(.x == "", NA, .x)),
                            id_auto = paste0("id_", sprintf("%04d", 1:n())))
      }
    }
    return(df)
  })
  
  # Reactive value to store user-defined aggregations
  efa_aggregations <- reactiveVal(list())
  
  # New data_user reactive that applies aggregations on top of raw_data_user
  data_user <- reactive({
    df <- raw_data_user()
    req(df)
    
    aggs <- efa_aggregations()
    if (length(aggs) > 0) {
      for (agg in aggs) {
        vars <- agg$vars
        name <- agg$name
        method <- agg$method
        
        valid_vars <- intersect(vars, names(df))
        if (length(valid_vars) > 0) {
          if (method == "mean") {
            df[[name]] <- rowMeans(df[, valid_vars, drop = FALSE], na.rm = TRUE)
          } else if (method == "sum") {
            df[[name]] <- rowSums(df[, valid_vars, drop = FALSE], na.rm = TRUE)
          }
        }
      }
    }
    return(df)
  })
  
  # Handle aggregation button click
  observeEvent(input$efa_btn_aggregate, {
    req(input$efa_agg_vars, input$efa_agg_name)
    
    # Check if name is valid and doesn't exist already
    new_name <- make.names(input$efa_agg_name)
    
    new_agg <- list(
      vars = input$efa_agg_vars,
      name = new_name,
      method = input$efa_agg_method
    )
    
    # Append to existing aggregations
    current_aggs <- efa_aggregations()
    current_aggs[[new_name]] <- new_agg
    efa_aggregations(current_aggs)
    
    showNotification(paste("Aggregated variable", new_name, "created using", input$efa_agg_method, "method."), type = "message")
    
    # Clear inputs
    updateTextInput(session, "efa_agg_name", value = "")
    updateSelectizeInput(session, "efa_agg_vars", selected = "")
  })
  
  # Handle reset aggregation button
  observeEvent(input$efa_btn_reset_agg, {
    efa_aggregations(list())
    showNotification("All aggregations have been reset.", type = "warning")
  })
  
  # Populate aggregation UI choices
  observe({
    req(raw_data_user())
    # We populate with raw data columns + existing aggregated columns
    df_names <- names(data_user())
    updateSelectizeInput(session, "efa_agg_vars", choices = df_names)
  })
  

  # ==== Pilih ID ====
  output$id_select_ui_fa <- renderUI({
    req(data_user())
    shinyWidgets::pickerInput(
      "id_lca",
      label = "Select ID Columns (Optional):",
      choices = names(data_user()),
      selected = names(data_user())[str_detect(names(data_user()), "id")],
      multiple = TRUE,
      options = list(`actions-box` = TRUE, `live-search` = TRUE, placeholder = 'Choose one or more ID columns')
    )
  })
  
  # ==== Pilih Variabel (selain ID) ====
  output$var_select_ui <- renderUI({
    req(data_user())
    all_vars <- names(data_user())
    id_cols <- input$id_lca
    
    # Hilangkan kolom ID dari pilihan variabel
    available_vars <- setdiff(all_vars, id_cols)
    shinyWidgets::pickerInput(
      "selected_vars",
      label = "Select Variables:",
      choices = available_vars,
      selected = available_vars,  # default pilih semua yang tersisa
      multiple = TRUE,
      options = list(`actions-box` = TRUE, `live-search` = TRUE, `selected-text-format` = "count > 3")
    )
  })

  output$data_preview_fa <- renderDT({
    req(data_user(),input$selected_vars)
    df <- data_user()[,input$selected_vars]
    df <- df %>%
      dplyr::mutate(across(where(is.numeric), ~ round(.x, 3)))
    
    DT::datatable(
      head(df, 50),extensions = 'Buttons',
      options = list(dom='Brtp',scrollX = TRUE, pageLength = 25,  
                     buttons = list(
                       list(extend = 'csv',
                            text = 'Export CSV',
                            filename = 'Data EFA'
                       ),
                       list(extend = 'excel',
                            text = 'Export Excel',
                            filename = 'Data EFA'
                       ))),
      rownames = FALSE
    )
  }, server = FALSE)
  
  # ====== Data Preview Visualizations ======
  output$efa_summary_table <- renderUI({
    vars_to_use <- input$selected_vars
    if (is.null(vars_to_use) || length(vars_to_use) == 0) vars_to_use <- names(data_user())
    if (is.null(vars_to_use) || length(vars_to_use) == 0) vars_to_use <- names(data_user())
    valid_vars <- intersect(vars_to_use, names(data_user()))
    req(length(valid_vars) > 0)
    
    df <- data_user()[, valid_vars, drop = FALSE]
    
    summary_list <- lapply(names(df), function(v) {
      x <- df[[v]]
      x_na <- na.omit(x)
      n <- length(x_na)
      
      if (is.numeric(x)) {
        mean_val <- round(mean(x_na), 2)
        sd_val <- round(sd(x_na), 2)
        min_val <- round(min(x_na), 2)
        max_val <- round(max(x_na), 2)
        type <- "Scale"
        
        data.frame(
          Variable = v,
          Type = type,
          N = n,
          `Mean/Mode` = as.character(mean_val),
          `SD/Count` = as.character(sd_val),
          Min = as.character(min_val),
          Max = as.character(max_val),
          check.names = FALSE
        )
      } else {
        tbl <- table(x_na)
        if(length(tbl) > 0) {
          mode_val <- names(tbl)[which.max(tbl)]
          count_val <- max(tbl)
        } else {
          mode_val <- "-"
          count_val <- "-"
        }
        type <- if (is.ordered(x)) "Ordinal" else "Nominal"
        
        data.frame(
          Variable = v,
          Type = type,
          N = n,
          `Mean/Mode` = as.character(mode_val),
          `SD/Count` = as.character(count_val),
          Min = "-",
          Max = "-",
          check.names = FALSE
        )
      }
    })
    
    summary_df <- do.call(rbind, summary_list)
    
    html_out <- summary_df %>%
      kable(format = "html", escape = FALSE, caption = "<b>Table 1</b><br><i>Descriptive Statistics</i>", align = "llccccc") %>%
      kable_styling(bootstrap_options = "none", full_width = FALSE, position = "left") %>%
      row_spec(0, bold = FALSE, extra_css = "border-top: 1.5px solid black; border-bottom: 1.5px solid black; font-family: 'Times New Roman', Times, serif; font-size: 11pt;") %>%
      row_spec(nrow(summary_df), extra_css = "border-bottom: 1.5px solid black;") %>%
      column_spec(1:ncol(summary_df), extra_css = "padding: 4px 12px; font-size: 11pt; line-height: 1; font-family: 'Times New Roman', Times, serif;")
    
    HTML(as.character(html_out))
  })

  output$efa_data_summary_plot <- renderPlot({
    vars_to_use <- input$selected_vars
    if (is.null(vars_to_use) || length(vars_to_use) == 0) vars_to_use <- names(data_user())
    if (is.null(vars_to_use) || length(vars_to_use) == 0) vars_to_use <- names(data_user())

    valid_vars <- intersect(vars_to_use, names(data_user()))
    if (length(valid_vars) == 0) {
      return(NULL)
    }

    df <- data_user()[, valid_vars, drop = FALSE]

    types <- sapply(df, function(x) class(x)[1])
    type_df <- as.data.frame(table(types))
    names(type_df) <- c("DataType", "Count")
    
    theme_choice <- input$efa_summary_theme
    if (is.null(theme_choice)) theme_choice <- "default"
    
    bar_colors <- switch(theme_choice,
      "pastel" = scale_fill_brewer(palette = "Pastel1"),
      "blue"   = scale_fill_brewer(palette = "Blues"),
      "dark"   = scale_fill_brewer(palette = "Dark2"),
      scale_fill_brewer(palette = "Set2")
    )

    ggplot(type_df, aes(x = DataType, y = Count, fill = DataType)) +
      geom_col(width = 0.5, show.legend = FALSE, alpha = 0.9, color = "transparent") +
      geom_text(aes(label = Count), vjust = -0.5, fontface = "bold", color = "#495057") +
      bar_colors +
      theme_minimal(base_size = 14) +
      theme(
        plot.title = element_text(hjust = 0.5, face = "bold", size = 14, color = "#343a40"),
        axis.title.x = element_text(face = "bold", color = "#495057", margin = margin(t = 10)),
        axis.title.y = element_text(face = "bold", color = "#495057", margin = margin(r = 10)),
        axis.text.x = element_text(angle = 45, hjust = 1, vjust = 1), # Prevent overlap
        panel.grid.major.x = element_blank(),
        plot.margin = margin(15, 15, 15, 15),
        panel.background = element_rect(fill = "transparent", color = NA),
        plot.background = element_rect(fill = "transparent", color = NA)
      ) +
      labs(title = "Data Type Composition", x = "Data Type", y = "Number of Variables")
  }, bg="transparent")

  output$efa_data_heatmap <- renderPlot({
    vars_to_use <- input$selected_vars
    if (is.null(vars_to_use) || length(vars_to_use) == 0) vars_to_use <- names(data_user())
    if (is.null(vars_to_use) || length(vars_to_use) == 0) vars_to_use <- names(data_user())

    valid_vars <- intersect(vars_to_use, names(data_user()))
    if (length(valid_vars) == 0) {
      return(NULL)
    }

    df <- data_user()[, valid_vars, drop = FALSE]

    num_df <- df[, sapply(df, is.numeric), drop = FALSE]
    if (ncol(num_df) < 2) {
      plot.new()
      title(main = "Not enough numeric variables for correlation heatmap", col.main = "#6c757d")
      return(NULL)
    }

    cor_matrix <- cor(num_df, use = "pairwise.complete.obs")
    cor_matrix[upper.tri(cor_matrix)] <- NA

    cor_melted <- as.data.frame(as.table(cor_matrix))
    cor_melted <- cor_melted[!is.na(cor_melted$Freq), ]
    names(cor_melted) <- c("Var1", "Var2", "value")

    theme_choice <- input$efa_summary_theme
    if (is.null(theme_choice)) theme_choice <- "default"
    
    heatmap_colors <- switch(theme_choice,
      "pastel" = scale_fill_gradient2(low = "#b3cde3", high = "#fbb4ae", mid = "white", midpoint = 0, limit = c(-1,1), name="Pearson\nCorrelation"),
      "blue"   = scale_fill_gradient2(low = "#deebf7", high = "#3182bd", mid = "white", midpoint = 0, limit = c(-1,1), name="Pearson\nCorrelation"),
      "dark"   = scale_fill_gradient2(low = "#1b9e77", high = "#d95f02", mid = "white", midpoint = 0, limit = c(-1,1), name="Pearson\nCorrelation"),
      scale_fill_gradient2(low = "#4575b4", high = "#d73027", mid = "white", midpoint = 0, limit = c(-1,1), name="Pearson\nCorrelation")
    )

    ggplot(cor_melted, aes(Var1, Var2, fill = value)) +
      geom_tile(color = "white", size = 0.5) +
      heatmap_colors +
      theme_minimal(base_size = 12) +
      theme(
        axis.text.x = element_text(angle = 45, vjust = 1, hjust = 1, color = "#495057"),
        axis.text.y = element_text(color = "#495057"),
        axis.title.x = element_blank(),
        axis.title.y = element_blank(),
        panel.grid.major = element_blank(),
        panel.border = element_blank(),
        panel.background = element_blank(),
        axis.ticks = element_blank(),
        plot.title = element_text(hjust = 0.5, face = "bold", size = 14, color = "#343a40"),
        plot.background = element_rect(fill = "transparent", color = NA)
      ) +
      coord_fixed() +
      labs(title = "Correlation Heatmap (Numeric Variables)")
  }, bg="transparent")
  
  # === Ketika user memilih CFA ===
  observeEvent(input$fa_type, {
    if (input$fa_type == "cfa") {
      message("Running CFA server...")
      cfa_server(input, output, session, data_user = data_user)
    }
  })
  
  
  
  
  # ==== Dynamic input nama faktor ====
  output$factor_names_ui <- renderUI({
    req(input$n_factors)
    n <- input$n_factors
    div(
      style = "
      display: flex;
      justify-content: center;
      flex-wrap: wrap;
      gap: 8px;
      margin-top: 5px;
    ",
      
      lapply(1:n, function(i) {
        textInput(
          inputId = paste0("factor_name_", i),
          label = NULL,
          value = paste0("Factor ", i),
          width = "100px"
        )
      }),
      br(),
    )
  })
  
  # ==== EFA ====
  efa_result <- reactiveVal(NULL)
  
  observeEvent(c(input$run_efa, input$n_factors), {
    req(data_user(), input$selected_vars)
    req(input$run_efa > 0 || !is.null(efa_result())) # Only run if button clicked or already loaded
    
    withProgress(message = "Running Exploratory Factor Analysis (EFA)...", value = 0, {
      incProgress(0.1, detail = "Preparing data...")
      
      df <- as.data.frame(data_user())[ , input$selected_vars, drop = FALSE]
      
      # ensure numeric columns
      nonnum <- which(!sapply(df, is.numeric))
      if (length(nonnum) > 0) {
        df[nonnum] <- lapply(df[nonnum], function(x) as.numeric(as.character(x)))
      }
      
      # handle missing
      if (input$missing_method_efa == "mean") {
        for (j in seq_along(df)) df[is.na(df[, j]), j] <- mean(df[, j], na.rm = TRUE)
        missing_arg <- "no"
      } else if (input$missing_method_efa == "pairwise") {
        missing_arg <- "pairwise"
      } else {
        df <- na.omit(df)
        missing_arg <- "listwise"
      }
      
      incProgress(0.3, detail = "Computing KMO and Bartlett tests...")
      # compute KMO and Bartlett
      kmo_res <- tryCatch(psych::KMO(df), error = function(e) NULL)
      bartlett_res <- tryCatch(
        psych::cortest.bartlett(cor(df, use = "pairwise.complete.obs"), n = nrow(df)),
        error = function(e) NULL
      )
      
      incProgress(0.6, detail = "Performing Parallel Analysis...")
      par <- tryCatch(
        psych::fa.parallel(df, n.iter = 20, main = "Parallel Analysis (fa.parallel)"),
        error = function(e) NULL
      )
      
      incProgress(0.9, detail = paste("Extracting", input$n_factors, "factors..."))
      fa_res <- tryCatch(
        psych::fa(df, nfactors = input$n_factors, rotate = input$rotation_method),
        error = function(e) NULL
      )
      
      incProgress(1, detail = "Finalizing results...")
      
      efa_out <- list(
        kmo = kmo_res,
        bartlett = bartlett_res,
        parallel = par,
        fa = fa_res,
        loadings = if (!is.null(fa_res)) fa_res$loadings else NULL,
        df_used = df,
        scores = as.data.frame(psych::factor.scores(df, fa_res, Phi = NULL,rho=NULL,missing=FALSE,impute="mean")$scores)
      )
      
      efa_result(efa_out)
    })
  }, ignoreInit = TRUE)

  
  # ==== Pindah ke tab hasil EFA setelah dijalankan ====
  observeEvent(input$run_efa, {
    updateTabsetPanel(session, "main_tab_fa", selected = "KMO & Bartlett Tests")
  })
  
  
  # ==== Tampil hasil tes EFA dengan gaya HTML ====
  output$efa_tests <- renderUI({
    req(efa_result())
    out <- efa_result()
    n_fact <- if (!is.null(out$parallel$nfact)) out$parallel$nfact else NA
    n_comp <- if (!is.null(out$parallel$ncomp)) out$parallel$ncomp else NA
    bart <- out$bartlett
    kmo <- out$kmo
    
    # ==== Format hasil KMO ====
    if (!is.null(kmo)) {
      MSA_overall <- round(kmo$MSA, 3)
      interpret <- if (MSA_overall >= 0.90) {
        "Excellent sampling adequacy — the data are highly suitable for factor analysis."
      } else if (MSA_overall >= 0.80) {
        "Meritorious sampling adequacy — the data are suitable for factor analysis."
      } else if (MSA_overall >= 0.70) {
        "Middling sampling adequacy — acceptable for factor analysis."
      } else if (MSA_overall >= 0.60) {
        "Mediocre sampling adequacy — factor analysis may still be appropriate."
      } else if (MSA_overall >= 0.50) {
        "Miserable sampling adequacy — consider improving data quality or sample size."
      } else {
        "Unacceptable sampling adequacy — factor analysis is not recommended."
      }
      
      kmo_html <- paste0(
        "<ul style='margin-top:4px;'>",
        "<li><b>Overall MSA:</b> ", MSA_overall, "</li>",
        "</ul>",
        "<p style='margin-left:10px; color:#2c3e50;'>", interpret, "</p>"
      )
    } else {
      kmo_html <- "<p style='color:#999;'>KMO test could not be computed (check data).</p>"
    }
    
    # ==== Format hasil Bartlett ====
    if (!is.null(bart)) {
      pval <- bart$p.value
      signif_text <- if (pval < 0.05) {
        "<span style='color:#2ecc71;font-weight:500;'>Significant (p < 0.05)</span> — the correlation matrix is <b>not an identity matrix</b>, indicating that factor analysis is appropriate."
      } else {
        "<span style='color:#e74c3c;font-weight:500;'>Not significant (p ≥ 0.05)</span> — the correlation matrix is <b>close to an identity matrix</b>, suggesting that factor analysis may <b>not be suitable</b> for this data."
      }
      
      bartlett_html <- paste0(
        "<ul style='margin-top:4px;'>",
        "<li><b>Chi-Square:</b> ", round(bart$chisq, 2), "</li>",
        "<li><b>df:</b> ", bart$df, "</li>",
        "<li><b>p-value:</b> ", format.pval(pval, digits = 3), "</li>",
        "</ul>",
        "<p style='margin-left:10px;'>", signif_text, "</p>"
      )
    } else {
      bartlett_html <- "<p style='color:#999;'>Bartlett test could not be computed (check data).</p>"
    }
    
    # ==== Gabungkan semua hasil ====
    HTML(paste0(
      "<div style='font-family:Segoe UI, sans-serif; line-height:1.6; background:#f9f9f9;
               border-radius:12px; padding:12px 18px; border:1px solid #ddd;'>",
      
      # --- Parallel Analysis ---
      "<h4 style='font-weight:bold; color:#2c3e50; margin-bottom:6px;'>📊 Parallel Analysis Recommendation</h4>",
      "<p style='margin-left:10px;'>
       <a>Suggested Number of Factors =</a> ", n_fact, "<br>
       <a>Suggested Number of Components =</a> ", n_comp, "
     </p>",
      
      "<hr style='border: none; border-top: 1px solid #ccc;'>",
      
      # --- KMO and MSA ---
      "<h4 style='font-weight:bold; color:#2c3e50; margin-bottom:6px;'>🧮 Kaiser-Meyer-Olkin (KMO) Measure</h4>",
      kmo_html,
      
      "<hr style='border: none; border-top: 1px solid #ccc;'>",
      
      # --- Bartlett's Test ---
      "<h4 style='font-weight:bold; color:#2c3e50; margin-bottom:6px;'>🧪 Bartlett's Test of Sphericity</h4>",
      bartlett_html,
      
      "</div>"
    ))
  })
  
  
  # ==== Plot KMO per Item (Lollipop Chart) ====
  output$kmo_item <- renderPlot({
    req(efa_result())
    out <- efa_result()
    kmo_res <- out$kmo
    req(!is.null(kmo_res))
    # Ambil nilai MSA per item (measure of sampling adequacy)
    kmo_values <- kmo_res$MSAi
    MSA_all <- data.frame(Item = 'Overall', KMO= kmo_res$MSA)
    kmo_data <- data.frame(
      Item = names(kmo_values),
      KMO = as.numeric(kmo_values)
    ) %>% rbind(MSA_all)
    
    # Tambahkan kategori kualitas KMO
    kmo_data <- kmo_data %>%
      dplyr::mutate(Kategori = dplyr::case_when(
        KMO < 0.5 ~ "Unacceptable",
        KMO < 0.6 ~ "Miserable",
        KMO < 0.7 ~ "Mediocre",
        KMO < 0.8 ~ "Meritorious",
        TRUE ~ "Marvelous"
      ))
    
    # Skema warna kategori
    kategori_colors <- c(
      "Unacceptable" = "#d73027",
      "Miserable"    = "#fc8d59",
      "Mediocre"     = "#fee08b",
      "Meritorious"  = "#4575b4",
      "Marvelous"    = "blue"
    )
    kmo_data$Item <- factor(kmo_data$Item, levels = kmo_data$Item)
    
    # Plot lollipop
    ggplot(kmo_data, aes(x = KMO, y = Item)) +
      geom_segment(aes(x = 0, xend = KMO, yend = Item, color = Kategori), linewidth = 1.5) +
      geom_point(aes(color = Kategori), size = 4) +
      # === Tambahkan label nilai di ujung ===
      geom_text(aes(label = round(KMO, 2)), 
                hjust = -0.3,  # geser sedikit ke kanan
                size = 3, 
                color = "black") +
      geom_vline(xintercept = 0.5, linetype = "dashed", color = "red", linewidth = 1) +
      scale_color_manual(values = kategori_colors) +
      labs(
        title = "Kaiser-Meyer-Olkin (KMO) per Item",
        subtitle = "Measure of Sampling Adequacy (MSA)",
        x = "MSA Value",
        y = "Item",
        color = "Category"
      ) +
      xlim(0, 1) +
      theme_minimal(base_size = 13) +
      theme(
        plot.title = element_text(face = "bold", size = 15),
        plot.subtitle = element_text(size = 11),
        axis.text.y = element_text(face = "bold"),
        legend.position = "bottom"
      )
  })
  
  # Scree plot behaviour:
  output$scree_plot <- renderPlot({
    req(efa_result())
    out <- efa_result()
    df <- out$df_used
    invisible(capture.output(
      psych::fa.parallel(df, fm = "ml", n.iter = 20, main = "Parallel Analysis (fa.parallel)")
    ))    
  })

  # ==== Output Ringkasan Hasil FA (Centered) ====
  output$efa_summary <- renderUI({
    req(efa_result())
    out <- efa_result()
    
    if (is.null(out$fa)) {
      return(tags$p("No factor analysis result available.", style = "color: #999; font-style: italic; text-align:center;"))
    }
    
    fa <- out$fa
    n_factors <- fa$factors
    method <- fa$method
    rotation <- fa$rotation
    total_var <- sum(fa$Vaccounted["Proportion Var", ]) * 100
    
    # --- Ambil nama faktor dari input teks ---
    factor_names <- sapply(1:n_factors, function(i) {
      input[[paste0("factor_name_", i)]] %||% paste0("Factor ", i)
    })
    
    # --- Matriks loading ---
    L <- as.matrix(fa$loadings)
    if (is.null(colnames(L))) colnames(L) <- paste0("Factor", seq_len(ncol(L)))
    if (length(factor_names) == ncol(L)) colnames(L) <- factor_names
    rown <- rownames(L)
    
    # Tentukan loading terbesar per item
    absmat <- abs(L)
    max.idx <- apply(absmat, 1, function(x) if (all(is.na(x))) NA_integer_ else which.max(x))
    
    # --- Buat tabel loading HTML ---
    hdr <- paste0(
      "<tr><th style='text-align:left;padding:6px;border:1px solid #ccc;background:#f8f8f8;'>Item</th>",
      paste0("<th style='padding:6px;border:1px solid #ccc;background:#f8f8f8;'>", colnames(L), "</th>", collapse = ""),
      "</tr>"
    )
    
    rows_html <- vapply(seq_len(nrow(L)), FUN.VALUE = character(1), function(i) {
      cells <- vapply(seq_len(ncol(L)), FUN.VALUE = character(1), function(j) {
        val <- L[i, j]
        if (is.na(val)) txt <- "" else txt <- format(round(val, 3), nsmall = 3)
        if (!is.na(max.idx[i]) && j == max.idx[i] && abs(L[i, j]) > 0) {
          paste0("<td style='background:#b2f0b2;padding:6px;border:1px solid #ccc;text-align:center;font-weight:600;'>", txt, "</td>")
        } else {
          paste0("<td style='padding:6px;border:1px solid #ccc;text-align:center;'>", txt, "</td>")
        }
      }, USE.NAMES = FALSE)
      paste0("<tr><td style='text-align:left;padding:6px;border:1px solid #ccc;'>", rown[i], "</td>", paste(cells, collapse = ""), "</tr>")
    })
    
    tbl_html <- paste0("<table style='margin:auto;border-collapse:collapse;width:auto;font-size:13px;font-family:Arial,Helvetica,sans-serif;'>",
                       hdr, paste(rows_html, collapse = ""), "</table>")
    
    # --- Variance explained ---
    variance_df <- as.data.frame(round(fa$Vaccounted * 100, 2))
    variance_df <- tibble::rownames_to_column(variance_df, var = "Metric")
    if (length(factor_names) == ncol(variance_df) - 1)
      colnames(variance_df)[-1] <- factor_names
    
    # Gunakan table HTML agar tetap center
    variance_tbl <- paste0(
      "<table style='margin:auto;border-collapse:collapse;width:auto;font-size:13px;font-family:Arial,Helvetica,sans-serif;'>",
      paste0(
        "<tr><th style='padding:6px;border:1px solid #ccc;background:#f0f0f0;'>", 
        paste(colnames(variance_df), collapse = "</th><th style='padding:6px;border:1px solid #ccc;background:#f0f0f0;'>"), 
        "</th></tr>"
      ),
      paste(
        apply(variance_df, 1, function(row) {
          paste0("<tr>", paste0("<td style='text-align:center;padding:6px;border:1px solid #ccc;'>", row, "</td>", collapse = ""), "</tr>")
        }),
        collapse = ""
      ),
      "</table>"
    )
    
    tagList(
      div(
        style = "text-align:center;",
        tags$h4("Exploratory Factor Analysis Summary", style = "font-weight:bold;color:#2c3e50;"),
        tags$p(HTML(paste0(
          "<b>Extraction method:</b> ", method, "<br>",
          "<b>Rotation:</b> ", rotation, "<br>",
          "<b>Number of factors extracted:</b> ", n_factors, "<br>",
          "<b>Total variance explained:</b> ", sprintf("%.2f%%", total_var)
        ))),
        tags$h5("Factor Loadings", style = "margin-top:15px;font-weight:bold;"),
        HTML(tbl_html),
        tags$h5("Variance Explained by Each Factor", style = "margin-top:20px;font-weight:bold;"),
        HTML(variance_tbl)
      )
    )
  })
  
  # =====Factor Scores EFA =====
  output$efa_scores_table <- DT::renderDataTable({
    req(efa_result())
    out <- efa_result()
    
    if (is.null(out$fa)) {
      return(tags$p("No factor analysis result available.", style = "color: #999; font-style: italic; text-align:center;"))
    }
    n_factors <- out$fa$factors
    
    # --- Ambil nama faktor dari input teks ---
    factor_names <- sapply(1:n_factors, function(i) {
      input[[paste0("factor_name_", i)]] %||% paste0("Factor ", i)
    })
    
    scores <- efa_result()$scores
    if (length(factor_names) == ncol(scores))
      colnames(scores) <- factor_names
    datatable(
      round(as.data.frame(scores), 3),extensions = 'Buttons',
      options = list(dom='Brtp',scrollX = TRUE, pageLength = 25,  
                     buttons = list(
                       list(extend = 'csv',
                            text = 'Export CSV',
                            filename = 'Factor Scores EFA'
                       ),
                       list(extend = 'excel',
                            text = 'Export Excel',
                            filename = 'Factor Scores EFA'
                       ))),
      caption = "Estimated Factor Scores for Each Observation"
    )
  })
  # ==== 3. DOWNLOAD BUTTONS ====
  output$download_loading <- downloadHandler(
    filename = function() {
      paste0("efa_loadings_", Sys.Date(), ".csv")
    },
    content = function(file) {
      res <- efa_result()
      load_tab <- as.data.frame(round(res$fa$loadings[, ], 3))
      load_tab$Item <- rownames(res$fa$loadings)
      load_tab <- load_tab[, c(ncol(load_tab), 1:(ncol(load_tab)-1))]
      write.csv(load_tab, file, row.names = FALSE)
    }
  )
  
  output$download_scores <- downloadHandler(
    filename = function() {
      paste0("efa_factor_scores_", Sys.Date(), ".csv")
    },
    content = function(file) {
      scores <- efa_result()$scores
      write.csv(round(as.data.frame(scores), 3), file, row.names = FALSE)
    }
  )

  # ==== 4. R Console Output & Model Export ====
  observeEvent(efa_result(), {
    req(efa_result())
    out_res <- efa_result()
    if (!is.null(out_res$fa)) {
      out_text <- paste(capture.output(print(out_res$fa)), collapse = "\n")
      console_context$text <- out_text
    } else {
      console_context$text <- "Error in EFA model computation."
    }
  })
  
  output$export_efa_rds <- downloadHandler(
    filename = function() {
      paste0("EFA_Workspace_", Sys.Date(), ".rds")
    },
    content = function(file) {
      req(efa_result())
      workspace <- list(
        type = "projectLSA_workspace",
        module = "EFA",
        raw_data = raw_data_user(),
        efa_result = efa_result(),
        efa_aggregations = efa_aggregations()
      )
      saveRDS(workspace, file)
    }
  )

  # ==== 5. Score New Data ====
  output$download_efa_template <- downloadHandler(
    filename = function() { "EFA_template.xlsx" },
    content = function(file) {
      req(efa_result())
      items <- rownames(efa_result()$fa$loadings)
      df <- data.frame(matrix(ncol = length(items), nrow = 0))
      colnames(df) <- items
      writexl::write_xlsx(df, file)
    }
  )

  efa_newscores_reactive <- eventReactive(input$efa_score_newdata_btn, {
    req(efa_result(), input$efa_newdata)
    ext <- tools::file_ext(input$efa_newdata$name)
    df <- switch(
      ext,
      "csv" = read.csv(input$efa_newdata$datapath),
      "xlsx" = readxl::read_excel(input$efa_newdata$datapath),
      "xls" = readxl::read_excel(input$efa_newdata$datapath),
      stop("Invalid file format")
    )
    
    # Apply user-defined calculated variables to the new data
    aggs <- efa_aggregations()
    if (length(aggs) > 0) {
      for (agg in aggs) {
        vars <- agg$vars
        name <- agg$name
        method <- agg$method

        valid_vars <- intersect(vars, names(df))
        if (length(valid_vars) > 0) {
          if (method == "mean") {
            df[[name]] <- rowMeans(df[, valid_vars, drop = FALSE], na.rm = TRUE)
          } else if (method == "sum") {
            df[[name]] <- rowSums(df[, valid_vars, drop = FALSE], na.rm = TRUE)
          }
        }
      }
    }
    
    # Calculate scores
    fa_model <- efa_result()$fa
    # Make sure we only use the columns matching the items in the model
    items <- rownames(fa_model$loadings)
    df_used <- df[, items, drop = FALSE]
    
    scores <- psych::factor.scores(df_used, fa_model, missing = FALSE, impute = "mean")$scores
    
    n_factors <- fa_model$factors
    factor_names <- sapply(1:n_factors, function(i) {
      input[[paste0("factor_name_", i)]] %||% paste0("Factor ", i)
    })
    
    if (length(factor_names) == ncol(scores)) {
      colnames(scores) <- factor_names
    }
    
    as.data.frame(scores)
  })

  output$efa_newscores_table <- DT::renderDataTable({
    req(efa_newscores_reactive())
    datatable(round(efa_newscores_reactive(), 3), options = list(scrollX = TRUE))
  })

  output$download_efa_newscores <- downloadHandler(
    filename = function() { paste0("EFA_newscores_", Sys.Date(), ".csv") },
    content = function(file) {
      write.csv(round(efa_newscores_reactive(), 3), file, row.names = FALSE)
    }
  )
  # ==== AI Assistant ====
  # Update global AI context whenever results change
  observe({
    res_text <- ""
    if (!is.null(efa_result())) {
      fit <- efa_result()
      res_text <- paste(capture.output(print(fit)), collapse = "\n")
    }
    ai_context$results_text <- res_text
    ai_context$module <- "Exploratory Factor Analysis (EFA)"
  })

  # Per-session directory, so concurrent users never share a report file
  efa_report_res <- session_report_dir(session, "efa")
  efa_report_path <- reactiveVal(NULL)
  
  # Renders the EFA report and returns the path of the generated HTML file
  # (NULL on failure). Shared by the preview button and the download handler.
  generate_efa_report <- function(progress_label = "Generating Report...") {
    if (is.null(efa_result())) {
      return(NULL)
    }

    report_path <- file.path(system.file("app", package = "projectLSA"), "efa_report.Rmd")
    if (report_path == "" || !file.exists(report_path)) {
      report_path <- "efa_report.Rmd"
    }
    
    tempReport <- file.path(efa_report_res$path, "efa_report.Rmd")
    file.copy(report_path, tempReport, overwrite = TRUE)
    
    out_html <- file.path(efa_report_res$path, "efa_report_out.html")
    
    showModal(modalDialog(progress_label, footer = NULL))
    ok <- tryCatch({
      rmarkdown::render(tempReport, output_file = out_html,
        params = list(
          efa_res = efa_result(),
          console_out = console_context$text,
          ai_summary = if (is.null(ai_context$ai_report_text)) "" else ai_context$ai_report_text
        ),
        envir = new.env(parent = globalenv())
      )
      efa_report_path(out_html)
      TRUE
    }, error = function(e) {
      showNotification(paste("Error rendering report:", e$message), type = "error")
      FALSE
    }, finally = {
      removeModal()
    })

    if (isTRUE(ok)) out_html else NULL
  }

  observeEvent(input$efa_generate_preview, {
    req(efa_result())
    generate_efa_report("Generating Report Preview...")
  })

  output$efa_report_preview_frame <- renderUI({
    req(efa_report_path())
    tags$iframe(
      src = paste0(efa_report_res$prefix, "/efa_report_out.html?v=", as.integer(Sys.time())),
      width = "100%", height = "800px", style = "border: none;"
    )
  })

  output$download_report_efa <- downloadHandler(
    filename = function() {
      paste0("EFA_Report_", Sys.Date(), ".html")
    },
    content = function(file) {
      req(efa_result())
      path <- efa_report_path()
      if (is.null(path) || !file.exists(path)) {
        path <- generate_efa_report("Generating Report...")
      }
      req(path)
      file.copy(path, file, overwrite = TRUE)
    },
    contentType = "text/html"
  )
}

# ==== server_lpa.R ====
server_lpa <- function(input, output, session, ai_context, console_context) {
  library(tidyLPA)
  library(mclust)
  library(ggplot2)
  library(stats)
  library(haven)
  library(data.table)
  set.seed(100)

  # Results are held in reactiveVal (instead of eventReactive) so that a saved
  # workspace can be restored straight into them.
  fitcompare <- reactiveVal(NULL)
  models <- reactiveVal(NULL)

  # Variables selected in a restored workspace, applied once the picker exists
  lpa_restored_vars <- reactiveVal(NULL)

  # --- Upload data ----
  data_user <- reactive({
    if (input$data_source == "pisaUSA15") {
      df <- tidyLPA::pisaUSA15[1:500,]%>% mutate(id_auto = paste0("id_", sprintf("%04d", 1:n())))
    } else if (input$data_source == "curry_mac") {
      df <- tidyLPA::curry_mac %>% mutate(id_auto = paste0("id_", sprintf("%04d", 1:n())))

    } else if (input$data_source == "id_edu") {
      df <- tidyLPA::id_edu %>% mutate(id_auto = paste0("id_", sprintf("%04d", 1:n())))
    } else {
      req(input$datafile)
      ext <- tolower(tools::file_ext(input$datafile$name))
      showModal(modalDialog(title = NULL, "Reading Your File, Please wait...", footer = NULL, easyClose = FALSE))

      if (ext == "rds") {
        res <- readRDS(input$datafile$datapath)
        if (is.list(res) && !is.data.frame(res) && identical(res$type, "projectLSA_workspace")) {
          if (!identical(res$module, "LPA")) {
            removeModal()
            stop("Uploaded workspace belongs to a different module: ", res$module)
          }
          # Restore workspace state
          fitcompare(res$fitcompare)
          models(res$models)
          df <- res$raw_data

          # Restore UI inputs
          st <- res$input_state
          if (!is.null(st)) {
            if (!is.null(st$min_profiles)) updateNumericInput(session, "min_profiles", value = st$min_profiles)
            if (!is.null(st$max_profiles)) updateNumericInput(session, "max_profiles", value = st$max_profiles)
            if (!is.null(st$model_to_run)) updateSelectInput(session, "model_to_run", selected = st$model_to_run)
            if (!is.null(st$model_type))   updateSelectInput(session, "model_type", selected = st$model_type)
            if (!is.null(st$best_k))       updateNumericInput(session, "best_k", value = st$best_k)
            if (!is.null(st$selected_vars)) {
              # var_select_ui is rendered from the data, so defer the selection
              lpa_restored_vars(st$selected_vars)
            }
          }
          showNotification("LPA Workspace restored successfully!", type = "message")
        } else {
          df <- res
        }
      } else {
        df <- switch(
          ext,
          "csv"  = data.table::fread(input$datafile$datapath,data.table = FALSE),
          "xls"  = readxl::read_excel(input$datafile$datapath),
          "xlsx" = readxl::read_excel(input$datafile$datapath),
          "sav"  = haven::read_sav(input$datafile$datapath),
          stop("Unsupported file type. Please upload CSV, Excel, SPSS (.sav), or RDS file.")
        )
      }
      removeModal()
      if (!"id_auto" %in% names(df)) {
        df <- df %>% mutate(across(everything(), ~ifelse(.x == "", NA, .x)),
                            id_auto = paste0("id_", sprintf("%04d", 1:n())))
      }
    }
    return(df)
  })
  observeEvent(input$data_source, {
    if (input$data_source != "upload") {
      showNotification(paste("Using built-in dataset:", input$data_source), type = "message")
    }
  })
  output$data_description <- renderUI({
    desc_html <- if (input$data_source == "pisaUSA15") {
      desc_pisa
    } else if (input$data_source == "curry_mac") {
      desc_curry_text
    } else if (input$data_source == "id_edu") {
      desc_id_edu
    } else { ""
    }
    
    div(
      class = "badge-info",
      HTML(desc_html)
    )
  })
  # ---  ID ---
  output$id_select_ui <- renderUI({
    req(data_user())
    selectInput(
      "id_vars",
      label = "Select ID Columns (Optional):",
      choices = names(data_user()),
      selected = names(data_user())[str_detect(names(data_user()), "id")],
      multiple = TRUE
      )
  })
  
  # --- Pilih variabel ---
  output$var_select_ui <- renderUI({
    req(data_user())
    restored <- lpa_restored_vars()
    sel <- if (!is.null(restored)) {
      intersect(restored, names(data_user()))
    } else {
      names(data_user())[1:min(3, ncol(data_user()))]
    }
    selectInput(
      "selected_vars",
      label = "Select Variables for LPA:",
      choices = names(data_user()),
      selected = sel,
      multiple = TRUE
      )
  })

  observeEvent(c(input$data_source, input$datafile), {
    # Keep the selection that came from a restored workspace
    if (is.null(lpa_restored_vars())) {
      updateSelectInput(session, "selected_vars", selected = "")
    }
  }, ignoreInit = TRUE)
  
  # --- Data preview ----
  output$data_preview <- renderDT({
    req(data_user(), input$selected_vars)
    df <- data_user() %>% 
      dplyr::select(input$selected_vars)
    numeric_cols <- which(sapply(df, function(x) is.numeric(x)))
    datatable(df,
              extensions = 'Buttons',
              options = list(dom='Brtp',scrollX = TRUE, #pageLength = nrow(df),
                             buttons = list(
                               list(extend = 'csv',
                                    text = 'Export CSV',
                                    filename = 'Data LPA'
                               ),
                               list(extend = 'excel',
                                    text = 'Export Excel',
                                    filename = 'Data LPA'
                               ))),
      rownames = T) %>% 
      formatRound(columns = numeric_cols, digits = 2)
  }, server = TRUE)
  
  output$data_summary_lpa <- DT::renderDT({
    req(data_user(), input$selected_vars)
    
    df <- data_user() %>% 
      dplyr::select(all_of(input$selected_vars))
    
    table_summary <- data.frame(
      N        = sapply(df, function(x) sum(!is.na(x))),
      Missing  = sapply(df, function(x) sum(is.na(x))),
      Mean     = sapply(df, function(x) mean(x, na.rm = TRUE)),
      SD       = sapply(df, function(x) sd(x, na.rm = TRUE)),
      Variance = sapply(df, function(x) var(x, na.rm = TRUE)),
      Min      = sapply(df, function(x) min(x, na.rm = TRUE)),
      Max      = sapply(df, function(x) max(x, na.rm = TRUE))
    )
    
    # Transpose supaya Stat = baris, Variabel = kolom
    table_summary <- t(table_summary)
    # Round Mean, SD, Variance ke 2 desimal
    table_summary[c("Mean", "SD", "Variance","Min", "Max"), ] <- round(table_summary[c("Mean", "SD", "Variance", "Min", "Max"), ], 2)
    
    DT::datatable(
      table_summary,
      extensions = "Buttons",
      options = list(
        scrollX = TRUE,
        dom = "Bt",
        buttons = list(
          list(
            extend = "csv",
            text = "Export CSV",
            filename = "Data_LPA_Summary"
          ),
          list(
            extend = "excel",
            text = "Export Excel",
            filename = "Data_LPA_Summary"
          )
        )
      ),
      rownames = TRUE
    )
  }, server = FALSE)
 
  
  # --- Jalankan LPA ----
  observeEvent(input$run_lpa, {
    req(data_user(), input$selected_vars)
    updateTabsetPanel(session, "main_tab_lpa", selected = "fit_tab")
  })
  observeEvent(input$run_lpa, {
    req(data_user(), input$selected_vars, input$model_to_run)
    # A fresh run invalidates any restored workspace selection
    lpa_restored_vars(NULL)
    showModal(modalDialog(title = NULL, "Please wait, running LPA...", footer = NULL, easyClose = FALSE))
    df <- data_user() %>% dplyr::select(input$selected_vars)
    df <- df %>% mutate(across(everything(), as.numeric))
    min_k <- input$min_profiles
    max_k <- input$max_profiles
    
    model_spec <- tibble::tibble(
      model = c(1, 2, 3, 6),
      variances = c("equal", "varying", "equal", "varying"),
      covariances = c("zero", "zero", "equal", "varying")
    )

    model_to_run <- input$model_to_run
    
    spec_selected <- model_spec %>%
      dplyr::filter(model %in% model_to_run) %>%
      dplyr::arrange(match(model, model_to_run))
    
    variances   <- spec_selected$variances
    covariances <- spec_selected$covariances
    
    withProgress(message = "Running...", {
      incProgress(1/(max_k-min_k+1), detail = paste("Compare LPA results"))
      fit_res <- df %>%
        tidyLPA::single_imputation() %>%   # ← ini kuncinya
        tidyLPA::estimate_profiles(
          n_profiles = min_k:max_k,
          variances  = variances,
          covariances = covariances
          #model = model_to_run
        ) %>%
        tidyLPA::get_fit()
      fit_res <- as.data.frame(fit_res) %>% dplyr::rename(n_profiles=Classes)
      removeModal()
      showNotification("LPA completed successfully!", type = "message")
    })
    fitcompare(fit_res)
  })

  # Also re-estimate when a new comparison run finishes, so the Best Model tab is
  # populated straight after "Run LPA" instead of waiting for the user to touch a control.
  observeEvent(c(input$best_k, input$model_type, fitcompare()), {
    req(data_user(), fitcompare(),input$selected_vars, input$best_k)
    df <- data_user()[, input$selected_vars]
    df <- df %>% mutate(across(everything(), as.numeric))
    min_k <- input$min_profiles
    max_k <- input$max_profiles
    model_type <- input$model_type
    variances <- ifelse(input$model_type %in% c(1, 3), "equal", "varying")
    covariances <- ifelse(input$model_type %in% c(1,2), "zero",
                          ifelse(input$model_type %in% c(3), "equal", "varying"))
    model_list <- list()
    for (k in input$best_k) {
      withProgress(message = "Running...", {
      incProgress(1/(max_k-min_k+1), detail = paste("Estimate LPA Model for ", k, " Profiles"))
      model_list[[as.character(k)]] <- df %>%
        tidyLPA::single_imputation() %>%   # ← ini kuncinya
        tidyLPA::estimate_profiles(
          n_profiles = k,
          variances = variances,
          covariances = covariances
        )
      showNotification("Estimation completed successfully!", type = "message")
      })
    }
    models(model_list)
   
  })
  
  # --- Fit table ----
  output$fit_table <- renderDT({
    req(fitcompare())
    fitcompare <- fitcompare() %>%
      dplyr::mutate(across(where(is.numeric), ~round(.x, 2))) %>%
      dplyr::select(-c(AWE, CAIC, CLC, KIC, SABIC, ICL))
    prob_min_vals <- sort(unique(fitcompare$prob_min), decreasing = TRUE)
    prob_max_vals <- sort(unique(fitcompare$prob_max),decreasing = TRUE)
    aic_vals <- sort(unique(fitcompare$AIC))
    bic_vals <- sort(unique(fitcompare$BIC))
    ent_vals <- sort(unique(fitcompare$Entropy), decreasing = TRUE)
    
    datatable(fitcompare, extensions = 'Buttons',
              options = list(dom='B', scrollX = TRUE, pageLength = 40,
                             buttons = list(
                               list(extend='csv', text='Export CSV', filename='FIT Comparison LPA'),
                               list(extend='excel', text='Export Excel', filename='FIT Comparison LPA')
                             )),
              rownames = FALSE) %>%
      formatStyle('AIC',
                  backgroundColor = styleEqual(
                    c(aic_vals[1], aic_vals[2]),
                    c('lightgreen', 'khaki')
                  ),
                  fontWeight = styleEqual(aic_vals[1], 'bold')
      ) %>%
      formatStyle('BIC',
                  backgroundColor = styleEqual(
                    c(bic_vals[1], bic_vals[2]),
                    c('lightgreen', 'khaki')
                  ),
                  fontWeight = styleEqual(bic_vals[1], 'bold')
      ) %>%
      formatStyle('Entropy',
                  backgroundColor = styleInterval(c(0.7, 0.8), c('lightcoral', 'khaki', 'lightgreen')),
                  fontWeight      = styleInterval(c(0.7, 0.8), c('bold', 'bold',''))
      ) %>%
      formatStyle('prob_min',
                  backgroundColor = styleEqual(
                    c(prob_min_vals[1], prob_min_vals[2]),
                    c('lightgreen', 'khaki')
                  ),
                  fontWeight = styleEqual(ent_vals[1], 'bold')
      ) %>%
      formatStyle('prob_max',
                  backgroundColor = styleEqual(
                    c(prob_max_vals[1], prob_max_vals[2]),
                    c('lightgreen', 'khaki')
                  ),
                  fontWeight = styleEqual(ent_vals[1], 'bold')
      ) %>%
      formatStyle('n_min',
                  backgroundColor = styleInterval(
                    c(0.05, 0.07),
                    c('lightcoral', 'khaki', 'lightgreen'))
                  ) %>%
      formatStyle('BLRT_p',
                  backgroundColor = styleInterval(0.05, c('lightgreen', 'lightcoral')),
                  fontWeight = styleInterval(0.05, c('bold', ''))
      )
  }, server = FALSE)
  fit_bic_reactive <- reactive({
    req(fitcompare())
    fit_bic <- fitcompare() %>%
      dplyr::select(Model, n_profiles, BIC) %>%
      dplyr::mutate(n_profiles = factor(n_profiles), Model = factor(Model))
    
    ggplot(fit_bic, aes(n_profiles, BIC, group = Model, shape = Model, color = Model)) +
      geom_line(size = 1.2, linetype = 4) + geom_point(size = 3) +
      geom_text(aes(label = round(BIC, 2)), vjust = -0.6, size = 4, show.legend = FALSE) +
      labs(subtitle  = "BIC Comparison Across Models", x = "Number of Profiles", y = "BIC") +
      theme_minimal(base_size = 14) +
      theme(legend.position = "bottom", axis.line = element_line(color = "black"))
  })
  fit_aic_reactive <- reactive({
    req(fitcompare())
    fit_aic <- fitcompare() %>%
      dplyr::select(Model, n_profiles, AIC) %>%
      dplyr::mutate(n_profiles = factor(n_profiles), Model = factor(Model))
    
    ggplot(fit_aic, aes(n_profiles, AIC, group = Model, shape = Model, color = Model)) +
      geom_line(size = 1.2, linetype = 5) + geom_point(size = 3) +
      geom_text(aes(label = round(AIC, 2)), vjust = -0.6, size = 4, show.legend = FALSE) +
      labs(subtitle = "AIC Comparison Across Models", x = "Number of Profiles", y = "AIC") +
      theme_minimal(base_size = 14) +
      theme(legend.position = "bottom", axis.line = element_line(color = "black"))
  })
  fit_entropy_reactive <- reactive({
    req(fitcompare())
    fit_entropy <- fitcompare() %>%
      dplyr::select(Model, n_profiles, Entropy) %>%
      dplyr::mutate(n_profiles = factor(n_profiles), Model = factor(Model))
    
    ggplot(fit_entropy, aes(n_profiles, Entropy, group = Model, shape = Model,color = Model)) +
      geom_line(size = 1.2, linetype = 2) + geom_point(size = 3) +
      geom_text(aes(label = round(Entropy, 2)), vjust = -0.6, size = 4, show.legend = FALSE) +
      labs(subtitle = "Entropy Comparison Across Models", x = "Number of Profiles", y = "Entropy") +
      theme_minimal(base_size = 14) +
      theme(legend.position = "bottom", axis.line = element_line(color = "black"))
  })
  fit_class_size_reactive <- reactive({
    req(fitcompare())
    fit_class_size <- fitcompare() %>%
      dplyr::select(Model, n_profiles, n_min) %>%
      dplyr::mutate(n_profiles = factor(n_profiles), Model = factor(Model))
    
    ggplot(fit_class_size, aes(n_profiles, n_min, group = Model, shape = Model, color = Model)) +
      geom_line(size = 1.2, linetype = 1) + geom_point(size = 4) +
      geom_text(aes(label = round(n_min, 2)), vjust = -0.6, size = 4, show.legend = FALSE) +
      labs(subtitle = "Min Class Size Comparison Across Models", x = "Number of Profiles", y = "Minimum Class Size") +
      theme_minimal(base_size = 14) +
      theme(legend.position = "bottom", axis.line = element_line(color = "black"))
  })
  
  # ==== Render Plot ====
  output$fit_bic <- renderPlot({
    fit_bic_reactive()
  })
  output$fit_aic <- renderPlot({
    fit_aic_reactive()
  })
  output$fit_entropy <- renderPlot({
    fit_entropy_reactive()
  })
  output$fit_class_size <- renderPlot({
    fit_class_size_reactive()
  })

  # ==== Download Buttons ====
  # output$download_plot_AicBic_LPA <- make_download_plot(
  #   plot_reactive = fit_plot_reactive,
  #   filename_prefix = "Fit AIC BIC Plot_LPA"
  # )
  # 
  # output$download_plot_entropy_LPA <- make_download_plot(
  #   plot_reactive = entropy_plot_reactive,
  #   filename_prefix = "EntropyPlot_LPA"
  # )
  # --- Input nama profil ---
  output$profile_name_inputs <- renderUI({
    req(input$best_k)
    lapply(1:input$best_k, function(i) {
      textInput(paste0("profile_name_", i), paste("Profile", i, "Name:"), value = paste("Profile", i))
    })
  })
  best_model_plot_reactive <- reactive({
    req(models(), input$best_k, input$selected_vars)
    
    k <- as.character(input$best_k)
    model_k <- models()[[k]]
    params  <- tidyLPA::get_estimates(model_k)
    
    # === Profile names (AMAN) ===
    profile_names <- sapply(1:input$best_k, function(i) {
      nm <- input[[paste0("profile_name_", i)]]
      if (is.null(nm) || nm == "") paste0("Profile ", i) else as.character(nm)
    })
    
    # === Ambil Means + SE ===
    plot_df <- params %>%
      dplyr::filter(
        Category == "Means",
        Parameter %in% input$selected_vars
      ) %>%
      dplyr::mutate(
        Profile = factor(
          Class,
          levels = seq_along(profile_names),
          labels = profile_names
        ),
        Variable = factor(Parameter, levels = input$selected_vars),
        Mean = Estimate,
        SE   = se
      )
    
    # === Plot ===
    ggplot(plot_df,
           aes(x = Variable, y = Mean,
               group = Profile,
               color = Profile,
               linetype = Profile,
               shape = Profile)) +
      geom_line(linewidth = 1.2) +
      geom_point(size = 3) +
      
      # === Error bar (Mean ± SE) ===
      geom_errorbar(
        aes(ymin = Mean - SE, ymax = Mean + SE),
        width = 0.15,
        linewidth = 0.8
      ) +
      
      geom_text(
        aes(label = round(Mean, 2)),
        vjust = -0.9,
        size = 4,
        show.legend = FALSE
      ) +
      
      labs(
        title = paste0("Profile Plot Based on Model Estimates (K = ", k, ")"),
        x = "Indicator",
        y = "Estimated Mean (± SE)"
      ) +
      
      theme_minimal(base_size = 14) +
      theme(
        legend.position = "bottom",
        axis.line = element_line(color = "black"),
        panel.grid.minor = element_blank()
      )
  })
  
  # RenderP lot
  output$best_model_plot <- renderPlot({
    best_model_plot_reactive()
  })
  
  # ==== Download Buttons ====
  output$download_plot_best_LPA <- make_download_plot(
    plot_reactive = best_model_plot_reactive,
    filename_prefix = "BestPlot_LPA"
  )
  #=======
  summary_data <- reactive({
    req(models(), input$best_k)
    
    model_k <- models()[[as.character(input$best_k)]]
    df_cls  <- get_data(model_k)
    params  <- tidyLPA::get_estimates(model_k)
  
    profile_names <- sapply(1:input$best_k, function(i) input[[paste0("profile_name_", i)]])
    df_cls$Profile <- factor(df_cls$Class, labels = profile_names)
    class_size <- df_cls %>%
          dplyr::count(Profile) %>%
          dplyr::mutate(Percent = round(100 * n / sum(n), 2))
    
    # === Mean & Variance (model-based) ===
    mean_var_tbl <- params %>%
      dplyr::filter(Category %in% c("Means", "Variances")) %>%
      dplyr::mutate(Profile = factor(Class, labels = profile_names)) %>%
      dplyr::select(Profile, Variable = Parameter, Category, Estimate) %>%
      tidyr::pivot_wider(
        names_from  = Category,
        values_from = Estimate
      ) %>%
      dplyr::mutate(
        MeanVar = sprintf("%.2f (%.2f)", Means, Variances)
      ) %>%
      dplyr::select(Profile, Variable, MeanVar)
    
    # === Bentuk tabel ringkas ===
    profile_summary <- mean_var_tbl %>%
      tidyr::pivot_wider(
        names_from  = Variable,
        values_from = MeanVar
      )
    
    # === Gabungkan dengan class size ===
    summary_table <- class_size %>%
      dplyr::left_join(profile_summary, by = "Profile")
    
    summary_table
  })
  # --- Render tabel ke UI ---
  output$summary_table <- renderDT({
    req(summary_data())
    df <- summary_data()
    DT::datatable(df,extensions = 'Buttons',
                  options = list(scrollX = TRUE, dom = 'B',
                                 buttons = list(
                                   list(
                                     extend = 'csv',
                                     text = 'Export CSV',
                                     filename = paste0('Summary LPA')
                                   ),
                                   list(
                                     extend = 'excel',
                                     text = 'Export Excel',
                                     filename = paste0('Summary LPA')
                                   ))),
                  rownames = FALSE)
  })

  
  df_out <- reactive({
    req(models(), input$best_k)
    df <- data_user()
    id_cols <- if (is.null(input$id_vars) || length(input$id_vars) == 0) "id_auto" else input$id_vars
    model_k <- models()[[as.character(input$best_k)]]
    cluster <- get_data(model_k)$Class
    profile_names <- sapply(1:input$best_k, function(i) input[[paste0("profile_name_", i)]])
    df$Profile <- factor(cluster, labels = profile_names)
    #df_out <- df[, c(id_cols, input$selected_vars, "Profile")]
    numeric_cols <- which(sapply(df_out, is.numeric))
    df
  })
  # --- Data profil per ID ---
  output$profile_table <- renderDT({
    req(models(), input$best_k)
    df <- df_out()
    id_cols <- if (is.null(input$id_vars) || length(input$id_vars) == 0) "id_auto" else input$id_vars
    df <- df[, c(id_cols, input$selected_vars, "Profile")]
    numeric_cols <- which(sapply(df, is.numeric))

    datatable(df,
              extensions = 'Buttons',
              options = list(dom='Brtp',scrollX = TRUE, pageLength = 10,
                             buttons = list(
                               list(extend = 'csv',
                                    text = 'Export CSV',
                                    filename = 'Data Profle LPA'
                               ),
                               list(extend = 'excel',
                                    text = 'Export Excel',
                                    filename = 'Data Profile LPA'
                               ))),
              rownames = T) %>%
      formatRound(columns = numeric_cols, digits = 2)
  }, server = FALSE)
  output$var_x_ui <- renderUI({
    req(df_out())
    selectInput(
      "var_x",
      label = "X Absis",
      choices = names(df_out()),
      selected = "Profile",
      multiple = FALSE 
    )
  })
  output$var_y_ui <- renderUI({
    req(df_out())
    selectInput(
      "var_y",
      label = "Y Absis",
      choices = setdiff(names(df_out()), "Profile"),
      selected = "",
      multiple = TRUE
    )
  })
  output$cross_latent <- renderPlot({
    req(models(), input$best_k, input$var_x, input$var_y, input$plot_type)
    df <- df_out()
    
    # =========================
    # 1. RINGKAS DATA
    # =========================
    Summary_long <- df %>%
      dplyr::select(all_of(c(input$var_x, input$var_y))) %>%
      tidyr::pivot_longer(
        cols = all_of(input$var_y),
        names_to = "Variable",
        values_to = "Value"
      ) %>%
      dplyr::group_by(
        Profile  = .data[[input$var_x]],
        Variable
      ) %>%
      dplyr::summarise(
        Mean = mean(Value, na.rm = TRUE),
        .groups = "drop"
      ) %>% 
      dplyr::mutate(
        Variable = factor(Variable, levels = input$var_y)
      )
    
    # =========================
    # 2. BASE PLOT
    # =========================
    p <- ggplot(
      Summary_long,
      aes(
        x = Variable,
        y = Mean,
        fill = Profile,
        color = Profile,
        group = Profile
      )
    )
    
    # =========================
    # 3. SWITCH BAR / LINE
    # =========================
    if (input$plot_type == "bar") {
      
      p <- p +
        geom_col(
          position = position_dodge(width = 0.7),
          width = 0.6
        )
      
    } else {
      
      p <- p +
        geom_line(
          aes(linetype = Profile),
          size = 1.2
        ) +
        geom_point(
          aes(shape = Profile),
          size = 3
        )
    }
    
    # =========================
    # 4. FINAL STYLE
    # =========================
    p +
      geom_text(
        aes(label = round(Mean, 2)),
        position = position_dodge(width = 0.7),
        vjust = -0.7,
        size = 4.5,
        show.legend = FALSE
      ) +
      labs(
        subtitle = paste0("Profile Plot for ", input$best_k, " Profiles"),
        x = "Variable",
        y = "Mean Value"
      ) +
      theme_minimal(base_size = 12) +
      theme(
        axis.text.x = element_text(angle = 90, hjust = 1, vjust = 0.5, size = 9),
        legend.position = "bottom",
        axis.line = element_line(color = "black")
      )
  })
  output$anova_lpa_ui <- renderUI({
    req(models(), input$best_k, input$var_x, input$var_y)
    
    if (isTRUE(input$check_anova)) {
      tagList(
        tags$h5(
          paste0(
            "Latent Profile Differences on ",
            paste(input$var_y, collapse = ", ")
          )
        ),
        tags$h6("Analysis of Variance"),
        tableOutput("anova_lpa_table"),
        uiOutput("posthoc_lpa_ui")
      )
    }
  })
  output$anova_lpa_table <- renderTable({
    req(df_out(), input$var_x, input$var_y)
    
    df <- df_out()
    x  <- input$var_x
    ys <- input$var_y
    
    # =========================
    # CASE 1: ONE-WAY ANOVA
    # =========================
    if (length(ys) == 1) {
      
      fml <- as.formula(paste(ys, "~", x))
      fit <- aov(fml, data = df)
      tbl <- summary(fit)[[1]]
      
      pval <- tbl["Pr(>F)"][1,1]
      
      out <- data.frame(
        Source = rownames(tbl),
        round(tbl, 4),
        row.names = NULL
      )
      
      out$Interpretation <- c(
        ifelse(
          pval < 0.05,
          "Significant difference",
          "No significant difference"
        ),
        rep("", nrow(out) - 1)
      )
      
      out
      
    } else {
      
      # =========================
      # CASE 2: MANOVA
      # =========================
      
      fml <- as.formula(
        paste(
          "cbind(",
          paste(ys, collapse = ","),
          ") ~ ",
          x
        )
      )
      
      fit <- manova(fml, data = df)
      tbl <- summary(fit, test = "Wilks")$stats
      pval <- tbl[1, "Pr(>F)"]
      
      out <- data.frame(
        Effect = rownames(tbl),
        round(tbl, 4),
        row.names = NULL
      )
      
      out$Interpretation <- ifelse(
        out$Effect == x,
        ifelse(
          pval < 0.05,
          "Significant difference",
          "No significant difference"
        ),
        ""
      )
      
      out
    }
  })
  output$posthoc_lpa_ui <- renderUI({
    req(df_out(), input$var_x, input$var_y)
    
    if (length(input$var_y) == 1) {
      tagList(
        tags$h6("Post-hoc Test (Tukey HSD)"),
        tableOutput("posthoc_lpa_table")
      )
    }
  })
  output$posthoc_lpa_table <- renderTable({
    req(df_out(), input$var_x, input$var_y)
    
    # Post-hoc hanya untuk ANOVA
    if (length(input$var_y) != 1) return(NULL)
    
    df <- df_out()
    x  <- input$var_x
    y  <- input$var_y
    
    fml <- as.formula(paste(y, "~", x))
    fit <- aov(fml, data = df)
    
    # p-value ANOVA utama
    p_main <- summary(fit)[[1]]["Pr(>F)"][1, 1]
    
    # Jika tidak signifikan → tampilkan pesan
    if (is.na(p_main) || p_main >= 0.05) {
      return(
        data.frame(
          Note = "Post-hoc test was not conducted because the ANOVA result was not statistically significant."
        )
      )
    }
    
    tuk <- TukeyHSD(fit)[[1]]
    tuk <- round(as.data.frame(tuk), 4)
    
    data.frame(
      Comparison = rownames(tuk),
      tuk,
      Interpretation = ifelse(
        tuk$`p adj` < 0.05,
        "Significant difference",
        "Not significant"
      ),
      row.names = NULL
    )
  })
  output$crosstab_ui <- renderUI({
    req(input$crosstab_result)
    if (!input$crosstab_result) return(NULL)
    
    req(df_out())
    vars <- setdiff(names(df_out()), "Profile")
    fluidRow(
      column(
        12,
        h5("Cross Tabulation Settings"),
        selectInput(
          "crosstab_vars",
          "Select Variables:",
          choices = vars,
          multiple = TRUE
        )
      ),
      column(
        6,
        h5("Cross Tabulation For Categorical Variables"),
        tableOutput("crosstab_cat"),
        plotOutput("crosstab_cat_plot")
      ),
      column(
        6,
        h5("Mean & SD for Continuous Variables"),
        tableOutput("crosstab_cont"),
        plotOutput("crosstab_cont_plot")
      ),
      br(), br(), br()
    )
  })
  output$crosstab_cat <- renderTable({
    req(input$crosstab_result, input$crosstab_vars)
    if (!input$crosstab_result) return(NULL)
    
    df <- df_out()
    
    vars_cat <- input$crosstab_vars[
      sapply(df[input$crosstab_vars], function(x)
        is.factor(x) || is.character(x))
    ]
    
    if (length(vars_cat) == 0) return(NULL)
    
    do.call(rbind, lapply(vars_cat, function(v) {
      tab <- as.data.frame(
        table(Profile = df$Profile, Category = df[[v]])
      )
      
      tab <- tab |>
        dplyr::group_by(Category) |>
        dplyr::mutate(
          Percent = round(Freq / sum(Freq) * 100, 2)
        ) |>
        dplyr::ungroup()
      
      data.frame(
        Profile  = tab$Profile,
        Variable = v,
        Category = tab$Category,
        N        = tab$Freq,
        Percent  = tab$Percent
      )
    }))
  })
  output$crosstab_cont <- renderTable({
    req(df_out(), input$crosstab_vars)
    
    df <- df_out()
    
    vars_num <- input$crosstab_vars[
      sapply(df[input$crosstab_vars], is.numeric)
    ]
    
    if (length(vars_num) == 0) return(NULL)
    
    res <- lapply(vars_num, function(v) {
      aggregate(
        df[[v]],
        by = list(Profile = df$Profile),
        FUN = function(x)
          paste0(round(mean(x, na.rm = TRUE), 2),
                 " (", round(sd(x, na.rm = TRUE), 2), ")")
      ) |>
        setNames(c("Profile", v))
    })
    
    Reduce(function(x, y) merge(x, y, by = "Profile"), res)
  })
  output$crosstab_cat_plot <- renderPlot({
    req(input$crosstab_result, input$crosstab_vars)
    if (!input$crosstab_result) return(NULL)
    
    df <- df_out()
    
    vars_cat <- input$crosstab_vars[
      sapply(df[input$crosstab_vars], function(x)
        is.factor(x) || is.character(x))
    ]
    
    if (length(vars_cat) == 0) return(NULL)
    
    plot_data <- do.call(rbind, lapply(vars_cat, function(v) {
      tab <- as.data.frame(table(Profile = df$Profile, Category = df[[v]]))
      
      tab <- tab |>
        dplyr::group_by(Category) |>
        dplyr::mutate(
          Percent = round(Freq / sum(Freq) * 100, 2)
        ) |>
        dplyr::ungroup()
      
      tab$Variable <- v
      tab
    }))
    
    ggplot(plot_data,
           aes(x = Category, y = Percent, fill = Profile)) +
      geom_col(
        width = 0.6,
        position = position_dodge(width = 0.7)
      ) +
      geom_text(
        aes(label = paste0(Percent, "%")),
        position = position_dodge(width = 0.7),
        vjust = -0.25,
        size = 3
      ) +
      facet_wrap(~ Variable, scales = "free_x") +
      labs(
        title = "Categorical Cross-Tabulation by Profile",
        y = "Percentage (%)",
        x = NULL
      ) +
      theme_minimal() +
      theme(
        axis.text.x = element_text(angle = 90, hjust = 1, vjust = 0.5, size = 9),
        legend.position = "bottom",
        axis.line = element_line(color = "black")
      )
  })
  output$crosstab_cont_plot <- renderPlot({
    req(input$crosstab_result, input$crosstab_vars)
    if (!input$crosstab_result) return(NULL)
    
    df <- df_out()
    
    vars_num <- input$crosstab_vars[
      sapply(df[input$crosstab_vars], is.numeric)
    ]
    
    if (length(vars_num) == 0) return(NULL)
    
    plot_data <- df |>
      dplyr::select(Profile, dplyr::all_of(vars_num)) |>
      tidyr::pivot_longer(
        cols = -Profile,
        names_to = "Variable",
        values_to = "Value"
      ) |>
      dplyr::group_by(Profile, Variable) |>
      dplyr::summarise(
        Mean = mean(Value, na.rm = TRUE),
        SD   = sd(Value, na.rm = TRUE),
        .groups = "drop"
      )
    
    ggplot(plot_data, aes(x = Profile, y = Mean, fill = Profile)) +
      geom_col(width = 0.6) +
      geom_errorbar(
        aes(ymin = Mean - SD, ymax = Mean + SD),
        width = 0.2
      ) +
      facet_wrap(~ Variable, scales = "free_y") +
      labs(
        title = "Continuous Variables by Profile",
        y = "Mean (± SD)",
        x = NULL
      ) +
      theme_minimal() +
      theme(
        axis.text.x = element_text(angle = 90, hjust = 1, vjust = 0.5, size = 9),
        legend.position = "none",
        axis.line = element_line(color = "black")
      )
  })
  
  
  
  
  
  
  
  
  
  
  

  # ==== R Console Output ====
  observeEvent(models(), {
    req(models(), input$best_k)
    model_k <- models()[[as.character(input$best_k)]]
    if (!is.null(model_k)) {
      console_context$text <- paste(capture.output(print(model_k)), collapse = "\n")
    }
  })

  # ==== AI Assistant context ====
  observe({
    res_text <- ""
    if (!is.null(fitcompare())) {
      res_text <- paste0(
        "=== LPA MODEL FIT COMPARISON ===\n",
        paste(capture.output(print(as.data.frame(fitcompare()))), collapse = "\n"), "\n\n"
      )
    }
    if (!is.null(models()) && !is.null(input$best_k)) {
      model_k <- models()[[as.character(input$best_k)]]
      if (!is.null(model_k)) {
        res_text <- paste0(
          res_text,
          "=== SELECTED MODEL (", input$best_k, " PROFILES) ===\n",
          paste(capture.output(print(model_k)), collapse = "\n"), "\n\n",
          "=== PROFILE PARAMETER ESTIMATES ===\n",
          paste(capture.output(print(as.data.frame(tidyLPA::get_estimates(model_k)))), collapse = "\n"), "\n"
        )
      }
    }
    ai_context$results_text <- res_text
    ai_context$module <- "Latent Profile Analysis (LPA)"
  })

  # ==== Save Project (workspace .rds) ====
  output$export_lpa_rds <- downloadHandler(
    filename = function() paste0("LPA_Workspace_", Sys.Date(), ".rds"),
    content = function(file) {
      req(fitcompare())
      workspace <- list(
        type = "projectLSA_workspace",
        module = "LPA",
        version = "0.1.1",
        saved_at = Sys.time(),
        raw_data = data_user(),
        fitcompare = fitcompare(),
        models = models(),
        input_state = list(
          selected_vars = input$selected_vars,
          min_profiles = input$min_profiles,
          max_profiles = input$max_profiles,
          model_to_run = input$model_to_run,
          model_type = input$model_type,
          best_k = input$best_k
        )
      )
      saveRDS(workspace, file)
    }
  )

  # ==== HTML Report ====
  # Per-session directory, so concurrent users never share a report file
  lpa_report_res <- session_report_dir(session, "lpa")
  lpa_report_path <- reactiveVal(NULL)

  generate_lpa_report <- function(progress_label = "Generating Report...") {
    if (is.null(fitcompare())) return(NULL)

    report_path <- file.path(system.file("app", package = "projectLSA"), "lpa_report.Rmd")
    if (report_path == "" || !file.exists(report_path)) {
      report_path <- "lpa_report.Rmd"
    }
    tempReport <- file.path(lpa_report_res$path, "lpa_report.Rmd")
    file.copy(report_path, tempReport, overwrite = TRUE)

    out_html <- file.path(lpa_report_res$path, "lpa_report_out.html")

    best_k <- input$best_k
    model_k <- if (!is.null(models())) models()[[as.character(best_k)]] else NULL

    showModal(modalDialog(progress_label, footer = NULL))
    ok <- tryCatch({
      rmarkdown::render(tempReport,
        output_file = out_html,
        params = list(
          fit_compare = as.data.frame(fitcompare()),
          best_k = best_k,
          model_best = model_k,
          profile_plot = tryCatch(best_model_plot_reactive(), error = function(e) NULL),
          summary_tbl = tryCatch(summary_data(), error = function(e) NULL),
          selected_vars = input$selected_vars,
          console_out = console_context$text,
          ai_summary = if (is.null(ai_context$ai_report_text)) "" else ai_context$ai_report_text
        ),
        envir = new.env(parent = globalenv())
      )
      lpa_report_path(out_html)
      TRUE
    }, error = function(e) {
      showNotification(paste("Error rendering report:", e$message), type = "error")
      FALSE
    }, finally = {
      removeModal()
    })

    if (isTRUE(ok)) out_html else NULL
  }

  observeEvent(input$lpa_generate_preview, {
    req(fitcompare())
    generate_lpa_report("Generating Report Preview...")
  })

  output$lpa_report_preview_frame <- renderUI({
    req(lpa_report_path())
    tags$iframe(
      src = paste0(lpa_report_res$prefix, "/lpa_report_out.html?v=", as.integer(Sys.time())),
      width = "100%", height = "800px", style = "border: none;"
    )
  })

  output$download_report_lpa <- downloadHandler(
    filename = function() paste0("LPA_Report_", Sys.Date(), ".html"),
    content = function(file) {
      req(fitcompare())
      path <- lpa_report_path()
      if (is.null(path) || !file.exists(path)) {
        path <- generate_lpa_report("Generating Report...")
      }
      req(path)
      file.copy(path, file, overwrite = TRUE)
    },
    contentType = "text/html"
  )
}

console_widget_ui <- function() {
  absolutePanel(
    id = "console_floating_panel",
    fixed = TRUE,
    draggable = FALSE,
    bottom = 20,
    left = 20,
    width = "auto",
    height = "auto",
    class = "floating-widget",
    style = "z-index: 9999; border-radius: 8px; box-shadow: 0 4px 15px rgba(0,0,0,0.2); background: white; border: 1px solid #ccc; display: flex; flex-direction: column; overflow: hidden;",
    
    # Header
    div(
      id = "console_widget_header",
      style = "background: #343a40; color: white; padding: 12px 15px; cursor: pointer; display: flex; justify-content: space-between; align-items: center; font-weight: bold; border-top-left-radius: 8px; border-top-right-radius: 8px;",
      tags$span(icon("r-project", style = "font-size:1.2em;")),
      actionButton("toggle_console_widget", label = "", icon = icon("chevron-up"), 
                   style = "background: transparent; border: none; color: white; box-shadow: none; padding: 0;")
    ),
    
    # Body
    div(
      id = "console_widget_body_wrapper",
      style = "display: none;", # Start minimized
      div(
        id = "console_widget_body",
        style = "resize: both; height: 350px; width: 450px; display: flex; flex-direction: column; background: #2b2b2b; color: #a9b7c6; font-family: Consolas, monospace; overflow: hidden;",
        
        # Toolbar for resize and copy
        div(
          style = "display: flex; justify-content: flex-end; background: #1e1e1e; padding: 4px 8px; gap: 6px; border-bottom: 1px solid #444;",
          tags$button(id = "decrease_font", type = "button", class = "btn btn-xs", style = "background: #444; color: white; border: none; padding: 2px 8px; font-size: 11px; border-radius: 4px;", "- A"),
          tags$button(id = "increase_font", type = "button", class = "btn btn-xs", style = "background: #444; color: white; border: none; padding: 2px 8px; font-size: 11px; border-radius: 4px;", "+ A"),
          tags$button(id = "copy_console", type = "button", class = "btn btn-xs", style = "background: #444; color: white; border: none; padding: 2px 8px; font-size: 11px; border-radius: 4px;", tagList(icon("copy"), "Copy"))
        ),

        div(
          style = "flex-grow: 1; overflow: auto; padding: 10px;",
          # We need to style the verbatimTextOutput to have a dark theme to match the body, and use Consolas font
          tags$style(HTML("
            #global_console_output {
              background-color: transparent;
              color: #a9b7c6;
              border: none;
              font-family: Consolas, monospace;
              font-size: 11px;
              padding: 0;
              white-space: pre-wrap;
            }
          ")),
          verbatimTextOutput("global_console_output")
        )
      )
    ),
    
    # Simple JS to change icon on toggle
    tags$script(HTML("
      $(document).on('click', '#toggle_console_widget', function(e) {
        e.preventDefault();
        var icon = $(this).find('i');
        var wrapper = $('#console_widget_body_wrapper');
        if (wrapper.is(':visible')) {
          wrapper.hide();
          icon.removeClass('fa-chevron-down').addClass('fa-chevron-up');
        } else {
          wrapper.show();
          icon.removeClass('fa-chevron-up').addClass('fa-chevron-down');
        }
      });
      // Click header to toggle
      $(document).on('click', '#console_widget_header', function(e) {
        if ($(e.target).closest('button').length === 0) {
          var wrapper = $('#console_widget_body_wrapper');
          var icon = $('#toggle_console_widget i');
          if (wrapper.is(':visible')) {
            wrapper.hide();
            icon.removeClass('fa-chevron-down').addClass('fa-chevron-up');
          } else {
            wrapper.show();
            icon.removeClass('fa-chevron-up').addClass('fa-chevron-down');
          }
        }
      });
      
      // Font resizing logic
      $(document).on('click', '#increase_font', function(e) {
        e.stopPropagation();
        var el = $('#global_console_output');
        var size = parseInt(el.css('font-size'));
        el.css('font-size', (size + 1) + 'px');
      });

      $(document).on('click', '#decrease_font', function(e) {
        e.stopPropagation();
        var el = $('#global_console_output');
        var size = parseInt(el.css('font-size'));
        el.css('font-size', (size - 1) + 'px');
      });

      // Copy logic
      $(document).on('click', '#copy_console', function(e) {
        e.stopPropagation();
        var text = $('#global_console_output').text();
        
        var $btn = $(this);
        var originalHTML = $btn.html();
        
        var textArea = document.createElement('textarea');
        textArea.value = text;
        textArea.style.position = 'fixed';
        textArea.style.left = '-999999px';
        document.body.appendChild(textArea);
        textArea.focus();
        textArea.select();
        try {
            document.execCommand('copy');
            $btn.html('<i class=\"fa fa-check\"></i> Copied!');
            setTimeout(function() {
              $btn.html(originalHTML);
            }, 1500);
        } catch (err) {
            console.error('Copy failed', err);
        }
        document.body.removeChild(textArea);
      });
    "))
  )
}

# ==== Reactive Data LCA ====
server_lca <- function(input, output, session, ai_context, console_context) {
  library(poLCA)
  library(dplyr)
  library(tidyr)
  library(ggplot2)
  library(purrr)
  library(tibble)
  library(ggiraph)
  library(data.table)
  library(glca)
  library(stats)
  library(haven)

  best_c_r <- reactiveVal(NULL)
  set.seed(100)

  # Fitted models are held in a reactiveVal (instead of eventReactive) so that a
  # saved workspace can be restored straight into them.
  lca_models <- reactiveVal(NULL)

  # Variables selected in a restored workspace, applied once the picker exists
  lca_restored_vars <- reactiveVal(NULL)
observeEvent(input$run_lca, {
  req(data_lca(), input$vars_lca)
  updateTabsetPanel(session, "main_tab_lca", selected = "fit_tab_lca")
})
data_source <- reactive({
  if (input$data_source_lca == "cheat") {
    data("cheating", package = "poLCA")
    df <- cheating %>% rownames_to_column("id_auto") 
  } else if (input$data_source_lca == "lazar") {
    freq <- data.frame(
      V1   = c(1, 1, 1, 1, 2, 2, 2, 2),
      V2   = c(1, 1, 2, 2, 1, 1, 2, 2),
      V3   = c(1, 2, 1, 2, 1, 2, 1, 2),
      Freq = c(220, 160, 60, 160, 60, 60, 60, 220)
    )
    df <- freq[rep(seq_len(nrow(freq)), freq$Freq), c("V1", "V2", "V3")]
    rownames(df) <- NULL
    df$id_auto <- seq_len(nrow(df))
  
  } else if(input$data_source_lca == "gss82") {
    data("gss82", package = "poLCA")
    df <- gss82 %>% rownames_to_column("id_auto") %>%
      dplyr::mutate(
        PURPOSE  = dplyr::recode(PURPOSE,  "Waste of time"=1,"Depends"=2,"Good"=3),
        ACCURACY = dplyr::recode(ACCURACY, "Not true"=1,"Mostly true"=2),
        UNDERSTA = dplyr::recode(UNDERSTA, "Fair/Poor"=1,"Good"=2),
        COOPERAT = dplyr::recode(COOPERAT, "Impatient"=1,"Cooperative"=2,"Interested"=3)
      )
    
  } else {
    req(input$datafile_lca)
    showModal(modalDialog(title = NULL, "Reading Your File, Please wait...", footer = NULL, easyClose = FALSE))
    ext <- tolower(tools::file_ext(input$datafile_lca$name))

    if (ext == "rds") {
      res <- readRDS(input$datafile_lca$datapath)
      if (is.list(res) && !is.data.frame(res) && identical(res$type, "projectLSA_workspace")) {
        if (!identical(res$module, "LCA")) {
          removeModal()
          stop("Uploaded workspace belongs to a different module: ", res$module)
        }
        # Restore workspace state
        lca_models(res$lca_models)
        df <- res$raw_data

        st <- res$input_state
        if (!is.null(st)) {
          if (!is.null(st$min_class_lca))  updateNumericInput(session, "min_class_lca", value = st$min_class_lca)
          if (!is.null(st$max_class_lca))  updateNumericInput(session, "max_class_lca", value = st$max_class_lca)
          if (!is.null(st$best_class_lca)) updateNumericInput(session, "best_class_lca", value = st$best_class_lca)
          if (!is.null(st$use_cov_lca))    updateCheckboxInput(session, "use_cov_lca", value = st$use_cov_lca)
          if (!is.null(st$cov_lca))        updateSelectInput(session, "cov_lca", selected = st$cov_lca)
          if (!is.null(st$vars_lca))       lca_restored_vars(st$vars_lca)
        }
        showNotification("LCA Workspace restored successfully!", type = "message")
      } else {
        df <- res
      }
    } else {
      df <- switch(
        ext,
        "csv"  = data.table::fread(input$datafile_lca$datapath,data.table = FALSE),
        "xls"  = readxl::read_excel(input$datafile_lca$datapath),
        "xlsx" = readxl::read_excel(input$datafile_lca$datapath),
        "sav"  = haven::read_sav(input$datafile_lca$datapath),
        stop("Unsupported file type. Please upload CSV, Excel, SPSS (.sav), or RDS file.")
      )
    }
    removeModal()
    if (!"id_auto" %in% names(df)) {
      df <- df %>% mutate(across(everything(), ~ifelse(.x=="", NA, .x)),
                          id_auto = paste0("id_", sprintf("%04d", 1:n())))
    }
  }
  return(df)
})
data_lca <- reactive({
  df <- data_source()
  # Jika pakai covariat → buang baris NA di covariat
  if (isTRUE(input$use_cov_lca) && length(input$cov_lca) > 0) {
    df <- df %>% tidyr::drop_na(all_of(input$cov_lca))
  }
  
  df
})

# ==== Pilih ID ====
output$id_select_ui_lca <- renderUI({
  req(data_source())
  selectInput(
    "id_lca",
    label = "Select ID Columns (Optional):",
    choices = names(data_lca()),
    selected = names(data_source())[str_detect(names(data_source()), "id")],
    multiple = TRUE
    )
})

# ==== Pilih variabel ====
output$var_select_ui_lca <- renderUI({
  req(data_source())
  restored <- lca_restored_vars()
  sel <- if (!is.null(restored)) {
    intersect(restored, names(data_source()))
  } else {
    names(data_source()%>% dplyr::select(-c(id_auto)))[1:min(2,ncol(data_source()))]
  }
  selectInput(
    "vars_lca",
    label = "Select Variables for LCA:",
    choices = names(data_source()),
    selected = sel,
    multiple = TRUE  )
})

output$cov_lca_ui <- renderUI({
  req(data_source())
  
  tagList(
    checkboxInput(
      "use_cov_lca",
      label = "Include Covariate?",
      value = FALSE
    ),
    
    conditionalPanel(
      condition = "input.use_cov_lca == true",
      selectInput(
        "cov_lca",
        label = "Select Covariate (Numeric):",
        choices = names(data_source()),
        multiple = TRUE
      )
    )
  )
})


observeEvent(c(input$data_source_lca, input$datafile_lca), {
  # Keep the selection that came from a restored workspace
  if (is.null(lca_restored_vars())) {
    updateSelectInput(session, "vars_lca", selected = "")
  }
}, ignoreInit = TRUE)

# ==== Preview Data ====
output$data_preview_lca <- DT::renderDT({
  req(data_lca(), input$vars_lca)
  df <- data_lca() %>% dplyr::select(input$vars_lca)
  numeric_cols <- which(sapply(df, function(x) is.numeric(x)))
  
  DT::datatable(df,#extensions = 'Buttons',
                options = list(scrollX = TRUE, dom = 'rtp',
                               # buttons = list(
                               #   list(
                               #     extend = 'csv',
                               #     text = 'Export CSV',
                               #     filename = paste0('Data LCA')  
                               #   ),
                               #   list(
                               #     extend = 'excel',
                               #     text = 'Export Excel',
                               #     filename = paste0('Data LCA')
                               #   ))
                               ),
                rownames = TRUE) %>% 
    formatRound(columns = numeric_cols, digits = 0)
}, server = TRUE)


output$data_summary_lca <- DT::renderDT({
  req(data_lca(), input$vars_lca)
  df <- data_lca() %>% dplyr::select(input$vars_lca)
  # Pastikan semuanya character (aman untuk join)
  df_chr <- df %>% dplyr::mutate(across(everything(), as.character))
  # Buat kolom join (kombinasi respons)
  df_chr$Respon <- apply(df_chr, 1, paste0, collapse = "-")
  
  # Tabel frekuensi
  freq_table <- df_chr %>%
    dplyr::count(Respon, name = "Freq") %>%
    dplyr::mutate(
      Percent = round(100 * Freq / sum(Freq), 2)
    ) %>%
    dplyr::arrange(dplyr::desc(Freq))
  
  DT::datatable(freq_table,extensions = 'Buttons',
                options = list(scrollX = TRUE, dom = 'Brtp',
                               buttons = list(
                                 list(
                                   extend = 'csv',
                                   text = 'Export CSV',
                                   filename = paste0('Data LCA')  
                                 ),
                                 list(
                                   extend = 'excel',
                                   text = 'Export Excel',
                                   filename = paste0('Data LCA')
                                 ))),
                rownames = TRUE)
}, server = FALSE)

output$data_description <- renderUI({
  desc_html <- if (input$data_source_lca == "cheat") {
    desc_cheating
  } else if (input$data_source_lca == "lazar") {
    desc_lazar
  } else if (input$data_source_lca == "gss82") {
    desc_gss82
  } else {
    ""
  }
  
  div(
    class = "badge-info",
    HTML(desc_html)
  )
})

# ==== Fit LCA ====
observeEvent(input$run_lca, {
  req(data_lca(), input$vars_lca)
  # A fresh run invalidates any restored workspace selection
  lca_restored_vars(NULL)
  df <- data_lca()
  showModal(modalDialog(title = NULL, "Please wait, (Running LCA)...", footer = NULL, easyClose = FALSE))
  
  # =========================
  # 1. Tentukan kolom yang dipakai
  # =========================
  vars_ind  <- input$vars_lca
  vars_cov  <- if (isTRUE(input$use_cov_lca)) input$cov_lca else NULL

  used_vars <- unique(c(vars_ind, vars_cov))

  dat <- df[, used_vars, drop = FALSE]

  # =========================
  # 2. Konversi indikator → factor
  # =========================
  dat <- dat %>%
    mutate(across(all_of(vars_ind), as.factor))

  # =========================
  # 3. Kovariat: JANGAN dipaksa factor
  #    (biarkan numeric tetap numeric)
  # =========================
  # OPTIONAL: jika mau auto-detect kovariat nominal
  if (!is.null(vars_cov)) {
    dat <- dat %>%
      mutate(across(
        all_of(vars_cov),
        ~ if (is.numeric(.x) && dplyr::n_distinct(.x) <= 3) {
          as.factor(.x)
        } else {
          .x
        }
      ))
  }

  # formula <- as.formula(paste("cbind(", paste(input$vars_lca, collapse=","), ") ~ 1"))
  # =========================
  # FORMULA (dengan / tanpa covariat)
  # =========================
  if (isTRUE(input$use_cov_lca) && length(input$cov_lca) > 0) {

    formula_polca <- as.formula(
      paste("cbind(", paste(input$vars_lca, collapse = ","), ") ~ ",
        paste(input$cov_lca)
      )
    )
    formula_glca <- as.formula(
      paste("item(", paste(input$vars_lca, collapse = ","), ") ~ ",
            paste(input$cov_lca)
      )
    )
  } else {
    formula_polca <- as.formula(
      paste("cbind(", paste(input$vars_lca, collapse = ","), ") ~ 1")
    )
    formula_glca <- as.formula(
      paste("item(", paste(input$vars_lca, collapse = ","), ") ~ 1")
    )
  }

  min_k <- input$min_class_lca
  max_k <- input$max_class_lca
  # model_list <- list()
  polca_models <- list()
  glca_models  <- list()
  withProgress(message = "Estimating LCA models...", {
    for (k in min_k:max_k) {
      
      incProgress(1 / (max_k - min_k + 1),
                  detail = paste("Classes:", k))
      
      polca_models[[as.character(k)]] <- tryCatch(
        poLCA::poLCA(formula = formula_polca, data= dat, nclass = k, verbose = FALSE),
        error = function(e) NULL
      )
      
      glca_models[[as.character(k)]] <- tryCatch(
        glca::glca(formula=formula_glca, data=dat, nclass = k, seed = 1, verbose = FALSE),
        error = function(e) NULL
      )
    }
  })
  # =========================
  # GOFGLCA (SEMUA MODEL SEKALIGUS)
  # =========================
  withProgress(message = "Comparing Multiple Model...", {
  glca_valid <- glca_models[!sapply(glca_models, is.null)]
  gof <- if (length(glca_valid) >= 2) {
    do.call(
      glca::gofglca,
      c(
        unname(glca_valid), 
        list(test = "boot", seed = 1)
      )
    )
  } else {
    NULL
  }
  gtable <- as.data.frame(gof$gtable) %>% dplyr::rename(p_Gsq = `Boot p-value`)
  dtable <- as.data.frame(gof$dtable) %>% dplyr::rename(p_Deviance = `Boot p-value`) %>% dplyr::select(Deviance,p_Deviance)
  gof_glca <- cbind(gtable,dtable)
  if (input$min_class_lca == 1) {
    fit_1_class <- data.frame(
      loglik     = polca_models[["1"]]$llik,
      AIC        = polca_models[["1"]]$aic,
      BIC        = polca_models[["1"]]$bic,
      entropy    = NA,
      df         = polca_models[["1"]]$resid.df,
      Gsq        = polca_models[["1"]]$Gsq,
      p_Gsq      = NA,
      Deviance   = NA,
      p_Deviance = NA)
    gof_glca <- rbind(fit_1_class, gof_glca)
  }
  })
  removeModal()
  lca_models(list(
    polca = polca_models,
    glca   = gof_glca
  ))
})


# FUNCTION TO CALCULATION OF LCA -----
APCP_poLCA <- function(fit) {
  post <- fit$posterior
  cls  <- fit$predclass
  K <- ncol(post)
  APCP_class <- sapply(1:K, function(k) {
    mean(post[cls == k, k], na.rm=TRUE) })
  APCP_overall <- mean(post[cbind(1:nrow(post), cls)])
  APCP_byclass <- mean(APCP_class,  na.rm = TRUE)
  min_Prob <- min(post[cbind(1:nrow(post), cls)])
  list(
    APCP_per_class = APCP_class,
    APCP_overall  = APCP_overall,
    APCP_byClass = APCP_byclass,
    min_Prob = min_Prob
  )
}
# Smallest Class Size ===
smallest_class <- reactive({ 
  req(lca_models())
  purrr::map_df(names(lca_models()$polca), function(k) {
    model_k <- lca_models()$polca[[k]]
    membership <- model_k$predclass
    class_table <- as.data.frame(table(membership))
    names(class_table) <- c("Class", "N")
    
    class_table$Percent <- round(100 * class_table$N / sum(class_table$N), 1)
    min_class <- class_table %>% slice_min(N, n = 1)
    min_class$N_class <- as.numeric(k)
    min_class
  })
})


# ==== Fit Table ====

# fit_lca <- reactive({
#   req(lca_models())
#   
#   fit <- data.frame(
#     N_class = integer(),
#     Smallest_class_size = numeric(),
#     Av_Prob = numeric()
#   )
#   
#   gof <- lca_models()$glca
#   model_polca <- lca_models()$polca
#   smallest_df <- smallest_class()
#   
#   for (k in names(model_polca)) {
#     n <- model_polca[[k]]
#     if (!is.null(n)) {
#       
#       scs <- smallest_df %>%
#         dplyr::filter(N_class == as.numeric(k)) %>%
#         dplyr::pull(Percent)
#       
#       fit <- rbind(
#         fit,
#         data.frame(
#           N_class = as.numeric(k),
#           Av_Prob = round(APCP_poLCA(n)$APCP_byClass, 2),
#           Smallest_class_size = scs
#         )
#       )
#     }
#   }
#   
#   cbind(fit, gof)
# })

fit_lca <- reactive({
  req(lca_models())
  
  fit <- data.frame(
    N_class = integer(),
    Smallest_class_size = numeric(),
    Av_Prob = numeric()
  )
  
  gof <- lca_models()$glca
  model_polca <- lca_models()$polca
  smallest_df <- smallest_class()
  
  for (k in names(model_polca)) {
    n <- model_polca[[k]]
    if (!is.null(n)) {
      
      scs <- smallest_df %>%
        dplyr::filter(N_class == as.numeric(k)) %>%
        dplyr::pull(Percent)
      
      fit <- rbind(
        fit,
        data.frame(
          N_class = as.numeric(k),
          Av_Prob = round(APCP_poLCA(n)$APCP_byClass, 2),
          Smallest_class_size = scs
        )
      )
    }
  }
  
  cbind(fit, gof)
})


# ==== Fit Table ====
output$fit_table_lca <- renderDT({
    df <- fit_lca() %>% 
      dplyr::select(N_class, loglik:p_Deviance, Smallest_class_size, Av_Prob) %>% 
      dplyr::mutate(
        dplyr::across(
          .cols = -c(N_class,df),
          .fns  = ~ round(.x, 2)
        )
      )
    aic_vals <- sort(unique(df$AIC))
    bic_vals <- sort(unique(df$BIC))
    ent_vals <- sort(unique(df$entropy), decreasing = TRUE)
    apcp_vals  <- sort(unique(df$Av_Prob), decreasing = TRUE)

    DT::datatable(df,extensions = 'Buttons',
                  options = list(scrollX = TRUE, dom = 'B',
                                 buttons = list(
                                   list(
                                     extend = 'csv',
                                     text = 'Export CSV',
                                     filename = paste0('Fit Comparison LCA')  
                                   ),
                                   list(
                                     extend = 'excel',
                                     text = 'Export Excel',
                                     filename = paste0('Fit Comparison LCA')
                                   ))), 
                  rownames = FALSE) %>% 
      formatStyle('AIC',
                  backgroundColor = styleEqual(
                    c(aic_vals[1], aic_vals[2]),
                    c('lightgreen', 'khaki')
                  ),
                  fontWeight = styleEqual(aic_vals[1], 'bold')
      ) %>%
      formatStyle('BIC',
                  backgroundColor = styleEqual(
                    c(bic_vals[1], bic_vals[2]),
                    c('lightgreen', 'khaki')
                  ),
                  fontWeight = styleEqual(bic_vals[1], 'bold')
      ) %>%
      formatStyle('entropy',
                  backgroundColor = styleInterval(c(0.7, 0.8), c('lightcoral', 'khaki', 'lightgreen')),
                  fontWeight      = styleInterval(c(0.7, 0.8), c('bold', 'bold',''))
      ) %>%
      formatStyle('Av_Prob',
                    backgroundColor = styleInterval(c(0.8, 0.9), c('lightcoral', 'khaki', 'lightgreen')),
                    fontWeight      = styleInterval(c(0.8, 0.9), c('bold', 'bold',''))
      ) %>% 
      formatStyle(
        'p_Gsq',
        backgroundColor = styleInterval(0.05, c('lightcoral', 'lightgreen')),
        fontWeight      = styleInterval(0.05, c('bold', ''))
      ) %>%
      formatStyle(
        'p_Deviance',
        backgroundColor = styleInterval(0.05, c('lightgreen', 'lightcoral')),
        fontWeight      = styleInterval(0.05, c('', 'bold'))
      ) %>% 
      formatStyle(
        'Smallest_class_size',
        backgroundColor = styleInterval(c(5, 7), c('lightcoral', 'khaki', 'lightgreen')),
        fontWeight      = styleInterval(c(5, 7), c('bold', 'bold',''))
      )
}, server = FALSE)

# ==== Fit Plot (AIC/BIC) ====
fit_plot_lca_reactive <- reactive({
  req(lca_models())
  fit <- fit_lca() %>% dplyr::select(N_class, BIC, AIC)
  fit_long <- fit %>% pivot_longer(-N_class, names_to="Index", values_to="Value")
  ggplot(fit_long, aes(x=N_class, y=Value, color=Index, group=Index)) +
    geom_line(size = 1.2) + geom_point(size = 3) +
    geom_text(aes(label = round(Value, 2)), vjust = -0.6, size = 3) +
    labs(title = "BIC & AIC Comparison", x = "Number of Class", y = "Fit Index") +
    theme_minimal(base_size = 14) + 
    theme(legend.position = "bottom", axis.line = element_line(color = "black"))
})
# Render Plot
output$fit_plot_lca <- renderPlot({
  fit_plot_lca_reactive()
})

# ==== Download Buttons ====
output$download_plot_AicBic_LCA <- make_download_plot(
  plot_reactive = fit_plot_lca_reactive,
  filename_prefix = "Fit BIC & AIC Plot_LCA"
)

# ==== Entropy Plot  ====
entropy_plot_lca_reactive <- reactive({
  req(lca_models())
  
  fit <- fit_lca() %>%
    dplyr::select(N_class, entropy, Av_Prob) %>%
    tidyr::pivot_longer(-N_class, names_to = "Index", values_to = "Value")
  
  ggplot(
    fit,
    aes(
      N_class, Value,
      color = Index, linetype = Index, shape = Index, group = Index
    )
  ) +
    geom_line(size = 1.2) +
    geom_point(size = 3) +
    geom_text(aes(label = round(Value, 2)), vjust = -0.6, size = 3, show.legend = FALSE) +
    scale_color_manual(values = c(entropy = "black", Av_Prob = "blue")) +
    scale_shape_manual(values = c(entropy = 17, Av_Prob = 15)) +
    scale_linetype_manual(values = c(entropy = "solid", Av_Prob = "dashed")) +
    labs(
      title = "Entropy & Average Posterior Probability",
      x = "Number of Class",
      y = "Value"
    ) +
    theme_minimal(base_size = 14) +
    theme(legend.position = "bottom", axis.line = element_line(color = "black"))
})

# Render Plot
output$entropy_plot_lca <- renderPlot({
  entropy_plot_lca_reactive()
})

# ==== Download Buttons ====
output$download_plot_entropy <- make_download_plot(
  plot_reactive = entropy_plot_lca_reactive,
  filename_prefix = "Entropy & Av. Posterior Probability Plot_LCA"
)

observeEvent(input$best_class_lca, {
  req(input$best_class_lca)
  best_c_r(as.numeric(input$best_class_lca))
}, ignoreInit = TRUE)

# ==== Input nama class ====
output$class_name_inputs_lca <- renderUI({
  req(input$best_class_lca)
  lapply(1:input$best_class_lca, function(i) {
    textInput(paste0("class_name_", i), paste("Class", i, "Name:"), value = paste("Class", i))
  })
})

# === 1. Fungsi pembuat plot ===
build_best_model_plot_lca <- function(lca_models, k, vars_lca, class_names, interactive = TRUE) {
  model_k <- lca_models[[as.character(k)]]
  probs <- model_k$probs
  
  prob <- data.frame(matrix("", ncol = 2 + as.numeric(k), nrow = 0))
  names(prob) <- c("Var", "Level", paste0("Prob.class", 1:as.numeric(k)))
  
  for (j in seq_along(vars_lca)) {
    a <- data.frame(
      Var = vars_lca[j],
      Level = colnames(probs[[j]]),
      t(probs[[j]])
    )
    a <- setNames(a, names(prob))
    prob <- rbind(prob, a)
  }
  
  prob_gathered <- prob %>%
    tidyr::gather(key = "class", value = "probability", 3:ncol(prob)) %>%
    dplyr::mutate(
      class = gsub("Prob.", "", class),
      probability = round(as.numeric(probability), 3),
      Var = factor(Var, levels = vars_lca)
    )# %>%
    # dplyr::arrange(Var)
  
  prob_gathered$class <- factor(
    prob_gathered$class,
    levels = paste0("class", 1:as.numeric(k)),
    labels = class_names
  )
  
  if (interactive) {
    gg <- ggplot(prob_gathered,
                 aes(
                   x = Var, y = probability, fill = Level,
                   tooltip = paste0(
                     "Class: ", class,
                     "<br>Variable: ", Var,
                     "<br>Level: ", Level,
                     "<br>Probability: ", probability
                   ),
                   data_id = class
                 )) +
      geom_bar_interactive(stat = "identity", position = "stack")
  } else {
    gg <- ggplot(prob_gathered,
                 aes(x = Var, y = probability, fill = Level)) +
      geom_bar(stat = "identity", position = "stack")
  }
  
  gg +
    facet_grid(~ class) +
    ylab("Probability") + xlab("") +
    theme_minimal(base_size = 13) +
    theme(
      axis.text.x = element_text(angle = 90, hjust = 1, vjust = 0.5, size = 8),
      panel.grid.major.x = element_blank(),
      panel.grid.minor.x = element_blank(),
      strip.text = element_text(face = "bold", size = 11)
    )
}

# === 2. Reactive untuk plot interaktif ===
best_model_plot_lca_reactive <- reactive({
  req(lca_models(), input$best_class_lca, input$vars_lca)
  k <- input$best_class_lca
  class_names <- sapply(1:k, function(i) input[[paste0("class_name_", i)]])
  if (length(class_names) == 0) class_names <- paste("Class", 1:k)
  
  build_best_model_plot_lca(lca_models()$polca, k, input$vars_lca, class_names, interactive = TRUE)
})

# === 3. Render plot interaktif ===
output$best_model_plot_lca <- renderGirafe({
  k <- input$best_class_lca
  filename_plot <- paste0("LCA_best_model_", k, "class")
  
  ggiraph::girafe(
    ggobj = best_model_plot_lca_reactive(),
    width_svg = 9, height_svg = 5,
    options = list(
      opts_hover(css = "fill-opacity:1;"),
      opts_tooltip(css = "background-color:white;color:black;
                    border:1px solid #ccc;padding:6px;
                    font-size:11px;border-radius:5px;"),
      opts_toolbar(saveaspng = TRUE, pngname = filename_plot)
    )
  )
})

# === 4. Reactive versi statis untuk download ===
best_model_plot_lca_static <- reactive({
  req(lca_models(), input$best_class_lca, input$vars_lca)
  k <- input$best_class_lca
  class_names <- sapply(1:k, function(i) input[[paste0("class_name_", i)]])
  if (length(class_names) == 0) class_names <- paste("Class", 1:k)
  
  build_best_model_plot_lca(lca_models()$polca, k, input$vars_lca, class_names, interactive = FALSE)
})


# === 3. Download handler panggil plot statis ===
output$download_plot_best_LCA <- make_download_plot(
  plot_reactive = best_model_plot_lca_static,
  filename_prefix = paste0("LCA_best_model_", input$best_class_lca, "class")
)

# ==== Summary table ====
summary_data_lca <- reactive({
  req(lca_models(), input$best_class_lca, input$vars_lca)
  
  k <- as.numeric(input$best_class_lca)
  model_k <- lca_models()$polca[[as.character(k)]]
  req(model_k)
  
  # Ambil nama class dari input (fallback jika belum diisi)
  class_names <- sapply(1:k, function(i) {
    nm <- input[[paste0("class_name_", i)]]
    if (is.null(nm) || nm == "") paste0("Class ", i) else nm
  })
  
  # === 1. Ukuran kelas ===
  class_size <- data.frame(
    base::table(model_k$predclass)) %>%
    dplyr::rename(Class = Var1, N = Freq) %>%
    dplyr::mutate(Percent=round(100*N/sum(N),2))
  class_size$Class <- class_names[class_size$Class]
  
  # === 2. Probabilitas kategori (adaptasi dari plot Girafe) ===
  probs <- model_k$probs
  listvar <- input$vars_lca
  
  prob <- data.frame(matrix("", ncol = 2 + k, nrow = 0))
  names(prob) <- c("Variable", "Category", paste0("Prob.", class_names))
  
  for (j in seq_along(listvar)) {
    a <- data.frame(
      Variable = listvar[j],
      Category = colnames(probs[[j]]),
      t(probs[[j]])
    )
    names(a) <- names(prob)
    prob <- rbind(prob, a)
  }
  
  prob <- prob %>%
    mutate(across(starts_with("Prob."), as.numeric)) %>%
    arrange(Variable)
  
  # Gabungkan class size + probabilitas (tidak merge, tapi list agar bisa dua tabel)
  list(
    class_size = class_size,
    probability = prob
  )
})


# ==== Render table ====
output$summary_table_lca <- renderUI({
  req(summary_data_lca())
  dat <- summary_data_lca()
  
  tagList(
    tags$h5("Class Size"),
    DTOutput("class_size_table_lca"),
    tags$h5("Item-Categories Probabilities"),
    DTOutput("probability_table_lca")
  )
})

output$class_size_table_lca <- renderDT({
  df <- summary_data_lca()$class_size
  DT::datatable(df,extensions = 'Buttons',
                options = list(scrollX = TRUE, dom = 'B',
                               buttons = list(
                                 list(
                                   extend = 'csv',
                                   text = 'Export CSV',
                                   filename = paste0('Class Size')
                                 ),
                                 list(
                                   extend = 'excel',
                                   text = 'Export Excel',
                                   filename = paste0('Class Size')
                                 ))),
                rownames = FALSE)
})

output$probability_table_lca <- renderDT({
  df <- summary_data_lca()$probability
  numeric_cols <- which(sapply(df, function(x) is.numeric(x)))
  DT::datatable(df,extensions = 'Buttons',
                options = list(scrollX = TRUE, pageLength=25,
                               dom = 'Brtp',
                               buttons = list(
                                 list(
                                   extend = 'csv',
                                   text = 'Export CSV',
                                   filename = paste0('Class Probability')
                                 ),
                                 list(
                                   extend = 'excel',
                                   text = 'Export Excel',
                                   filename = paste0('Class Probability')
                                 ))),
                rownames = FALSE) %>%
    formatRound(columns = numeric_cols, digits = 2)
})

# df_out <- reactive({
#   req(lca_models(), input$best_class_lca)
#   k <- as.numeric(input$best_class_lca)
#   # Ambil nama class dari input (fallback jika belum diisi)
#   class_names <- sapply(1:k, function(i) {
#     nm <- input[[paste0("class_name_", i)]]
#     if (is.null(nm) || nm == "") paste0("Class ", i) else nm
#   })
#   model_k <- lca_models()$polca[[as.character(input$best_class_lca)]]
#   req(model_k)
#   
#   df <- data_lca()
#   df$Class <- model_k$predclass
#   df$Class <- class_names[df$Class]
#   df$Respon <- df %>%
#     tidyr::unite("Respon", all_of(input$vars_lca), sep = "-", remove = FALSE) %>%
#     dplyr::pull(Respon)
#   
#   # --- Posterior probabilities ---
#   posterior_df <- as.data.frame(round(model_k$posterior,2))
#   colnames(posterior_df) <- paste0("Posterior_Prob_", class_names)
#   numeric_cols <- which(sapply(posterior_df, is.numeric))
#   
#   # Gabungkan: ID + vars + posterior + Class
#   df <- cbind(
#     df,
#     posterior_df
#   )
#   df
#   
# })
df_out <- reactive({
  req(lca_models(), input$best_class_lca)
  
  k <- as.numeric(input$best_class_lca)
  
  # --- Class names ---
  class_names <- sapply(1:k, function(i) {
    nm <- input[[paste0("class_name_", i)]]
    if (is.null(nm) || nm == "") paste0("Class ", i) else nm
  })
  
  model_k <- lca_models()$polca[[as.character(input$best_class_lca)]]
  req(model_k)
  
  df <- data_lca()
  n <- nrow(df)
  
  # --- Tentukan baris yang benar-benar dipakai LCA ---
  vars_used <- input$vars_lca
  valid_idx <- complete.cases(df[, vars_used])
  
  # --- Inisialisasi kolom hasil (panjang = data asli) ---
  df$Class <- NA_character_
  
  # Isi hanya untuk baris valid
  df$Class[valid_idx] <- class_names[model_k$predclass]
  
  # --- Respon pattern (aman walau ada NA) ---
  df$Respon <- df |>
    tidyr::unite("Respon", all_of(vars_used), sep = "-", remove = FALSE) |>
    dplyr::pull(Respon)
  
  # --- Posterior probabilities ---
  posterior_mat <- model_k$posterior
  posterior_df <- matrix(NA, nrow = n, ncol = k)
  posterior_df[valid_idx, ] <- round(posterior_mat, 2)
  
  posterior_df <- as.data.frame(posterior_df)
  colnames(posterior_df) <- paste0("Posterior_Prob_", class_names)
  
  # --- Gabungkan ke data asli ---
  df <- cbind(df, posterior_df)
  
  df
})

# Data Summary Respon_With Class
output$data_summary_lca_with_class <- DT::renderDT({
  req(df_out(), input$best_class_lca)
  df <- df_out()
  # Tabel frekuensi
  freq_table <- df %>%
    dplyr::count(Respon,Class, name = "Freq") %>%
    dplyr::mutate(
      Percent = round(100 * Freq / sum(Freq), 2)
    ) %>%
    dplyr::arrange(dplyr::desc(Freq))
  
  DT::datatable(freq_table,extensions = 'Buttons',
                options = list(scrollX = TRUE, pageLength=25,
                               dom = 'Brtp',
                               buttons = list(
                                 list(
                                   extend = 'csv',
                                   text = 'Export CSV',
                                   filename = paste0('Data LCA')  
                                 ),
                                 list(
                                   extend = 'excel',
                                   text = 'Export Excel',
                                   filename = paste0('Data LCA')
                                 ))),
                rownames = TRUE)
}, server = FALSE)


# ==== Summary Table & Class per ID ====
output$profile_table_lca <- renderDT({
  req(lca_models(), input$best_class_lca)
  df <- df_out()
  DT::datatable(df,extensions = 'Buttons',
                options = list(scrollX = TRUE, pageLength=10,
                               dom = 'Brtp',
                               buttons = list(
                                 list(
                                   extend = 'csv',
                                   text = 'Export CSV',
                                   filename = paste0('LCA Result with ', input$best_class_lca, ' Classes')
                                 ),
                                 list(
                                   extend = 'excel',
                                   text = 'Export Excel',
                                   filename = paste0('LCA Result with', input$best_class_lca, ' Classes')
                                 ))),
                rownames = TRUE)
  
}, server = FALSE)

output$var_x_ui <- renderUI({
  req(df_out())
  selectInput(
    "var_x",
    label = "X Absis",
    choices = names(df_out()),
    selected = "Class",
    multiple = FALSE 
  )
})
output$var_y_ui <- renderUI({
  req(df_out())
  selectInput(
    "var_y",
    label = "Y Absis",
    choices = setdiff(names(df_out()), "Class"),
    selected = "",
    multiple = TRUE
  )
})
output$cross_latent <- renderPlot({
  req(lca_models(), input$best_class_lca, input$var_x, input$var_y)
  df <- df_out()
  # =========================
  # 1. RINGKAS DATA
  # =========================
  Summary_long <- df %>%
    dplyr::select(all_of(c(input$var_x, input$var_y))) %>%
    tidyr::pivot_longer(
      cols = all_of(input$var_y),
      names_to = "Variable",
      values_to = "Value"
    ) %>%
    dplyr::group_by(
      Class  = .data[[input$var_x]],
      Variable
    ) %>%
    dplyr::summarise(
      Mean = mean(Value, na.rm = TRUE),
      .groups = "drop"
    ) %>% 
    dplyr::mutate(
      Variable = factor(Variable, levels = input$var_y)
    )
  
  # =========================
  # 2. BASE PLOT
  # =========================
  p <- ggplot(
    Summary_long,
    aes(
      x = Variable,
      y = Mean,
      fill = Class,
      color = Class,
      group = Class
    )
  )
  # =========================
  # 3. SWITCH BAR / LINE
  # =========================
  if (input$plot_type == "bar") {
    
    p <- p +
      geom_col(
        position = position_dodge(width = 0.7),
        width = 0.6
      )
  } else {
    p <- p +
      geom_line(
        aes(linetype = Class),
        size = 1.2
      ) +
      geom_point(
        aes(shape = Class),
        size = 3
      )
  }
  
  # =========================
  # 4. FINAL STYLE
  # =========================
  p +
    geom_text(
      aes(label = round(Mean, 2)),
      position = position_dodge(width = 0.7),
      vjust = -0.7,
      size = 4.5,
      show.legend = FALSE
    ) +
    labs(
      title = paste0("Class Plot for ", input$best_class_lca, " Classes"),
      x = "Variable",
      y = "Mean Value"
    ) +
    ylim(0, max(Summary_long$Mean, na.rm = TRUE) * 1.18) +
    
    theme_minimal(base_size = 10) +
    theme(
      axis.text.x = element_text(angle = 90, hjust = 1, vjust = 0.5, size = 9),
      legend.position = "bottom",
      axis.line = element_line(color = "black")
    )
  })

output$reg_lca_summary <- renderTable ({
  req(lca_models(), input$best_class_lca, input$use_cov_lca)
  
  model_k <- lca_models()$polca[[as.character(input$best_class_lca)]]
  req(model_k$coeff)
  
  coef_mat <- model_k$coeff
  coef_df <- as.data.frame(coef_mat)
  coef_df$Covariate <- rownames(coef_df)
  
  coef_df <- coef_df %>%
    relocate(Covariate) %>%
    mutate(across(where(is.numeric), round, 3))
  coef_df
})
output$reg_lca_plot <- renderPlot({
  req(
    lca_models(),
    input$best_class_lca,
    input$use_cov_lca,
    input$cov_lca
  )
  
  # === HANYA UNTUK 2 KELAS ===
  if (as.numeric(input$best_class_lca) != 2) {
    return(NULL)
  }
  
  model_k <- lca_models()$polca[[as.character(input$best_class_lca)]]
  beta <- model_k$coeff
  coef_names <- rownames(beta)
  
  # === Pilih kovariat ===
  cov_name <- input$cov_lca[1]
  cov_vals <- sort(unique(data_lca()[[cov_name]]))
  
  # === Design matrix X ===
  X <- data.frame(matrix(0, nrow = length(cov_vals), ncol = length(coef_names)))
  colnames(X) <- coef_names
  
  if ("(Intercept)" %in% coef_names) {
    X[["(Intercept)"]] <- 1
  }
  if (cov_name %in% coef_names) {
    X[[cov_name]] <- cov_vals
  }
  
  # === Probabilities ===
  eta <- as.matrix(X) %*% beta
  p_class2 <- as.vector(exp(eta) / (1 + exp(eta)))
  p_class1 <- 1 - p_class2
  
  prob_df <- data.frame(
    Covariate = cov_vals,
    `Class 1` = p_class1,
    `Class 2` = p_class2
  )
  
  prob_long <- tidyr::pivot_longer(
    prob_df,
    cols = c("Class 1", "Class 2"),
    names_to = "Class",
    values_to = "Probability"
  )
  
  # === Plot ===
  ggplot(prob_long, aes(Covariate, Probability, color = Class)) +
    geom_line(size = 1.2) +
    geom_point(size = 3) +
    scale_y_continuous(limits = c(0, 1)) +
    labs(
      x = cov_name,
      y = "Predicted Class Membership Probability",
      title = "Effect of Covariate on Latent Class Membership (2-Class Model)"
    ) +
    theme_minimal(base_size = 14) +
    theme(
      legend.position = "bottom",
      axis.line = element_line(color = "black")
    )
})
output$anova_lca_ui <- renderUI({
  req(lca_models(), input$best_class_lca, input$var_x, input$var_y)
  
  if (isTRUE(input$check_anova)) {
    tagList(
      tags$h5(
        paste0(
          "Latent Class Differences on ",
          paste(input$var_y, collapse = ", ")
        )
      ),
      tags$h6("Analysis of Variance"),
      tableOutput("anova_lca_table"),
      uiOutput("posthoc_lca_ui")
    )
  }
})
output$anova_lca_table <- renderTable({
  req(df_out(), input$var_x, input$var_y)
  
  df <- df_out()
  x  <- input$var_x
  ys <- input$var_y
  
  # =========================
  # CASE 1: ONE-WAY ANOVA
  # =========================
  if (length(ys) == 1) {
    
    fml <- as.formula(paste(ys, "~", x))
    fit <- aov(fml, data = df)
    tbl <- summary(fit)[[1]]
    
    pval <- tbl["Pr(>F)"][1,1]
    
    out <- data.frame(
      Source = rownames(tbl),
      round(tbl, 4),
      row.names = NULL
    )
    
    out$Interpretation <- c(
      ifelse(
        pval < 0.05,
        "Significant difference",
        "No significant difference"
      ),
      rep("", nrow(out) - 1)
    )
    
    out
    
  } else {
    
    # =========================
    # CASE 2: MANOVA
    # =========================
    
    fml <- as.formula(
      paste(
        "cbind(",
        paste(ys, collapse = ","),
        ") ~ ",
        x
      )
    )
    
    fit <- manova(fml, data = df)
    tbl <- summary(fit, test = "Wilks")$stats
    pval <- tbl[1, "Pr(>F)"]
    
    out <- data.frame(
      Effect = rownames(tbl),
      round(tbl, 4),
      row.names = NULL
    )
    
    out$Interpretation <- ifelse(
      out$Effect == x,
      ifelse(
        pval < 0.05,
        "Significant difference",
        "No significant difference"
      ),
      ""
    )
    
    out
  }
})
output$posthoc_lca_ui <- renderUI({
  req(df_out(), input$var_x, input$var_y)
  
  if (length(input$var_y) == 1) {
    tagList(
      tags$h6("Post-hoc Test (Tukey HSD)"),
      tableOutput("posthoc_lca_table")
    )
  }
})
output$posthoc_lca_table <- renderTable({
  req(df_out(), input$var_x, input$var_y)
  
  # Post-hoc hanya untuk ANOVA
  if (length(input$var_y) != 1) return(NULL)
  
  df <- df_out()
  x  <- input$var_x
  y  <- input$var_y
  
  fml <- as.formula(paste(y, "~", x))
  fit <- aov(fml, data = df)
  
  # p-value ANOVA utama
  p_main <- summary(fit)[[1]]["Pr(>F)"][1, 1]
  
  # Jika tidak signifikan → tampilkan pesan
  if (is.na(p_main) || p_main >= 0.05) {
    return(
      data.frame(
        Note = "Post-hoc test was not conducted because the ANOVA result was not statistically significant."
      )
    )
  }
  
  tuk <- TukeyHSD(fit)[[1]]
  tuk <- round(as.data.frame(tuk), 4)
  
  data.frame(
    Comparison = rownames(tuk),
    tuk,
    Interpretation = ifelse(
      tuk$`p adj` < 0.05,
      "Significant difference",
      "Not significant"
    ),
    row.names = NULL
  )
})
output$reg_lca_ui <- renderUI({
  req(input$use_cov_lca)
  
  if (isTRUE(input$use_cov_lca) && length(input$cov_lca) > 0) {
    tagList(
      tags$h5("Covariate Effects on Class Membership"),
      tableOutput("reg_lca_summary"),
      br(),
      plotOutput("reg_lca_plot", height = "400px")
    )
  }
})

# CrossTab====
output$crosstab_ui <- renderUI({
  req(input$crosstab_result)
  if (!input$crosstab_result) return(NULL)
  
  req(df_out())
  vars <- setdiff(names(df_out()), "Class")
  fluidRow(
    column(
      12,
      h5("Cross Tabulation Settings"),
      selectInput(
        "crosstab_vars",
        "Select Variables:",
        choices = vars,
        multiple = TRUE
      )
    ),
    column(
      6,
      h5("Cross Tabulation For Categorical Variables"),
      tableOutput("crosstab_cat"),
      plotOutput("crosstab_cat_plot")
    ),
    column(
      6,
      h5("Mean & SD for Continuous Variables"),
      tableOutput("crosstab_cont"),
      plotOutput("crosstab_cont_plot")
    ),
    br(), br(), br()
  )
})
output$crosstab_cat <- renderTable({
  req(input$crosstab_result, input$crosstab_vars)
  if (!input$crosstab_result) return(NULL)
  
  df <- df_out()
  
  vars_cat <- input$crosstab_vars[
    sapply(df[input$crosstab_vars], function(x)
      is.factor(x) || is.character(x))
  ]
  
  if (length(vars_cat) == 0) return(NULL)
  
  do.call(rbind, lapply(vars_cat, function(v) {
    tab <- as.data.frame(
      table(Class = df$Class, Category = df[[v]])
    )
    
    tab <- tab |>
      dplyr::group_by(Category) |>
      dplyr::mutate(
        Percent = round(Freq / sum(Freq) * 100, 2)
      ) |>
      dplyr::ungroup()
    
    data.frame(
      Class  = tab$Class,
      Variable = v,
      Category = tab$Category,
      N        = tab$Freq,
      Percent  = tab$Percent
    )
  }))
})
output$crosstab_cont <- renderTable({
  req(df_out(), input$crosstab_vars)
  
  df <- df_out()
  
  vars_num <- input$crosstab_vars[
    sapply(df[input$crosstab_vars], is.numeric)
  ]
  
  if (length(vars_num) == 0) return(NULL)
  
  res <- lapply(vars_num, function(v) {
    aggregate(
      df[[v]],
      by = list(Class = df$Class),
      FUN = function(x)
        paste0(round(mean(x, na.rm = TRUE), 2),
               " (", round(sd(x, na.rm = TRUE), 2), ")")
    ) |>
      setNames(c("Class", v))
  })
  
  Reduce(function(x, y) merge(x, y, by = "Class"), res)
})
output$crosstab_cat_plot <- renderPlot({
  req(input$crosstab_result, input$crosstab_vars)
  if (!input$crosstab_result) return(NULL)
  
  df <- df_out()
  
  vars_cat <- input$crosstab_vars[
    sapply(df[input$crosstab_vars], function(x)
      is.factor(x) || is.character(x))
  ]
  
  if (length(vars_cat) == 0) return(NULL)
  
  plot_data <- do.call(rbind, lapply(vars_cat, function(v) {
    tab <- as.data.frame(table(Class = df$Class, Category = df[[v]]))
    
    tab <- tab |>
      dplyr::group_by(Category) |>
      dplyr::mutate(
        Percent = round(Freq / sum(Freq) * 100, 2)
      ) |>
      dplyr::ungroup()
    
    tab$Variable <- v
    tab
  }))
  
  ggplot(plot_data,
         aes(x = Category, y = Percent, fill = Class)) +
    geom_col(
      width = 0.6,
      position = position_dodge(width = 0.7)
    ) +
    geom_text(
      aes(label = paste0(Percent, "%")),
      position = position_dodge(width = 0.7),
      vjust = -0.25,
      size = 3
    ) +
    facet_wrap(~ Variable, scales = "free_x") +
    labs(
      title = "Categorical Cross-Tabulation by Class",
      y = "Percentage (%)",
      x = NULL
    ) +
    theme_minimal() +
    theme(
      axis.text.x = element_text(angle = 90, hjust = 1, vjust = 0.5, size = 9),
      legend.position = "bottom",
      axis.line = element_line(color = "black")
    )
})
output$crosstab_cont_plot <- renderPlot({
  req(input$crosstab_result, input$crosstab_vars)
  if (!input$crosstab_result) return(NULL)
  
  df <- df_out()
  
  vars_num <- input$crosstab_vars[
    sapply(df[input$crosstab_vars], is.numeric)
  ]
  
  if (length(vars_num) == 0) return(NULL)
  
  plot_data <- df |>
    dplyr::select(Class, dplyr::all_of(vars_num)) |>
    tidyr::pivot_longer(
      cols = -Class,
      names_to = "Variable",
      values_to = "Value"
    ) |>
    dplyr::group_by(Class, Variable) |>
    dplyr::summarise(
      Mean = mean(Value, na.rm = TRUE),
      SD   = sd(Value, na.rm = TRUE),
      .groups = "drop"
    )
  
  ggplot(plot_data, aes(x = Class, y = Mean, fill = Class)) +
    geom_col(width = 0.6) +
    geom_errorbar(
      aes(ymin = Mean - SD, ymax = Mean + SD),
      width = 0.2
    ) +
    facet_wrap(~ Variable, scales = "free_y") +
    labs(
      title = "Continuous Variables by Class",
      y = "Mean (± SD)",
      x = NULL
    ) +
    theme_minimal() +
    theme(
      axis.text.x = element_text(angle = 90, hjust = 1, vjust = 0.5, size = 9),
      legend.position = "none",
      axis.line = element_line(color = "black")
    )
})


# ==== R Console Output ====
observeEvent(lca_models(), {
  req(lca_models(), input$best_class_lca)
  model_k <- lca_models()$polca[[as.character(input$best_class_lca)]]
  if (!is.null(model_k)) {
    console_context$text <- paste(capture.output(print(model_k)), collapse = "\n")
  }
})

# ==== AI Assistant context ====
observe({
  res_text <- ""
  if (!is.null(lca_models())) {
    gof <- lca_models()$glca
    if (!is.null(gof)) {
      res_text <- paste0(
        "=== LCA MODEL FIT COMPARISON ===\n",
        paste(capture.output(print(as.data.frame(gof))), collapse = "\n"), "\n\n"
      )
    }
    if (!is.null(input$best_class_lca)) {
      model_k <- lca_models()$polca[[as.character(input$best_class_lca)]]
      if (!is.null(model_k)) {
        res_text <- paste0(
          res_text,
          "=== SELECTED MODEL (", input$best_class_lca, " CLASSES) ===\n",
          paste(capture.output(print(model_k)), collapse = "\n"), "\n\n",
          "=== CLASS PREVALENCE ===\n",
          paste(capture.output(print(round(model_k$P, 4))), collapse = "\n"), "\n\n",
          "=== ITEM-RESPONSE PROBABILITIES ===\n",
          paste(capture.output(print(lapply(model_k$probs, function(x) round(x, 4)))), collapse = "\n"), "\n"
        )
      }
    }
  }
  ai_context$results_text <- res_text
  ai_context$module <- "Latent Class Analysis (LCA)"
})

# ==== Save Project (workspace .rds) ====
output$export_lca_rds <- downloadHandler(
  filename = function() paste0("LCA_Workspace_", Sys.Date(), ".rds"),
  content = function(file) {
    req(lca_models())
    workspace <- list(
      type = "projectLSA_workspace",
      module = "LCA",
      version = "0.1.1",
      saved_at = Sys.time(),
      raw_data = data_source(),
      lca_models = lca_models(),
      input_state = list(
        vars_lca = input$vars_lca,
        cov_lca = input$cov_lca,
        use_cov_lca = input$use_cov_lca,
        min_class_lca = input$min_class_lca,
        max_class_lca = input$max_class_lca,
        best_class_lca = input$best_class_lca
      )
    )
    saveRDS(workspace, file)
  }
)

# ==== HTML Report ====
# Per-session directory, so concurrent users never share a report file
lca_report_res <- session_report_dir(session, "lca")
lca_report_path <- reactiveVal(NULL)

generate_lca_report <- function(progress_label = "Generating Report...") {
  if (is.null(lca_models())) return(NULL)

  report_path <- file.path(system.file("app", package = "projectLSA"), "lca_report.Rmd")
  if (report_path == "" || !file.exists(report_path)) {
    report_path <- "lca_report.Rmd"
  }
  tempReport <- file.path(lca_report_res$path, "lca_report.Rmd")
  file.copy(report_path, tempReport, overwrite = TRUE)

  out_html <- file.path(lca_report_res$path, "lca_report_out.html")

  k <- input$best_class_lca
  model_k <- lca_models()$polca[[as.character(k)]]

  showModal(modalDialog(progress_label, footer = NULL))
  ok <- tryCatch({
    rmarkdown::render(tempReport,
      output_file = out_html,
      params = list(
        fit_compare = tryCatch(as.data.frame(fit_lca()), error = function(e) NULL),
        best_k = k,
        model_best = model_k,
        class_plot = tryCatch(best_model_plot_lca_static(), error = function(e) NULL),
        summary_tbl = tryCatch(summary_data_lca(), error = function(e) NULL),
        selected_vars = input$vars_lca,
        console_out = console_context$text,
        ai_summary = if (is.null(ai_context$ai_report_text)) "" else ai_context$ai_report_text
      ),
      envir = new.env(parent = globalenv())
    )
    lca_report_path(out_html)
    TRUE
  }, error = function(e) {
    showNotification(paste("Error rendering report:", e$message), type = "error")
    FALSE
  }, finally = {
    removeModal()
  })

  if (isTRUE(ok)) out_html else NULL
}

observeEvent(input$lca_generate_preview, {
  req(lca_models())
  generate_lca_report("Generating Report Preview...")
})

output$lca_report_preview_frame <- renderUI({
  req(lca_report_path())
  tags$iframe(
    src = paste0(lca_report_res$prefix, "/lca_report_out.html?v=", as.integer(Sys.time())),
    width = "100%", height = "800px", style = "border: none;"
  )
})

output$download_report_lca <- downloadHandler(
  filename = function() paste0("LCA_Report_", Sys.Date(), ".html"),
  content = function(file) {
    req(lca_models())
    path <- lca_report_path()
    if (is.null(path) || !file.exists(path)) {
      path <- generate_lca_report("Generating Report...")
    }
    req(path)
    file.copy(path, file, overwrite = TRUE)
  },
  contentType = "text/html"
)
}

ai_widget_ui <- function() {
  absolutePanel(
    id = "ai_floating_panel",
    fixed = TRUE,
    draggable = FALSE,
    bottom = 20,
    right = 20,
    width = "auto",
    height = "auto",
    class = "floating-widget",
    style = "z-index: 9999; border-radius: 8px; box-shadow: 0 4px 15px rgba(0,0,0,0.2); background: white; border: 1px solid #ccc; display: flex; flex-direction: column; overflow: hidden;",

    # Header
    div(
      id = "ai_widget_header",
      style = "background: #2563eb; color: white; padding: 8px 12px; cursor: pointer; display: flex; justify-content: space-between; align-items: center; font-weight: bold; border-top-left-radius: 8px; border-top-right-radius: 8px;",
      tags$span(tags$img(src = "logoProjectLSA.png", style = "width: 18px; height: 18px; margin-right: 5px; vertical-align: middle;"), " AI Assistant"),
      tags$div(style = "display: flex; align-items: center; gap: 12px;",
        actionButton("dock_ai_widget",
          label = "", icon = icon("external-link-alt"), title = "View in Tab",
          style = "background: transparent; border: none; color: white; box-shadow: none; padding: 0; font-size: 13px; line-height: 1;"
        ),
        actionButton("toggle_ai_widget",
          label = "", icon = icon("chevron-up"),
          style = "background: transparent; border: none; color: white; box-shadow: none; padding: 0; font-size: 13px; line-height: 1;"
        )
      )
    ),

    # Body
    div(
      id = "ai_widget_body_wrapper",
      style = "display: none;", # Start minimized
      div(
        id = "ai_widget_body",
        style = "width: 380px; height: 450px; display: flex; flex-direction: column; background: #fff; resize: both; overflow: auto; min-width: 300px; min-height: 300px;",
        tabsetPanel(
          id = "ai_widget_tabs",
          type = "tabs",

          # Chat Tab
          tabPanel("Ask AI",
                   conditionalPanel(
                     condition = "input.ai_api_key == '' || input.ai_api_key == null",
                     div(style = "padding: 20px; text-align: center; height: 100%; display: flex; flex-direction: column; justify-content: center; color: #888;",
                         tags$h4(icon("lock", class="fa-2x")),
                         tags$p("Please configure your API Key in the Settings tab first.")
                     )
                   ),
                   conditionalPanel(
                     condition = "input.ai_api_key != '' && input.ai_api_key != null",
                     div(style = "display: flex; flex-direction: column; height: 100%;",
                         # Action row
                         div(style = "display: flex; justify-content: flex-end; align-items: center; padding: 5px 10px; background: #f8f9fa; border-bottom: 1px solid #eee; flex-shrink: 0;",
                             actionLink("ai_global_chat_clear", "Clear chat", style = "font-size: 12px; color: #dc3545; margin-right: auto;"),
                             tags$button(
                               class = "btn btn-default btn-xs", icon("download"), " Export HTML",
                               onclick = "downloadAiChat()",
                               style = "font-size: 12px;"
                             )
                         ),
                         # History
                         div(
                           id = "ai_chat_history_container",
                           style = "flex: 1; overflow-y: auto; padding: 15px; background: #ffffff; font-size: 13px;",
                           uiOutput("ai_global_chat_history")
                         ),
                         # Input area
                         div(
                           style = "padding: 10px; border-top: 1px solid #ddd; background: #f8f9fa; display: flex; gap: 8px; align-items: center; flex-shrink: 0;",
                           textInput("ai_global_chat_input", label = NULL, placeholder = "Type a question... (press Enter)", width = "100%"),
                           actionButton("ai_global_chat_send", label = "", icon = icon("paper-plane"), class = "btn-primary")
                         )
                     )
                   )
          ),
          
          # Summary Tab
          tabPanel("Create Summary",
                   conditionalPanel(
                     condition = "input.ai_api_key == '' || input.ai_api_key == null",
                     div(style = "padding: 20px; text-align: center; height: 100%; display: flex; flex-direction: column; justify-content: center; color: #888;",
                         tags$h4(icon("lock", class="fa-2x")),
                         tags$p("Please configure your API Key in the Settings tab first.")
                     )
                   ),
                   conditionalPanel(
                     condition = "input.ai_api_key != '' && input.ai_api_key != null",
                     div(style = "padding: 15px; height: 100%; display: flex; flex-direction: column; flex-grow: 1; background: #ffffff;",
                         # Context Inputs
                         bsCollapse(id = "ai_context_collapse",
                           bsCollapsePanel("Research Context (Optional)", style = "info",
                             textAreaInput("ai_context_text", "Theoretical background, hypotheses, etc.:", rows = 3, width = "100%", placeholder = "E.g., The goal of this research is to measure..."),
                             fileInput("ai_context_file", "Or upload document (txt/pdf/docx)", accept = c(".txt", ".pdf", ".docx"), width = "100%")
                           )
                         ),
                         
                         # Inputs
                         div(style = "display: flex; gap: 10px; margin-bottom: 10px;",
                             div(style="flex:1;",
                                 selectInput("ai_summary_format", "Format:", choices = c("Manuscript (Narrative)" = "Manuscript", "APA 7" = "APA 7", "Paragraph" = "Paragraf", "Bullet" = "Poin-poin", "Table" = "Tabel"), width="100%")
                             ),
                             div(style="flex:1;",
                                 selectInput("ai_summary_lang", "Language:", choices = c("English", "Indonesian"), width="100%")
                             )
                         ),
                         # Generate button
                         actionButton("ai_global_summary_generate", "Generate Summary from Analysis Results", class = "btn-success", icon = icon("wand-magic-sparkles"), style = "width: 100%; margin-bottom: 10px; font-weight: bold;"),
                         
                         # Result tools
                         div(style = "display: flex; justify-content: flex-end; gap: 8px; margin-bottom: 5px;",
                             tags$button(
                               id = "copy_summary_btn", class = "btn btn-default btn-xs", icon("copy"), " Copy Text",
                               onclick = "
                                 let temp = $('<textarea>'); $('body').append(temp);
                                 temp.val($('#ai_global_summary_result')[0].innerText).select();
                                 document.execCommand('copy'); temp.remove();
                                 let btn = $(this); let originalText = btn.html();
                                 btn.html('<i class=\"fa fa-check\"></i> Copied!');
                                 setTimeout(function() { btn.html(originalText); }, 2000);
                               ",
                               style = "font-size: 12px;"
                             ),
                             actionButton("ai_add_to_report", tagList(icon("file-medical"), " Add to Report"), class = "btn btn-default btn-xs", style = "font-size: 12px; padding: 1px 5px;")
                         ),
                         # Summary Output
                         div(
                           id = "ai_summary_result_container",
                           style = "flex: 1; overflow-y: auto; border: 1px solid #ddd; padding: 12px; border-radius: 4px; background: #f8f9fa; font-size: 13px;",
                           uiOutput("ai_global_summary_result")
                         )
                     )
                   )
          ),
          
          # Settings Tab
          tabPanel("Settings",
            icon = icon("cog"),
            div(
              style = "padding: 15px; text-align: left; height: 400px; display: flex; flex-direction: column; overflow-y: auto;",
              tags$b(icon("cog"), " AI Assistant Setup"),
              tags$hr(style = "margin-top: 10px; margin-bottom: 15px;"),
              selectInput("ai_provider", "Provider:",
                choices = c(
                  "Google Gemini" = "gemini",
                  "OpenAI" = "openai",
                  "Groq" = "groq",
                  "OpenRouter" = "openrouter"
                ),
                width = "100%"
              ),
              textInput("ai_model", "Model:", value = "gemini-2.5-flash", placeholder = "e.g., gemini-2.5-flash", width = "100%"),
              passwordInput("ai_api_key", "API Key:", placeholder = "Enter your API key here", width = "100%"),
              div(
                style = "display: flex; gap: 10px; align-items: center; margin-top: 10px;",
                actionButton("test_ai_connection", "Test Connection", icon = icon("plug"), class = "btn-primary"),
                uiOutput("ai_connection_status_ui")
              ),
              tags$hr(style = "margin-top: 15px; margin-bottom: 15px;"),
              div(
                style = "font-size: 11px; margin-bottom: 15px; background: #f8f9fa; padding: 10px; border-radius: 5px;",
                tags$b("How to get an API Key:"),
                tags$ul(
                  style = "padding-left: 15px; margin-bottom: 0;",
                  tags$li("Gemini: ", tags$a(href = "https://aistudio.google.com/app/apikey", target = "_blank", "Google AI Studio")),
                  tags$li("OpenAI: ", tags$a(href = "https://platform.openai.com/api-keys", target = "_blank", "OpenAI Platform")),
                  tags$li("Groq: ", tags$a(href = "https://console.groq.com/keys", target = "_blank", "Groq Console")),
                  tags$li("OpenRouter: ", tags$a(href = "https://openrouter.ai/keys", target = "_blank", "OpenRouter"))
                )
              ),
              tags$small(style = "color: #666;", "Required to use the AI interpretation feature across all modules.")
            )
          )
        )
      )
    ),

    # Simple JS to change icon on toggle
    tags$script(HTML("
      // Export the current chat transcript as a standalone HTML file
      function downloadAiChat() {
        var el = document.getElementById('ai_chat_history_container');
        var body = el ? el.innerHTML : '';
        if (!body || body.trim() === '') {
          alert('There is no conversation to export yet.');
          return;
        }
        var stamp = new Date().toISOString().slice(0, 10);
        var html = '<!DOCTYPE html><html><head><meta charset=\"utf-8\">' +
                   '<title>projectLSA AI Assistant Chat</title>' +
                   '<style>body{font-family:Segoe UI,Arial,sans-serif;font-size:13px;' +
                   'line-height:1.6;max-width:860px;margin:30px auto;padding:0 20px;color:#222;}' +
                   'h1{font-size:18px;border-bottom:2px solid #2563eb;padding-bottom:6px;}' +
                   'b{color:#2563eb;}</style></head><body>' +
                   '<h1>projectLSA - AI Assistant Chat (' + stamp + ')</h1>' +
                   body + '</body></html>';
        var blob = new Blob([html], {type: 'text/html;charset=utf-8'});
        var a = document.createElement('a');
        a.href = URL.createObjectURL(blob);
        a.download = 'projectLSA_AI_chat_' + stamp + '.html';
        document.body.appendChild(a);
        a.click();
        document.body.removeChild(a);
        URL.revokeObjectURL(a.href);
      }

      // Send chat on Enter
      $(document).on('keydown', '#ai_global_chat_input', function(e) {
        if (e.which === 13 && !e.shiftKey) {
          e.preventDefault();
          $('#ai_global_chat_send').click();
        }
      });

      $(document).on('click', '#toggle_ai_widget', function(e) {
        e.preventDefault();
        var icon = $(this).find('i');
        var wrapper = $('#ai_widget_body_wrapper');
        if (wrapper.is(':visible')) {
          wrapper.hide();
          icon.removeClass('fa-chevron-down').addClass('fa-chevron-up');
        } else {
          wrapper.show();
          icon.removeClass('fa-chevron-up').addClass('fa-chevron-down');
        }
      });
      // Click header to toggle
      $(document).on('click', '#ai_widget_header', function(e) {
        if ($(e.target).closest('button').length === 0) {
          var wrapper = $('#ai_widget_body_wrapper');
          var icon = $('#toggle_ai_widget i');
          if (wrapper.is(':visible')) {
            wrapper.hide();
            icon.removeClass('fa-chevron-down').addClass('fa-chevron-up');
          } else {
            wrapper.show();
            icon.removeClass('fa-chevron-up').addClass('fa-chevron-down');
          }
        }
      });
      // Scroll to bottom on new message
      Shiny.addCustomMessageHandler('scroll_ai_chat', function(msg) {
        setTimeout(function() {
          var objDiv = document.getElementById('ai_chat_history_container');
          if (objDiv) objDiv.scrollTop = objDiv.scrollHeight;
        }, 100);
      });
    ")),

    # CSS to make sure the tabsetPanel and tab-content take up full height
    tags$style(HTML("
      #ai_widget_body .tab-content {
        flex: 1;
        display: flex;
        flex-direction: column;
        overflow: hidden;
      }
      #ai_widget_body .tab-pane.active {
        flex: 1;
        display: flex;
        flex-direction: column;
        height: 100%;
      }
    ")),
    tags$script(HTML("
      // Locate the module's main tabset.
      // The CFA/SEM module nests its tabset inside a col-sm-9; LPA, LCA, LTA and
      // EFA put their tabset at the top level of the module, so fall back to the
      // first visible tabset that is not part of the AI widget itself.
      function findHostTabset() {
        let candidates = $('.col-sm-9:visible').filter(function() {
          return $(this).find('.nav-tabs').length > 0;
        });
        let navTabs = null, tabContent = null;
        if (candidates.length) {
          navTabs = candidates.first().find('.nav-tabs').first();
          tabContent = candidates.first().find('.tab-content').first();
        } else {
          navTabs = $('.nav-tabs:visible').not('#ai_widget_body .nav-tabs').first();
          tabContent = navTabs.length ? navTabs.nextAll('.tab-content').first() : $();
        }
        return {navTabs: navTabs, tabContent: tabContent};
      }

      $(document).on('click', '#dock_ai_widget', function(e) {
        e.preventDefault();
        let aiBody = $('#ai_widget_body');

        let host = findHostTabset();
        let navTabs = host.navTabs;
        let tabContent = host.tabContent;

        if (navTabs.length && tabContent.length) {
          let tabId = 'ai_assistant_docked_tab_' + Date.now();
          aiBody.attr('data-docked-tab-id', tabId);

          navTabs.append('<li><a href=\"#' + tabId + '\" data-toggle=\"tab\">AI Assistant <span class=\"undock-ai-btn\" style=\"margin-left:8px;cursor:pointer;color:#888;\" title=\"Float Assistant\"><i class=\"fa fa-window-restore\"></i></span></a></li>');
          tabContent.append('<div class=\"tab-pane\" id=\"' + tabId + '\" style=\"padding-top: 15px;\"></div>');

          aiBody.css({
             'width': '100%',
             'height': '75vh',
             'resize': 'none',
             'border': '1px solid #ddd',
             'box-shadow': 'none'
          });

          $('#' + tabId).append(aiBody);
          $('#ai_floating_panel').hide();

          navTabs.find('a[href=\"#' + tabId + '\"]').tab('show');
        } else {
          alert('Cannot dock AI in this module layout. Try using it here.');
        }
      });

      // Undock logic
      $(document).on('click', '.undock-ai-btn', function(e) {
        e.preventDefault(); e.stopPropagation();
        let aiBody = $('#ai_widget_body');

        aiBody.css({
           'width': '380px',
           'height': '450px',
           'resize': 'both',
           'border': 'none'
        });

        $('#ai_widget_header').next().append(aiBody);
        $('#ai_floating_panel').show();

        let tabId = aiBody.attr('data-docked-tab-id');
        if (tabId) {
           $('a[href=\"#' + tabId + '\"]').parent('li').remove();
           $('#' + tabId).remove();
        }

        // Go back to the first tab of the host module
        let host = findHostTabset();
        if (host.navTabs.length) {
            host.navTabs.find('li:first-child a').tab('show');
        }
      });
    "))
  )
}

# Fungsi untuk menu Siswa
lta_ui <- function(project) {
    tabsetPanel(
      id = "main_tab_lta",
      
      # --- TAB 1: Prepare Data & Model ----
      tabPanel(
        title = tagList(icon("upload"), "Prepare Data & Model"),
        value = "prepare_tab_lta", 
        sidebarLayout(
          sidebarPanel(
            width = 3,
            # Tombol kembali ke halaman utama
            actionButton("go_home", 
                         label = tagList(icon("home"), "Main Menu"), 
                         class = "btn btn-danger btn-block",
                         style = "width: 100% !important;"),
            br(),
            #uiOutput("data_source_lta_ui"),
            selectInput("data_source_lta", 
              "Select Data Source:",choices = c(
                "UPLOAD DATA" = "upload",
                "Built in Data: Dichotomous" = "diko",
                "Built in Data: Polytomous" = "poli"),
              selected = "upload"
            ),
            conditionalPanel(
              condition = "input.data_source_lta == 'upload'",
              fileInput("datafile_lta", "Upload Data/Workspace (csv/xlsx/sav/rds)", accept = c(".csv", ".xlsx", ".sav", ".rds")),
              selectInput("datatype", 
                          "Select Data Type:",
                          choices = c("Dicotomous" = "diko",
                                      "Polytomous" = "poli"),
                          selected = 'poli')
            ),
            selectInput("dimension",
                        "Dimension:",
                        choices = c("Unidimensional" = "uni",
                                    "Multidimensional" = "multi"),
                        selected = "uni"),
            uiOutput("id_select_ui"),
            uiOutput("var_select_ui"),
            conditionalPanel(
              condition = "input.dimension == 'multi'",
              tags$label("Define Dimension Name & Item:"),
              uiOutput("lta_model_ui")
            ),
            selectInput("fit_stats", 
                        "Itemfit Stat:",
                        choices = c("S_X2 (complete cases only)" = "S_X2",
                                    "X2" = "X2",
                                    "X2*" = "X2*"),
                        selected = "S_X2"),
            
            actionButton("run_lta",
                         label = tagList(icon("play"), "Run LTA"), 
                         class = "btn btn-success",
                         style = "width: 100% !important; margin-bottom: 10px;"),
            downloadButton("export_lta_rds", "Export Model (.rds)", 
                           class = "btn btn-primary", 
                           style = "width: 100% !important;")
          ),
          mainPanel(
            width = 9,
            h5(icon("table"), "Data Preview"),
            DTOutput("data_preview_lta")
          )
        )
      ),
      # ==== Model Comparison
      tabPanel(
        title = tagList(icon("chart-line"), "Model Summary"),
        value = "summary_tab", 
        br(),
        
        column(
          12,
          # === 1️⃣ Model Comparison ===
          h4(icon("table"), "Model Fit Comparison"),
          uiOutput("fit_comparison"),
         
           tags$hr(style = "border-top: 2px solid #bbb;"),
          
          # === 2️⃣ Pilihan Model Terbaik ===
          column(4,
            h4(icon("cog"), "Select the Best Model"),
            p(
              "Choose the model you want to inspect in detail from the options below. ",
              "The best model can be determined by observing the smallest AIC/BIC and the highest item fit count above."),
              uiOutput('itemtype_ui')
            ),
          # === 3️⃣ Item Parameters & Fit ===
          column(8,
          h4(icon("chart-bar"), "Item Parameters & Item Fit Statistics"),
          uiOutput('item_summary')
          ),
          br(),
          br(),
          tags$hr(style = "border-top: 2px solid #bbb;"),
          column(12,
          h4(icon("chart-line"), "Item Characteristic Curve (ICC) and Information Function"),
          uiOutput('icc_info')
          ),
          br(),
          br(),
          br(),
        )
      ),
      tabPanel(
        title=tagList(icon("brain"), "Factor Scores"),
        column(12, uiOutput('fscoreLTA') )
      ),
      tabPanel(
        title = tagList(icon("info"), "Information & Reliability"),
        value = "info_rel_tab",
        uiOutput("info_rel_ui")
      ),
      tabPanel(
        title = tagList(icon("people-arrows"), "DIF Analysis"),
        value = "dif_tab",
        uiOutput("dif_ui")
      ),

      tabPanel(
        title = tagList(icon("file-import"), "Score New Data"),
        sidebarLayout(
          sidebarPanel(
            width = 3,
            h4("Score New Data"),
            p("Upload new data to calculate factor scores using the fitted LTA/IRT model."),
            downloadButton("download_lta_template", "Download Data Template (Excel)", class = "btn-info btn-block"),
            br(),br(),
            fileInput("lta_newdata", "Upload New Data (Excel/CSV)", accept = c(".csv", ".xlsx", ".xls")),
            actionButton("lta_score_newdata_btn", "Calculate Scores", icon = icon("calculator"), class = "btn-success btn-block")
          ),
          mainPanel(
            width = 9,
            div(style = "text-align: right; margin-bottom: 5px;",
                downloadButton("download_lta_newscores", "Download New Scores (.csv)", class = "btn-primary btn-sm")),
            DTOutput("lta_newscores_table")
          )
        )
      ),

      # =====================================
      # Report Preview
      # =====================================
      tabPanel(
        title = tagList(icon("file-alt"), " Report Preview"),
        column(12,
               br(),
               div(style = "display: flex; gap: 10px; margin-bottom: 15px;",
                   actionButton("lta_generate_preview", tagList(icon("sync"), " Generate Report Preview"), class = "btn btn-success"),
                   downloadButton("download_report_lta", "Download HTML Report", class = "btn btn-primary")
               ),
               div(
                 style = "border: 1px solid #ddd; border-radius: 4px; padding: 5px; background: #f9f9f9;",
                 uiOutput("lta_report_preview_frame")
               )
        )
      ),
      
      # ===== INFO ======
      tabPanel(
        title = tagList(icon("info-circle"), "About"),
        fluidRow(
          column(
            width = 8, offset = 2,
            br(),
            div(
              style = "text-align:center;",
              tags$hr(),
              tags$h5("projectLSA Was Developed By:"),
              tags$p(
                tags$a(
                  href = "https://scholar.google.com/citations?user=PSAwkTYAAAAJ&hl=id",
                  target = "_blank",
                  "Dr. Hasan Djidu, M.Pd."),
                tags$br(),
                "Universitas Sembilanbelas November Kolaka"
              ),
              tags$h5("Supervised By:"),
              tags$p(
                tags$a(
                  href = "https://scholar.google.com/citations?user=7CzPTYIAAAAJ&hl=id",
                  target = "_blank",
                  "Prof. Dr. Heri Retnawati, M.Pd."),
                tags$br(),
                "Universitas Negeri Yogyakarta"
              ),
              tags$p(tags$a(
                href = "https://scholar.google.com/citations?hl=id&user=VGKeBm0AAAAJ",
                target = "_blank",
                "Prof. Dr. Samsul Hadi"),
                tags$br(),
                "Universitas Negeri Yogyakarta"
              ),
              tags$p(tags$a(
                href = "https://scholar.google.com/citations?hl=id&user=k4MA8XgAAAAJ",
                target = "_blank",
                "Dr. Drs. Ir. Haryanto, M.Pd., M.T."),
                tags$br(),
                "Universitas Negeri Yogyakarta"
              ),
              tags$b("Contact:"),
              tags$br(),
              tags$a("hasandjidu@gmail.com"),
              tags$hr()
            )
          ),
          column(
            width = 8, offset = 2,
            h4("References (R Packages)"),
            uiOutput("package_references_lta"),
            br(),
            div(
              style = "text-align:center;",
              tags$p(
                style = "font-size:13px; color:#777;",
                format(Sys.Date(), "%Y"), 
                "projectLSA. All rights reserved."
              ) ))
        )
      )
    )
  }

library(shiny)
# ==== styleCSS.R ====
styleCSS <- tags$head(
  # Import FontAwesome untuk ikon
  tags$link(
    rel = "stylesheet",
    href = "https://cdnjs.cloudflare.com/ajax/libs/font-awesome/6.5.0/css/all.min.css"
  ),
  
  tags$style(HTML("
.container-fluid {
    padding-left: 5px !important;
    padding-right: 5px !important;
  }
  /* ================================
     STYLE UNTUK DATATABLE
  ================================ */
  table.dataTable {
    font-size: 12px !important;
    line-height: 0.7 !important;
    letter-spacing: -0.3px !important;
    border-collapse: collapse !important;
  }

  table.dataTable th,
  table.dataTable td {
    padding: 4px 6px !important;
  }

  table.dataTable thead th {
    background-color: darkcyan !important;
    color: white !important;
    height: 0.7 !important;
    font-weight: bold !important;
  }

  table.dataTable tbody tr:nth-child(even) {
    background-color: #f2f6fa !important;
  }

  /* ================================
     STYLE UNTUK TABEL BIASA
  ================================ */
  table, th, td {
    font-size: 12px !important;
    line-height: 0.7 !important;
    letter-spacing: -0.3px !important;
    border-collapse: collapse !important;
  }

  th {
    background-color: darkcyan !important;
    color: white !important;
    font-weight: bold !important;
  }

  tr:nth-child(even) {
    background-color: #f2f6fa !important;
  }

  td, th {
    padding: 4px 6px !important;
  }

  /* ================================
     STYLE UNTUK TOMBOL (BUTTON)
  ================================ */
  .btn {
    width: auto !important;
    padding: 5px 10px !important;
    font-size: 12px !important;
    line-height: 1 !important;
    border-radius: 8px !important;
    font-weight: 700 !important;
    transition: all 0.1s ease-in-out !important;
    display: block;
    margin: 0 auto;
  }

  .btn i {
    margin-right: 6px !important;
  }

  .btn:hover {
    box-shadow: 0 2px 6px rgba(0,0,0,0.2) !important;
  }

  .btn:active {
    transform: scale(0.97) !important;
    box-shadow: 0 1px 3px rgba(0,0,0,0.3) !important;
  }
  
  @keyframes pulse-glow {
    0% { box-shadow: 0 0 0 0 rgba(255, 193, 7, 0.7); }
    70% { box-shadow: 0 0 0 10px rgba(255, 193, 7, 0); }
    100% { box-shadow: 0 0 0 0 rgba(255, 193, 7, 0); }
  }
  
  .btn-glow {
    animation: pulse-glow 2s infinite !important;
  }

  /* Warna Tombol Khusus */
  .btn-primary {
    background-color: #0077dd !important;
    border: none !important;
  }
  .btn-primary:hover {
    background-color: #0056b3 !important;
  }

  .btn-danger {
    background-color: #fd4084 !important;
    border: none !important;
  }
  .btn-danger:hover {
    background-color: #a71d2a !important;
  }

  .btn-success {
    background-color: #28a745 !important;
    border: none !important;
  }
  .btn-success:hover {
    background-color: #1e7e34 !important;
  }
  /* =======================================
     STYLE   DT BUTTONS
   ======================================= */

.dt-button {
  padding: 4px 10px !important;    
  font-size: 9px !important;        
  border-radius: 4px !important; 
  background-color: black !important;
  color: white !important;
  border: none !important;
  font-weight: 600 !important;
  margin: 3px !important;
  line-height: 1 !important;
}

/* Hover */
.dt-button:hover {
  background-color: #005bb0 !important;
}

/* Active */
.dt-button:active {
  transform: scale(0.97) !important;
}

.dataTables_wrapper .dataTables_paginate .paginate_button {
    font-size: 10px !important; 
    padding: 2px 6px !important;     
    border-radius: 6px !important;  
  }

  .dataTables_wrapper .dataTables_paginate {
    margin-top: 2px !important;      
  }

  /* ================================
     STYLE UNTUK INPUT (select, numeric, file)
  ================================ */
  select.form-control,
  input.form-control,
  .selectize-input,
  .selectize-dropdown-content {
    border-radius: 12px !important;
    font-size: 12px !important;
    line-height: 1.2 !important;
    border: 1px solid #ddd !important;
  }

  .selectize-input {
    padding: 6px 10px !important;
    min-height: 32px !important;
  }

  .selectize-dropdown-content {
    font-size: 12px !important;
  }

  /* ================================
     STYLE UNTUK HOMEPAGE / LANDING PAGE
  ================================ */
  body {
    background: #fff;
    font-family: 'Poppins', sans-serif;
    margin: 0;
    padding: 0;
  }

  .landing {
    text-align: center;
    margin-top: 25px;
  }

  .hero {
    display: flex;
    flex-direction: column;
    align-items: center;
    gap: 5px;
  }

  .welcome {
    font-size: 16px;
    font-weight: 800;
    color: #444;
    letter-spacing: 1px;
  }

.app-name { font-size:36px; font-weight:900; margin-top:1px; color:transparent; -webkit-text-fill-color:black; -webkit-background-clip:text; background:none; }
.app-name { text-shadow:-2px -2px 3px #fff,2px -2px 3px #fff,-2px 2px 3px #fff,2px 2px 3px #fff,0 0 5px rgba(0,0,0,.45),0 0 10px rgba(0,0,0,.35),0 0 15px rgba(0,0,0,.25); }

  .subtitle{
    font-size: 13px;
    font-weight: 600;
    color: #444;
  }

  h4.page-subtitle {
    margin-top: 20px;
    color: #333;
  }

  .quad-container {
    position: relative;
    z-index: 0;
    width: 90vw;
    height: 60vh;
    margin: 10px auto;
    display: grid;
    grid-template-columns: 1fr 1fr;
    grid-template-rows: 1fr 1fr;
    gap: 10px;
    justify-items: center;
    align-items: center;
  }

  .quad-container::before,
  .quad-container::after {
    content: '';
    position: absolute;
    z-index: 1;
  }

  
.quad-container::before{
  top:50%;left:0;width:100%;height:3px;
  background:linear-gradient(90deg,blue,red);
}

.quad-container::after{
  left:50%;top:0;width:3px;height:100%;
  background:linear-gradient(180deg,green,red);
}

  .axis-label {
    position: absolute;
    font-weight: bold;
    color: black;
    font-size: clamp(12px, 2vw, 14px);
    background: white;
    border-radius: 6px;
    z-index: 5;
  }

  .axis-label.x-top {
    top: 0;
    left: 50%;
    transform: translateX(-50%);
  }

  .axis-label.x-bottom {
    bottom: 0;
    left: 50%;
    transform: translateX(-50%);
  }

  .axis-label.y-left {
    left: 0;
    top: 50%;
    transform: translateY(-50%);
  }

  .axis-label.y-right {
    right: 0;
    top: 50%;
    transform: translateY(-50%);
  }

  .center-logo {
    position: absolute;
    top: 50%;
    left: 50%;
    transform: translate(-50%, -50%);
    z-index: 6;
    padding: 6px;
  }

  .center-logo img {
    width: clamp(100px, 2vw, 200px);
    height: auto;
    border-radius: 50%;
  }

  .quad-card {
    width: 90%;
    height: 80%;
    border-radius: 21px;
    box-shadow: 0 4px 10px rgba(0,0,0,0.15);
    padding: 1.5vw;
    display: flex;
    flex-direction: column;
    justify-content: space-between;
    align-items: center;
    text-align: center;
    z-index: 2;
    transition: transform .5s;
  }

  .quad-card:hover {
    transform: scale(1.05);
  }
#card_lca { background: #eef3ff; border: 2px solid #c7d4ff; }   /* Blue */
#card_lpa { background: #edf7f6; border: 2px solid #c6e6e2; }   /* Teal */
#card_fa  { background: #f3effa; border: 2px solid #d9cff0; }   /* Purple */
#card_lta { background: #fbeef2; border: 2px solid #f2cdd8; }   /* Rose */
/* Warna & tema masing-masing card */
#card_lpa .project-icon { background: #dff1ee; color: #138f87; } /* Teal */
#card_lca .project-icon { background: #dde6ff; color: #2a55e6; } /* Blue */
#card_fa  .project-icon { background: #e6e0f5; color: #6b4cc2; } /* Purple */
#card_lta .project-icon { background: #f6dde6; color: #c23b63; } /* Rose */

#card_lca .btn-pill { background: #2a55e6; }  /* Blue */
#card_lpa .btn-pill { background: #138f87; }  /* Teal */
#card_fa  .btn-pill { background: #6b4cc2; }  /* Purple */
#card_lta .btn-pill { background: #c23b63; }  /* Rose */

  .project-title {
    font-size: clamp(12px, 1.5vw, 27px);
    font-weight: 600;
    margin-bottom: 4px;
  }

  .project-desc {
    color: #444;
    font-size: clamp(12px, 1vw, 120px);
    line-height: 1.3em;
    padding: 0 .5vw;
  }

  .project-icon {
    font-size: clamp(25px, 2vw, 40px);
    width: clamp(40px, 4vw, 60px);
    height: clamp(40px, 4vw, 60px);
    display: flex;
    align-items: center;
    justify-content: center;
    border-radius: 12px;
  }



  .btn-pill {
    display: block !important;
    width: 100% !important;
    padding: clamp(5px, .8vw, 10px) 0;
    border-radius: 8px;
    text-align: center;
    text-decoration: none;
    margin-top: 3px;
    color: #fff;
    font-size: clamp(9px, 1vw, 18px);
    transition: .1s;
  }

  .btn-pill:hover {
    opacity: .85;
    transform: scale(1.001);
  }
  
   /* Styling untuk labels */
         .shiny-input-container label {
           font-size: 12px !important;
           font-weight: normal !important;
           color: #555555 !important;
           margin-bottom: 3px !important;
         }
         
         /* Styling untuk input fields */
         .form-control {
           font-size: 12px !important;
           height: 30px !important;
           padding: 4px 8px !important;
         }
         
         /* Styling khusus untuk numeric input */
         input[type='number'] {
           font-size: 12px !important;
           height: 28px !important;

         }
         
         /* Styling untuk select input */
         select.form-control {
           font-size: 12px !important;
           height: 30px !important;
           padding: 4px 8px !important;
         }
/* === Style khusus untuk selectInput besar (itemtype) === */
.select-large .selectize-input {
  font-size: 18px !important;
  font-weight: bold !important;
  text-align: center !important;
  padding: 12px 16px !important;
  border-radius: 14px !important;
  background-color: lightgreen !important;
  border: 2px solid #cfcfcf !important;
  box-shadow: 0 3px 6px rgba(0,0,0,0.1) !important;
  transition: all 0.2s ease-in-out !important;
}

.select-large .selectize-input:hover {
  border-color: #007bff !important;
  box-shadow: 0 0 8px rgba(0,123,255,0.3) !important;
}

.select-large .selectize-input.focus {
  border-color: #0056b3 !important;
  box-shadow: 0 0 0 0.2rem rgba(0,123,255,0.25) !important;
}

.select-large .option {
  text-align: center !important;
  font-size: 16px !important;
  font-weight: bold !important;
}

/* === Style selectInput mini tanpa highlight pilihan (versi tinggi lebih pendek) === */
.select-mini .selectize-input {
  font-size: 12px !important;
  font-weight: 600 !important;
  text-align: center !important;
  justify-content: center !important;

  padding: 1px 6px !important;          
  line-height: 12px !important;  
  min-height: 18px !important;     
  height: 18px !important;
  
  border-radius: 8px !important;
  background: lightgreen !important;
  border: 1.5px solid #b8bced !important;
  box-shadow: 0 2px 3px rgba(0,0,0,0.07) !important;
  transition: all 0.2s ease-in-out !important;
}

.select-mini .selectize-input:hover {
  border-color: #756bff !important;
  box-shadow: 0 0 5px rgba(117,107,255,0.33) !important;
}

.select-mini .selectize-input.focus {
  border-color: #574cff !important;
  box-shadow: 0 0 0 0.15rem rgba(87,76,255,0.22) !important;
}

.select-mini .option {
  text-align: center !important;
  font-size: 12px !important;
  font-weight: 600 !important;
}

/* Hilangkan highlight */
.select-mini .selectize-dropdown .highlight {
  background: none !important;
  color: inherit !important;
  box-shadow: none !important;
}


         /* Styling untuk text input */
         input[type='text'] {
           font-size: 12px !important;
         }
         
         /* Styling untuk checkbox labels */
         .checkbox label {
           font-size: 12px !important;
           font-weight: normal !important;
           color: #555555 !important;
         }
  
  
  .well {
        background-color: #f8f9fa;
        border: 1px solid #dee2e6;
      }
      .shiny-output-error { color: #dc3545; }
      .shiny-output-error:before { content: 'Error: '; }
      .small-input { max-width: 100px; }

  @media (max-width: 700px) {
    .quad-container {
      transform: scale(1);
      transform-origin: center top;
    }
  }
  
.badge-warning{
  background:#fee2e2;
  border-left:4px solid #ef4444;
  color:#7f1d1d;
  padding:8px 12px;
  border-radius:5px;
  font-size:12px;
  line-height:1.4;
}

.badge-info{
  background:#e0e7ff;
  border-left:4px solid #3b82f6;
  color:#1e3a8a;  
  padding:10px 14px;
  border-radius:4px;
  font-size:13px;
  line-height:1.6;
}

.badge-appinfo{
  background:#fffef9;            /* ivory lembut */
  border-left:4px solid #334155; /* slate blue soft */
  color:#374151;                 /* dark gray (BUKAN hitam) */
  padding:10px 14px;
  border-radius:4px;
  font-size:13px;
  line-height:1.6;
}


/* Background + border modal */
.modal-content{
  background:#fffdf7;        /* ivory */
  border:2px solid #1e293b;  /* biru kehitaman */
  border-radius:16px;
  box-shadow:0 12px 30px rgba(15,23,42,.25);
}

/* Header & footer lebih bersih */
.modal-header, .modal-footer{
  border:none;
}

.modal.fade .modal-dialog {
  transform: translateY(-30px) scale(1);
  transition: transform 1.3s cubic-bezier(0.22, 1, 0.36, 1);
}

.modal.show .modal-dialog {
  transform: translateY(0) scale(1);
}

/* ==== Floating widgets (AI Assistant & R Console) ==== */
.floating-widget {
  opacity: 0.6;
  transition: opacity 0.3s ease-in-out;
}
.floating-widget:hover {
  opacity: 1.0;
}

  "))
)

# fa_ui.R (revisi)
efa_ui <- function(project) {
  tabsetPanel(
    id = "main_tab_fa",
    
    # === TAB 1: Prepare Data & Model ====
    tabPanel(
      title = tagList(icon("upload"), "Prepare Data & Model"),
      sidebarLayout(
        sidebarPanel(
          width = 3,
          # Tombol kembali
          actionButton("go_home", 
                       label = tagList(icon("home"), "Main Menu"), 
                       class = "btn btn-danger btn-block",
                       style = "width: 100% !important;"),
          br(),
          
          # === Input sumber data ===
          selectInput("data_source", "Select Data Source:",
                      choices = c("Upload Data" = "upload",
                                  "Built-in: bfi" = "bfi",
                                  "Built-in: HolzingerSwineford1939" = "HolzingerSwineford1939"
                                  
                                 )),
          
          conditionalPanel(
            condition = "input.data_source == 'upload'",
            fileInput("datafile", "Upload Data/Workspace (csv/xlsx/sav/rds)", accept = c(".csv", ".xlsx", ".sav", ".rds"))),
          uiOutput("id_select_ui_fa"),
          uiOutput("var_select_ui"),
          br(),
          selectInput("rotation_method", "Rotation Method:", 
                      choices = c("varimax", "oblimin", "promax", "none")),
          selectInput("missing_method_efa", "Handle Missing Data:",
                      choices = c("Listwise Deletion" = "listwise",
                                  "Pairwise Deletion" = "pairwise",
                                  "Mean Imputation" = "mean")),
          br(),
          actionButton("run_efa", label = tagList(icon("play"), "Run EFA"),
                       class = "btn btn-success",
                       style = "width: 100% !important; margin-bottom: 10px;"),
          downloadButton("export_efa_rds", "Export Model (.rds)", 
                         class = "btn btn-primary", 
                         style = "width: 100% !important;")
        ),
        
        mainPanel(
          width = 9,
          DTOutput("data_preview_fa"),
          tags$hr(),
          bsCollapse(id = "efa_data_preview_collapse", multiple = TRUE,
            bsCollapsePanel("Data Summary", style = "default",
              fluidRow(
                column(12, selectInput("efa_summary_theme", "Chart Theme:", choices = c("Default" = "default", "Pastel" = "pastel", "Blue" = "blue", "Dark" = "dark")))
              ),
              tabsetPanel(
                tabPanel("Summary Table", 
                         br(),
                         uiOutput("efa_summary_table")
                ),
                tabPanel("Visualizations", 
                         br(),
                         fluidRow(
                           column(6, plotOutput("efa_data_summary_plot", height = "350px")),
                           column(6, plotOutput("efa_data_heatmap", height = "350px"))
                         )
                )
              )
            ),
            bsCollapsePanel("Calculate Variable", style = "default",
              fluidRow(
                column(4, shinyWidgets::pickerInput("efa_agg_vars", "Variables to Calculate:", choices = NULL, multiple = TRUE, width = "100%", options = list(`actions-box` = TRUE, `live-search` = TRUE))),
                column(3, textInput("efa_agg_name", "New Variable Name:", placeholder = "e.g., Factor1_Score")),
                column(3, selectInput("efa_agg_method", "Calculation Method:", choices = c("Mean" = "mean", "Sum" = "sum"))),
                column(2, 
                       actionButton("efa_btn_aggregate", "Calculate", class = "btn-primary", style = "margin-top: 25px; width: 100%;"),
                       actionButton("efa_btn_reset_agg", "Reset", class = "btn-danger btn-sm", style = "margin-top: 5px; width: 100%;")
                )
              )
            )
          )
        )
      )
    ),
    # ===== Pre-Analisis  ======
    tabPanel(
      "KMO & Bartlett Tests",
      
      fluidRow(
        column(12, uiOutput("efa_tests")),
        column(6, plotOutput("kmo_item" )),
        column(6, plotOutput("scree_plot"))
      )
      ),
    tabPanel(
      "EFA Results",
      fluidRow(
        
        # ==================== KIRI: EFA SUMMARY =====================
        column(
          6,
          div(
            class = "card shadow-sm p-3 mb-3 bg-white rounded",
            style = "height: 80vh; overflow-y: auto;",
            
            # ==== Header & Input ====
            div(
              style = "display: flex;flex-direction: column;align-items: center;justify-content: flex-start;text-align: center;width: 100%;padding: 10px;",
              # === Input Number of Factors (di tengah) ===
              div(
                style = "display: flex; flex-direction: column; align-items: center;",
                tags$h4(icon('chart-simple'), "Exploratory Factor Analysis Summary", style = "font-weight:bold;color:#2c3e50;"),
                tags$b("Number of Factors:"),
                numericInput("n_factors", label = NULL, value = 3, min = 1, max = 10, width = '120px')
              ),
              # === Dynamic factor names (juga di tengah) ===
              tags$b("Factors' Names:"),
              uiOutput("factor_names_ui")
            ),
            
            hr(),
            # ==== Tombol download di kanan ====
            div(
              style = "text-align: right; margin-bottom: 5px; width: 100%;",
              downloadButton(
                "download_loading",
                "Download Loadings (.csv)",
                class = "btn btn-primary btn-block btn-sm"
              )
            ),
            
            # ==== Output ringkasan ====
            uiOutput("efa_summary")
          )
        ),
        
        # ==================== KANAN: FACTOR SCORES =====================
        column(
          6,
          div(
            class = "card shadow-sm p-3 mb-3 bg-white rounded",
            style = "height: 80vh; overflow-y: auto;",
            div(
              style = "display: flex; flex-direction: column; align-items: center;justify-content: flex-start;text-align: center;width: 100%;padding: 10px;",
              tags$h4(icon("table"), "Factor Scores", style = "font-weight:bold;color:#2c3e50;"),
              tags$p("These are estimated latent factor scores for each observation.")
            ),
            div(
              class = "card shadow-sm p-3 mb-3 bg-white rounded",
              style = "height: 80vh; overflow-y: auto;",
              # ==== Tombol download skor ====
              div(
                style = "text-align: right; margin-bottom: 5px; width: 100%;",
                downloadButton(
                  "download_scores",
                  "Download Factor Scores (.csv)",
                  class = "btn btn-primary btn-block btn-sm"
                )
              ),
              
              # ==== Output skor faktor ====
              DTOutput ("efa_scores_table")
            )
          )
        )
      )
    ),

    tabPanel(
      "Score New Data",
      sidebarLayout(
        sidebarPanel(
          width = 3,
          h4("Score New Data"),
          p("Upload new data to calculate factor scores using the fitted EFA model."),
          downloadButton("download_efa_template", "Download Data Template (Excel)", class = "btn-info btn-block"),
          br(),br(),
          fileInput("efa_newdata", "Upload New Data (Excel/CSV)", accept = c(".csv", ".xlsx", ".xls")),
          actionButton("efa_score_newdata_btn", "Calculate Scores", icon = icon("calculator"), class = "btn-success btn-block")
        ),
        mainPanel(
          width = 9,
          div(style = "text-align: right; margin-bottom: 5px;",
              downloadButton("download_efa_newscores", "Download New Scores (.csv)", class = "btn-primary btn-sm")),
          DTOutput("efa_newscores_table")
        )
      )
    ),

    tabPanel(
      title = tagList(icon("file-alt"), " Report Preview"),
      column(12,
             br(),
             div(style = "display: flex; gap: 10px; margin-bottom: 15px;",
                 actionButton("efa_generate_preview", tagList(icon("sync"), " Generate Report Preview"), class = "btn btn-success"),
                 downloadButton("download_report_efa", "Download HTML Report", class = "btn btn-primary")
             ),
             div(
               style = "border: 1px solid #ddd; border-radius: 4px; padding: 5px; background: #f9f9f9;",
               uiOutput("efa_report_preview_frame")
             )
      )
    ),

    tabPanel(
      title = tagList(icon("info-circle"), "About"),
      fluidRow(
        column(
          width = 8, offset = 2,
          br(),
          div(
            style = "text-align:center;",
            tags$hr(),
            tags$h5("projectLSA Was Developed By:"),
            tags$p(
              tags$a(
                href = "https://scholar.google.com/citations?user=PSAwkTYAAAAJ&hl=id",
                target = "_blank",
                "Dr. Hasan Djidu, M.Pd."),
              tags$br(),
              "Universitas Sembilanbelas November Kolaka"
            ),
            tags$h5("Supervised By:"),
            tags$p(
              tags$a(
                href = "https://scholar.google.com/citations?user=7CzPTYIAAAAJ&hl=id",
                target = "_blank",
                "Prof. Dr. Heri Retnawati, M.Pd."),
              tags$br(),
              "Universitas Negeri Yogyakarta"
            ),
            tags$p(tags$a(
              href = "https://scholar.google.com/citations?hl=id&user=VGKeBm0AAAAJ",
              target = "_blank",
              "Prof. Dr. Samsul Hadi"),
              tags$br(),
              "Universitas Negeri Yogyakarta"
            ),
            tags$p(tags$a(
              href = "https://scholar.google.com/citations?hl=id&user=k4MA8XgAAAAJ",
              target = "_blank",
              "Dr. Drs. Ir. Haryanto, M.Pd., M.T."),
              tags$br(),
              "Universitas Negeri Yogyakarta"
            ),
            tags$b("Contact:"),
            tags$br(),
            tags$a("hasandjidu@gmail.com"),
            tags$hr()
          )
        ),
        column(
          width = 8, offset = 2,
          h4("References (R Packages)"),
          uiOutput("package_references_efa"),
          br(),
          div(
            style = "text-align:center;",
            tags$p(
              style = "font-size:13px; color:#777;",
              format(Sys.Date(), "%Y"), 
              "projectLSA. All rights reserved."
            )
          )
        )
      )
    )
  )
}

# cfa_ui.R - projectLSA CFA/SEM Module UI
cfa_ui <- function(project) {
  fluidRow(
    column(width = 3,
           
           wellPanel(
             actionButton("go_home", 
                        label = tagList(icon("home"), "Main Menu"), 
                        class = "btn btn-danger btn-block",
                        style = "width: 100% !important;"),
             br(),
             selectInput("data_source", "Select Data Source:",
                         choices = c("Upload Data" = "upload",
                                     "Built-in: bfi (CFA/EFA)" = "bfi",
                                     "Built-in: HolzingerSwineford1939 (MGCFA/CFA)" = "HolzingerSwineford1939",
                                     "Built-in: PoliticalDemocracy (SEM)" = "PoliticalDemocracy",
                                     "Built-in: Demo.growth (LGM)" = "Demo.growth"),
                         selected = "upload"),
             conditionalPanel(condition = "input.data_source == 'upload'",
                              fileInput("datafile", "Upload Data/Workspace (csv/xlsx/sav/rds)", accept = c(".csv", ".xlsx", ".sav", ".rds"))),
             uiOutput("id_select_ui"),
             uiOutput("var_select_ui"),
             tags$label("Model (Lavaan syntax):"),
             uiOutput("cfa_model_ui"),
             
             tags$div(
               style = "margin-top: 15px; margin-bottom: 5px;",
               actionLink("cfa_method_guide", " Methodological Guide", icon = icon("info-circle"), 
                          style = "font-weight: bold; color: #17a2b8; font-size: 13px;")
             ),
             
             selectInput("cfa_estimator", "Estimator:",
                         choices = c("ML" = "ML", "GLS" = "GLS", "WLS" = "WLS", 
                                     "DWLS" = "DWLS", "ULS" = "ULS", "DLS" = "DLS", 
                                     "PML" = "PML", "MLM" = "MLM", "MLMVS" = "MLMVS", 
                                     "MLMV" = "MLMV", "MLF" = "MLF", "MLR" = "MLR", 
                                     "WLSM" = "WLSM", "WLSMVS" = "WLSMVS", "WLSMV" = "WLSMV", 
                                     "ULSM" = "ULSM", "ULSMVS" = "ULSMVS", "ULSMV" = "ULSMV"),
                         selected = "ML"),
             selectInput("cfa_missing", "Missing data:",
                         choices = c("Listwise" = "listwise", "Pairwise" = "pairwise", "FIML" = "fiml"),
                         selected = "listwise"),
             checkboxInput("cfa_std_est", "Standardized estimates", TRUE),
             checkboxInput("htmt_opt", "Heterotrait–Monotrait Ratio (HTMT)", FALSE),
             
             actionButton("run_cfa", label = tagList(icon("play"), "Run CFA/SEM"),
                          class = "btn btn-success",
                          style = "width: 100% !important; margin-bottom: 10px;"),
             downloadButton("export_cfa_rds", "Export Model (.rds)",
                            class = "btn btn-primary",
                            style = "width: 100% !important; margin-bottom: 10px;"),
             actionButton("btn_export_modal", "Export Report (HTML)",
                          class = "btn btn-warning btn-block btn-glow",
                          style = "width: 100%; font-weight: bold;",
                          onclick = "$(this).removeClass('btn-glow');")
           )
    ), 
    column(width = 9,
           tabsetPanel(
             id = "main_tab_cfa",
             
             tabPanel(
               title = tagList(icon("upload"), "Data Preview"),
               br(),
               
               DTOutput("data_preview"),
               tags$hr(),
               
               bsCollapse(id = "cfa_data_preview_collapse", multiple = TRUE,
                 bsCollapsePanel("Data Summary", style = "default",
                    fluidRow(
                      column(12, selectInput("cfa_summary_theme", "Chart Theme:", choices = c("Default" = "default", "Pastel" = "pastel", "Blue" = "blue", "Dark" = "dark")))
                    ),
                   tabsetPanel(
                     tabPanel("Summary Table", 
                              br(),
                              uiOutput("cfa_summary_table")
                     ),
                     tabPanel("Visualizations", 
                              br(),
                              fluidRow(
                                column(6, plotOutput("cfa_data_summary_plot", height = "350px")),
                                column(6, plotOutput("cfa_data_heatmap", height = "350px"))
                              )
                     )
                   )
                 ),
                 bsCollapsePanel("Calculate Variable", style = "default",
                   fluidRow(
                     column(5,
                       tags$label("Calculation Syntax:"),
                       tags$p(style = "font-size: 11px; color: #666; margin-bottom: 2px;", "Format: NewVar = var1, var2, var3"),
                       textAreaInput("cfa_agg_syntax", label = NULL, rows = 6, placeholder = "A = x1, x2, x3\nB = y1, y2")
                     ),
                     column(4,
                       tags$label("Insert Variables:"),
                       shinyWidgets::pickerInput("cfa_agg_vars_insert", label = NULL, choices = NULL, multiple = TRUE, width = "100%", options = list(`actions-box` = TRUE, `live-search` = TRUE)),
                       actionButton("cfa_btn_insert_agg", "Insert to Syntax", icon = icon("arrow-left"), class = "btn-info btn-sm", style = "width: 100%;")
                     ),
                     column(3,
                       selectInput("cfa_agg_method", "Method (Global):", choices = c("Mean" = "mean", "Sum" = "sum")),
                       actionButton("cfa_btn_aggregate", "Calculate All", class = "btn-primary", style = "width: 100%; margin-bottom: 5px;"),
                       actionButton("cfa_btn_reset_agg", "Reset Data", class = "btn-danger btn-sm", style = "width: 100%;")
                     )
                   )
                 )
               )),
             # ====Model Summary =====
             tabPanel(
               title = tagList(icon("chart-line"), "Model Summary"),
                      value = "fit_tab_cfa", 
                      br(),
                      fluidRow(
                        uiOutput("fit_comparison")

                      )),
             tabPanel(
               title = tagList(icon("table"), "Parameter Estimates"),
               br(), 
               tags$h4("Factor Loadings & Regression Paths"),
               DTOutput("loadings_table"),
               br(), tags$hr(), br(),
               tags$h4("Variances & Covariances"),
               uiOutput("variance_warning"),
               DTOutput("variances_table")
               ),
             tabPanel(
               title = tagList(icon("calculator"), "Factor Scores"),
               br(),
               tabsetPanel(
                 tabPanel("Current Data Scores",
                          br(),
                          DTOutput("fscores_cfa")
                 ),
                 tabPanel("Score New Data",
                          br(),
                          wellPanel(
                            style = "background: #f4f6f9; border: 1px solid #e2e8f0; padding-bottom: 5px;",
                            fluidRow(
                              column(12,
                                tags$h4(icon("file-import"), " Score New Data"),
                                tags$p("Upload new data to calculate factor scores using the fitted CFA model.")
                              ),
                              column(4,
                                downloadButton("download_cfa_template", "1. Download Template (Excel)", class = "btn-info", style = "width: 100%; margin-top: 10px;")
                              ),
                              column(4,
                                fileInput("cfa_newdata", "2. Upload New Data (Excel/CSV)", accept = c(".csv", ".xlsx", ".xls"), width = "100%")
                              ),
                              column(4,
                                actionButton("cfa_score_newdata_btn", "3. Calculate Scores", icon = icon("calculator"), class = "btn-success", style = "width: 100%; margin-top: 25px;")
                              )
                            )
                          ),
                          br(),
                          bsCollapse(id = "cfa_newdata_collapse",
                            bsCollapsePanel("Calculate Variable (Optional)", style = "info",
                              fluidRow(
                                column(5,
                                  tags$label("Calculation Syntax:"),
                                  tags$p(style = "font-size: 11px; color: #666; margin-bottom: 2px;", "Format: NewVar = var1, var2, var3"),
                                  textAreaInput("cfa_newdata_agg_syntax", label = NULL, rows = 6, placeholder = "A = x1, x2, x3\nB = y1, y2")
                                ),
                                column(4,
                                  tags$label("Insert Variables:"),
                                  shinyWidgets::pickerInput("cfa_newdata_agg_vars_insert", label = NULL, choices = NULL, multiple = TRUE, width = "100%", options = list(`actions-box` = TRUE, `live-search` = TRUE)),
                                  actionButton("cfa_newdata_btn_insert_agg", "Insert to Syntax", icon = icon("arrow-left"), class = "btn-info btn-sm", style = "width: 100%;")
                                ),
                                column(3,
                                  selectInput("cfa_newdata_agg_method", "Method (Global):", choices = c("Mean" = "mean", "Sum" = "sum")),
                                  actionButton("cfa_newdata_btn_aggregate", "Calculate All", class = "btn-primary", style = "width: 100%; margin-bottom: 5px;"),
                                  actionButton("cfa_newdata_btn_reset_agg", "Reset Data", class = "btn-danger btn-sm", style = "width: 100%;")
                                )
                              )
                            )
                          ),
                          br(),
                          div(style = "text-align: right; margin-bottom: 5px;",
                              downloadButton("download_cfa_newscores", "Download New Scores (.csv)", class = "btn-primary btn-sm")),
                          DTOutput("cfa_newscores_table")
                 )
               )
             ),
             tabPanel(
               title = tagList(icon("cogs"), "Advanced Analysis"),
               value = "invariance_tab_cfa",
               br(),
               fluidRow(
                 column(4, 
                   wellPanel(
                     tags$h4(icon("sliders"), " Analysis Mode"),
                     selectInput("cfa_analysis_mode", "Analysis Mode:",
                                 choices = c("Standard CFA/SEM" = "standard",
                                             "Multi-group CFA" = "mgcfa",
                                             "Latent Growth Model" = "lgm")),
                     conditionalPanel("input.cfa_analysis_mode == 'mgcfa'",
                       uiOutput("mgcfa_group_ui"),
                       selectInput("mgcfa_invariance", "Invariance Level:",
                                   choices = c("Configural" = "configural",
                                               "Metric (Weak)" = "loadings",
                                               "Scalar (Strong)" = "loadings, intercepts",
                                               "Strict" = "loadings, intercepts, residuals")),
                       tags$hr(style="margin-top:10px; margin-bottom:10px;"),
                       tags$p(style="font-size:11px; color:#666;", "Or run all levels to compare:"),
                       actionButton("run_auto_invariance", "Auto Invariance Test", class = "btn-info btn-block btn-sm", style="width:100%;")
                     ),
                     conditionalPanel("input.cfa_analysis_mode == 'lgm'",
                       tags$p(style="font-size:11px; color:#666;", 
                              "Note: LGM uses lavaan::growth(). Ensure your syntax defines intercept (i) and slope (s) factors.")
                     )
                   )
                 ),
                 column(8,
                   tags$h4("Advanced Analysis Results"),
                   uiOutput("invariance_results")
                 )
               )
             ),
            # ====Plot =====
             tabPanel(
               title = tagList(icon("project-diagram"), "Path Plot"),
                      br(),
                      fluidRow(
                        column(3,
                        bsCollapse(
                          id = "plot_settings",
                          
                          bsCollapsePanel( style = 'info',
                                           "1. General & Layout",
                                     fluidRow(
                                       column(12, selectInput("plot_model_scope", "Display Scope:", 
                                                              choices = c("Full Model (Measurement + Structural)" = "full", 
                                                                          "Structural Model Only" = "structural"), 
                                                              selected = "full"))
                                     ),
                                     fluidRow(
                                       column(6, selectInput("plot_style", "Style:", choices = c("lisrel", "ram", "mx", "OpenMx"), selected = "lisrel")),
                                       column(6, selectInput("plot_layout", "Layout:", choices = c("tree", "tree2", "tree3", "spring", "circle", "circle2"), selected = "tree2"))
                                     ),
                                   fluidRow(
                                     column(6, numericInput("plotwidth", "Width:", value = 5, min = 1, max = 15, step = 0.3)),
                                     column(6, numericInput("plotheight", "Height:", value = 5, min = 1, max = 20, step = 0.3))
                                   ),
                                   
                                   fluidRow(
                                     column(6, numericInput("plot_rotation", "Rotation:", value = 4, min = 1, max = 4, step = 1)),
                                     column(6, textInput("bifactor", "Bifactor:", value = NULL, placeholder = "Gen. Factor"))
                                   )
                                 ),
                                 
                          bsCollapsePanel( style = 'info',
                                           "2. Node & Edge Sizes",
                                   fluidRow(
                                     column(6, numericInput("plot_nodesize_lat", "Latent Size:", value = 8, min = 1, max = 15, step = 0.2)),
                                     column(6, numericInput("plot_nodesize_man", "Obs(W) Size:", value = 8, min = 1, max = 15, step = 0.2))
                                   ),
                                   fluidRow(
                                     column(6, numericInput("plot_nodesize_man2", "Obs(H) Size:", value = 4, min = 0.1, max = 10, step = 0.1)),
                                     column(6, numericInput("plot_edge_label_size", "Label Size:", value = 0.75, min = 0.2, max = 2, step = 0.05))
                                   ),
                                   fluidRow(
                                     column(12, numericInput("edgewidth", "Edge Width:", value = 0.3, min = 0.1, max = 5, step = 0.05))
                                   )
                                 ),
                          bsCollapsePanel( style = 'info',
                                           "3. Measurement Model",
                                   checkboxInput("plot_curve", "Curve", TRUE),
                                   checkboxInput("plot_layout_split", "Split Layout", FALSE),
                                   
                                   fluidRow(
                                     column(6, numericInput("subScale_Wi", "Sub_Wid:", value = 0.2, min = 0.2, max = 3, step = 0.1)),
                                     column(6, numericInput("subScale_He", "Sub_Hei:", value = 0.2, min = 0.2, max = 3, step = 0.1))
                                   )
                                 ),
                          bsCollapsePanel( style = 'info',
                                           "4. Colors & Display Options",
                                   selectInput("plot_color_scheme", "Color scheme:",
                                               choices = c("Blue-Yellow", "Ocean", "Forest", "Rainbow", "Pastel", "Greyscale", "Earth", "Vibrant", "Monochrome", "Sunset", "Rose", "Mint", "Custom"),
                                               selected = "Blue-Yellow"),
                                   conditionalPanel(
                                     condition = "input.plot_color_scheme == 'Custom'",
                                     fluidRow(
                                       column(6, colourpicker::colourInput("mancolour", "Manifest:", value = "#A1E3F9", showColour = "background", palette = "square")),
                                       column(6, colourpicker::colourInput("latcolour", "Latent:", value = "#FFFFBA", showColour = "background", palette = "square"))
                                     )
                                   ),
                                   checkboxInput("plot_standardized", "Standardized estimates", TRUE),
                                   checkboxInput("plot_residuals", "Show residuals", FALSE),
                                   checkboxInput("plot_exoCov", "Show exogenous covariances", FALSE)
                                 ),
                          bsCollapsePanel( style = 'info',
                                           "5. Fit Indices",
                                           div(style = "text-align: center;",
                                             column(12, shinyWidgets::pickerInput("fit_indices_selected", "Select Fit Indices to Show:", 
                                       choices = c("SELECT ALL", "chisq", "df", "pvalue", "cmindf", "rmsea", "cfi", "tli", "srmr", "gfi", "nfi"), 
                                       selected = c("chisq", "df", "pvalue", "rmsea", "cfi", "tli", "srmr"), 
                                       multiple = TRUE, options = list(`actions-box` = TRUE, `live-search` = TRUE), width="100%")),
                                             column(12, checkboxInput("cfa_use_robust_fit", "Use Robust/Scaled Indices (if available)", TRUE))
                                           )
                          )
                        )
                        ),
                        column(9, 
                               downloadButton("download_cfa_plot", "Download Plot (PNG)", class = "btn btn-primary btn-glow"),
                               br(),
                               plotOutput("path_plot", height = "700px")
                        )
                      )),

             # --- Settings ---
             tabPanel(
               title = icon("sliders-h"),
               value = "settings_tab_cfa",
               br(),
               tags$h3(icon("sliders-h"), " General Settings"),
               tags$hr(),
               fluidRow(
                 column(6,
                   wellPanel(
                     tags$h4("Data & Output Formatting"),
                     radioButtons("cfa_dec_sep", "Decimal Separator:", 
                                  choices = c("Dot (.)" = ".", "Comma (,)" = ","),
                                  selected = ".", inline = TRUE)
                   )
                 )
               )
             ),
             # --- Report Preview ---
               tabPanel(
                 title = tagList(icon("file-alt"), " Report Preview"),
                 br(),
                 div(style = "display: flex; gap: 10px; margin-bottom: 15px;",
                     actionButton("cfa_generate_preview", tagList(icon("sync"), " Generate Report Preview"), class = "btn btn-success"),
                     downloadButton("download_report_html", "Download HTML Report", class = "btn btn-primary")
                 ),
                 div(
                   style = "border: 1px solid #ddd; border-radius: 4px; padding: 5px; background: #f9f9f9;",
                   uiOutput("cfa_report_preview_frame")
                 )
               ),

             # --- About ----
            tabPanel(
              title = tagList(icon("info-circle"), "About"),
              fluidRow(
                column(
                  width = 8, offset = 2,
                  br(),
                  div(
                    style = "text-align:center;",
                    tags$hr(),
                    tags$h5("projectLSA Was Developed By:"),
                    tags$p(
                      tags$a(
                        href = "https://scholar.google.com/citations?user=PSAwkTYAAAAJ&hl=id",
                        target = "_blank",
                        "Dr. Hasan Djidu, M.Pd."),
                      tags$br(),
                      "Universitas Sembilanbelas November Kolaka"
                    ),
                    tags$h5("Supervised By:"),
                    tags$p(
                      tags$a(
                        href = "https://scholar.google.com/citations?user=7CzPTYIAAAAJ&hl=id",
                        target = "_blank",
                        "Prof. Dr. Heri Retnawati, M.Pd."),
                      tags$br(),
                      "Universitas Negeri Yogyakarta"
                    ),
                    tags$p(tags$a(
                      href = "https://scholar.google.com/citations?hl=id&user=VGKeBm0AAAAJ",
                      target = "_blank",
                      "Prof. Dr. Samsul Hadi"),
                      tags$br(),
                      "Universitas Negeri Yogyakarta"
                    ),
                    tags$p(tags$a(
                      href = "https://scholar.google.com/citations?hl=id&user=k4MA8XgAAAAJ",
                      target = "_blank",
                      "Dr. Drs. Ir. Haryanto, M.Pd., M.T."),
                      tags$br(),
                      "Universitas Negeri Yogyakarta"
                    ),
                    tags$b("Contact:"),
                    tags$br(),
                    tags$a("hasandjidu@gmail.com"),
                    tags$hr()
                  )
                ),
                column(
                  width = 8, offset = 2,
                  h4("References (R Packages)"),
                   uiOutput("package_references_cfa"),
                   br(),
                   div(
                     style = "text-align:center;",
                     tags$p(
                       style = "font-size:13px; color:#777;",
                       format(Sys.Date(), "%Y"), 
                       "projectLSA. All rights reserved."
                     )
                   )
                 )
               )
             )
             
           )
    )
  )  
}

# ==== HELPER FUNCTIONS (GLOBAL IN SERVER) ====
# CR AVE & HTMT ====
calc_ave_cr <- function(fit) {
  std <- lavaan::standardizedSolution(fit)

  loading <- std[std$op == "=~", c("lhs", "rhs", "est.std")]
  loading$err <- 1 - loading$est.std^2

  result <- lapply(split(loading, loading$lhs), function(x) {
    lambda2 <- x$est.std^2
    theta <- x$err

    AVE <- sum(lambda2) / (sum(lambda2) + sum(theta))
    CR <- (sum(x$est.std))^2 /
      ((sum(x$est.std))^2 + sum(theta))

    c(AVE = AVE, CR = CR)
  })

  as.data.frame(do.call(rbind, result))
}
calc_htmt <- function(data, model_syntax,
                      missing = "listwise",
                      ordered = NULL,
                      absolute = TRUE) {
  ## -------------------------------------------------
  ## 1. Parse lavaan model syntax
  ## -------------------------------------------------
  lines <- unlist(strsplit(model_syntax, "\n"))
  fac_lines <- lines[grepl("=~", lines)]

  blocks <- lapply(fac_lines, function(x) {
    trimws(unlist(strsplit(strsplit(x, "=~")[[1]][2], "\\+")))
  })
  names(blocks) <- trimws(sub("=~.*", "", fac_lines))

  # Merge items for factors defined across multiple lines (duplicate factor names)
  unique_factors <- unique(names(blocks))
  merged_blocks <- lapply(unique_factors, function(f) {
    unique(unlist(blocks[names(blocks) == f]))
  })
  names(merged_blocks) <- unique_factors
  blocks <- merged_blocks

  ## -------------------------------------------------
  ## 2. Lavcor
  ## -------------------------------------------------
  R <- lavaan::lavCor(
    data,
    missing = missing,
    ordered = ordered,
    output  = "cor"
  )

  if (absolute) R <- abs(R)

  ## -------------------------------------------------
  ## 3. Indicator check
  ## -------------------------------------------------
  blocks <- lapply(blocks, function(x) intersect(x, colnames(R)))

  ## -------------------------------------------------
  ## 4. calculate HTMT2 (GEOMETRIC MEAN)
  ## -------------------------------------------------
  k <- length(blocks)
  htmt <- matrix(NA, k, k, dimnames = list(names(blocks), names(blocks)))

  for (i in seq_len(k)) {
    for (j in seq_len(k)) {
      if (i == j) {
        htmt[i, j] <- 1
        next
      }

      Xi <- blocks[[i]]
      Xj <- blocks[[j]]

      # heterotrait correlations
      r_het <- R[Xi, Xj]

      # monotrait correlations
      r_mono_i <- R[Xi, Xi][lower.tri(R[Xi, Xi])]
      r_mono_j <- R[Xj, Xj][lower.tri(R[Xj, Xj])]

      htmt[i, j] <-
        exp(mean(log(r_het))) /
          sqrt(
            exp(mean(log(r_mono_i))) *
              exp(mean(log(r_mono_j)))
          )
    }
  }

  htmt
}
# ====END of CR AVE and HTMT ====
server_cfa <- function(input, output, session, ai_context, console_context) {
  library(lavaan)
  library(semPlot)
  library(readxl)
  library(psych)
  library(tibble)
  library(semptools)
  # library(semTools)
  library(data.table)
  library(kableExtra)

  raw_data_user <- reactive({
    req(input$data_source)
    if (input$data_source == "bfi") {
      df <- psych::bfi %>%
        dplyr::select(A1:O5) %>%
        tibble::rownames_to_column("id_auto")
    }
    if (input$data_source == "HolzingerSwineford1939") {
      df <- lavaan::HolzingerSwineford1939 %>%
        dplyr::select(x1:x9, school) %>%
        tibble::rownames_to_column("id_auto")
    }
    if (input$data_source == "PoliticalDemocracy") {
      df <- lavaan::PoliticalDemocracy %>%
        dplyr::select(y1:y8, x1:x3) %>%
        tibble::rownames_to_column("id_auto")
    }
    if (input$data_source == "Demo.growth") {
      df <- lavaan::Demo.growth %>%
        dplyr::select(t1, t2, t3, t4, x1, x2, c1, c2) %>%
        tibble::rownames_to_column("id_auto")
    }

    if (input$data_source == "upload") {
      req(input$datafile)
      ext <- tolower(tools::file_ext(input$datafile$name))
      showModal(modalDialog(title = NULL, "Reading Your File, Please wait...", footer = NULL, easyClose = FALSE))

      if (ext == "rds") {
        res <- readRDS(input$datafile$datapath)
        if (is.list(res) && !is.data.frame(res) && identical(res$type, "projectLSA_workspace")) {
          if (identical(res$module, "CFA")) {
            # Restore Workspace State
            cfa_aggregations(res$cfa_aggregations)
            cfa_fit_list(res$cfa_fit_list)
            cfa_current_id(res$cfa_current_id)
            df <- res$raw_data
            
            # Restore UI Inputs
            if (!is.null(res$input_state)) {
              updateSelectInput(session, "cfa_estimator", selected = res$input_state$cfa_estimator)
              updateSelectInput(session, "cfa_missing", selected = res$input_state$cfa_missing)
              updateCheckboxInput(session, "cfa_std_est", value = res$input_state$cfa_std_est)
              updateCheckboxInput(session, "htmt_opt", value = res$input_state$htmt_opt)
              updateSelectInput(session, "cfa_analysis_mode", selected = res$input_state$cfa_analysis_mode)
              updateSelectInput(session, "mgcfa_invariance", selected = res$input_state$mgcfa_invariance)
              updateSelectInput(session, "mgcfa_group", selected = res$input_state$mgcfa_group)
              updateTextAreaInput(session, "cfa_model_text", value = res$input_state$cfa_model_text)
              updateSelectInput(session, "cfa_summary_theme", selected = res$input_state$cfa_summary_theme)
              updateSelectInput(session, "plot_model_scope", selected = res$input_state$plot_model_scope)
              updateSelectInput(session, "plot_style", selected = res$input_state$plot_style)
              updateSelectInput(session, "plot_layout", selected = res$input_state$plot_layout)
              updateSelectInput(session, "plot_color_scheme", selected = res$input_state$plot_color_scheme)
              updateCheckboxInput(session, "plot_standardized", value = res$input_state$plot_standardized)
              shinyWidgets::updatePickerInput(session, "fit_indices_selected", selected = res$input_state$fit_indices_selected)
              if (!is.null(res$input_state$cfa_agg_syntax)) updateTextAreaInput(session, "cfa_agg_syntax", value = res$input_state$cfa_agg_syntax)
              if (!is.null(res$input_state$cfa_agg_method)) updateSelectInput(session, "cfa_agg_method", selected = res$input_state$cfa_agg_method)
            }
            
            showNotification("CFA Workspace restored successfully!", type = "message")
          } else {
            stop("Uploaded workspace belongs to a different module: ", res$module)
          }
        } else {
          df <- res
        }
      } else {
        df <- switch(ext,
          "csv"  = data.table::fread(input$datafile$datapath, data.table = FALSE),
          "xls"  = readxl::read_excel(input$datafile$datapath),
          "xlsx" = readxl::read_excel(input$datafile$datapath),
          "sav"  = haven::read_sav(input$datafile$datapath),
          stop("Unsupported file type. Please upload CSV, Excel, SPSS (.sav), or RDS file.")
        )
      }
      removeModal()

      # Process df (only if it doesn't already have id_auto, e.g. newly uploaded raw data)
      if (!"id_auto" %in% names(df)) {
        df <- df %>% mutate(across(everything(), ~ ifelse(.x == "", NA, .x)),
          id_auto = paste0("id_", sprintf("%04d", 1:n()))
        )
      }
    }
    return(df)
  })

  # Reactive value to store user-defined aggregations
  cfa_aggregations <- reactiveVal(list())

  # New data_user reactive that applies aggregations on top of raw_data_user
  data_user <- reactive({
    df <- raw_data_user()
    req(df)

    aggs <- cfa_aggregations()
    if (length(aggs) > 0) {
      for (agg in aggs) {
        vars <- agg$vars
        name <- agg$name
        method <- agg$method

        valid_vars <- intersect(vars, names(df))
        if (length(valid_vars) > 0) {
          if (method == "mean") {
            df[[name]] <- rowMeans(df[, valid_vars, drop = FALSE], na.rm = TRUE)
          } else if (method == "sum") {
            df[[name]] <- rowSums(df[, valid_vars, drop = FALSE], na.rm = TRUE)
          }
        }
      }
    }
    return(df)
  })

  # Handle aggregation button click
  observeEvent(input$cfa_btn_aggregate, {
    syntax <- input$cfa_agg_syntax
    req(syntax)
    lines <- strsplit(syntax, "\n")[[1]]
    
    current_aggs <- cfa_aggregations()
    count <- 0
    for (line in lines) {
      if (grepl("=", line)) {
        parts <- strsplit(line, "=")[[1]]
        if (length(parts) == 2) {
          name <- make.names(trimws(parts[1]))
          vars_raw <- strsplit(parts[2], "[, ]+")[[1]]
          vars <- trimws(vars_raw[vars_raw != ""])
          
          if (length(vars) > 0 && name != "") {
            new_agg <- list(
              vars = vars,
              name = name,
              method = input$cfa_agg_method
            )
            current_aggs[[name]] <- new_agg
            count <- count + 1
          }
        }
      }
    }
    
    cfa_aggregations(current_aggs)
    if (count > 0) {
      showNotification(paste(count, "variables calculated using", input$cfa_agg_method, "method."), type = "message")
      shinyWidgets::updatePickerInput(session, "cfa_agg_vars_insert", selected = character(0))
    }
  })

  # Handle insert button
  observeEvent(input$cfa_btn_insert_agg, {
    vars <- input$cfa_agg_vars_insert
    if (length(vars) > 0) {
      current <- input$cfa_agg_syntax
      prefix <- ifelse(!is.null(current) && nchar(current) > 0 && !endsWith(current, "\n"), "\n", "")
      new_text <- paste0(current, prefix, "NewVar = ", paste(vars, collapse = ", "))
      updateTextAreaInput(session, "cfa_agg_syntax", value = new_text)
    }
  })

  # Handle reset aggregation button
  observeEvent(input$cfa_btn_reset_agg, {
    cfa_aggregations(list())
    showNotification("All aggregations have been reset.", type = "warning")
  })

  # Populate aggregation UI choices
  observe({
    req(raw_data_user())
    # We populate with raw data columns + existing aggregated columns
    df_names <- names(data_user())
    shinyWidgets::updatePickerInput(session, "cfa_agg_vars_insert", choices = df_names)
    updateSelectizeInput(session, "cfa_summary_vars", choices = df_names)
  })

  output$id_select_ui <- renderUI({
    req(data_user())
    shinyWidgets::pickerInput("id_cols", "ID columns (optional)",
      choices = names(data_user()), multiple = TRUE,
      selected = names(data_user())[grepl("id", names(data_user()), ignore.case = TRUE)],
      options = list(`actions-box` = TRUE, `live-search` = TRUE)
    )
  })

  output$var_select_ui <- renderUI({
    req(data_user())
    all_vars <- names(data_user())
    id_cols <- input$id_cols
    choices <- setdiff(all_vars, id_cols)
    shinyWidgets::pickerInput("selected_vars", "Select Variables (used for CFA):", 
                              choices = choices, multiple = TRUE, selected = choices,
                              options = list(`actions-box` = TRUE, `live-search` = TRUE, `selected-text-format` = "count > 3"))
  })

  output$mgcfa_group_ui <- renderUI({
    req(data_user())
    all_vars <- names(data_user())
    # Try to find categorical columns
    cat_vars <- all_vars[sapply(data_user(), function(x) is.factor(x) || is.character(x) || length(unique(x)) <= 5)]
    selectInput("mgcfa_group_var", "Grouping Variable:", choices = cat_vars)
  })

  output$data_preview <- renderDT(
    {
      req(data_user(), input$selected_vars)
      df <- data_user()[, input$selected_vars, drop = FALSE]
      # Identifikasi kolom numeric
      numeric_cols <- which(sapply(df, is.numeric))
      # df <- df %>% mutate(across(where(is.numeric), ~ round(.x, 3)))
      datatable(df,
        extensions = "Buttons",
        options = list(
          scrollX = TRUE, dom = "Brtp", pageLength = 25,
          buttons = list(
            list(
              extend = "csv",
              text = "Export CSV",
              filename = paste0("Data CFA")
            ),
            list(
              extend = "excel",
              text = "Export Excel",
              filename = paste0("Data CFA")
            )
          )
        ), rownames = T
      ) %>%
        formatRound(columns = numeric_cols, digits = 2)
    },
    server = FALSE
  )


  # ===== CFA MODEL =====
  output$cfa_model_ui <- renderUI({
    model_text <- switch(input$data_source,
      "bfi" = "A =~ A1 + A2 + A3 + A4 + A5\nC =~ C1 + C2 + C3 + C4 + C5\nE =~ E1 + E2 + E3 + E4 + E5\nN =~ N1 + N2 + N3 + N4 + N5\nO =~ O1 + O2 + O3 + O4 + O5",
      "HolzingerSwineford1939" = "Visual =~ x1 + x2 + x3\nTextual =~ x4 + x5 + x6\nSpeed =~ x7 + x8 + x9\nG =~ Visual+Textual+Speed",
      "PoliticalDemocracy" = "# Measurement model\nind60 =~ x1 + x2 + x3\ndem60 =~ y1 + y2 + y3 + y4\ndem65 =~ y5 + y6 + y7 + y8\n\n# Structural model\ndem60 ~ ind60\ndem65 ~ ind60 + dem60\n\n# Residual correlations\ny1 ~~ y5\ny2 ~~ y4 + y6\ny3 ~~ y7\ny4 ~~ y8\ny6 ~~ y8",
      "Demo.growth" = "# Intercept and slope with fixed coefficients\ni =~ 1*t1 + 1*t2 + 1*t3 + 1*t4\ns =~ 0*t1 + 1*t2 + 2*t3 + 3*t4\n\n# Regressions (optional predictors)\n# i ~ x1 + x2\n# s ~ x1 + x2",
      "# Enter your lavaan model syntax here\n# Example:\n# factor1 =~ item1 + item2 + item3\n# factor2 =~ item4 + item5 + item6"
    )

    tags$textarea(id = "cfa_model_text", rows = 8, style = "width:100%; font-family: monospace; font-size:11.5px", model_text)
  })

  # ---- CFA storage ----
  cfa_fit_list <- reactiveVal(list())
  cfa_current_id <- reactiveVal(NULL)
  cfa_model_counter <- reactiveVal(0)
  run_lavaan <- function(model_text, data, estimator = "ML", missing = "listwise",
                         mode = "standard", group = NULL, group_equal = NULL) {
    tryCatch(
      {
        # Enforce group variable inclusion if MGCFA
        if (mode == "mgcfa" && !is.null(group) && group != "") {
          data_to_use <- data %>% dplyr::select(where(is.numeric) | all_of(group))
        } else {
          data_to_use <- data %>% dplyr::select(where(is.numeric))
        }

        # Determine function
        lavaan_fun <- if (mode == "lgm") lavaan::growth else lavaan::sem

        # Additional args
        args <- list(
          model = model_text,
          data = data_to_use,
          estimator = estimator,
          ov.order = "model",
          missing = if (missing == "fiml") "fiml" else if (missing == "pairwise") "pairwise" else "listwise"
        )

        if (mode == "mgcfa" && !is.null(group) && group != "") {
          args$group <- group
          if (!is.null(group_equal) && group_equal != "configural") {
            # group_equal comes as a string like "loadings" or "loadings, intercepts"
            args$group.equal <- strsplit(group_equal, ", ")[[1]]
          }
        }

        do.call(lavaan_fun, args)
      },
      error = function(e) {
        showNotification(paste("Model error:", e$message), type = "error")
        return(NULL)
      }
    )
  }
  observeEvent(input$run_cfa, {
    req(data_user(), input$cfa_model_text, input$selected_vars)
    updateTabsetPanel(session, "main_tab_cfa", selected = "fit_tab_cfa")
  })
  # Main run CFA
  observeEvent(input$run_cfa, {
    req(data_user(), input$cfa_model_text, input$selected_vars)

    if (nchar(trimws(input$cfa_model_text)) == 0) {
      showNotification("Please enter a CFA model", type = "error")
      return()
    }

    df_subset <- data_user()[, input$selected_vars, drop = FALSE]

    # If MGCFA, we need the group column too
    if (input$cfa_analysis_mode == "mgcfa" && !is.null(input$mgcfa_group_var)) {
      df <- cbind(df_subset, data_user()[, input$mgcfa_group_var, drop = FALSE])
    } else {
      df <- df_subset
    }

    # Only items should be forced to numeric
    # df_subset <- df_subset %>% mutate(across(everything(), as.numeric))


    showModal(modalDialog("Running CFA ...", footer = NULL))

    fit <- run_lavaan(
      model_text = input$cfa_model_text,
      data = df,
      estimator = input$cfa_estimator,
      missing = input$cfa_missing,
      mode = input$cfa_analysis_mode,
      group = if (input$cfa_analysis_mode == "mgcfa") input$mgcfa_group_var else NULL,
      group_equal = if (input$cfa_analysis_mode == "mgcfa") input$mgcfa_invariance else NULL
    )

    removeModal()

    if (is.null(fit)) {
      return()
    }

    measures <- tryCatch(
      {
        fm <- lavaan::fitMeasures(fit)
        
        use_robust <- is.null(input$cfa_use_robust_fit) || input$cfa_use_robust_fit
        has_scaled <- "chisq.scaled" %in% names(fm)
        
        get_val <- function(std, scaled, robust) {
          if (use_robust) {
            if (!missing(robust) && robust %in% names(fm) && !is.na(fm[robust])) {
              return(unname(fm[robust]))
            }
            if (!missing(scaled) && scaled %in% names(fm) && !is.na(fm[scaled])) {
              return(unname(fm[scaled]))
            }
          }
          if (std %in% names(fm)) {
            return(unname(fm[std]))
          }
          return(NA)
        }

        c(
          chisq = get_val("chisq", "chisq.scaled"),
          df = get_val("df", "df.scaled"),
          pvalue = get_val("pvalue", "pvalue.scaled"),
          rmsea = get_val("rmsea", "rmsea.scaled", "rmsea.robust"),
          cfi = get_val("cfi", "cfi.scaled", "cfi.robust"),
          tli = get_val("tli", "tli.scaled", "tli.robust"),
          srmr = get_val("srmr", "srmr_nomean"),
          gfi = get_val("gfi"),
          agfi = get_val("agfi"),
          nfi = get_val("nfi", "nfi.scaled"),
          nnfi = get_val("nnfi", "nnfi.scaled", "nnfi.robust"),
          is_robust = as.numeric(use_robust && has_scaled)
        )
      },
      error = function(e) {
        c(chisq = NA, df = NA, pvalue = NA, rmsea = NA, cfi = NA, tli = NA, srmr = NA, gfi = NA, agfi = NA, nfi = NA, nnfi = NA, is_robust = 0)
      }
    )


    # AVE <- tryCatch({
    #   semTools::AVE(object = fit)
    #   }, error = function(e) {
    #   '<NA>'
    # })
    # RelSEM <- tryCatch({
    #   semTools::compRelSEM(object = fit)
    # }, error = function(e) {
    #   '<NA>'
    # })
    # HTMT <- tryCatch({
    #   semTools::htmt(model = input$cfa_model_text, data = df)
    # }, error = function(e) {
    #   '<NA>'
    # })

    ave_cr <- tryCatch(
      {
        calc_ave_cr(fit)
      },
      error = function(e) {
        NA
      }
    )
    AVE <- setNames(ave_cr$AVE, rownames(ave_cr))
    RelSEM <- setNames(ave_cr$CR, rownames(ave_cr))
    HTMT <- tryCatch(
      {
        calc_htmt(df, input$cfa_model_text)
      },
      error = function(e) {
        NA
      }
    )

    scoreCfa <- tryCatch(
      {
        fs <- as.data.frame(lavaan::predict(fit))
        for (col in names(fs)) {
          min_val <- min(fs[[col]], na.rm = TRUE)
          max_val <- max(fs[[col]], na.rm = TRUE)
          if (max_val > min_val) {
            fs[[paste0(col, "_0_100")]] <- (fs[[col]] - min_val) / (max_val - min_val) * 100
          } else {
            fs[[paste0(col, "_0_100")]] <- 100
          }
        }
        fs
      },
      error = function(e) {
        "<NA>"
      }
    )

    current_count <- cfa_model_counter() + 1
    cfa_model_counter(current_count)
    new_id <- paste0("Model_", current_count)

    cur <- cfa_fit_list()

    # Auto-generate modification note
    auto_note <- ""
    if (length(cur) == 0) {
      auto_note <- "Initial Model"
    } else {
      last_model <- cur[[length(cur)]]
      old_syntax <- last_model$spec
      new_syntax <- input$cfa_model_text

      old_lines <- trimws(unlist(strsplit(old_syntax, "\n")))
      new_lines <- trimws(unlist(strsplit(new_syntax, "\n")))
      old_lines <- old_lines[old_lines != ""]
      new_lines <- new_lines[new_lines != ""]

      added <- setdiff(new_lines, old_lines)
      removed <- setdiff(old_lines, new_lines)

      changes <- c()
      if (length(added) > 0) changes <- c(changes, paste("Added:", paste(added, collapse = ", ")))
      if (length(removed) > 0) changes <- c(changes, paste("Removed:", paste(removed, collapse = ", ")))

      if (length(changes) == 0) {
        auto_note <- "Modified formatting only"
      } else {
        auto_note <- paste(changes, collapse = " | ")
      }
    }

    # Check for Heywood Cases (Negative Variances)
    has_heywood <- FALSE
    try(
      {
        pe <- lavaan::parameterEstimates(fit)
        if (any(pe$op == "~~" & pe$lhs == pe$rhs & pe$est < 0)) {
          has_heywood <- TRUE
          auto_note <- paste0(auto_note, " | \u26A0 Heywood")
        }
      },
      silent = TRUE
    )

    cur[[new_id]] <- list(
      id = new_id,
      spec = input$cfa_model_text,
      fit = fit,
      measures = measures,
      ave = AVE,
      relsem = RelSEM,
      htmt = HTMT,
      scoreCfa = as.data.frame(scoreCfa),
      note = auto_note,
      estimator = input$cfa_estimator,
      has_heywood = has_heywood,
      time = Sys.time()
    )

    cfa_fit_list(cur)
    cfa_current_id(new_id)
    showNotification(paste0("CFA finished: ", new_id), type = "message")
  })

  # ====== Fit Comparison Table =======
  output$fit_comparison <- renderUI({
    req(cfa_fit_list())
    req(cfa_fit_list())
    lst <- cfa_fit_list()
    if (length(lst) == 0) {
      return(tags$p("No models run yet. Run CFA to see results."))
    }

    # Buat tabel HTML manual
    column(
      12,
      tags$div(
        style = "margin-top: 0px; font-size: 13px;",
        tags$b("Fit Model Comparison")
      ),
      # ---Render table DT =====
      DTOutput("fit_comparison_dt"),
      tags$div(
        style = "margin-top: 0px; font-size: 12px; color: #6c757d;",
        tags$b("Interpretation Guidelines:"),
        tags$ul(
          tags$li(tags$span(style = "color: green;", "Excellent:"), "χ²/df < 2 AND RMSEA ≤ 0.05 AND CFI ≥ 0.95 AND TLI ≥ 0.95 AND SRMR ≤ 0.05 (Hu & Bentler, 1999)"),
          tags$li(tags$span(style = "color: green;", "Good:"), "χ²/df < 3 AND RMSEA < 0.08 AND CFI ≥ 0.90 AND TLI ≥ 0.90 AND SRMR ≤ 0.08 (Schumacker & Lomax, 2016; Kline, 2016)"),
          tags$li(tags$span(style = "color: orange;", "Acceptable:"), "χ²/df < 5 AND (RMSEA < 0.10 OR CFI ≥ 0.85 OR SRMR ≤ 0.10) OR ≥ 3 indices meet 'Good' (Hair et al., 2019; Brown, 2015)"),
          tags$li(tags$span(style = "color: red;", "Poor:"), "Fails to meet Acceptable criteria"),
          tags$li(tags$span(style = "color: red;", "⚠ Heywood:"), "Negative variance detected (Chen et al., 2001)")
        )
      ),
      tags$hr(),
      column(
        6,
        tags$div(
          style = "margin-top: 0px; font-size: 13px;",
          tags$b("Average Variance Explained (AVE):"),
          DTOutput("ave_table") # Table AVE
        )
      ),
      column(
        6,
        tags$div(
          style = "margin-top: 1px; font-size: 13px;",
          tags$b("Composite Reliability (CR):"),
          DTOutput("reliability_table"), # Table Reliability
        )
      ),
      tags$div(
        style = "margin-top: 0px; font-size: 12px; color: #6c757d;",
        tags$b("Interpretation Guidelines:"),
        tags$ul(
          tags$li(tags$span(style = "color: green;", "Good:"), "AVE ≥ 0.5; CR ≥ 0.7 (Hair et al., 2010; Fornell & Larcker, 1981)"),
          tags$li(tags$span(style = "color: orange;", "Acceptable:"), "AVE ≥ 0.4; CR ≥ 0.6 (Fornell & Larcker, 1981)"),
          tags$li(tags$span(style = "color: red;", "Poor:"), "Outside acceptable ranges")
        )
      ),
      conditionalPanel(
        condition = "input.htmt_opt == true",
        tags$hr(),
        tags$div(
          style = "margin-top: 0px; font-size: 13px;",
          tags$b("Heterotrait–Monotrait Ratio (HTMT) for Current Model")
        ),
        DTOutput("htmt_table"),
        tags$div(
          style = "margin-top: 0px; font-size: 12px; color: #6c757d;",
          tags$b("Interpretation Guidelines:"),
          tags$ul(
            tags$li(tags$span(style = "color: green;", "Good discriminant validity:"), "HTMT < 0.7 "),
            tags$li(tags$span(style = "color: orange;", "Acceptable discriminant validity:"), "HTMT < 0.8"),
            tags$li(tags$span(style = "color: red;", "Potential discriminant validity issues:"), "HTMT ≥ 0.8")
          )
        )
      ),
      tags$hr(),
      column(
        12,
        tags$div(
          style = "margin-top: 0px; font-size: 13px;",
          tags$b("Modification Indices")
        ),
        tags$div(
          class = "alert alert-warning", style = "padding: 10px; font-size: 12px; margin-top: 10px;",
          tags$b(icon("exclamation-triangle"), " Warning: "),
          "Data-driven specification searches using Modification Indices should not be treated as prescriptive. Over-relying on MI without substantive theoretical justification can lead to overfitted models that do not generalize well. Only apply modifications that make theoretical sense (e.g., MacCallum et al., 1992)."
        ),
        DTOutput("mi_table")
      )
    )
  })

  output$fit_comparison_dt <- renderDT({
    req(cfa_fit_list())
    lst <- cfa_fit_list()
    if (length(lst) == 0) {
      return(NULL)
    }

    df <- do.call(rbind, lapply(lst, function(x) {
      m <- x$measures
      hw <- ifelse(!is.null(x$has_heywood) && x$has_heywood, TRUE, FALSE)
      status_info <- get_fit_status(
        m["chisq"], m["df"], m["pvalue"],
        m["rmsea"], m["cfi"], m["srmr"], m["tli"], hw
      )
      data.frame(
        Model = x$id,
        Estimator = ifelse(is.null(x$estimator), "Unknown", x$estimator),
        Note = ifelse(is.null(x$note) || x$note == "", "-", x$note),
        chisq = round(m["chisq"], 2),
        df = m["df"],
        chisq_df_ratio = ifelse(is.na(m["chisq"]), NA, sprintf("%.2f", m["chisq"] / m["df"])),
        pvalue = ifelse(is.na(m["pvalue"]), NA, sprintf("%.3f", m["pvalue"])),
        RMSEA = ifelse(is.na(m["rmsea"]), NA, sprintf("%.3f", m["rmsea"])),
        CFI = ifelse(is.na(m["cfi"]), NA, sprintf("%.3f", m["cfi"])),
        TLI = ifelse(is.na(m["tli"]), NA, sprintf("%.3f", m["tli"])),
        GFI = ifelse(is.na(m["gfi"]), NA, sprintf("%.3f", m["gfi"])),
        SRMR = ifelse(is.na(m["srmr"]), NA, sprintf("%.3f", m["srmr"])),
        NFI = ifelse(is.na(m["nfi"]), NA, sprintf("%.3f", m["nfi"])),
        Heywood = ifelse(hw, "Yes", "No"),
        Status = status_info$status,
        StatusColor = status_info$color,
        stringsAsFactors = FALSE
      )
    }))

    # Add Action column
    df$Action <- sapply(seq_len(nrow(df)), function(i) {
      as.character(
        actionButton(
          inputId = paste0("del_model_btn_", i),
          label = icon("trash"),
          class = "btn-danger btn-sm",
          style = "font-size: 11px !important; padding: 2px 10px !important;",
          onclick = paste0('Shiny.setInputValue("del_model_selected", "', df$Model[i], '", {priority: "event"})')
        )
      )
    })

    # We want Note column to be editable. Columns are 0-indexed in DT when rownames=FALSE.
    # Note is the 3rd column -> index 2.
    # We disable all except index 2.
    # Columns in df: Model(0), Estimator(1), Note(2), chisq(3), df(4), chisq_df_ratio(5), pvalue(6), RMSEA(7), CFI(8), TLI(9), GFI(10), SRMR(11), NFI(12), Heywood(13), Status(14), Action(15) (we will exclude StatusColor)

    # Drop StatusColor for display
    display_df <- df[, c("Model", "Estimator", "Note", "chisq", "df", "chisq_df_ratio", "pvalue", "RMSEA", "CFI", "TLI", "GFI", "SRMR", "NFI", "Heywood", "Status", "Action")]

    datatable(display_df,
      rownames = FALSE,
      escape = FALSE,
      selection = "none",
      editable = list(target = "cell", disable = list(columns = c(0, 1, 3:15))),
      extensions = "Buttons",
      options = list(
        dom = "Brt",
        scrollX = TRUE,
        pageLength = 20,
        buttons = list(
          list(
            extend = "excel",
            text = "Export Excel",
            filename = paste0("Fit Model Comparison")
          )
        ),
        columnDefs = list(list(className = "dt-center", targets = 3:12))
      )
    ) %>%
      formatStyle("chisq_df_ratio", color = styleInterval(c(2, 5), c("green", "orange", "red")), fontWeight = "bold", textAlign = "center") %>%
      formatStyle("pvalue", color = styleInterval(c(0.05, 0.1), c("red", "green", "green")), fontWeight = "bold", textAlign = "center") %>%
      formatStyle("RMSEA", color = styleInterval(c(0.08, 0.1), c("green", "orange", "red")), fontWeight = "bold", textAlign = "center") %>%
      formatStyle("CFI", color = styleInterval(c(0.88, 0.9), c("red", "orange", "green")), fontWeight = "bold", textAlign = "center") %>%
      formatStyle("TLI", color = styleInterval(c(0.88, 0.9), c("red", "orange", "green")), fontWeight = "bold", textAlign = "center") %>%
      formatStyle("GFI", color = styleInterval(c(0.88, 0.9), c("red", "orange", "green")), fontWeight = "bold", textAlign = "center") %>%
      formatStyle("SRMR", color = styleInterval(c(0.05, 0.08), c("green", "orange", "red")), fontWeight = "bold", textAlign = "center") %>%
      formatStyle("NFI", color = styleInterval(c(0.88, 0.9), c("red", "orange", "green")), fontWeight = "bold", textAlign = "center") %>%
      formatStyle("Status", backgroundColor = styleEqual(df$Status, df$StatusColor), fontWeight = "bold", color = "black", textAlign = "center") %>%
      formatStyle("Heywood", color = styleEqual(c("Yes", "No"), c("red", "green")), fontWeight = "bold", textAlign = "center")
  })

  # Handle note edit in fit comparison table
  observeEvent(input$fit_comparison_dt_cell_edit, {
    info <- input$fit_comparison_dt_cell_edit
    str(info)
    # info$row is the row number (1-indexed in R)
    # info$col is the column number (0-indexed from JS)
    # info$value is the new value
    req(info)

    cur_list <- cfa_fit_list()
    # The models are ordered the same as in the table
    if (info$row <= length(cur_list)) {
      model_names <- names(cur_list)
      model_id <- model_names[info$row]

      # Update the note
      cur_list[[model_id]]$note <- as.character(info$value)

      # Save back to reactive val
      cfa_fit_list(cur_list)
    }
  })

  # Handle model deletion
  observeEvent(input$del_model_selected, {
    req(input$del_model_selected)

    cur_list <- cfa_fit_list()
    model_id <- input$del_model_selected

    if (model_id %in% names(cur_list)) {
      cur_list[[model_id]] <- NULL
      cfa_fit_list(cur_list)

      # Update current ID
      if (length(cur_list) > 0) {
        cfa_current_id(names(cur_list)[length(cur_list)])
      } else {
        cfa_current_id(NULL)
      }

      showNotification(paste("Deleted", model_id), type = "message")
    }
  })

  # === Output: HTMT Table =====
  output$htmt_table <- renderDT({
    lst <- cfa_fit_list()
    if (length(lst) == 0) {
      return(NULL)
    }

    # Ambil model terakhir
    last_model <- lst[[length(lst)]]

    if (!is.null(last_model$htmt) && !identical(last_model$htmt, "<NA>")) {
      # Konversi HTMT matrix ke data frame
      htmt_matrix <- as.matrix(last_model$htmt)
      htmt_df <- as.data.frame(htmt_matrix)

      # Tambahkan nama baris sebagai kolom pertama
      htmt_df <- cbind(Variable = rownames(htmt_df), htmt_df)
      rownames(htmt_df) <- NULL

      # Identifikasi kolom numeric (HTMT values)
      numeric_cols <- which(sapply(htmt_df, is.numeric))

      datatable(htmt_df,
        rownames = FALSE, extensions = "Buttons",
        options = list(
          dom = "Bt",
          pageLength = nrow(htmt_df),
          buttons = list(
            list(
              extend = "csv",
              text = "Export CSV",
              filename = paste0("HTMT")
            ),
            list(
              extend = "excel",
              text = "Export Excel",
              filename = paste0("HTMT")
            )
          )
        )
      ) %>%
        formatStyle(
          columns = numeric_cols,
          color = styleInterval(c(0.7, 0.8), c("green", "orange", "red"))
        ) %>%
        formatStyle(
          columns = "Variable",
          backgroundColor = "#f8f9fa",
          fontWeight = "bold"
        ) %>%
        formatRound(columns = numeric_cols, digits = 3)
    } else {
      retu(data.frame(Message = "HTMT not available for the last model"))
    }
  })

  # ===== TABEL AVE =====
  output$ave_table <- renderDT({
    lst <- cfa_fit_list()
    if (length(lst) == 0) {
      return(datatable(data.frame(Message = "No models available")))
    }

    # Pastikan dplyr tersedia
    if (!require(dplyr, quietly = TRUE)) {
      stop("Package dplyr is required")
    }

    ave_data <- lapply(lst, function(x) {
      if (!is.null(x$ave) && !identical(x$ave, "<NA>")) {
        # Konversi AVE ke data frame
        ave_df <- as.data.frame(as.list(round(x$ave, 3)))
        ave_df$Model <- x$id
        return(ave_df)
      } else {
        data.frame(Model = x$id, Note = "AVE not available")
      }
    })

    # Gabungkan dengan full_join berantai
    combined_data <- Reduce(function(x, y) {
      dplyr::full_join(x, y, by = intersect(names(x), names(y)))
    }, ave_data)

    # Pastikan Model ada di kolom pertama
    if ("Model" %in% names(combined_data)) {
      model_col <- combined_data$Model
      combined_data$Model <- NULL
      combined_data <- data.frame(Model = model_col, combined_data)
    }

    datatable(combined_data,
      extensions = "Buttons",
      rownames = FALSE,
      options = list(
        dom = "Bt",
        buttons = list(
          list(
            extend = "csv",
            text = "Export CSV",
            filename = paste0("AVE")
          ),
          list(
            extend = "excel",
            text = "Export Excel",
            filename = paste0("AVE")
          )
        )
      )
    ) %>%
      formatStyle(
        columns = which(!names(combined_data) %in% c("Model", "Note")),
        color = styleInterval(c(0.45, 0.5), c("red", "orange", "green"))
      ) %>%
      formatRound(columns = which(sapply(combined_data, is.numeric)), digits = 3)
  })

  # ===== Table  Reliability =====
  output$reliability_table <- renderDT({
    lst <- cfa_fit_list()
    if (length(lst) == 0) {
      return(datatable(data.frame(Message = "No models available")))
    }

    rel_data <- lapply(lst, function(x) {
      if (!is.null(x$relsem) && !identical(x$relsem, "<NA>")) {
        rel_df <- as.data.frame(as.list(round(x$relsem, 3)), stringsAsFactors = FALSE)
        rel_df$Model <- x$id
        return(rel_df)
      } else {
        data.frame(Model = x$id, Note = "Reliability not available", stringsAsFactors = FALSE)
      }
    })

    combined_data <- Reduce(function(x, y) {
      common_cols <- intersect(names(x), names(y))
      if (length(common_cols) > 0) {
        dplyr::full_join(x, y, by = common_cols)
      } else {
        # Jika tidak ada kolom common, gabungkan secara manual
        cbind(x, y[, setdiff(names(y), names(x)), drop = FALSE])
      }
    }, rel_data)

    # Pastikan Model ada di kolom pertama
    if ("Model" %in% names(combined_data)) {
      model_col <- combined_data$Model
      combined_data$Model <- NULL
      combined_data <- data.frame(Model = model_col, combined_data, stringsAsFactors = FALSE)
    }

    # Identifikasi kolom numeric (Reliability values)
    numeric_cols <- which(sapply(combined_data, is.numeric))

    datatable(combined_data,
      rownames = FALSE, extensions = "Buttons",
      options = list(
        dom = "Bt",
        buttons = list(
          list(
            extend = "csv",
            text = "Export CSV",
            filename = paste0("Composite Realiability")
          ),
          list(
            extend = "excel",
            text = "Export Excel",
            filename = paste0("Composite Reliability")
          )
        )
      )
    ) %>%
      formatStyle(
        columns = numeric_cols,
        color = styleInterval(c(0.6, 0.7), c("red", "orange", "green"))
      ) %>%
      formatStyle(
        columns = "Model",
        backgroundColor = "#f8f9fa"
      ) %>%
      formatRound(columns = numeric_cols, digits = 3)
  })

  # ===== Table  FSCORES =====
  output$fscores_cfa <- renderDT(
    {
      req(input$run_cfa)
      lst <- cfa_fit_list()
      cur_id <- cfa_current_id()
      if (is.null(cur_id)) {
        return(NULL)
      }
      scoreCfa <- cfa_fit_list()[[cur_id]]$scoreCfa
      numeric_cols <- which(sapply(scoreCfa, is.numeric))
      datatable(scoreCfa,
        rownames = TRUE, extensions = "Buttons",
        options = list(
          dom = "Brtp",
          buttons = list(
            list(
              extend = "csv",
              text = "Export CSV",
              filename = paste0("Factor Scores")
            ),
            list(
              extend = "excel",
              text = "Export Excel",
              filename = paste0("Factor Scores")
            )
          )
        )
      ) %>%
        formatRound(columns = numeric_cols, digits = 3)
    },
    server = FALSE
  )

  # ====== Data Preview Visualizations ======
  output$cfa_summary_table <- renderUI({
    req(data_user())
    vars_to_use <- input$selected_vars
    if (is.null(vars_to_use) || length(vars_to_use) == 0) vars_to_use <- names(data_user())
    valid_vars <- intersect(vars_to_use, names(data_user()))
    req(length(valid_vars) > 0)
    
    df <- data_user()[, valid_vars, drop = FALSE]
    
    summary_list <- lapply(names(df), function(v) {
      x <- df[[v]]
      x_na <- na.omit(x)
      n <- length(x_na)
      
      if (is.numeric(x)) {
        mean_val <- round(mean(x_na), 2)
        sd_val <- round(sd(x_na), 2)
        min_val <- round(min(x_na), 2)
        max_val <- round(max(x_na), 2)
        type <- "Scale"
        
        data.frame(
          Variable = v,
          Type = type,
          N = n,
          `Mean/Mode` = as.character(mean_val),
          `SD/Count` = as.character(sd_val),
          Min = as.character(min_val),
          Max = as.character(max_val),
          check.names = FALSE
        )
      } else {
        tbl <- table(x_na)
        if(length(tbl) > 0) {
          mode_val <- names(tbl)[which.max(tbl)]
          count_val <- max(tbl)
        } else {
          mode_val <- "-"
          count_val <- "-"
        }
        type <- if (is.ordered(x)) "Ordinal" else "Nominal"
        
        data.frame(
          Variable = v,
          Type = type,
          N = n,
          `Mean/Mode` = as.character(mode_val),
          `SD/Count` = as.character(count_val),
          Min = "-",
          Max = "-",
          check.names = FALSE
        )
      }
    })
    
    summary_df <- do.call(rbind, summary_list)
    
    html_out <- summary_df %>%
      kable(format = "html", escape = FALSE, caption = "<b>Table 1</b><br><i>Descriptive Statistics</i>", align = "llccccc") %>%
      kable_styling(bootstrap_options = "none", full_width = FALSE, position = "left") %>%
      row_spec(0, bold = FALSE, extra_css = "border-top: 1.5px solid black; border-bottom: 1.5px solid black; font-family: 'Times New Roman', Times, serif; font-size: 11pt;") %>%
      row_spec(nrow(summary_df), extra_css = "border-bottom: 1.5px solid black;") %>%
      column_spec(1:ncol(summary_df), extra_css = "padding: 4px 12px; font-size: 11pt; line-height: 1; font-family: 'Times New Roman', Times, serif;")
    
    HTML(as.character(html_out))
  })
  output$cfa_data_summary_plot <- renderPlot({
    req(data_user())
    vars_to_use <- input$cfa_summary_vars
    if (is.null(vars_to_use) || length(vars_to_use) == 0) vars_to_use <- input$selected_vars
    if (is.null(vars_to_use) || length(vars_to_use) == 0) vars_to_use <- names(data_user())

    valid_vars <- intersect(vars_to_use, names(data_user()))
    if (length(valid_vars) == 0) {
      return(NULL)
    }

    df <- data_user()[, valid_vars, drop = FALSE]

    types <- sapply(df, function(x) class(x)[1])
    type_df <- as.data.frame(table(types))
    names(type_df) <- c("DataType", "Count")
    
    theme_choice <- input$cfa_summary_theme
    if (is.null(theme_choice)) theme_choice <- "default"
    
    bar_colors <- switch(theme_choice,
      "pastel" = scale_fill_brewer(palette = "Pastel1"),
      "blue"   = scale_fill_brewer(palette = "Blues"),
      "dark"   = scale_fill_brewer(palette = "Dark2"),
      scale_fill_brewer(palette = "Set2")
    )

    ggplot(type_df, aes(x = DataType, y = Count, fill = DataType)) +
      geom_col(width = 0.5, show.legend = FALSE, alpha = 0.9, color = "transparent") +
      geom_text(aes(label = Count), vjust = -0.5, fontface = "bold", color = "#495057") +
      bar_colors +
      theme_minimal(base_size = 14) +
      theme(
        plot.title = element_text(hjust = 0.5, face = "bold", size = 14, color = "#343a40"),
        axis.title.x = element_text(face = "bold", color = "#495057", margin = margin(t = 10)),
        axis.title.y = element_text(face = "bold", color = "#495057", margin = margin(r = 10)),
        axis.text.x = element_text(angle = 45, hjust = 1, vjust = 1), # Prevent overlap
        panel.grid.major.x = element_blank(),
        plot.margin = margin(15, 15, 15, 15),
        panel.background = element_rect(fill = "transparent", color = NA),
        plot.background = element_rect(fill = "transparent", color = NA)
      ) +
      labs(title = "Data Type Composition", x = "Data Type", y = "Number of Variables")
  }, bg="transparent")

  output$cfa_data_heatmap <- renderPlot({
    req(data_user())
    vars_to_use <- input$selected_vars
    if (is.null(vars_to_use) || length(vars_to_use) == 0) vars_to_use <- names(data_user())

    valid_vars <- intersect(vars_to_use, names(data_user()))
    if (length(valid_vars) == 0) {
      return(NULL)
    }

    df <- data_user()[, valid_vars, drop = FALSE]

    num_df <- df[, sapply(df, is.numeric), drop = FALSE]
    if (ncol(num_df) < 2) {
      plot.new()
      title(main = "Not enough numeric variables for correlation heatmap", col.main = "#6c757d")
      return(NULL)
    }

    cor_matrix <- cor(num_df, use = "pairwise.complete.obs")
    # Set upper triangle to NA to display only lower triangle for cleaner look
    cor_matrix[upper.tri(cor_matrix)] <- NA

    cor_melted <- as.data.frame(as.table(cor_matrix))
    cor_melted <- cor_melted[!is.na(cor_melted$Freq), ]
    names(cor_melted) <- c("Var1", "Var2", "value")

    theme_choice <- input$cfa_summary_theme
    if (is.null(theme_choice)) theme_choice <- "default"
    
    heatmap_colors <- switch(theme_choice,
      "pastel" = scale_fill_gradient2(low = "#b3cde3", high = "#fbb4ae", mid = "white", midpoint = 0, limit = c(-1,1), name="Pearson\nCorrelation"),
      "blue"   = scale_fill_gradient2(low = "#deebf7", high = "#3182bd", mid = "white", midpoint = 0, limit = c(-1,1), name="Pearson\nCorrelation"),
      "dark"   = scale_fill_gradient2(low = "#1b9e77", high = "#d95f02", mid = "white", midpoint = 0, limit = c(-1,1), name="Pearson\nCorrelation"),
      scale_fill_gradient2(low = "#4575b4", high = "#d73027", mid = "white", midpoint = 0, limit = c(-1,1), name="Pearson\nCorrelation")
    )

    ggplot(cor_melted, aes(Var1, Var2, fill = value)) +
      geom_tile(color = "white", size = 0.5) +
      heatmap_colors +
      theme_minimal(base_size = 12) +
      theme(
        axis.text.x = element_text(angle = 45, vjust = 1, hjust = 1, color = "#495057"),
        axis.text.y = element_text(color = "#495057"),
        axis.title.x = element_blank(),
        axis.title.y = element_blank(),
        panel.grid.major = element_blank(),
        panel.border = element_blank(),
        panel.background = element_blank(),
        axis.ticks = element_blank(),
        plot.title = element_text(hjust = 0.5, face = "bold", size = 14, color = "#343a40"),
        plot.background = element_rect(fill = "transparent", color = NA)
      ) +
      coord_fixed() +
      labs(title = "Correlation Heatmap (Numeric Variables)")
  }, bg="transparent")

  # Helper function untuk menentukan status fit ======
  get_fit_status <- function(chisq, df, pvalue, rmsea, cfi, srmr, tli, has_heywood = FALSE) {
    if (is.na(chisq) || is.na(rmsea) || is.na(cfi) || is.na(srmr)) {
      status_res <- list(status = "Unknown", color = "#e2e3e5")
    } else {
      c_df <- chisq / df
      tli_val <- ifelse(is.na(tli), cfi, tli) # Fallback to CFI if TLI missing

      if (c_df < 2 && rmsea <= 0.05 && cfi >= 0.95 && tli_val >= 0.95 && srmr <= 0.05) {
        status_res <- list(status = "Excellent", color = "lightgreen")
      } else if (c_df < 3 && rmsea < 0.08 && cfi >= 0.90 && tli_val >= 0.90 && srmr <= 0.08) {
        status_res <- list(status = "Good", color = "#d4edda")
      } else if (c_df < 5 && (rmsea < 0.10 || cfi >= 0.85 || srmr <= 0.10) ||
        sum(c(c_df < 3, rmsea < 0.08, cfi >= 0.90, tli_val >= 0.90, srmr <= 0.08)) >= 3) {
        status_res <- list(status = "Acceptable", color = "#fff3cd")
      } else {
        status_res <- list(status = "Poor", color = "#f8d7da")
      }
    }

    if (has_heywood) {
      status_res$status <- paste(status_res$status, "(\u26A0 Heywood)")
      status_res$color <- "#ffcccc" # Overwrite with light red if Heywood case
    }

    return(status_res)
  }


  # Loadings table
  output$loadings_table <- renderDT(
    {
      cur_id <- cfa_current_id()
      if (is.null(cur_id)) {
        return(NULL)
      }

      fit <- cfa_fit_list()[[cur_id]]$fit

      if (input$cfa_std_est) {
        sol <- tryCatch(lavaan::standardizedSolution(fit), error = function(e) NULL)
      } else {
        sol <- tryCatch(lavaan::parameterEstimates(fit), error = function(e) NULL)
      }

      if (is.null(sol)) {
        return(datatable(data.frame(Message = "No parameter estimates available")))
      }

      loadings <- sol[sol$op == "=~", ]

      if (nrow(loadings) == 0) {
        return(datatable(data.frame(Message = "No factor loadings found")))
      }

      if (input$cfa_std_est) {
        display_df <- loadings[, c("lhs", "rhs", "est.std", "pvalue")]
        colnames(display_df) <- c("Factor", "Item", "StdLoading", "pvalue")
        display_df$StdLoading <- round(display_df$StdLoading, 3)
      } else {
        display_df <- loadings[, c("lhs", "rhs", "est", "se", "pvalue")]
        colnames(display_df) <- c("Factor", "Item", "Estimate", "SE", "pvalue")
        display_df$Estimate <- round(display_df$Estimate, 3)
        display_df$SE <- round(display_df$SE, 3)
      }

      display_df$pvalue <- format.pval(display_df$pvalue, digits = 3)

      datatable(display_df,
        extensions = "Buttons",
        options = list(
          pageLength = 15, scrollX = TRUE, dom = "Brtp",
          buttons = list(
            list(
              extend = "csv",
              text = "Export CSV",
              filename = paste0("Factor Loading")
            ),
            list(
              extend = "excel",
              text = "Export Excel",
              filename = paste0("Factor Loading")
            )
          )
        )
      ) %>%
        formatStyle(
          columns = "StdLoading",
          fontWeight = "bold",
          color = styleInterval(c(0.3, 0.5, 0.7), c("red", "orange", "blue", "green"))
        )
    },
    server = FALSE
  )

  # Modification indices
  mi_df <- reactive({
    cur_id <- cfa_current_id()
    if (is.null(cur_id)) {
      return(NULL)
    }

    fit <- cfa_fit_list()[[cur_id]]$fit
    mi <- tryCatch(lavaan::modindices(fit, sort. = TRUE, minimum.value = 1), error = function(e) NULL)

    if (is.null(mi) || nrow(mi) == 0) {
      return(NULL)
    }

    if (nrow(mi) > 20) mi <- mi[1:20, ]

    mi
  })

  output$mi_table <- renderDT(
    {
      mi <- mi_df()
      # mi <- mi %>% dplyr::mutate(across(where(is.numeric), ~ round(.x, 3)))
      if (is.null(mi)) {
        return(datatable(data.frame(Message = "No modification indices available")))
      }

      mi$Action <- sapply(1:nrow(mi), function(i) {
        as.character(
          actionButton(
            class = "btn-primary sm",
            style = "font-size: 11px !important; padding: 2px 10px !important;",
            inputId = paste0("mi_btn_", i),
            label = "Use This",
            onclick = paste0('Shiny.setInputValue("mi_selected", "', i, '", {priority: "event"})')
          )
        )
      })

      display_mi <- mi[, c("lhs", "op", "rhs", "mi", "epc", "Action")]
      colnames(display_mi) <- c("From", "Operator", "To", "MI", "EPC", "Action")

      datatable(display_mi,
        rownames = FALSE, escape = FALSE,
        options = list(
          pageLength = 10, scrollX = TRUE, dom = "rtp",
          # Atur ukuran font dan styling
          initComplete = JS(
            "function(settings, json) {",
            "$(this.api().table().header()).css({'font-size': '5px', 'font-weight': 'bold'});",
            "$(this.api().table().body()).css({'font-size': '5px', 'line-height': '1'});",
            "}"
          )
        )
      ) %>%
        formatRound(columns = which(sapply(display_mi, is.numeric)), digits = 3)
    },
    server = FALSE
  )

  # Handle MI button clicks
  observeEvent(input$mi_selected, {
    req(input$mi_selected)

    mi <- mi_df()
    if (is.null(mi)) {
      return()
    }

    idx <- as.numeric(input$mi_selected)
    if (idx > nrow(mi)) {
      return()
    }

    row <- mi[idx, ]

    if (row$op == "=~") {
      new_line <- paste(row$lhs, "=~", row$rhs)
    } else if (row$op == "~~") {
      new_line <- paste(row$lhs, "~~", row$rhs)
    } else if (row$op == "~") {
      new_line <- paste(row$lhs, "~", row$rhs)
    } else {
      new_line <- paste(row$lhs, row$op, row$rhs)
    }

    current_model <- input$cfa_model_text
    updated_model <- paste(current_model, "\n", new_line)

    updateTextAreaInput(session, "cfa_model_text", value = updated_model)

    showNotification(paste("Added MI suggestion to model:", new_line), type = "message")
  })

  # Update fungsi get_color_scheme
  get_color_scheme <- function(scheme_name, custom_man = NULL, custom_lat = NULL) {
    if (scheme_name == "Custom" && !is.null(custom_man) && !is.null(custom_lat)) {
      return(list(man = custom_man, lat = custom_lat))
    }

    switch(scheme_name,
      "Blue-Yellow" = list(man = "#A1E3F9", lat = c("#FFFFBA")),
      "Ocean" = list(man = "#B3E5FC", lat = c("#81D4FA", "#4FC3F7", "#29B6F6")),
      "Forest" = list(man = "#C8E6C9", lat = c("#A5D6A7", "#81C784", "#4CAF50")),
      "Rainbow" = list(man = "#B3E5FC", lat = c("#FF8A65", "#81D4FA", "#AED581", "#CE93D8", "#FFF176")),
      "Pastel" = list(man = "#F8ECE5", lat = c("#E2E2E2", "#E8EAF6", "#F3E5F5", "#E0F2F1")),
      "Greyscale" = list(man = "#EEEEEE", lat = c("#DDDDDD", "#CCCCCC", "#BBBBBB", "#AAAAAA")),
      "Earth" = list(man = "#D7CCC8", lat = c("#BCAAA4", "#A1887F", "#8D6E63", "#795548")),
      "Vibrant" = list(man = "#FFE0B2", lat = c("#FF5252", "#E040FB", "#7C4DFF", "#536DFE")),
      "Monochrome" = list(man = "#E0E0E0", lat = c("#9E9E9E", "#757575", "#616161", "#424242")),
      "Sunset" = list(man = "#FFCCBC", lat = c("#FF8A65", "#FF7043", "#F4511E", "#E64A19")),
      "Rose" = list(man = "#F8BBD0", lat = c("#F06292", "#EC407A", "#E91E63", "#D81B60")),
      "Mint" = list(man = "#B2DFDB", lat = c("#4DB6AC", "#26A69A", "#009688", "#00897B"))
    )
  }

  # Get factor names safely
  get_factor_names <- function(fit_obj) {
    tryCatch(
      {
        if (inherits(fit_obj, "lavaan")) {
          params <- lavaan::parameterEstimates(fit_obj)
          factors <- unique(params$lhs[params$op == "=~"])
          if (length(factors) > 0) {
            return(factors)
          }

          model_text <- fit_obj@Model@model.syntax
          if (is.character(model_text)) {
            lines <- strsplit(model_text, "\n")[[1]]
            factor_lines <- grep("=~", lines, value = TRUE)
            factors <- sapply(strsplit(factor_lines, "=~"), function(x) trimws(x[1]))
            return(factors)
          }
        }
        return("Unknown")
      },
      error = function(e) {
        return("Error getting factors")
      }
    )
  }

  # Helper to generate the fit text
  get_fit_text <- function(m, selected, dec_sep = ".") {
    all_indices <- c("chisq", "df", "pvalue", "cmindf", "rmsea", "cfi", "gfi", "srmr", "tli", "nfi")
    if ("SELECT ALL" %in% selected) selected <- all_indices
    selected <- intersect(all_indices, selected)
    
    # Check if robust indices were used
    rob_pfx <- if ("is_robust" %in% names(m) && !is.na(m["is_robust"]) && m["is_robust"] == 1) " (R)" else ""
    
    labels <- c(chisq = paste0("χ²", rob_pfx), df = paste0("df", rob_pfx), pvalue = "p", 
                cmindf = paste0("CMIN/DF", rob_pfx), rmsea = paste0("RMSEA", rob_pfx), 
                cfi = paste0("CFI", rob_pfx), gfi = "GFI", srmr = "SRMR", 
                tli = paste0("TLI", rob_pfx), nfi = paste0("NFI", rob_pfx))
    
    # Calculate CMIN/DF if requested
    if ("cmindf" %in% selected && !is.na(m["chisq"]) && !is.na(m["df"]) && m["df"] > 0) {
      m["cmindf"] <- m["chisq"] / m["df"]
    } else if ("cmindf" %in% selected) {
      m["cmindf"] <- NA
    }
    
    format_val <- function(name, val) {
      if (is.na(val)) {
        return("NA")
      }
      fmt_with_sep <- function(x) {
        if (dec_sep == ",") gsub("\\.", ",", x) else x
      }
      if (name == "pvalue") {
        return(fmt_with_sep(format.pval(val, digits = 3)))
      }
      if (name == "chisq") {
        return(fmt_with_sep(as.character(round(val, 2))))
      }
      if (name == "df") {
        return(val)
      }
      return(fmt_with_sep(sprintf("%.3f", val)))
    }
    paste(sapply(selected, function(n) paste0(labels[n], "=", format_val(n, m[n]))), collapse = "; ")
  }

  # Variances table
  output$variances_table <- renderDT(
    {
      cur_id <- cfa_current_id()
      if (is.null(cur_id)) {
        return(NULL)
      }

      fit <- cfa_fit_list()[[cur_id]]$fit

      if (input$cfa_std_est) {
        sol <- tryCatch(lavaan::standardizedSolution(fit), error = function(e) NULL)
      } else {
        sol <- tryCatch(lavaan::parameterEstimates(fit), error = function(e) NULL)
      }

      if (is.null(sol)) {
        return(datatable(data.frame(Message = "No parameter estimates available")))
      }

      variances <- sol[sol$op == "~~" & sol$lhs == sol$rhs, ]

      if (nrow(variances) == 0) {
        return(datatable(data.frame(Message = "No variances found")))
      }

      if (input$cfa_std_est) {
        display_df <- variances[, c("lhs", "est.std", "se", "pvalue")]
        colnames(display_df) <- c("Variable", "StdVariance", "SE", "pvalue")
        display_df$StdVariance <- round(display_df$StdVariance, 3)
        display_df$SE <- round(display_df$SE, 3)
      } else {
        display_df <- variances[, c("lhs", "est", "se", "pvalue")]
        colnames(display_df) <- c("Variable", "Variance", "SE", "pvalue")
        display_df$Variance <- round(display_df$Variance, 3)
        display_df$SE <- round(display_df$SE, 3)
      }

      display_df$pvalue <- format.pval(display_df$pvalue, digits = 3)

      datatable(display_df,
        extensions = "Buttons",
        options = list(
          pageLength = 15, scrollX = TRUE, dom = "Brtp",
          buttons = list(
            list(extend = "csv", text = "Export CSV", filename = "Variances"),
            list(extend = "excel", text = "Export Excel", filename = "Variances")
          )
        )
      )
    },
    server = FALSE
  )

  # Variance warning
  output$variance_warning <- renderUI({
    cur_id <- cfa_current_id()
    if (is.null(cur_id)) {
      return(NULL)
    }

    fit <- cfa_fit_list()[[cur_id]]$fit
    sol <- tryCatch(lavaan::parameterEstimates(fit), error = function(e) NULL)

    if (is.null(sol)) {
      return(NULL)
    }

    variances <- sol[sol$op == "~~" & sol$lhs == sol$rhs, ]
    has_negative <- any(variances$est < 0, na.rm = TRUE)

    if (has_negative) {
      tags$div(
        class = "alert alert-danger",
        icon("exclamation-triangle"),
        tags$b(" Warning: Heywood case detected! One or more variances are negative. This may indicate a problem with the model or data.")
      )
    } else {
      NULL
    }
  })

  # HTML Report Download Handler
  # Modal for HTML export
  # Make tempdir available for iframe
  # Per-session directory, so concurrent users never share a report file
  cfa_report_res <- session_report_dir(session, "cfa")
  
  cfa_report_path <- reactiveVal(NULL)

  # Renders the R Markdown report and returns the path of the generated HTML
  # file (or NULL on failure). Shared by the preview button and the download
  # handler, so a report can be downloaded without generating a preview first.
  generate_cfa_report <- function(progress_label = "Generating Report...") {
    if (is.null(cfa_current_id())) {
      return(NULL)
    }
    cur_id <- cfa_current_id()
    fit_info <- cfa_fit_list()[[cur_id]]

    report_path <- "cfa_report.Rmd"
    if (!file.exists(report_path)) {
      report_path <- file.path(system.file("app", package = "projectLSA"), "cfa_report.Rmd")
    }

    tempReport <- file.path(cfa_report_res$path, "cfa_report.Rmd")
    file.copy(report_path, tempReport, overwrite = TRUE)

    # Get plot colors safely
    if (input$plot_color_scheme == "Custom") {
      man_col <- input$mancolour
      lat_col <- input$latcolour
    } else {
      colors <- get_color_scheme(input$plot_color_scheme)
      man_col <- colors$man
      lat_col <- colors$lat
    }

    # Get all models for history
    lst <- cfa_fit_list()
    fit_initial <- lst[[1]]$fit

    model_history_df <- do.call(rbind, lapply(lst, function(x) {
      m <- x$measures
      hw <- ifelse(!is.null(x$has_heywood) && x$has_heywood, TRUE, FALSE)
      status_info <- get_fit_status(
        m["chisq"], m["df"], m["pvalue"],
        m["rmsea"], m["cfi"], m["srmr"], m["tli"], hw
      )
      data.frame(
        Model = x$id,
        Note = ifelse(is.null(x$note) || x$note == "", "-", x$note),
        chisq = m["chisq"],
        df = m["df"],
        pvalue = m["pvalue"],
        RMSEA = m["rmsea"],
        CFI = m["cfi"],
        TLI = m["tli"],
        SRMR = m["srmr"],
        Heywood = ifelse(hw, "Yes", "No"),
        Status = status_info$status,
        stringsAsFactors = FALSE
      )
    }))

    out_html <- file.path(cfa_report_res$path, "cfa_report_out.html")

    # Detect model type
    pe <- lavaan::parameterEstimates(fit_info$fit)
    model_type <- if (any(pe$op == "~")) "SEM" else "CFA"

    showModal(modalDialog(progress_label, footer = NULL))
    ok <- tryCatch({
      rmarkdown::render(tempReport,
        output_file = out_html,
        params = list(
          model_type = model_type,
          fit_final = fit_info$fit,
          fit_initial = fit_initial,
          model_history_df = model_history_df,
          console_out = console_context$text,
          ai_summary = if (is.null(ai_context$ai_report_text)) "" else ai_context$ai_report_text,
          dec = input$cfa_dec_sep,
          ave = fit_info$ave,
          relsem = fit_info$relsem,
          htmt = fit_info$htmt,
          plot_standardized = input$plot_standardized,
          plot_style = input$plot_style,
          plot_layout = input$plot_layout,
          plot_residuals = input$plot_residuals,
          plot_exoCov = input$plot_exoCov,
          plot_edge_label_size = input$plot_edge_label_size,
          plot_nodesize_lat = input$plot_nodesize_lat,
          plot_nodesize_man = input$plot_nodesize_man,
          plot_nodesize_man2 = input$plot_nodesize_man2,
          plot_rotation = input$plot_rotation,
          man_col = man_col,
          lat_col = lat_col,
          bifactor = input$bifactor,
          edgewidth = input$edgewidth,
          plotwidth = input$plotwidth,
          plotheight = input$plotheight
        ),
        envir = new.env(parent = globalenv())
      )
      cfa_report_path(out_html)
      TRUE
    }, error = function(e) {
      showNotification(paste("Error generating report:", e$message), type = "error")
      FALSE
    }, finally = {
      removeModal()
    })

    if (isTRUE(ok)) out_html else NULL
  }

  observeEvent(input$cfa_generate_preview, {
    req(cfa_current_id())
    generate_cfa_report("Generating Report Preview...")
  })

  output$cfa_report_preview_frame <- renderUI({
    req(cfa_report_path())
    # Cache-busting query string so a regenerated report is actually reloaded
    tags$iframe(
      src = paste0(cfa_report_res$prefix, "/cfa_report_out.html?v=", as.integer(Sys.time())),
      width = "100%", height = "800px", style = "border: none;"
    )
  })

  # Export Report modal (also carries the support/donation message)
  observeEvent(input$btn_export_modal, {
    showModal(modalDialog(
      title = "Export HTML Report",
      tags$div(
        style = "text-align: center;",
        tags$h4("Your report is ready to download!"),
        tags$p("If you find projectLSA helpful for your research, consider supporting its continuous development. Your support helps us keep the project alive!"),
        tags$div(
          style = "background-color: #f8f9fa; border: 2px dashed #17a2b8; padding: 20px; border-radius: 10px; margin-bottom: 20px;",
          tags$strong("Support via Bank BNI:", style = "color: #17a2b8; font-size: 16px;"), tags$br(),
          tags$span("0331355050", style = "font-size: 24px; font-weight: bold; color: #d32f2f; letter-spacing: 2px;"), tags$br(),
          tags$em("Account Name: Hasan Djidu"), tags$br(), tags$br(),
          tags$strong("International Transfer (SWIFT):", style = "color: #17a2b8; font-size: 14px;"), tags$br(),
          tags$span("BNINIDJA", style = "font-size: 16px; font-weight: bold; color: #d32f2f;")
        ),
        downloadButton("download_report_html_modal", "Download Report Now", class = "btn btn-success btn-lg")
      ),
      footer = modalButton("Close"),
      easyClose = TRUE
    ))
  })

  cfa_report_filename <- function() {
    cur_id <- cfa_current_id()
    if (is.null(cur_id)) {
      return(paste0("Report_", Sys.Date(), ".html"))
    }

    fit_obj <- cfa_fit_list()[[cur_id]]$fit
    pe <- lavaan::parameterEstimates(fit_obj)

    # Detect if SEM
    is_sem <- any(pe$op == "~")
    prefix <- if (is_sem) "SEM_Report_" else "CFA_Report_"

    latents <- unique(pe$lhs[pe$op == "=~"])
    latent_name <- if (length(latents) > 0) latents[length(latents)] else "Model"

    paste0(prefix, latent_name, "_", Sys.Date(), ".html")
  }

  cfa_report_content <- function(file) {
    req(cfa_current_id())
    # Render on demand when the user downloads without generating a preview
    path <- cfa_report_path()
    if (is.null(path) || !file.exists(path)) {
      path <- generate_cfa_report("Generating Report...")
    }
    req(path)
    file.copy(path, file, overwrite = TRUE)
  }

  output$download_report_html <- downloadHandler(
    filename = cfa_report_filename,
    content = cfa_report_content,
    contentType = "text/html"
  )

  output$download_report_html_modal <- downloadHandler(
    filename = cfa_report_filename,
    content = cfa_report_content,
    contentType = "text/html"
  )

  # Refactored plot rendering (shared between display and download)
  render_cfa_plot_impl <- function() {
    cur_id <- cfa_current_id()
    if (is.null(cur_id)) {
      plot(1, 1, type = "n", xlab = "", ylab = "", axes = FALSE, main = "No CFA model available")
      text(1, 1, "Please run a CFA model first\n\n1. Select 'Built-in: HolzingerSwineford1939'\n2. Keep the example model\n3. Click 'Run CFA'", cex = 1.2, col = "darkred")
      return(invisible(NULL))
    }

    obj <- cfa_fit_list()[[cur_id]]
    fit_obj <- obj$fit
    m <- obj$measures

    if (input$plot_color_scheme == "Custom") {
      colors <- list(man = input$mancolour, lat = input$latcolour)
    } else {
      colors <- get_color_scheme(input$plot_color_scheme)
    }

    b_margin <- 6
    A <- semPlot::semPaths(
      fit_obj,
      what = ifelse(input$plot_standardized, "std", "est"),
      structural = (input$plot_model_scope == "structural"),
      style = input$plot_style,
      layout = input$plot_layout,
      layoutSplit = input$plot_layout_split,
      curvePivot = input$plot_curve,
      subScale = input$subScale_Wi,
      subScale2 = input$subScale_He,
      residuals = input$plot_residuals,
      exoCov = input$plot_exoCov,
      edge.label.cex = input$plot_edge_label_size,
      nCharNodes = 6,
      nCharEdges = 6,
      sizeLat = input$plot_nodesize_lat,
      sizeMan = input$plot_nodesize_man,
      sizeMan2 = input$plot_nodesize_man2,
      rotation = input$plot_rotation,
      color = colors,
      label.color = "black",
      mar = c(b_margin, 15 - input$plotwidth, 20 - input$plotheight, 15 - input$plotwidth),
      bifactor = input$bifactor,
      esize = input$edgewidth,
      edge.color = "black",
      freeStyle = c(1, "black"),
      fixedStyle = c(2, "black"),
      fade = FALSE,
      DoNotPlot = TRUE
    )

    A <- semptools::mark_sig(
      semPaths_plot = A, object = fit_obj,
      alphas = c("*" = 0.05, "**" = 0.01, "***" = 0.001)
    )

    # Apply decimal separator
    if (input$cfa_dec_sep == ",") {
      A$graphAttributes$Edges$labels <- gsub("\\.", ",", A$graphAttributes$Edges$labels)
    }

    plot(A)

    # Add fit info inside a box
    factor_names <- get_factor_names(fit_obj)
    fit_text_raw <- get_fit_text(m, input$fit_indices_selected, dec_sep = input$cfa_dec_sep)
    wrapped_fit <- paste(strwrap(paste0("Fit indices: ", fit_text_raw), width = 110), collapse = "\n")

    estimator_text <- paste0("Estimator = ", input$cfa_estimator, " | Factors: ", paste(factor_names, collapse = ", "))
    sig_sign <- '*** : p ≤ 0.001", "** : p ≤ 0.01", "* : p ≤ 0.05'

    legend_text <- c(wrapped_fit, estimator_text, sig_sign)
    text_colors <- c("#000000", "blue", "#00002d")

    leg_info <- legend("bottom", legend = legend_text, plot = FALSE, inset = c(0, 0), cex = 0.9, y.intersp = 1.0)

    max_w <- max(strwidth(legend_text, cex = 0.9))
    legend("bottom",
      legend = rep("", length(legend_text)), bg = "#f8f9fa", box.col = "#e2e8f0",
      inset = c(0, 0), cex = 0.9, xpd = TRUE, y.intersp = 1.0, text.width = max_w
    )

    center_x <- leg_info$rect$left + leg_info$rect$w / 2
    y_coords <- leg_info$text$y
    for (i in seq_along(legend_text)) {
      text(x = center_x, y = y_coords[i], labels = legend_text[i], col = text_colors[i], cex = 0.9, xpd = TRUE, adj = c(0.5, 0.5))
    }
  }

  output$path_plot <- renderPlot({
    tryCatch(
      {
        render_cfa_plot_impl()
      },
      error = function(e) {
        plot(1, 1, type = "n", xlab = "", ylab = "", axes = FALSE, main = "Plot Error")
        text(1, 1, paste("Error creating plot:\n", e$message), cex = 1.2, col = "red")
      }
    )
  })

  output$download_cfa_plot <- downloadHandler(
    filename = function() {
      paste0("CFA_Plot_", Sys.Date(), ".png")
    },
    content = function(file) {
      png(file, width = 12, height = 10, units = "in", res = 300)
      tryCatch({
        render_cfa_plot_impl()
      }, finally = {
        dev.off()
      })
    }
  )

  observeEvent(input$fit_indices_selected, {
    sel <- input$fit_indices_selected
    if ("SELECT ALL" %in% sel && length(sel) > 1) {
      updateSelectInput(session, "fit_indices_selected",
        selected = if (tail(sel, 1) == "SELECT ALL") {
          "SELECT ALL"
        } else {
          setdiff(sel, "SELECT ALL")
        }
      )
    }
  })

  # Fit information below plot
  output$plot_fit_info <- renderUI({
    cur_id <- cfa_current_id()
    if (is.null(cur_id)) {
      return(NULL)
    }

    obj <- cfa_fit_list()[[cur_id]]
    m <- obj$measures
    fit_obj <- obj$fit

    factor_names <- get_factor_names(fit_obj)
    selected <- input$fit_indices_selected
    fit_text <- get_fit_text(m, selected, dec_sep = input$cfa_dec_sep)

    estimator_text <- paste0(
      "Estimator = ", input$cfa_estimator,
      " | Factors: ", paste(factor_names, collapse = ", ")
    )
    sig_sign <- paste0(
      '*** : p ≤ 0.001", "** : p ≤ 0.01", "* : p ≤ 0.05'
    )

    tagList(
      tags$div(
        style = "margin-top: 0px; padding: 5px; background-color: #f8f9fa; border-radius: 5px; text-align: center;",
        tags$p(
          style = "margin: 0; font-size: 11px; color: #2c3e50;",
          tags$b("Fit indices: "), fit_text
        ),
        tags$p(
          style = "margin: 0; font-size: 11px; color: darkblue;",
          estimator_text
        ),
        tags$p(
          style = "margin: 0; font-size: 10px; color: #6c757d;",
          sig_sign
        )
      )
    )
  })

  # Download handlers
  output$download_measures <- downloadHandler(
    filename = function() paste0("cfa_measures_", Sys.Date(), ".csv"),
    content = function(file) {
      lst <- cfa_fit_list()
      if (length(lst) == 0) {
        return(NULL)
      }

      df_list <- lapply(lst, function(x) {
        data.frame(Model = x$id, as.list(x$measures), stringsAsFactors = FALSE)
      })

      df <- do.call(rbind, df_list)
      write.csv(df, file, row.names = FALSE)
    }
  )

  output$download_loadings <- downloadHandler(
    filename = function() paste0("cfa_loadings_", Sys.Date(), ".csv"),
    content = function(file) {
      cur_id <- cfa_current_id()
      if (is.null(cur_id)) {
        return(NULL)
      }

      fit <- cfa_fit_list()[[cur_id]]$fit

      if (input$cfa_std_est) {
        sol <- lavaan::standardizedSolution(fit)
      } else {
        sol <- lavaan::parameterEstimates(fit)
      }

      write.csv(sol, file, row.names = FALSE)
    }
  )

  output$download_lavaan <- downloadHandler(
    filename = function() paste0("cfa_summary_", Sys.Date(), ".txt"),
    content = function(file) {
      cur_id <- cfa_current_id()
      if (is.null(cur_id)) {
        return(NULL)
      }

      fit <- cfa_fit_list()[[cur_id]]$fit

      sink(file)
      print(summary(fit, fit.measures = TRUE, standardized = TRUE))
      sink()
    }
  )

  output$download_scores_cfa <- downloadHandler(
    filename = function() paste0("Factor Scores_", Sys.Date(), ".csv"),
    content = function(file) {
      cur_id <- cfa_current_id()
      if (is.null(cur_id)) {
        return(NULL)
      }

      scoreCfa <- cfa_fit_list()[[cur_id]]$scoreCfa

      write.csv(scoreCfa, file, row.names = FALSE)
    }
  )

  # ====== AUTOMATED MEASUREMENT INVARIANCE ======
  observeEvent(input$run_auto_invariance, {
    req(data_user(), input$cfa_model_text, input$selected_vars, input$mgcfa_group_var)

    if (nchar(trimws(input$cfa_model_text)) == 0) {
      showNotification("Please enter a CFA model", type = "error")
      return()
    }

    updateTabsetPanel(session, "main_tab_cfa", selected = "invariance_tab_cfa")

    df_subset <- data_user()[, input$selected_vars, drop = FALSE]
    df <- cbind(df_subset, data_user()[, input$mgcfa_group_var, drop = FALSE])

    showModal(modalDialog("Running Automated Invariance Testing (Configural -> Metric -> Scalar -> Strict)...", footer = NULL))

    # 1. Configural
    fit_config <- run_lavaan(model_text = input$cfa_model_text, data = df, estimator = input$cfa_estimator, missing = input$cfa_missing, mode = "mgcfa", group = input$mgcfa_group_var, group_equal = "configural")
    # 2. Metric
    fit_metric <- run_lavaan(model_text = input$cfa_model_text, data = df, estimator = input$cfa_estimator, missing = input$cfa_missing, mode = "mgcfa", group = input$mgcfa_group_var, group_equal = "loadings")
    # 3. Scalar
    fit_scalar <- run_lavaan(model_text = input$cfa_model_text, data = df, estimator = input$cfa_estimator, missing = input$cfa_missing, mode = "mgcfa", group = input$mgcfa_group_var, group_equal = "loadings, intercepts")
    # 4. Strict
    fit_strict <- run_lavaan(model_text = input$cfa_model_text, data = df, estimator = input$cfa_estimator, missing = input$cfa_missing, mode = "mgcfa", group = input$mgcfa_group_var, group_equal = "loadings, intercepts, residuals")

    removeModal()

    output$invariance_results <- renderUI({
      extract_fit <- function(f, name) {
        if (is.null(f)) {
          return(data.frame(Model = name, Chisq = NA, df = NA, pvalue = NA, CFI = NA, RMSEA = NA))
        }
        fm <- lavaan::fitMeasures(f)
        get_val <- function(std, scaled, robust) {
          if (!missing(robust) && robust %in% names(fm) && !is.na(fm[robust])) {
            return(fm[robust])
          }
          if (!missing(scaled) && scaled %in% names(fm) && !is.na(fm[scaled])) {
            return(fm[scaled])
          }
          if (std %in% names(fm)) {
            return(fm[std])
          }
          return(NA)
        }
        chisq <- get_val("chisq", "chisq.scaled")
        df <- get_val("df", "df.scaled")
        pvalue <- get_val("pvalue", "pvalue.scaled")
        cfi <- get_val("cfi", "cfi.scaled", "cfi.robust")
        rmsea <- get_val("rmsea", "rmsea.scaled", "rmsea.robust")
        data.frame(Model = name, Chisq = chisq, df = df, pvalue = pvalue, CFI = cfi, RMSEA = rmsea)
      }

      res <- rbind(
        extract_fit(fit_config, "Configural (Base)"),
        extract_fit(fit_metric, "Metric (Weak)"),
        extract_fit(fit_scalar, "Scalar (Strong)"),
        extract_fit(fit_strict, "Strict (Residual)")
      )

      res$Delta_Chisq <- c(NA, diff(res$Chisq))
      res$Delta_df <- c(NA, diff(res$df))
      res$p_Delta_Chisq <- pchisq(abs(res$Delta_Chisq), abs(res$Delta_df), lower.tail = FALSE)
      res$Delta_CFI <- c(NA, diff(res$CFI))
      res$Delta_RMSEA <- c(NA, diff(res$RMSEA))

      # Fix signs for Delta CFI and RMSEA for better readability
      res_table <- datatable(res, rownames = FALSE, options = list(dom = "t", scrollX = TRUE)) %>%
        formatRound(c("Chisq", "CFI", "RMSEA", "Delta_Chisq", "Delta_CFI", "Delta_RMSEA"), digits = 3) %>%
        formatRound(c("pvalue", "p_Delta_Chisq"), digits = 4)

      tagList(
        tags$h4("Measurement Invariance Testing Results"),
        tags$p("Comparison of nested models: Configural vs Metric vs Scalar vs Strict"),
        res_table,
        br(),
        tags$b("Interpretation Guidelines (Chen, 2007; Cheung & Rensvold, 2002):"),
        tags$ul(
          tags$li("Invariance is supported if ", tags$b("ΔCFI ≥ -0.010"), " and ", tags$b("ΔRMSEA ≤ 0.015"), " (or non-significant Δχ²)."),
          tags$li("Strict invariance is often too restrictive and rarely achieved in practice.")
        )
      )
    })
  })

  # ==== R Console Output & Model Export ====
  observeEvent(cfa_current_id(), {
    cur_id <- cfa_current_id()
    if (is.null(cur_id)) {
      console_context$text <- "No model available."
    } else {
      all_models <- cfa_fit_list()
      full_out <- ""
      
      for (i in seq_along(all_models)) {
        mod <- all_models[[i]]
        out <- paste0("\n========================================\n", 
                      "           ", mod$id, "\n",
                      "========================================\n")
        if (mod$note != "") out <- paste0(out, "Note: ", mod$note, "\n\n")
        
        out <- paste0(out, paste(capture.output(print(summary(mod$fit, fit.measures = TRUE, standardized = TRUE))), collapse = "\n"))
        
        if (requireNamespace("semTools", quietly = TRUE)) {
          try({
            rel <- semTools::reliability(mod$fit)
            out <- paste0(out, "\n\n=== RELIABILITY & AVE ===\n", paste(capture.output(print(rel)), collapse = "\n"))
          }, silent = TRUE)
        }
        full_out <- paste0(full_out, out, "\n")
      }
      
      console_context$text <- full_out
    }
  })

  output$export_cfa_rds <- downloadHandler(
    filename = function() { paste0("CFA_Workspace_", Sys.Date(), ".rds") },
    content = function(file) {
      cur_id <- cfa_current_id()
      req(cur_id)
      workspace <- list(
        type = "projectLSA_workspace",
        module = "CFA",
        raw_data = raw_data_user(),
        cfa_fit_list = cfa_fit_list(),
        cfa_current_id = cfa_current_id(),
        cfa_aggregations = cfa_aggregations(),
        input_state = list(
          cfa_estimator = input$cfa_estimator,
          cfa_missing = input$cfa_missing,
          cfa_std_est = input$cfa_std_est,
          htmt_opt = input$htmt_opt,
          cfa_analysis_mode = input$cfa_analysis_mode,
          mgcfa_invariance = input$mgcfa_invariance,
          mgcfa_group = input$mgcfa_group,
          cfa_model_text = input$cfa_model_text,
          cfa_summary_theme = input$cfa_summary_theme,
          plot_model_scope = input$plot_model_scope,
          plot_style = input$plot_style,
          plot_layout = input$plot_layout,
          plot_color_scheme = input$plot_color_scheme,
          plot_standardized = input$plot_standardized,
          fit_indices_selected = input$fit_indices_selected,
          cfa_agg_syntax = input$cfa_agg_syntax,
          cfa_agg_method = input$cfa_agg_method
        )
      )
      saveRDS(workspace, file)
    }
  )

  # ==== Score New Data ====
  output$download_cfa_template <- downloadHandler(
    filename = function() {
      "CFA_template.xlsx"
    },
    content = function(file) {
      cur_id <- cfa_current_id()
      req(cur_id)
      fit <- cfa_fit_list()[[cur_id]]$fit
      # extract item names from lavaan model
      items <- lavaan::lavNames(fit, type = "ov")
      df <- data.frame(matrix(ncol = length(items), nrow = 0))
      colnames(df) <- items
      writexl::write_xlsx(df, file)
    }
  )

  # Handle New Data Upload and Calculation
  cfa_newdata_raw <- reactiveVal(NULL)

  observeEvent(input$cfa_newdata, {
    req(input$cfa_newdata)
    ext <- tools::file_ext(input$cfa_newdata$name)
    df <- switch(ext,
      "csv" = read.csv(input$cfa_newdata$datapath),
      "xlsx" = readxl::read_excel(input$cfa_newdata$datapath),
      "xls" = readxl::read_excel(input$cfa_newdata$datapath),
      stop("Invalid file format")
    )
    cfa_newdata_raw(df)
    updatePickerInput(session, "cfa_newdata_agg_vars", choices = names(df))
  })

  observeEvent(input$cfa_newdata_btn_aggregate, {
    df <- cfa_newdata_raw()
    syntax <- input$cfa_newdata_agg_syntax
    req(df, syntax)
    
    lines <- strsplit(syntax, "\n")[[1]]
    count <- 0
    method <- input$cfa_newdata_agg_method
    
    for (line in lines) {
      if (grepl("=", line)) {
        parts <- strsplit(line, "=")[[1]]
        if (length(parts) == 2) {
          name <- make.names(trimws(parts[1]))
          vars_raw <- strsplit(parts[2], "[, ]+")[[1]]
          vars <- trimws(vars_raw[vars_raw != ""])
          
          # Only use vars that exist in df
          valid_vars <- intersect(vars, names(df))
          if (length(valid_vars) > 0 && name != "") {
            if (method == "mean") {
              df[[name]] <- rowMeans(df[, valid_vars, drop = FALSE], na.rm = TRUE)
            } else if (method == "sum") {
              df[[name]] <- rowSums(df[, valid_vars, drop = FALSE], na.rm = TRUE)
            }
            count <- count + 1
          }
        }
      }
    }
    
    if (count > 0) {
      cfa_newdata_raw(df)
      shinyWidgets::updatePickerInput(session, "cfa_newdata_agg_vars_insert", choices = names(df))
      showNotification(paste(count, "variables calculated and added to new data."), type = "message")
      shinyWidgets::updatePickerInput(session, "cfa_newdata_agg_vars_insert", selected = character(0))
    }
  })

  # Handle insert button for new data
  observeEvent(input$cfa_newdata_btn_insert_agg, {
    vars <- input$cfa_newdata_agg_vars_insert
    if (length(vars) > 0) {
      current <- input$cfa_newdata_agg_syntax
      prefix <- ifelse(!is.null(current) && nchar(current) > 0 && !endsWith(current, "\n"), "\n", "")
      new_text <- paste0(current, prefix, "NewVar = ", paste(vars, collapse = ", "))
      updateTextAreaInput(session, "cfa_newdata_agg_syntax", value = new_text)
    }
  })

  observeEvent(input$cfa_newdata_btn_reset_agg, {
    req(input$cfa_newdata)
    ext <- tools::file_ext(input$cfa_newdata$name)
    df <- switch(ext,
      "csv" = read.csv(input$cfa_newdata$datapath),
      "xlsx" = readxl::read_excel(input$cfa_newdata$datapath),
      "xls" = readxl::read_excel(input$cfa_newdata$datapath),
      stop("Invalid file format")
    )
    cfa_newdata_raw(df)
    updatePickerInput(session, "cfa_newdata_agg_vars", choices = names(df))
    showNotification("New data has been reset to its original uploaded state.", type = "message")
  })

  cfa_current_newdata_view <- reactiveVal(NULL)

  observe({
    cfa_current_newdata_view(cfa_newdata_raw())
  })

  observeEvent(input$cfa_score_newdata_btn, {
    df <- cfa_newdata_raw()
    req(df)
    cur_id <- cfa_current_id()
    req(cur_id)
    fit <- cfa_fit_list()[[cur_id]]$fit
    
    tryCatch({
      fs <- as.data.frame(lavaan::lavPredict(fit, newdata = df))
      for (col in names(fs)) {
        min_val <- min(fs[[col]], na.rm = TRUE)
        max_val <- max(fs[[col]], na.rm = TRUE)
        if (max_val > min_val) {
          fs[[paste0(col, "_0_100")]] <- (fs[[col]] - min_val) / (max_val - min_val) * 100
        } else {
          fs[[paste0(col, "_0_100")]] <- 100
        }
      }
      res <- cbind(df, fs)
      cfa_current_newdata_view(res)
      showNotification("Factor scores calculated and appended to the data.", type = "message")
    }, error = function(e) {
      showNotification(paste("Error computing scores:", e$message), type = "error")
    })
  })

  output$cfa_newscores_table <- DT::renderDataTable({
    req(cfa_current_newdata_view())
    df <- cfa_current_newdata_view()
    
    # Round only numeric columns
    num_cols <- sapply(df, is.numeric)
    if (any(num_cols)) {
      df[num_cols] <- lapply(df[num_cols], round, 3)
    }
    
    datatable(df, options = list(scrollX = TRUE))
  })

  output$download_cfa_newscores <- downloadHandler(
    filename = function() {
      "CFA_New_Scores.csv"
    },
    content = function(file) {
      req(cfa_current_newdata_view())
      write.csv(cfa_current_newdata_view(), file, row.names = FALSE)
    }
  )

  # ==== AI Assistant ====
  # Update global AI context whenever results change
  observe({
    res_text <- ""
    cur_id <- cfa_current_id()
    if (!is.null(cur_id) && !is.null(cfa_fit_list()[[cur_id]]$fit)) {
      all_models <- cfa_fit_list()
      
      for (i in seq_along(all_models)) {
        mod <- all_models[[i]]
        fit <- mod$fit
        m <- mod$measures
        
        fit_text <- paste0("\n========================================\n",
                           "           ", mod$id, "\n",
                           "========================================\n")
        if (mod$note != "") {
           fit_text <- paste0(fit_text, "Modification Note: ", mod$note, "\n\n")
        }
        
        fit_text <- paste0(fit_text, "EXACT FIT INDICES (MUST USE THESE VALUES FOR INTERPRETATION!):\n")
        if (!is.null(m)) {
           fit_text <- paste0(fit_text, 
              "Chi-Square: ", round(m["chisq"],2), " (df=", m["df"], ", p=", round(m["pvalue"],3), ")\n",
              "RMSEA: ", round(m["rmsea"],3), "\n",
              "CFI: ", round(m["cfi"],3), "\n",
              "TLI: ", round(m["tli"],3), "\n",
              "SRMR: ", round(m["srmr"],3), "\n",
              "GFI: ", round(m["gfi"],3), "\n"
           )
        }
        summary_text <- paste(capture.output(summary(fit, fit.measures = TRUE, standardized = TRUE)), collapse = "\n")
        
        rel_text <- ""
        if (requireNamespace("semTools", quietly = TRUE)) {
          try({
            rel <- semTools::reliability(fit)
            rel_text <- paste0("\n\nRELIABILITY & AVE:\n", paste(capture.output(print(rel)), collapse = "\n"))
          }, silent = TRUE)
        }
        
        res_text <- paste0(res_text, fit_text, "\nRAW MODEL SUMMARY:\n", summary_text, rel_text, "\n")
      }
    }
    ai_context$results_text <- res_text
    ai_context$module <- "Confirmatory Factor Analysis (CFA)"
  })

  # ====== Methodological Guide Modal ======
  observeEvent(input$cfa_method_guide, {
    showModal(modalDialog(
      title = tags$b(icon("book"), " Methodological Guidelines"),
      size = "l",
      tags$div(
        tags$h4(tags$b("Estimator Selection")),
        tags$p("Choosing the right estimator is critical for valid inferences in CFA/SEM:"),
        tags$ul(
          tags$li(tags$b("ML (Maximum Likelihood):"), " Default for continuous data with multivariate normality."),
          tags$li(tags$b("MLR (Robust ML):"), " Recommended for continuous data that violate normality assumptions (computes robust standard errors and scaled test statistics)."),
          tags$li(tags$b("WLSMV (Weighted Least Squares Mean and Variance adjusted):"), " Best practice for ordinal or categorical data (e.g., Likert scales with < 5 points)."),
          tags$li(tags$b("Other WLS/ULS variants:"), " Used for specific covariance structure requirements (see Kline, 2016).")
        ),
        tags$hr(),
        tags$h4(tags$b("Missing Data Treatment")),
        tags$p("Improper handling of missing data can bias parameter estimates:"),
        tags$ul(
          tags$li(tags$b("Listwise Deletion:"), " Default. Removes any row with a missing value. Valid only if data is Missing Completely at Random (MCAR). Can severely reduce statistical power."),
          tags$li(tags$b("FIML (Full Information Maximum Likelihood):"), " Often the gold standard for Missing at Random (MAR) data. Uses all available data points without discarding cases (Enders, 2010)."),
          tags$li(tags$b("Pairwise Deletion:"), " Uses available pairs for correlation. Generally discouraged due to potential non-positive definite covariance matrices.")
        ),
        tags$hr(),
        tags$h5(tags$b("References:")),
        tags$ul(
          style = "font-size: 12px;",
          tags$li("Kline, R. B. (2016). Principles and practice of structural equation modeling (4th ed.). Guilford publications."),
          tags$li("Enders, C. K. (2010). Applied missing data analysis. Guilford press."),
          tags$li("MacCallum, R. C., Roznowski, M., & Necowitz, L. B. (1992). Model modifications in covariance structure analysis: The problem of capitalization on chance. Psychological Bulletin, 111(3), 490.")
        )
      ),
      easyClose = TRUE,
      footer = modalButton("Close")
    ))
  })
}

# ai_helper.R
# Helper function to call AI providers (Gemini, OpenAI, Groq, OpenRouter)

# Define providers and their default models
ai_providers <- function() {
  list(
    gemini = list(label = "Google Gemini", default_model = "gemini-2.5-flash"),
    openai = list(label = "OpenAI", default_model = "gpt-4o-mini", base_url = "https://api.openai.com/v1/chat/completions"),
    groq = list(label = "Groq", default_model = "llama-3.3-70b-versatile", base_url = "https://api.groq.com/openai/v1/chat/completions"),
    openrouter = list(label = "OpenRouter", default_model = "meta-llama/llama-3.3-70b-instruct", base_url = "https://openrouter.ai/api/v1/chat/completions")
  )
}

ask_ai <- function(prompt, provider = "gemini", model = "", api_key = "", system_prompt = NULL) {
  if (is.null(api_key) || api_key == "") {
    return("Error: API Key is missing. Please set it in the Settings on the Homepage.")
  }

  if (is.null(model) || model == "") {
    model <- ai_providers()[[provider]]$default_model
  }

  # Format messages
  base_system_prompt <- paste0(
    "You are an assistant for projectLSA, an R package for latent structure analysis (Latent Profile Analysis, Latent Class Analysis, Latent Trait Analysis/IRT, Exploratory Factor Analysis, and Confirmatory Factor Analysis/Structural Equation Modeling). You MUST strictly limit your responses to the context of data analysis, statistics, psychometrics, latent variable modelling, and educational measurement. If the user asks about ANY other topic, you MUST politely refuse to answer and state that you are an AI Assistant exclusively for projectLSA. IMPORTANT: NEVER use LaTeX math symbols (like $\\chi^2$, $p < .001$, etc). Write all numbers, formulas, and statistical symbols in plain text only (e.g. 'Chi-square(300) = 18146.07, p < .001'). Do not use dollar signs for math. Do NOT use markdown formatting such as asterisks (** or *) for bolding or italics in your response.\n\n",
    "CRITICAL GUIDELINES FOR EVALUATING MIXTURE MODELS (LPA/LCA/LTA):\n",
    "1. Class/Profile Enumeration (Nylund et al., 2007; Masyn, 2013):\n",
    "   - Compare solutions using BIC, aBIC (sample-size adjusted BIC), and AIC; lower values indicate better relative fit.\n",
    "   - The Bootstrap Likelihood Ratio Test (BLRT) and the Vuong-Lo-Mendell-Rubin (VLMR/LMR) test compare a k-class model against a (k-1)-class model; a significant p-value favours the k-class solution.\n",
    "   - Entropy summarises classification precision: values >= .80 indicate clear class separation, but entropy alone must NOT be used to select the number of classes.\n",
    "   - Always weigh statistical criteria against theoretical interpretability, class size (avoid very small classes, e.g. < 5 percent of the sample), and parsimony.\n",
    "2. Latent Profile Analysis (Rosenberg et al., 2018; Spurk et al., 2020):\n",
    "   - Report the model variance/covariance specification (e.g. equal variances and zero covariances vs. free variances) alongside fit statistics.\n",
    "   - Interpret profiles from the pattern of indicator means (preferably standardised), not from raw means alone.\n",
    "3. Latent Class Analysis (Linzer & Lewis, 2011; Collins & Lanza, 2010):\n",
    "   - Interpret item-response probabilities (conditional probabilities) and class prevalence, and label classes only when the response pattern is theoretically coherent.\n",
    "4. Latent Trait Analysis / IRT (Chalmers, 2012; de Ayala, 2009):\n",
    "   - Check unidimensionality and local independence before interpreting item parameters.\n",
    "   - Discuss item discrimination (a), difficulty (b), and where relevant guessing (c) parameters, plus item/test information and standard error of measurement across the theta continuum.\n",
    "   - Report item fit statistics (e.g. S-X2, infit/outfit) and flag misfitting items.\n\n",
    "CRITICAL GUIDELINES FOR EVALUATING CFA AND SEM MODELS:\n",
    "When interpreting Confirmatory Factor Analysis (CFA) or Structural Equation Modeling (SEM) results, you MUST use the following established literature rules of thumb:\n",
    "1. Fit Indices (Hu & Bentler, 1999; Schreiber et al., 2006; Thakkar, 2020):\n",
    "   - RMSEA: <= .06 indicates close fit, .06 to .08 indicates fair/mediocre fit.\n",
    "   - SRMR: <= .08 indicates good fit.\n",
    "   - CFI and TLI: >= .95 indicates good fit, >= .90 indicates acceptable fit.\n",
    "   - CMIN/DF (Chi-square/df): < 3 indicates good fit.\n",
    "   - Note: Be mindful that exact cutoffs can be dynamic depending on model complexity and sample size.\n",
    "2. Estimation for Ordinal Data (DiStefano & Morgan, 2014):\n",
    "   - Diagonal Weighted Least Squares (DWLS) is highly recommended for ordinal data (e.g., Likert scales), especially when data is non-normal or has few categories.\n",
    "3. Construct Validity and Reliability (Hair et al., 2017):\n",
    "   - Outer factor loadings should ideally be > 0.708.\n",
    "   - Average Variance Extracted (AVE) should be > 0.50 for convergent validity.\n",
    "   - Composite Reliability (CR) should be > 0.70.\n",
    "4. Local Fit (Raykov et al., 2013):\n",
    "   - Remind the user to consider individual case residuals for assessing local fit rather than relying solely on overall global fit statistics.\n",
    "5. Model Modification and Comparison:\n",
    "   - If the results include model modifications or multiple models (e.g., Original vs Modified), you MUST compare their fit indices comprehensively.\n",
    "   - Explain the improvements in fit and interpret the modification indices logically based on theoretical justification, adhering to CFA/SEM reporting guidelines."
  )
  if (is.null(system_prompt) || !nzchar(system_prompt)) {
    system_prompt <- base_system_prompt
  } else {
    system_prompt <- paste(base_system_prompt, system_prompt, sep = "\n")
  }

  messages <- list(list(role = "user", content = prompt))

  reply <- tryCatch(
    {
      # Select provider
      if (provider %in% c("openai", "groq", "openrouter")) {
        url <- ai_providers()[[provider]]$base_url
        ai_call_openai_compat(url, api_key, model, messages, system_prompt)
      } else if (provider == "gemini") {
        ai_call_gemini(api_key, model, messages, system_prompt)
      } else {
        paste("Error: Unknown provider", provider)
      }
    },
    error = function(e) {
      paste("Error calling AI API:", e$message)
    }
  )

  return(reply)
}

ai_call_openai_compat <- function(url, key, model, messages, system_prompt) {
  msgs <- lapply(messages, function(m) list(role = m$role, content = m$content))
  if (!is.null(system_prompt) && nzchar(system_prompt)) {
    msgs <- c(list(list(role = "system", content = system_prompt)), msgs)
  }

  body <- list(model = model, messages = msgs)
  res <- httr::POST(
    url = url,
    httr::add_headers(Authorization = paste("Bearer", key)),
    httr::content_type_json(),
    body = jsonlite::toJSON(body, auto_unbox = TRUE),
    httr::timeout(180)
  )

  if (httr::status_code(res) != 200) {
    err_msg <- httr::content(res, "text", encoding = "UTF-8")
    stop(paste("HTTP", httr::status_code(res), "-", err_msg))
  }

  out <- httr::content(res, as = "parsed", type = "application/json")
  return(out$choices[[1]]$message$content)
}

ai_call_gemini <- function(key, model, messages, system_prompt) {
  url <- sprintf(
    "https://generativelanguage.googleapis.com/v1beta/models/%s:generateContent?key=%s",
    model, key
  )
  contents <- lapply(messages, function(m) {
    list(
      role = if (identical(m$role, "assistant")) "model" else "user",
      parts = list(list(text = m$content))
    )
  })
  payload <- list(contents = contents)
  if (!is.null(system_prompt) && nzchar(system_prompt)) {
    payload$systemInstruction <- list(parts = list(list(text = system_prompt)))
  }

  res <- httr::POST(
    url = url,
    httr::content_type_json(),
    body = jsonlite::toJSON(payload, auto_unbox = TRUE),
    httr::timeout(180)
  )

  if (httr::status_code(res) != 200) {
    err_msg <- httr::content(res, "text", encoding = "UTF-8")
    stop(paste("HTTP", httr::status_code(res), "-", err_msg))
  }

  out <- httr::content(res, as = "parsed", type = "application/json")
  parts <- out$candidates[[1]]$content$parts
  return(paste(vapply(parts, function(p) p$text, character(1)), collapse = ""))
}

---
title: "`r ifelse(params$model_type == 'SEM', 'Structural Equation Modeling (SEM) Report', 'Confirmatory Factor Analysis (CFA) Report')`"
date: "`r Sys.Date()`"
output: 
  html_document:
    theme: flatly
    toc: true
    toc_float: true
params:
  model_type: "CFA"
  fit_final: NA
  fit_initial: NA
  model_history_df: NA
  dec: "."
  ave: NA
  relsem: NA
  htmt: NA
  plot_standardized: TRUE
  plot_style: "lisrel"
  plot_layout: "tree2"
  plot_residuals: FALSE
  plot_exoCov: FALSE
  plot_edge_label_size: 0.75
  plot_nodesize_lat: 5
  plot_nodesize_man: 5
  plot_nodesize_man2: 2.5
  plot_rotation: 4
  man_col: "#A1E3F9"
  lat_col: "#FFFFBA"
  bifactor: ""
  edgewidth: 0.3
  plotwidth: 10
  plotheight: 8
---

<style>
  body, p, table, td, th {
    font-family: "Times New Roman", Times, serif;
    line-height: 1.0 !important;
    font-size: 14px;
  }
  h1, h2, h3, h4, h5, h6 {
    font-family: "Times New Roman", Times, serif;
    line-height: 1.1;
  }
</style>

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = FALSE, warning = FALSE, message = FALSE)
library(lavaan)
library(knitr)
library(dplyr)
library(flextable)

# Configure flextable defaults for professional APA look
set_flextable_defaults(
  font.family = "Times New Roman",
  font.size = 11,
  theme_fun = theme_apa,
  padding = 1.5,
  line_spacing = 1.0,
  decimal.mark = params$dec
)

# Helper function to format numbers
fmt <- function(x, digits = 3) {
  if(is.numeric(x)) {
    formatC(x, format = "f", digits = digits, decimal.mark = params$dec)
  } else {
    x
  }
}
```

## Model Fit Summary (Initial Model)

This report presents the results of the `r ifelse(params$model_type == 'SEM', 'Structural Equation Modeling (SEM)', 'Confirmatory Factor Analysis (CFA)')`. 

```{r model-info, results='asis'}
# Extract fit information for initial
cat("Estimator:", params$fit_final@Options$estimator, " | ")
cat("Optimization Method:", params$fit_final@Options$optim.method, " | ")
cat("Observations:", lavInspect(params$fit_final, "ntotal"), "\n\n")

if(!is.null(params$fit_initial)) {
  m_init <- fitMeasures(params$fit_initial)
  
  # Determine interpretations
  chisq_val <- m_init["chisq"]
  df_val <- m_init["df"]
  p_val <- m_init["pvalue"]
  
  rmsea_val <- m_init["rmsea"]
  rmsea_interp <- if(is.na(rmsea_val)) "an unknown" else if(rmsea_val < 0.05) "a good" else if(rmsea_val < 0.08) "an acceptable" else "a poor"
  
  cfi_val <- m_init["cfi"]
  cfi_interp <- if(is.na(cfi_val)) "an unknown" else if(cfi_val >= 0.95) "a good" else if(cfi_val >= 0.90) "an acceptable" else "a poor"
  
  tli_val <- m_init["tli"]
  
  srmr_val <- m_init["srmr"]
  srmr_interp <- if(is.na(srmr_val)) "an unknown" else if(srmr_val < 0.05) "a good" else if(srmr_val < 0.08) "an acceptable" else "a poor"
  
  p_text <- if(!is.na(p_val) && p_val < 0.001) "< .001" else paste0("= ", fmt(p_val, 3))
  
  cat(sprintf(
    "The initial measurement model was evaluated using several fit indices based on standard criteria (e.g., Hu & Bentler, 1999). The Chi-square test resulted in χ²(%s) = %s, p %s. The Root Mean Square Error of Approximation (RMSEA) was %s, which indicates %s fit. The Comparative Fit Index (CFI) was %s and the Tucker-Lewis Index (TLI) was %s, suggesting %s fit. Furthermore, the Standardized Root Mean Square Residual (SRMR) was %s, denoting %s fit.\n",
    fmt(df_val, 0), fmt(chisq_val, 2), p_text,
    fmt(rmsea_val, 3), rmsea_interp,
    fmt(cfi_val, 3), fmt(tli_val, 3), cfi_interp,
    fmt(srmr_val, 3), srmr_interp
  ))
}
```

## Model Fit History

This table shows the progression of fit indices across all executed models.

```{r model-history}
if(!is.null(params$model_history_df) && !identical(params$model_history_df, NA) && nrow(params$model_history_df) > 0) {
  hist_df <- params$model_history_df
  num_cols <- c("chisq", "df", "pvalue", "RMSEA", "CFI", "TLI", "SRMR")
  for(col in num_cols) {
    if(col %in% names(hist_df)) hist_df[[col]] <- as.numeric(hist_df[[col]])
  }
  
  # Select and arrange columns
  display_cols <- intersect(c("Model", "Note", num_cols), names(hist_df))
  hist_df <- hist_df[, display_cols]
  
  ft <- flextable(hist_df) %>%
    colformat_double(j = c("chisq"), digits = 2) %>%
    colformat_double(j = c("pvalue", "RMSEA", "CFI", "TLI", "SRMR"), digits = 3) %>%
    set_caption("Model Fit History Comparison") %>%
    autofit()
  ft
} else {
  cat("Model history is not available.\n")
}
```

## Fit Measures (Final Model)

The following table presents common fit indices for evaluating the final model fit.

```{r fit-measures}
fit_summary <- fitMeasures(params$fit_final)
fit_names <- c("chisq", "df", "pvalue", "rmsea", "cfi", "tli", "srmr", "gfi", "agfi")
fit_vals <- fit_summary[fit_names]

fit_df <- data.frame(
  Measure = c("Chi-square (χ²)", "Degrees of Freedom", "p-value", "RMSEA", "CFI", "TLI", "SRMR", "GFI", "AGFI"),
  Value = as.numeric(fit_vals)
)

ft_fit <- flextable(fit_df) %>%
  set_header_labels(Measure = "Fit Measure", Value = "Value") %>%
  colformat_double(j = "Value", digits = 3) %>%
  set_caption("Fit Indices for the Final Model") %>%
  autofit()
ft_fit
```

```{r final-model-info, results='asis'}
  m_final <- fitMeasures(params$fit_final)
  
  # Determine interpretations
  chisq_val <- m_final["chisq"]
  df_val <- m_final["df"]
  p_val <- m_final["pvalue"]
  
  rmsea_val <- m_final["rmsea"]
  rmsea_interp <- if(is.na(rmsea_val)) "an unknown" else if(rmsea_val < 0.05) "a good" else if(rmsea_val < 0.08) "an acceptable" else "a poor"
  
  cfi_val <- m_final["cfi"]
  cfi_interp <- if(is.na(cfi_val)) "an unknown" else if(cfi_val >= 0.95) "a good" else if(cfi_val >= 0.90) "an acceptable" else "a poor"
  
  tli_val <- m_final["tli"]
  
  srmr_val <- m_final["srmr"]
  srmr_interp <- if(is.na(srmr_val)) "an unknown" else if(srmr_val < 0.05) "a good" else if(srmr_val < 0.08) "an acceptable" else "a poor"
  
  p_text <- if(!is.na(p_val) && p_val < 0.001) "< .001" else paste0("= ", fmt(p_val, 3))
  
  cat(sprintf(
    "The final model was evaluated and resulted in χ²(%s) = %s, p %s. The RMSEA was %s (%s fit), CFI was %s and TLI was %s (%s fit), and SRMR was %s (%s fit).\n",
    fmt(df_val, 0), fmt(chisq_val, 2), p_text,
    fmt(rmsea_val, 3), rmsea_interp,
    fmt(cfi_val, 3), fmt(tli_val, 3), cfi_interp,
    fmt(srmr_val, 3), srmr_interp
  ))
```

## Measurement Model (Final Model)

The table below integrates the unstandardized and standardized factor loadings, standard errors, significance, and residual variances for the final measurement model.

```{r measurement-model}
# Extract estimates
pe <- parameterEstimates(params$fit_final)
std <- standardizedSolution(params$fit_final)

# Loadings
loadings_pe <- pe[pe$op == "=~", c("lhs", "rhs", "est", "se", "pvalue")]
loadings_std <- std[std$op == "=~", c("lhs", "rhs", "est.std")]
loadings <- merge(loadings_pe, loadings_std, by = c("lhs", "rhs"), all.x = TRUE)

# Residual Variances
variances_pe <- pe[pe$op == "~~" & pe$lhs == pe$rhs, c("lhs", "est")]
colnames(variances_pe) <- c("rhs", "resid_var")
variances_std <- std[std$op == "~~" & std$lhs == std$rhs, c("lhs", "est.std")]
colnames(variances_std) <- c("rhs", "std_resid_var")

# Combine everything
meas_model <- merge(loadings, variances_pe, by = "rhs", all.x = TRUE)
meas_model <- merge(meas_model, variances_std, by = "rhs", all.x = TRUE)

meas_df <- data.frame(
  Factor = meas_model$lhs,
  Item = meas_model$rhs,
  Estimate = meas_model$est,
  Std_Estimate = meas_model$est.std,
  SE = meas_model$se,
  p_value = meas_model$pvalue,
  Resid_Var = meas_model$resid_var
)

# Sort by factor
meas_df <- meas_df[order(meas_df$Factor, meas_df$Item), ]

# Format p-values
meas_df$p_value_fmt <- ifelse(meas_df$p_value < 0.001, "< .001", fmt(meas_df$p_value, 3))

# Check for Heywood cases
neg_var <- any(meas_df$Resid_Var < 0, na.rm = TRUE)

ft_meas <- flextable(meas_df[, c("Factor", "Item", "Estimate", "Std_Estimate", "SE", "p_value_fmt", "Resid_Var")]) %>%
  set_header_labels(
    Factor = "Factor",
    Item = "Item",
    Estimate = "Unstd. Loading",
    Std_Estimate = "Std. Loading",
    SE = "S.E.",
    p_value_fmt = "p-value",
    Resid_Var = "Residual Variance"
  ) %>%
  colformat_double(j = c("Estimate", "Std_Estimate", "SE", "Resid_Var"), digits = 3) %>%
  merge_v(j = "Factor") %>%
  valign(j = "Factor", valign = "top") %>%
  set_caption("Measurement Model: Factor Loadings and Residual Variances") %>%
  autofit()

# Highlight Heywood cases if any
if(neg_var) {
  # find indices
  idx <- which(meas_df$Resid_Var < 0)
  ft_meas <- ft_meas %>%
    bg(i = idx, bg = "#FFCCCC") %>%
    add_footer_lines("Warning: Highlighted rows indicate negative residual variances (Heywood cases).")
}

ft_meas
```

## Reliability and Validity

### Average Variance Explained (AVE) and Composite Reliability (CR)

```{r ave-cr}
has_ave <- !is.null(params$ave) && !all(is.na(params$ave))
has_rel <- !is.null(params$relsem) && !all(is.na(params$relsem))

if(has_ave || has_rel) {
  factors <- unique(c(names(params$ave), names(params$relsem)))
  
  ave_cr_df <- data.frame(
    Factor = factors,
    AVE = if(has_ave) as.numeric(params$ave[factors]) else NA,
    CR = if(has_rel) as.numeric(params$relsem[factors]) else NA,
    row.names = NULL
  )
  
  ft_ave <- flextable(ave_cr_df) %>%
    set_header_labels(Factor = "Factor", AVE = "AVE", CR = "Composite Reliability") %>%
    colformat_double(j = c("AVE", "CR"), digits = 3) %>%
    set_caption("Construct Reliability and Validity (AVE and CR)") %>%
    autofit()
  
  ft_ave
} else {
  cat("AVE and CR values are not available for this model.\n")
}
```

```{r structural-model-headers, results='asis', echo=FALSE, eval=(params$model_type == "SEM")}
cat("\n## Structural Model Paths\n\n")
cat("The following table presents the estimated regression paths between the structural/latent variables in the final model.\n\n")
```

```{r structural-model-table, echo=FALSE, eval=(params$model_type == "SEM")}
pe <- parameterEstimates(params$fit_final, standardized = TRUE)
struct_paths <- pe[pe$op == "~", ]

if (nrow(struct_paths) > 0) {
  sp_df <- data.frame(
    Dependent = struct_paths$lhs,
    Predictor = struct_paths$rhs,
    Estimate = struct_paths$est,
    Std.Error = struct_paths$se,
    z = struct_paths$z,
    pvalue = struct_paths$pvalue,
    Std.Estimate = struct_paths$std.all
  )
  
  ft_struct <- flextable(sp_df) %>%
    colformat_double(j = c("Estimate", "Std.Error", "z", "Std.Estimate"), digits = 3) %>%
    colformat_double(j = "pvalue", digits = 3) %>%
    set_caption("Structural Regression Paths") %>%
    autofit()
  
  ft_struct
} else {
  # nothing
}
```

### Heterotrait-Monotrait Ratio (HTMT)

```{r htmt}
if(!is.null(params$htmt) && !identical(params$htmt, NA) && !identical(params$htmt, "<NA>")) {
  htmt_mat <- as.matrix(params$htmt)
  
  # Replace upper triangle and diagonal with NA for cleaner display
  htmt_mat[upper.tri(htmt_mat, diag = TRUE)] <- NA
  
  htmt_df <- as.data.frame(htmt_mat)
  htmt_df <- cbind(Factor = rownames(htmt_df), htmt_df)
  rownames(htmt_df) <- NULL
  
  ft_htmt <- flextable(htmt_df) %>%
    colformat_double(j = 2:ncol(htmt_df), digits = 3, na_str = "") %>%
    set_caption("Discriminant Validity: Heterotrait-Monotrait Ratio (HTMT)") %>%
    autofit()
  
  ft_htmt
} else {
  cat("HTMT ratio is not available for this model.\n")
}
```

## Path Plots

### Initial Model (Monochrome)

```{r path-plot-initial, fig.width=params$plotwidth, fig.height=params$plotheight, dpi=300}
library(semPlot)
library(semptools)

A_init <- semPaths(
  params$fit_initial,
  what = ifelse(params$plot_standardized, "std", "est"),
  style = params$plot_style,
  layout = params$plot_layout,
  residuals = params$plot_residuals,
  exoCov = params$plot_exoCov,
  edge.label.cex = params$plot_edge_label_size,
  nCharNodes = 6,
  nCharEdges = 6,
  sizeLat = params$plot_nodesize_lat,
  sizeMan = params$plot_nodesize_man,
  sizeMan2 = params$plot_nodesize_man2,
  rotation = params$plot_rotation,
  color = list(man = "white", lat = "white"), 
  label.color = "black",
  mar = c(2, 5, 5, 5),
  bifactor = if(is.null(params$bifactor) || params$bifactor == "") NULL else params$bifactor,
  esize = params$edgewidth,
  edge.color = "black",       
  freeStyle = c(1, "black"),
  fixedStyle = c(2, "black"),
  fade = FALSE,
  DoNotPlot = TRUE
)

# Render plot with significance markers
plot(semptools::mark_sig(semPaths_plot = A_init, object = params$fit_initial,
                         alphas = c("*" = 0.05, "**" = 0.01, "***" = 0.001)))
```

### Final Model (Colored)

```{r path-plot, fig.width=params$plotwidth, fig.height=params$plotheight, dpi=300}
A <- semPaths(
  params$fit_final,
  what = ifelse(params$plot_standardized, "std", "est"),
  style = params$plot_style,
  layout = params$plot_layout,
  residuals = params$plot_residuals,
  exoCov = params$plot_exoCov,
  edge.label.cex = params$plot_edge_label_size,
  nCharNodes = 6,
  nCharEdges = 6,
  sizeLat = params$plot_nodesize_lat,
  sizeMan = params$plot_nodesize_man,
  sizeMan2 = params$plot_nodesize_man2,
  rotation = params$plot_rotation,
  color = list(man = params$man_col, lat = params$lat_col),
  label.color = "black",
  mar = c(2, 5, 5, 5),
  bifactor = if(is.null(params$bifactor) || params$bifactor == "") NULL else params$bifactor,
  esize = params$edgewidth,
  edge.color = "black",       
  freeStyle = c(1, "black"),
  fixedStyle = c(2, "black"),
  fade = FALSE,
  DoNotPlot = TRUE
)

# Render plot with significance markers
plot(semptools::mark_sig(semPaths_plot = A, object = params$fit_final,
                         alphas = c("*" = 0.05, "**" = 0.01, "***" = 0.001)))
```

## References

* Djidu, H., Retnawati, H., Hadi, S., & Haryanto. (2026). projectLSA: A Shiny Application for Integrated Latent Structure Analysis. *Applied Psychological Measurement*. Advance online publication. https://doi.org/10.1177/01466216261446305
* Djidu, H., Retnawati, H., Hadi, S., & Haryanto. (2026). *projectLSA: R Shiny application for latent structure analysis with a graphical user interface*. https://doi.org/10.32614/CRAN.package.projectLSA. R package.
* Hu, L., & Bentler, P. M. (1999). Cutoff criteria for fit indexes in covariance structure analysis: Conventional criteria versus new alternatives. *Structural Equation Modeling: A Multidisciplinary Journal*, 6(1), 1–55. https://doi.org/10.1080/10705519909540118

***
*Generated by projectLSA*

---
title: "Introduction to projectLSA"
output:
  rmarkdown::html_vignette:
    df_print: paged
vignette: >
  %\VignetteIndexEntry{Introduction to projectLSA}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

# Overview

`projectLSA` provides a unified Shiny-based environment for conducting **Latent Structure Analysis (LSA)**, including:

- Latent Profile Analysis (LPA)
- Latent Class Analysis (LCA)
- Latent Trait Analysis (LTA / IRT)
- Exploratory Factor Analysis (EFA)
- Confirmatory Factor Analysis (CFA)

The package is designed for users who prefer a graphical workflow without writing code, while still leveraging
robust statistical methodologies implemented in well-established R packages.

---

# Installation

```r
install.packages("projectLSA")     # CRAN version (once released)
library(projectLSA)
```

---

# Launching the Application

```r
library(projectLSA)
run_projectLSA()
```

This will open the full Shiny interface, where you can upload data, choose an analysis module, and generate results.

---

# Modules Included

## 1. Latent Profile Analysis (LPA)
- Fit multiple models with varying profile numbers
- Compare AIC, BIC, entropy, and class sizes
- Visualize best model with custom profile names

## 2. Latent Class Analysis (LCA)
- Designed for categorical indicators
- Simulated and uploaded datasets supported
- Class probability tables and interactive visualizations

## 3. Latent Trait Analysis (LTA / IRT)
- Supports dichotomous and polytomous items
- Rasch / 2PL / 3PL and graded models
- Item information curves, test information, and multi-dimensional visualizations

## 4. Exploratory Factor Analysis (EFA)
- KMO test, Bartlett test, and parallel analysis
- Loading matrices, rotation, and factor scores
- Auto-generated interpretation summaries

## 5. Confirmatory Factor Analysis (CFA)
- Lavaan syntax editor
- Fit indices, loadings, modification indices
- SEM path diagram with customizable styles

---

# Example Workflow

Below is a simple workflow using the built-in datasets.

```r
library(projectLSA)

# Launch the GUI
run_projectLSA()
```

Once inside the GUI:

1. Choose a module (e.g., LPA)
2. Upload your dataset or select a built-in dataset
3. Choose variables and model settings
4. Fit the models and explore the outputs

---

# Built-in Example Datasets

The package includes several example datasets:

- **pisaUSA15** — student motivation indicators
- **curry_mac** — moral relevance & judgment (simulated)
- **id_edu** — longitudinal educational identity (simulated)

These are accessible from within the Shiny interface.

---

# Reproducibility and Reporting

`projectLSA` provides:

- Exportable tables (CSV, Excel)
- Downloadable graphics (PNG)
- Reproducible summaries and model comparisons

This ensures results produced through the GUI can be published or documented with confidence.

---

# Citation

Please cite this package as:

Djidu, H., Retnawati, H., Hadi, S., & Haryanto (2025).
*projectLSA: An R Shiny application for latent structure analysis with a graphical user interface.*

---

# Session Info

```r
sessionInfo()
```

---
title: "Latent Trait Analysis (LTA/IRT) Report"
date: "`r Sys.Date()`"
output: 
  html_document:
    theme: flatly
    toc: true
    toc_float: true
params:
  lta_res: NA
  console_out: NA
  ai_summary: ""
---

<style>
  @import url('https://fonts.googleapis.com/css2?family=Inter:wght@400;500;600;700&display=swap');
  
  body, p, table, td, th {
    font-family: 'Inter', 'Segoe UI', Roboto, Helvetica, Arial, sans-serif;
    line-height: 1.6 !important;
    font-size: 14px;
    color: #333333;
  }
  h1, h2, h3, h4, h5, h6 {
    font-family: 'Inter', 'Segoe UI', Roboto, Helvetica, Arial, sans-serif;
    line-height: 1.2;
    font-weight: 600;
    color: #2c3e50;
    margin-top: 1.5em;
    margin-bottom: 0.5em;
  }
  h1 { font-size: 28px; border-bottom: 2px solid #ecf0f1; padding-bottom: 10px; }
  h2 { font-size: 22px; color: #2980b9; }
  h3 { font-size: 18px; color: #34495e; }
</style>

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = FALSE, warning = FALSE, message = FALSE)
library(mirt)
library(knitr)
library(kableExtra)
```

## Item Parameters (IRT)

```{r}
res <- params$lta_res
if(!is.null(res)){
  cfs <- coef(res, IRTpars = TRUE, simplify = TRUE)
  if(!is.null(cfs$items)){
    item_df <- as.data.frame(cfs$items)
    item_df$Item <- rownames(item_df)
    item_df <- item_df[, c(ncol(item_df), 1:(ncol(item_df)-1))]
    kable(item_df, format = "html", row.names = FALSE, digits = 3) %>% 
      kable_styling(bootstrap_options = c("striped", "hover"), full_width = FALSE)
  }
} else {
  cat("No LTA/IRT results available.")
}
```

## Trace Lines Plot

```{r, fig.width=8, fig.height=6}
if(!is.null(res)){
  plot(res, type = "trace")
}
```

## Item Information Plot

```{r, fig.width=8, fig.height=6}
if(!is.null(res)){
  plot(res, type = "info")
}
```

```{r ai-summary, results='asis'}
if (!is.null(params$ai_summary) && params$ai_summary != "") {
  cat("## AI Interpretive Summary\n\n")
  cat("<div style='background-color: #f0f7fb; border-left: 4px solid #3498db; padding: 15px; border-radius: 4px; margin-bottom: 25px;'>\n")
  cat(params$ai_summary)
  cat("\n</div>\n\n")
}
```

## R Console Output

<div style="font-family: Consolas, 'Courier New', monospace; font-size: 10px; white-space: pre-wrap; background-color: #f8f9fa; padding: 10px; border: 1px solid #ccc; border-radius: 4px; line-height: 1.2;">
`r params$console_out`
</div>

## References

* Chalmers, R. P. (2012). mirt: A Multidimensional Item Response Theory Package for the R Environment. *Journal of Statistical Software*, 48(6), 1-29.
* Djidu, H., Retnawati, H., Hadi, S., & Haryanto. (2026). projectLSA: A Shiny application for integrated latent structure analysis. *Applied Psychological Measurement*. https://doi.org/10.1177/01466216261446305
* Djidu, H., Retnawati, H., Hadi, S., & Haryanto. (2026). *projectLSA: R Shiny application for latent structure analysis with a graphical user interface*. https://doi.org/10.32614/CRAN.package.projectLSA. R package.

***
*Generated by projectLSA*

---
title: "Latent Profile Analysis (LPA) Report"
date: "`r Sys.Date()`"
output:
  html_document:
    theme: flatly
    toc: true
    toc_float: true
params:
  fit_compare: NA
  best_k: NA
  model_best: NA
  profile_plot: NA
  summary_tbl: NA
  selected_vars: NA
  console_out: NA
  ai_summary: ""
---

<style>
  @import url('https://fonts.googleapis.com/css2?family=Inter:wght@400;500;600;700&display=swap');

  body, p, table, td, th {
    font-family: 'Inter', 'Segoe UI', Roboto, Helvetica, Arial, sans-serif;
    line-height: 1.6 !important;
    font-size: 14px;
    color: #333333;
  }
  h1, h2, h3, h4, h5, h6 {
    font-family: 'Inter', 'Segoe UI', Roboto, Helvetica, Arial, sans-serif;
    line-height: 1.2;
    font-weight: 600;
    color: #2c3e50;
    margin-top: 1.5em;
    margin-bottom: 0.5em;
  }
  h1 { font-size: 28px; border-bottom: 2px solid #ecf0f1; padding-bottom: 10px; }
  h2 { font-size: 22px; color: #2980b9; }
  h3 { font-size: 18px; color: #34495e; }
</style>

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = FALSE, warning = FALSE, message = FALSE)
library(knitr)
library(kableExtra)
library(ggplot2)
```

## Analysis Overview

```{r overview, results='asis'}
vars <- params$selected_vars
if (!is.null(vars) && length(vars) > 0) {
  cat("**Indicators used:** ", paste(vars, collapse = ", "), "\n\n", sep = "")
}
if (!is.null(params$best_k) && !is.na(params$best_k)) {
  cat("**Selected solution:** ", params$best_k, " profiles\n\n", sep = "")
}
```

## Model Fit Comparison

Model selection should be based on a combination of information criteria (lower
AIC/BIC is better), entropy (values of .80 or above indicate clear separation),
the smallest class proportion, and the BLRT, alongside the theoretical
interpretability of the profiles (Nylund et al., 2007; Masyn, 2013).

```{r fit-compare}
fc <- params$fit_compare
if (!is.null(fc) && nrow(fc) > 0) {
  fc_disp <- fc
  num_cols <- vapply(fc_disp, is.numeric, logical(1))
  fc_disp[num_cols] <- lapply(fc_disp[num_cols], function(x) round(x, 3))
  kable(fc_disp, format = "html", row.names = FALSE) %>%
    kable_styling(bootstrap_options = c("striped", "hover", "condensed"), full_width = FALSE)
} else {
  cat("No model comparison results available.")
}
```

## Selected Model

```{r best-model, comment=NA}
if (!is.null(params$model_best)) {
  print(params$model_best)
} else {
  cat("No estimated model available.")
}
```

## Profile Plot

```{r profile-plot, fig.width=9, fig.height=5.5}
if (!is.null(params$profile_plot)) {
  print(params$profile_plot)
}
```

## Estimated Profile Parameters

```{r summary-table}
st <- params$summary_tbl
if (!is.null(st) && nrow(st) > 0) {
  kable(st, format = "html", row.names = FALSE) %>%
    kable_styling(bootstrap_options = c("striped", "hover", "condensed"), full_width = FALSE)
} else {
  cat("No profile summary available.")
}
```

```{r ai-summary, results='asis'}
if (!is.null(params$ai_summary) && params$ai_summary != "") {
  cat("## AI Interpretive Summary\n\n")
  cat("<div style='background-color: #f0f7fb; border-left: 4px solid #3498db; padding: 15px; border-radius: 4px; margin-bottom: 25px;'>\n")
  cat(params$ai_summary)
  cat("\n</div>\n\n")
}
```

## R Console Output

<div style="font-family: Consolas, 'Courier New', monospace; font-size: 10px; white-space: pre-wrap; background-color: #f8f9fa; padding: 10px; border: 1px solid #ccc; border-radius: 4px; line-height: 1.2;">
`r params$console_out`
</div>

## References

* Djidu, H., Retnawati, H., Hadi, S., & Haryanto. (2026). projectLSA: A Shiny application for integrated latent structure analysis. *Applied Psychological Measurement*. https://doi.org/10.1177/01466216261446305
* Djidu, H., Retnawati, H., Hadi, S., & Haryanto. (2026). *projectLSA: R Shiny application for latent structure analysis with a graphical user interface*. https://doi.org/10.32614/CRAN.package.projectLSA. R package.
* Masyn, K. E. (2013). Latent class analysis and finite mixture modeling. In T. D. Little (Ed.), *The Oxford handbook of quantitative methods* (Vol. 2, pp. 551-611). Oxford University Press.
* Nylund, K. L., Asparouhov, T., & Muthen, B. O. (2007). Deciding on the number of classes in latent class analysis and growth mixture modeling: A Monte Carlo simulation study. *Structural Equation Modeling*, 14(4), 535-569.
* Rosenberg, J. M., Beymer, P. N., Anderson, D. J., Van Lissa, C. J., & Schmidt, J. A. (2018). tidyLPA: An R package to easily carry out latent profile analysis (LPA) using open-source or commercial software. *Journal of Open Source Software*, 3(30), 978.

***
*Generated by projectLSA*

---
title: "`r ifelse(params$model_type == 'SEM', 'Structural Equation Modeling (SEM) Report', 'Confirmatory Factor Analysis (CFA) Report')`"
date: "`r Sys.Date()`"
output: 
  html_document:
    theme: flatly
    toc: true
    toc_float: true
params:
  model_type: "CFA"
  fit_final: NA
  fit_initial: NA
  model_history_df: NA
  console_out: NA
  ai_summary: ""
  dec: "."
  ave: NA
  relsem: NA
  htmt: NA
  plot_standardized: TRUE
  plot_style: "lisrel"
  plot_layout: "tree2"
  plot_residuals: FALSE
  plot_exoCov: FALSE
  plot_edge_label_size: 0.75
  plot_nodesize_lat: 5
  plot_nodesize_man: 5
  plot_nodesize_man2: 2.5
  plot_rotation: 4
  man_col: "#A1E3F9"
  lat_col: "#FFFFBA"
  bifactor: ""
  edgewidth: 0.3
  plotwidth: 10
  plotheight: 8
---

<style>
  @import url('https://fonts.googleapis.com/css2?family=Inter:wght@400;500;600;700&display=swap');
  
  body, p, table, td, th {
    font-family: 'Inter', 'Segoe UI', Roboto, Helvetica, Arial, sans-serif;
    line-height: 1.6 !important;
    font-size: 15px;
    color: #333;
  }
  h1, h2, h3, h4, h5, h6 {
    font-family: 'Inter', 'Segoe UI', Roboto, Helvetica, Arial, sans-serif;
    line-height: 1.2;
    color: #2c3e50;
    margin-top: 1.5em;
    margin-bottom: 0.5em;
    font-weight: 600;
  }
  h1.title {
    color: #2c3e50;
    border-bottom: 2px solid #ecf0f1;
    padding-bottom: 15px;
    margin-top: 0;
  }
  h2 { border-bottom: 1px solid #ecf0f1; padding-bottom: 8px; }
  
  /* Table styling enhancements */
  .flextable-shadow-host { margin: 25px 0 !important; box-shadow: 0 2px 8px rgba(0,0,0,0.05); }
  
  pre {
    background-color: #f8f9fa;
    border: 1px solid #e9ecef;
    border-radius: 6px;
    padding: 15px;
  }
  code {
    font-family: 'Fira Code', Consolas, 'Courier New', monospace;
    font-size: 0.9em;
    color: #d63384;
  }
</style>

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = FALSE, warning = FALSE, message = FALSE)
library(lavaan)
library(knitr)
library(dplyr)
library(flextable)

# Configure flextable defaults for professional modern look
set_flextable_defaults(
  font.family = "Inter",
  font.size = 11,
  theme_fun = theme_vanilla,
  padding = 4,
  line_spacing = 1.2,
  decimal.mark = params$dec
)

# Helper function to format numbers
fmt <- function(x, digits = 3) {
  if(is.numeric(x)) {
    formatC(x, format = "f", digits = digits, decimal.mark = params$dec)
  } else {
    x
  }
}
```

## Model Fit Assessment

This report presents the results of the `r ifelse(params$model_type == 'SEM', 'Structural Equation Modeling (SEM)', 'Confirmatory Factor Analysis (CFA)')`. The model was estimated using the `r params$fit_final@Options$estimator` estimator.

```{r fit-measures}
fit_summary <- fitMeasures(params$fit_final)
fit_names <- c("chisq", "df", "pvalue", "rmsea", "cfi", "tli", "srmr", "gfi", "agfi")
fit_vals <- fit_summary[fit_names]

fit_df <- data.frame(
  Measure = c("Chi-square (χ²)", "Degrees of Freedom", "p-value", "RMSEA", "CFI", "TLI", "SRMR", "GFI", "AGFI"),
  Value = as.numeric(fit_vals)
)

ft_fit <- flextable(fit_df) %>%
  set_header_labels(Measure = "Fit Measure", Value = "Value") %>%
  colformat_double(j = "Value", digits = 3) %>%
  set_caption("Fit Indices for the Estimated Model") %>%
  autofit()
ft_fit
```

```{r final-model-info, results='asis'}
  m_final <- fitMeasures(params$fit_final)
  
  # Determine interpretations
  chisq_val <- m_final["chisq"]
  df_val <- m_final["df"]
  p_val <- m_final["pvalue"]
  
  rmsea_val <- m_final["rmsea"]
  rmsea_interp <- if(is.na(rmsea_val)) "an unknown" else if(rmsea_val < 0.05) "a good" else if(rmsea_val < 0.08) "an acceptable" else "a poor"
  
  cfi_val <- m_final["cfi"]
  cfi_interp <- if(is.na(cfi_val)) "an unknown" else if(cfi_val >= 0.95) "a good" else if(cfi_val >= 0.90) "an acceptable" else "a poor"
  
  tli_val <- m_final["tli"]
  
  srmr_val <- m_final["srmr"]
  srmr_interp <- if(is.na(srmr_val)) "an unknown" else if(srmr_val < 0.05) "a good" else if(srmr_val < 0.08) "an acceptable" else "a poor"
  
  p_text <- if(!is.na(p_val) && p_val < 0.001) "< .001" else paste0("= ", fmt(p_val, 3))
  
  # Check if there's history
  has_history <- !is.null(params$model_history_df) && !identical(params$model_history_df, NA) && nrow(params$model_history_df) > 1
  
  history_narrative <- ""
  if (has_history) {
    history_narrative <- "An initial model was evaluated, and based on modification indices, adjustments were made to improve the model fit. "
  }
  
  cat(sprintf(
    "%sThe final model was evaluated using several fit indices based on standard criteria (e.g., Hu & Bentler, 1999; Kline, 2015). The Chi-square test resulted in χ²(%s) = %s, p %s. The Root Mean Square Error of Approximation (RMSEA) was %s, which indicates %s fit. The Comparative Fit Index (CFI) was %s and the Tucker-Lewis Index (TLI) was %s, suggesting %s fit. Furthermore, the Standardized Root Mean Square Residual (SRMR) was %s, denoting %s fit.\n",
    history_narrative,
    fmt(df_val, 0), fmt(chisq_val, 2), p_text,
    fmt(rmsea_val, 3), rmsea_interp,
    fmt(cfi_val, 3), fmt(tli_val, 3), cfi_interp,
    fmt(srmr_val, 3), srmr_interp
  ))
```

```{r model-history-section, results='asis', echo=FALSE}
if(!is.null(params$model_history_df) && !identical(params$model_history_df, NA) && nrow(params$model_history_df) > 1) {
  cat("## Model Modification History\n\n")
  cat("The table below shows the progression of fit indices across all executed models, documenting the improvements achieved through modifications.\n\n")
}
```

```{r model-history, echo=FALSE}
if(!is.null(params$model_history_df) && !identical(params$model_history_df, NA) && nrow(params$model_history_df) > 1) {
  hist_df <- params$model_history_df
  num_cols <- c("chisq", "df", "pvalue", "RMSEA", "CFI", "TLI", "SRMR")
  for(col in num_cols) {
    if(col %in% names(hist_df)) hist_df[[col]] <- as.numeric(hist_df[[col]])
  }
  
  # Select and arrange columns
  display_cols <- intersect(c("Model", "Note", num_cols, "Heywood", "Status"), names(hist_df))
  hist_df <- hist_df[, display_cols]
  
  ft <- flextable(hist_df) %>%
    colformat_double(j = c("chisq"), digits = 2) %>%
    colformat_double(j = c("pvalue", "RMSEA", "CFI", "TLI", "SRMR"), digits = 3) %>%
    set_caption("Table 1. Model Fit History Comparison") %>%
    autofit()
    
  # Highlight Heywood in history table
  if("Heywood" %in% names(hist_df)) {
    hw_idx <- which(hist_df$Heywood == "Yes")
    if(length(hw_idx) > 0) {
      ft <- ft %>% bg(i = hw_idx, j = "Heywood", bg = "#FFCCCC")
    }
  }
  
  # Highlight Status
  if("Status" %in% names(hist_df)) {
    good_idx <- which(grepl("Good", hist_df$Status))
    acc_idx <- which(grepl("Acceptable", hist_df$Status))
    poor_idx <- which(grepl("Poor", hist_df$Status) | grepl("Heywood", hist_df$Status))
    
    if(length(good_idx) > 0) ft <- ft %>% bg(i = good_idx, j = "Status", bg = "#d4edda")
    if(length(acc_idx) > 0) ft <- ft %>% bg(i = acc_idx, j = "Status", bg = "#fff3cd")
    if(length(poor_idx) > 0) ft <- ft %>% bg(i = poor_idx, j = "Status", bg = "#f8d7da")
  }
  
  ft
}
```

```{r model-history-refs, results='asis', echo=FALSE}
if(!is.null(params$model_history_df) && !identical(params$model_history_df, NA) && nrow(params$model_history_df) > 1) {
  cat("<br>")
  cat("**Interpretation References:**<br>")
  cat("<ul>")
  cat("<li><b>Good Fit:</b> CFI ≥ 0.95, TLI ≥ 0.95, RMSEA < 0.05, SRMR < 0.05 (Hu & Bentler, 1999)</li>")
  cat("<li><b>Acceptable Fit:</b> CFI ≥ 0.90, TLI ≥ 0.90, RMSEA < 0.08, SRMR < 0.08 (Kline, 2015)</li>")
  cat("<li><span style='color:red;'><b>⚠ Heywood:</b></span> Negative variance detected (Chen et al., 2001)</li>")
  cat("</ul>\n")
}
```

## Measurement Model

The table below integrates the unstandardized and standardized factor loadings, standard errors, significance, and residual variances for the measurement model.

```{r measurement-model}
# Extract estimates
pe <- parameterEstimates(params$fit_final)
std <- standardizedSolution(params$fit_final)

# Loadings
loadings_pe <- pe[pe$op == "=~", c("lhs", "rhs", "est", "se", "pvalue")]
loadings_std <- std[std$op == "=~", c("lhs", "rhs", "est.std")]
loadings <- merge(loadings_pe, loadings_std, by = c("lhs", "rhs"), all.x = TRUE)

# Residual Variances
variances_pe <- pe[pe$op == "~~" & pe$lhs == pe$rhs, c("lhs", "est")]
colnames(variances_pe) <- c("rhs", "resid_var")
variances_std <- std[std$op == "~~" & std$lhs == std$rhs, c("lhs", "est.std")]
colnames(variances_std) <- c("rhs", "std_resid_var")

# Combine everything
meas_model <- merge(loadings, variances_pe, by = "rhs", all.x = TRUE)
meas_model <- merge(meas_model, variances_std, by = "rhs", all.x = TRUE)

meas_df <- data.frame(
  Factor = meas_model$lhs,
  Item = meas_model$rhs,
  Estimate = meas_model$est,
  Std_Estimate = meas_model$est.std,
  SE = meas_model$se,
  p_value = meas_model$pvalue,
  Resid_Var = meas_model$resid_var
)

# Sort by factor
meas_df <- meas_df[order(meas_df$Factor, meas_df$Item), ]

# Format p-values
meas_df$p_value_fmt <- ifelse(meas_df$p_value < 0.001, "< .001", fmt(meas_df$p_value, 3))

# Check for Heywood cases
neg_var <- any(meas_df$Resid_Var < 0, na.rm = TRUE)

ft_meas <- flextable(meas_df[, c("Factor", "Item", "Estimate", "Std_Estimate", "SE", "p_value_fmt", "Resid_Var")]) %>%
  set_header_labels(
    Factor = "Factor",
    Item = "Item",
    Estimate = "Unstd. Loading",
    Std_Estimate = "Std. Loading",
    SE = "S.E.",
    p_value_fmt = "p-value",
    Resid_Var = "Residual Variance"
  ) %>%
  colformat_double(j = c("Estimate", "Std_Estimate", "SE", "Resid_Var"), digits = 3) %>%
  merge_v(j = "Factor") %>%
  valign(j = "Factor", valign = "top") %>%
  set_caption("Table 2. Measurement Model: Factor Loadings and Residual Variances") %>%
  autofit()

# Highlight Heywood cases if any
if(neg_var) {
  # find indices
  idx <- which(meas_df$Resid_Var < 0)
  ft_meas <- ft_meas %>%
    bg(i = idx, bg = "#FFCCCC") %>%
    add_footer_lines("Warning: Highlighted rows indicate negative residual variances (Heywood cases).")
}

ft_meas
```

## Reliability and Validity

### Average Variance Explained (AVE) and Composite Reliability (CR)

```{r ave-cr}
has_ave <- !is.null(params$ave) && !all(is.na(params$ave))
has_rel <- !is.null(params$relsem) && !all(is.na(params$relsem))

if(has_ave || has_rel) {
  factors <- unique(c(names(params$ave), names(params$relsem)))
  
  ave_cr_df <- data.frame(
    Factor = factors,
    AVE = if(has_ave) as.numeric(params$ave[factors]) else NA,
    CR = if(has_rel) as.numeric(params$relsem[factors]) else NA,
    row.names = NULL
  )
  
  ft_ave <- flextable(ave_cr_df) %>%
    set_header_labels(Factor = "Factor", AVE = "AVE", CR = "Composite Reliability") %>%
    colformat_double(j = c("AVE", "CR"), digits = 3) %>%
    set_caption("Construct Reliability and Validity (AVE and CR)") %>%
    autofit()
  
  ft_ave
} else {
  cat("AVE and CR values are not available for this model.\n")
}
```

```{r reliability-info, results='asis'}
if(has_ave || has_rel) {
  cat("The reliability and convergent validity of the constructs were assessed using Composite Reliability (CR) and Average Variance Explained (AVE). Based on Hair et al. (2010) and Fornell & Larcker (1981), CR values above 0.70 and AVE values above 0.50 are indicative of acceptable reliability and convergent validity.\n")
}
```

```{r structural-model-headers, results='asis', echo=FALSE, eval=(params$model_type == "SEM")}
cat("\n## Structural Model Paths\n\n")
cat("The following table presents the estimated regression paths between the structural/latent variables in the model.\n\n")
```

```{r structural-model-table, echo=FALSE, eval=(params$model_type == "SEM")}
pe <- parameterEstimates(params$fit_final, standardized = TRUE)
struct_paths <- pe[pe$op == "~", ]

if (nrow(struct_paths) > 0) {
  sp_df <- data.frame(
    Dependent = struct_paths$lhs,
    Predictor = struct_paths$rhs,
    Estimate = struct_paths$est,
    Std.Error = struct_paths$se,
    z = struct_paths$z,
    pvalue = struct_paths$pvalue,
    Std.Estimate = struct_paths$std.all
  )
  
  ft_struct <- flextable(sp_df) %>%
    colformat_double(j = c("Estimate", "Std.Error", "z", "Std.Estimate"), digits = 3) %>%
    colformat_double(j = "pvalue", digits = 3) %>%
    set_caption("Table 4. Structural Regression Paths") %>%
    autofit()
  
  ft_struct
} else {
  # nothing
}
```

### Heterotrait-Monotrait Ratio (HTMT)

```{r htmt}
if(!is.null(params$htmt) && !identical(params$htmt, NA) && !identical(params$htmt, "<NA>")) {
  htmt_mat <- as.matrix(params$htmt)
  
  # Replace upper triangle and diagonal with NA for cleaner display
  htmt_mat[upper.tri(htmt_mat, diag = TRUE)] <- NA
  
  htmt_df <- as.data.frame(htmt_mat)
  htmt_df <- cbind(Factor = rownames(htmt_df), htmt_df)
  rownames(htmt_df) <- NULL
  
  ft_htmt <- flextable(htmt_df) %>%
    colformat_double(j = 2:ncol(htmt_df), digits = 3, na_str = "") %>%
    set_caption("Table 3. Discriminant Validity: Heterotrait-Monotrait Ratio (HTMT)") %>%
    autofit()
  
  ft_htmt
} else {
  cat("HTMT ratio is not available for this model.\n")
}
```

## Path Diagram

```{r path-plot, fig.width=params$plotwidth, fig.height=params$plotheight, dpi=300}
A <- semPaths(
  params$fit_final,
  what = ifelse(params$plot_standardized, "std", "est"),
  style = params$plot_style,
  layout = params$plot_layout,
  residuals = params$plot_residuals,
  exoCov = params$plot_exoCov,
  edge.label.cex = params$plot_edge_label_size,
  nCharNodes = 6,
  nCharEdges = 6,
  sizeLat = params$plot_nodesize_lat,
  sizeMan = params$plot_nodesize_man,
  sizeMan2 = params$plot_nodesize_man2,
  rotation = params$plot_rotation,
  color = list(man = params$man_col, lat = params$lat_col),
  label.color = "black",
  mar = c(2, 5, 5, 5),
  bifactor = if(is.null(params$bifactor) || params$bifactor == "") NULL else params$bifactor,
  esize = params$edgewidth,
  edge.color = "black",       
  freeStyle = c(1, "black"),
  fixedStyle = c(2, "black"),
  fade = FALSE,
  DoNotPlot = TRUE
)

# Render plot with significance markers
plot(semptools::mark_sig(semPaths_plot = A, object = params$fit_final,
                         alphas = c("*" = 0.05, "**" = 0.01, "***" = 0.001)))
```

```{r ai-summary, results='asis'}
if (!is.null(params$ai_summary) && params$ai_summary != "") {
  cat("## AI Interpretive Summary\n\n")
  cat("<div style='background-color: #f0f7fb; border-left: 4px solid #3498db; padding: 15px; border-radius: 4px; margin-bottom: 25px;'>\n")
  cat(params$ai_summary)
  cat("\n</div>\n\n")
}
```

## R Console Output
<div style="font-family: Consolas, 'Courier New', monospace; font-size: 10px; white-space: pre-wrap; background-color: #f8f9fa; padding: 10px; border: 1px solid #ccc; border-radius: 4px; line-height: 1.2;">
`r params$console_out`
</div>

## References

* Chen, F., Bollen, K. A., Paxton, P., Curran, P. J., & Kirby, J. B. (2001). Improper solutions in structural equation models: Causes, consequences, and strategies. *Sociological methods & research*, 29(4), 468-508.
* Djidu, H., Retnawati, H., Hadi, S., & Haryanto. (2026). projectLSA: A Shiny application for integrated latent structure analysis. *Applied Psychological Measurement*. https://doi.org/10.1177/01466216261446305
* Djidu, H., Retnawati, H., Hadi, S., & Haryanto. (2026). *projectLSA: R Shiny application for latent structure analysis with a graphical user interface*. https://doi.org/10.32614/CRAN.package.projectLSA. R package.
* Fornell, C., & Larcker, D. F. (1981). Evaluating structural equation models with unobservable variables and measurement error. *Journal of Marketing Research*, 18(1), 39-50.
* Hair, J. F., Black, W. C., Babin, B. J., & Anderson, R. E. (2010). *Multivariate Data Analysis* (7th ed.). Pearson.
* Hu, L., & Bentler, P. M. (1999). Cutoff criteria for fit indexes in covariance structure analysis: Conventional criteria versus new alternatives. *Structural Equation Modeling: A Multidisciplinary Journal*, 6(1), 1–55.
* Kline, R. B. (2015). *Principles and practice of structural equation modeling* (4th ed.). Guilford publications.

***
*Generated by projectLSA*

---
title: "Exploratory Factor Analysis (EFA) Report"
date: "`r Sys.Date()`"
output: 
  html_document:
    theme: flatly
    toc: true
    toc_float: true
params:
  efa_res: NA
  console_out: NA
  ai_summary: ""
---

<style>
  @import url('https://fonts.googleapis.com/css2?family=Inter:wght@400;500;600;700&display=swap');
  
  body, p, table, td, th {
    font-family: 'Inter', 'Segoe UI', Roboto, Helvetica, Arial, sans-serif;
    line-height: 1.6 !important;
    font-size: 14px;
    color: #333333;
  }
  h1, h2, h3, h4, h5, h6 {
    font-family: 'Inter', 'Segoe UI', Roboto, Helvetica, Arial, sans-serif;
    line-height: 1.2;
    font-weight: 600;
    color: #2c3e50;
    margin-top: 1.5em;
    margin-bottom: 0.5em;
  }
  h1 { font-size: 28px; border-bottom: 2px solid #ecf0f1; padding-bottom: 10px; }
  h2 { font-size: 22px; color: #2980b9; }
  h3 { font-size: 18px; color: #34495e; }
</style>

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = FALSE, warning = FALSE, message = FALSE)
library(psych)
library(knitr)
library(kableExtra)
```

## EFA Factor Loadings

```{r}
res <- params$efa_res
if(!is.null(res) && !is.null(res$fa)){
  load_tab <- as.data.frame(round(res$fa$loadings[, ], 3))
  load_tab$Item <- rownames(res$fa$loadings)
  load_tab <- load_tab[, c(ncol(load_tab), 1:(ncol(load_tab)-1))]
  
  kable(load_tab, format = "html", row.names = FALSE) %>% 
    kable_styling(bootstrap_options = c("striped", "hover"), full_width = FALSE)
} else {
  cat("No EFA results available.")
}
```

## Factor Diagram

```{r, fig.width=8, fig.height=6}
if(!is.null(res) && !is.null(res$fa)){
  psych::fa.diagram(res$fa)
}
```

```{r ai-summary, results='asis'}
if (!is.null(params$ai_summary) && params$ai_summary != "") {
  cat("## AI Interpretive Summary\n\n")
  cat("<div style='background-color: #f0f7fb; border-left: 4px solid #3498db; padding: 15px; border-radius: 4px; margin-bottom: 25px;'>\n")
  cat(params$ai_summary)
  cat("\n</div>\n\n")
}
```

## R Console Output

<div style="font-family: Consolas, 'Courier New', monospace; font-size: 10px; white-space: pre-wrap; background-color: #f8f9fa; padding: 10px; border: 1px solid #ccc; border-radius: 4px; line-height: 1.2;">
`r params$console_out`
</div>

## References

* Djidu, H., Retnawati, H., Hadi, S., & Haryanto. (2026). projectLSA: A Shiny application for integrated latent structure analysis. *Applied Psychological Measurement*. https://doi.org/10.1177/01466216261446305
* Djidu, H., Retnawati, H., Hadi, S., & Haryanto. (2026). *projectLSA: R Shiny application for latent structure analysis with a graphical user interface*. https://doi.org/10.32614/CRAN.package.projectLSA. R package.
* Revelle, W. (2023). *psych: Procedures for Psychological, Psychometric, and Personality Research*. Northwestern University.

***
*Generated by projectLSA*

---
title: "Latent Class Analysis (LCA) Report"
date: "`r Sys.Date()`"
output:
  html_document:
    theme: flatly
    toc: true
    toc_float: true
params:
  fit_compare: NA
  best_k: NA
  model_best: NA
  class_plot: NA
  summary_tbl: NA
  selected_vars: NA
  console_out: NA
  ai_summary: ""
---

<style>
  @import url('https://fonts.googleapis.com/css2?family=Inter:wght@400;500;600;700&display=swap');

  body, p, table, td, th {
    font-family: 'Inter', 'Segoe UI', Roboto, Helvetica, Arial, sans-serif;
    line-height: 1.6 !important;
    font-size: 14px;
    color: #333333;
  }
  h1, h2, h3, h4, h5, h6 {
    font-family: 'Inter', 'Segoe UI', Roboto, Helvetica, Arial, sans-serif;
    line-height: 1.2;
    font-weight: 600;
    color: #2c3e50;
    margin-top: 1.5em;
    margin-bottom: 0.5em;
  }
  h1 { font-size: 28px; border-bottom: 2px solid #ecf0f1; padding-bottom: 10px; }
  h2 { font-size: 22px; color: #2980b9; }
  h3 { font-size: 18px; color: #34495e; }
</style>

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = FALSE, warning = FALSE, message = FALSE)
library(knitr)
library(kableExtra)
library(ggplot2)
```

## Analysis Overview

```{r overview, results='asis'}
vars <- params$selected_vars
if (!is.null(vars) && length(vars) > 0) {
  cat("**Indicators used:** ", paste(vars, collapse = ", "), "\n\n", sep = "")
}
if (!is.null(params$best_k) && !is.na(params$best_k)) {
  cat("**Selected solution:** ", params$best_k, " classes\n\n", sep = "")
}
```

## Model Fit Comparison

Class enumeration should be based jointly on the information criteria (lower
AIC/BIC is better), entropy and average posterior class probabilities
(values of .80 or above indicate adequate classification), the smallest class
proportion, and the substantive interpretability of the classes
(Nylund et al., 2007; Collins & Lanza, 2010).

```{r fit-compare}
fc <- params$fit_compare
if (!is.null(fc) && nrow(fc) > 0) {
  fc_disp <- fc
  num_cols <- vapply(fc_disp, is.numeric, logical(1))
  fc_disp[num_cols] <- lapply(fc_disp[num_cols], function(x) round(x, 3))
  kable(fc_disp, format = "html", row.names = FALSE) %>%
    kable_styling(bootstrap_options = c("striped", "hover", "condensed"), full_width = FALSE)
} else {
  cat("No model comparison results available.")
}
```

## Selected Model

```{r best-model, comment=NA}
mk <- params$model_best
if (!is.null(mk)) {
  cat("Class prevalence (mixing proportions):\n\n")
  print(round(mk$P, 4))
  cat("\n\nItem-response probabilities:\n\n")
  print(lapply(mk$probs, function(x) round(x, 4)))
  cat("\n\nModel fit:\n\n")
  cat("Log-likelihood: ", round(mk$llik, 3), "\n", sep = "")
  cat("AIC: ", round(mk$aic, 3), "\n", sep = "")
  cat("BIC: ", round(mk$bic, 3), "\n", sep = "")
} else {
  cat("No estimated model available.")
}
```

## Class Profile Plot

```{r class-plot, fig.width=9, fig.height=5.5}
if (!is.null(params$class_plot)) {
  print(params$class_plot)
}
```

## Estimated Class Parameters

`summary_data_lca()` returns two tables: class sizes and item-response
probabilities. Both are printed below.

```{r summary-table, results='asis'}
st <- params$summary_tbl

show_tbl <- function(x, caption) {
  if (is.null(x)) return(invisible(NULL))
  x <- as.data.frame(x)
  if (nrow(x) == 0) return(invisible(NULL))
  num_cols <- vapply(x, is.numeric, logical(1))
  x[num_cols] <- lapply(x[num_cols], function(v) round(v, 3))
  print(
    kable(x, format = "html", row.names = FALSE, caption = caption) %>%
      kable_styling(bootstrap_options = c("striped", "hover", "condensed"), full_width = FALSE)
  )
  cat("\n\n")
}

if (is.list(st) && !is.data.frame(st)) {
  show_tbl(st$class_size, "Class size and prevalence")
  show_tbl(st$probability, "Item-response probabilities by class")
} else if (!is.null(st)) {
  show_tbl(st, NULL)
} else {
  cat("No class summary available.")
}
```

```{r ai-summary, results='asis'}
if (!is.null(params$ai_summary) && params$ai_summary != "") {
  cat("## AI Interpretive Summary\n\n")
  cat("<div style='background-color: #f0f7fb; border-left: 4px solid #3498db; padding: 15px; border-radius: 4px; margin-bottom: 25px;'>\n")
  cat(params$ai_summary)
  cat("\n</div>\n\n")
}
```

## R Console Output

<div style="font-family: Consolas, 'Courier New', monospace; font-size: 10px; white-space: pre-wrap; background-color: #f8f9fa; padding: 10px; border: 1px solid #ccc; border-radius: 4px; line-height: 1.2;">
`r params$console_out`
</div>

## References

* Collins, L. M., & Lanza, S. T. (2010). *Latent class and latent transition analysis: With applications in the social, behavioral, and health sciences*. Wiley.
* Djidu, H., Retnawati, H., Hadi, S., & Haryanto. (2026). projectLSA: A Shiny application for integrated latent structure analysis. *Applied Psychological Measurement*. https://doi.org/10.1177/01466216261446305
* Djidu, H., Retnawati, H., Hadi, S., & Haryanto. (2026). *projectLSA: R Shiny application for latent structure analysis with a graphical user interface*. https://doi.org/10.32614/CRAN.package.projectLSA. R package.
* Linzer, D. A., & Lewis, J. B. (2011). poLCA: An R package for polytomous variable latent class analysis. *Journal of Statistical Software*, 42(10), 1-29.
* Nylund, K. L., Asparouhov, T., & Muthen, B. O. (2007). Deciding on the number of classes in latent class analysis and growth mixture modeling: A Monte Carlo simulation study. *Structural Equation Modeling*, 14(4), 535-569.

***
*Generated by projectLSA*

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/projectLSA-package.R, R/zzz.R
\docType{package}
\name{projectLSA-package}
\alias{projectLSA-package}
\title{Latent Structure Analysis Toolkit}
\description{
Provides an interactive Shiny-based environment for performing Latent Profile
Analysis (LPA), Latent Class Analysis (LCA), Latent Trait Analysis (LTA; IRT),
Exploratory Factor Analysis (EFA), Confirmatory Factor Analysis (CFA), and
Structural Equation Modeling (SEM). All analyses are conducted through a
graphical user interface without requiring users to write code.

This file records package-level imports for roxygen2 so that NAMESPACE
will include the appropriate import directives.
}
\section{Main Function}{

The primary entry point is \code{\link{run_projectLSA}}, which launches the
full interactive application.
}

\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/hdmeasure/projectLSA}
  \item Report bugs at \url{https://github.com/hdmeasure/projectLSA/issues}
}

}
\author{
\strong{Maintainer}: Hasan Djidu \email{hasandjidu@gmail.com} (\href{https://orcid.org/0000-0003-1110-6815}{ORCID})

Authors:
\itemize{
  \item Hasan Djidu \email{hasandjidu@gmail.com} (\href{https://orcid.org/0000-0003-1110-6815}{ORCID})
}

Other contributors:
\itemize{
  \item Heri Retnawati (\href{https://orcid.org/0000-0002-1792-5873}{ORCID}) [contributor]
  \item Samsul Hadi (\href{https://orcid.org/0000-0003-3437-2542}{ORCID}) [contributor]
  \item Haryanto (\href{https://orcid.org/0000-0003-3322-904X}{ORCID}) [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_app.R
\name{run_projectLSA}
\alias{run_projectLSA}
\title{Launch the projectLSA Shiny Application}
\usage{
run_projectLSA()
}
\value{
Launches a Shiny application (no return value)
}
\description{
This function starts the Shiny app included in the projectLSA package.
}
\examples{
if (interactive()) {
  projectLSA::run_projectLSA()
}

}
